@@ -74,11 +74,11 @@ struct animmodel : model
         part *owner;
         Texture *tex, *masks, *envmap, *unlittex, *normalmap;
         Shader *shader;
-        float spec, ambient, glow, specglare, glowglare, fullbright, envmapmin, envmapmax, scrollu, scrollv, alphatest;
+        float spec, ambient, glow, glowdelta, glowpulse, specglare, glowglare, fullbright, envmapmin, envmapmax, scrollu, scrollv, alphatest;
         bool alphablend, cullface;
         int material, material2;
 
-        skin() : owner(0), tex(notexture), masks(notexture), envmap(NULL), unlittex(NULL), normalmap(NULL), shader(NULL), spec(1.0f), ambient(0.3f), glow(3.0f), specglare(1), glowglare(1), fullbright(0), envmapmin(0), envmapmax(0), scrollu(0), scrollv(0), alphatest(0.9f), alphablend(true), cullface(true), material(0), material2(0) {}
+        skin() : owner(0), tex(notexture), masks(notexture), envmap(NULL), unlittex(NULL), normalmap(NULL), shader(NULL), spec(1.0f), ambient(0.3f), glow(3.0f), glowdelta(0), glowpulse(0), specglare(1), glowglare(1), fullbright(0), envmapmin(0), envmapmax(0), scrollu(0), scrollv(0), alphatest(0.9f), alphablend(true), cullface(true), material(0), material2(0) {}
 
         bool multitextured() { return enableglow; }
         bool envmapped() { return hasCM && envmapmax>0 && envmapmodels && (renderpath!=R_FIXEDFUNCTION || maxtmus >= 3); }
@@ -163,16 +163,23 @@ struct animmodel : model
             }
             else
             {
-                float mincolor = as->cur.anim&ANIM_FULLBRIGHT ? fullbrightmodels/100.0f : 0.0f, 
-                      bias = max(mincolor-1.0f, 0.2f), scale = 0.5f*max(0.8f-bias, 0.0f), 
+                float mincolor = as->cur.anim&ANIM_FULLBRIGHT ? fullbrightmodels/100.0f : 0.0f,
+                      bias = max(mincolor-1.0f, 0.2f), scale = 0.5f*max(0.8f-bias, 0.0f),
                       minshade = scale*max(ambient, mincolor);
                 vec color = vec(lightcolor).max(mincolor);
                 glColor4f(color.x, color.y, color.z, trans);
                 setenvparamf("lightscale", SHPARAM_VERTEX, 2, scale - minshade, scale, minshade + bias);
                 setenvparamf("lightscale", SHPARAM_PIXEL, 2, scale - minshade, scale, minshade + bias);
             }
-            setenvparamf("maskscale", SHPARAM_PIXEL, 4, 0.5f*spec*lightmodels, 0.5f*glow*glowmodels, 16*specglare, 4*glowglare);
-            vec matcolor = material > 0 && lightmaterial ? lightmaterial[min(material, int(MAXLIGHTMATERIALS))-1].tocolor().mul(2) : vec(2, 2, 2), 
+            float curglow = glow;
+            if(glowpulse > 0)
+            {
+                float curpulse = lastmillis*glowpulse;
+                curpulse -= floor(curpulse);
+                curglow += glowdelta*2*fabs(curpulse - 0.5f);
+            }
+            setenvparamf("maskscale", SHPARAM_PIXEL, 4, 0.5f*spec*lightmodels, 0.5f*curglow*glowmodels, 16*specglare, 4*glowglare);
+            vec matcolor = material > 0 && lightmaterial ? lightmaterial[min(material, int(MAXLIGHTMATERIALS))-1].tocolor().mul(2) : vec(2, 2, 2),
                 matcolor2 = material2 > 0 && lightmaterial ? lightmaterial[min(material2, int(MAXLIGHTMATERIALS))-1].tocolor().mul(2) : vec(2, 2, 2);
             setenvparamf("lightmaterial", SHPARAM_PIXEL, 6, matcolor.x, matcolor.y, matcolor.z, 1);
             setenvparamf("lightmaterial2", SHPARAM_PIXEL, 7, matcolor2.x, matcolor2.y, matcolor2.z, 1);
@@ -267,18 +274,19 @@ struct animmodel : model
                 setshaderparams(b, as, m!=notexture, trans);
                 setshader(b, as, m!=notexture);
             }
+            int activetmu = 0;
             if(s!=lasttex)
             {
-                if(enableglow) glActiveTexture_(GL_TEXTURE1_ARB);
+                if(enableglow) { glActiveTexture_(GL_TEXTURE1_ARB); activetmu = 1; }
                 glBindTexture(GL_TEXTURE_2D, s->id);
-                if(enableglow) glActiveTexture_(GL_TEXTURE0_ARB);
                 lasttex = s;
             }
             if(n && n!=lastnormalmap)
             {
                 glActiveTexture_(GL_TEXTURE3_ARB);
+                activetmu = 3;
                 glBindTexture(GL_TEXTURE_2D, n->id);
-                glActiveTexture_(GL_TEXTURE0_ARB);
+                lastnormalmap = n;
             }
             if(trans < 1 && !enablealphablend)
             {
@@ -316,9 +324,9 @@ struct animmodel : model
             }
             if(m!=lastmasks && m!=notexture)
             {
-                if(!enableglow) glActiveTexture_(GL_TEXTURE1_ARB);
+                if(!enableglow) { glActiveTexture_(GL_TEXTURE1_ARB); activetmu = 1; }
+                else if(activetmu != 0) { glActiveTexture_(GL_TEXTURE0_ARB); activetmu = 0; }
                 glBindTexture(GL_TEXTURE_2D, m->id);
-                if(!enableglow) glActiveTexture_(GL_TEXTURE0_ARB);
                 lastmasks = m;
             }
             if((renderpath!=R_FIXEDFUNCTION || m!=notexture) && envmaptmu>=0 && envmapmax>0)
@@ -327,6 +335,7 @@ struct animmodel : model
                 if(!enableenvmap || lastenvmaptex!=emtex)
                 {
                     glActiveTexture_(GL_TEXTURE0_ARB+envmaptmu);
+                    activetmu = envmaptmu;
                     if(!enableenvmap)
                     {
                         glEnable(GL_TEXTURE_CUBE_MAP_ARB);
@@ -343,10 +352,10 @@ struct animmodel : model
                         if(!enablerescale) { glEnable(hasRN ? GL_RESCALE_NORMAL_EXT : GL_NORMALIZE); enablerescale = true; }
                     }
                     if(lastenvmaptex!=emtex) { glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, emtex); lastenvmaptex = emtex; }
-                    glActiveTexture_(GL_TEXTURE0_ARB);
                 }
             }
-            else if(enableenvmap) disableenvmap();
+            else if(enableenvmap) { disableenvmap(); activetmu = 0; }
+            if(activetmu != 0) glActiveTexture_(GL_TEXTURE0_ARB);
         }
     };
 
@@ -498,10 +507,8 @@ struct animmodel : model
         int shared;
         char *name;
         vector<mesh *> meshes;
-        float scale;
-        vec translate;
 
-        meshgroup() : next(NULL), shared(0), name(NULL), scale(1), translate(0, 0, 0)
+        meshgroup() : next(NULL), shared(0), name(NULL)
         {
         }
 
@@ -525,20 +532,22 @@ struct animmodel : model
             loopv(meshes) meshes[i]->gentris(frame, skins[i].tex && skins[i].tex->type&Texture::ALPHA ? skins[i].tex : NULL, tris, m);
         }
 
+        virtual void *animkey() { return this; }
         virtual int totalframes() const { return 1; }
         bool hasframe(int i) const { return i>=0 && i<totalframes(); }
         bool hasframes(int i, int n) const { return i>=0 && i+n<=totalframes(); }
         int clipframes(int i, int n) const { return min(n, totalframes() - i); }
 
         virtual void cleanup() {}
+        virtual void preload(part *p) {}
         virtual void render(const animstate *as, float pitch, const vec &axis, const vec &forward, dynent *d, part *p, modelattach *attached) {}
     };
 
-    virtual meshgroup *loadmeshes(char *name, va_list args) { return NULL; }
+    virtual meshgroup *loadmeshes(const char *name, va_list args) { return NULL; }
 
-    meshgroup *sharemeshes(char *name, ...)
+    meshgroup *sharemeshes(const char *name, ...)
     {
-        static hashtable<char *, meshgroup *> meshgroups;
+        static hashtable<const char *, meshgroup *> meshgroups;
         if(!meshgroups.access(name))
         {
             va_list args;
@@ -555,11 +564,11 @@ struct animmodel : model
     {
         part *p;
         int tag, anim, basetime;
-        vec translate;
+        vec translate, rotate;
         modelattach *attached;
         glmatrixf matrix;
 
-        linkedpart() : p(NULL), tag(-1), anim(-1), basetime(0), translate(0, 0, 0), attached(NULL) {}
+        linkedpart() : p(NULL), tag(-1), anim(-1), basetime(0), translate(0, 0, 0), rotate(0, 0, 0), attached(NULL) {}
     };
 
     struct part
@@ -618,12 +627,12 @@ struct animmodel : model
             }
         }
 
-        bool link(part *p, const char *tag, const vec &translate = vec(0, 0, 0), int anim = -1, int basetime = 0, modelattach *attached = NULL)
+        bool link(part *p, const char *tag, const vec &translate = vec(0, 0, 0), const vec &rotate = vec(0, 0, 0), int anim = -1, int basetime = 0, modelattach *attached = NULL)
         {
             int i = meshes ? meshes->findtag(tag) : -1;
             if(i<0)
             {
-                loopv(links) if(links[i].p && links[i].p->link(p, tag, translate, anim, basetime, attached)) return true;
+                loopv(links) if(links[i].p && links[i].p->link(p, tag, translate, rotate, anim, basetime, attached)) return true;
                 return false;
             }
             linkedpart &l = links.add();
@@ -632,6 +641,7 @@ struct animmodel : model
             l.anim = anim;
             l.basetime = basetime;
             l.translate = translate;
+            l.rotate = rotate;
             l.attached = attached;
             return true;
         }
@@ -669,6 +679,11 @@ struct animmodel : model
             loopv(skins) skins[i].preloadshader();
         }
 
+        void preloadmeshes()
+        {
+            if(meshes) meshes->preload(this);
+        }
+
         virtual void getdefaultanim(animinfo &info, int anim, int basetime, int basetime2, uint varseed, dynent *d)
         {
             info.frame = 0;
@@ -744,12 +759,13 @@ struct animmodel : model
             {
                 animinterpinfo &ai = d->animinterp[interp];
                 if((info.anim&ANIM_CLAMP)==ANIM_CLAMP) aitime = min(aitime, int(info.range*info.speed*0.5e-3f));
+                void *ak = meshes->animkey();
                 if(d->ragdoll && !(anim&ANIM_RAGDOLL))
                 {
                     ai.prev.range = ai.cur.range = 0;
                     ai.lastswitch = -1;
                 }
-                else if(ai.lastmodel!=this || ai.lastswitch<0 || lastmillis-d->lastrendered>aitime)
+                else if(ai.lastmodel!=ak || ai.lastswitch<0 || lastmillis-d->lastrendered>aitime)
                 {
                     ai.prev = ai.cur = info;
                     ai.lastswitch = lastmillis-aitime*2;
@@ -761,7 +777,7 @@ struct animmodel : model
                     ai.lastswitch = lastmillis;
                 }
                 else if(info.anim&ANIM_SETTIME) ai.cur.basetime = info.basetime;
-                ai.lastmodel = this;
+                ai.lastmodel = ak;
             }
             return true;
         }
@@ -849,7 +865,10 @@ struct animmodel : model
                 loopv(links)
                 {
                     linkedpart &link = links[i];
-                    link.matrix.transformedtranslate(links[i].translate, resize);
+                    link.matrix.transformedtranslate(link.translate, resize);
+                    if(link.rotate.x) link.matrix.rotate_around_z(link.rotate.x*RAD);
+                    if(link.rotate.z) link.matrix.rotate_around_x(-link.rotate.z*RAD);
+                    if(link.rotate.y) link.matrix.rotate_around_y(link.rotate.y*RAD);
 
                     matrixpos++;
                     matrixstack[matrixpos].mul(matrixstack[matrixpos-1], link.matrix);
@@ -919,14 +938,14 @@ struct animmodel : model
                 animmodel *m = (animmodel *)a[i].m;
                 if(!m || !m->loaded)
                 {
-                    if(a[i].pos) link(NULL, a[i].tag, vec(0, 0, 0), 0, 0, &a[i]);
+                    if(a[i].pos) link(NULL, a[i].tag, vec(0, 0, 0), vec(0, 0, 0), 0, 0, &a[i]);
                     continue;
                 }
                 part *p = m->parts[0];
                 switch(linktype(m))
                 {
                     case LINK_TAG:
-                        p->index = link(p, a[i].tag, vec(0, 0, 0), a[i].anim, a[i].basetime, &a[i]) ? index : -1;
+                        p->index = link(p, a[i].tag, vec(0, 0, 0), vec(m->offsetyaw + m->spinyaw*lastmillis/1000.0f, m->offsetpitch + m->spinpitch*lastmillis/1000.0f, m->offsetroll + m->spinroll*lastmillis/1000.0f), a[i].anim, a[i].basetime, &a[i]) ? index : -1;
                         break;
 
                     case LINK_COOP:
@@ -975,8 +994,9 @@ struct animmodel : model
     {
         if(!loaded) return;
 
-        yaw += spin*lastmillis/1000.0f;
-        pitch += offsetpitch;
+        yaw += spinyaw*lastmillis/1000.0f;
+        roll += spinroll*lastmillis/1000.0f;
+        pitch += offsetpitch + spinpitch*lastmillis/1000.0f;
 
         vec axis(0, -1, 0), forward(1, 0, 0);
 
@@ -1109,7 +1129,7 @@ struct animmodel : model
         m.identity();
         if(offsetyaw) m.rotate_around_z(offsetyaw*RAD);
         if(offsetroll) m.rotate_around_x(-offsetroll*RAD);
-        if(offsetpitch) m.rotate_around_y(-offsetpitch*RAD);
+        if(offsetpitch) m.rotate_around_y(offsetpitch*RAD);
     }
 
     void gentris(int frame, vector<BIH::tri> *tris)
@@ -1135,10 +1155,10 @@ struct animmodel : model
         return bih;
     }
 
-    bool link(part *p, const char *tag, const vec &translate = vec(0, 0, 0), int anim = -1, int basetime = 0, modelattach *attached = NULL)
+    bool link(part *p, const char *tag, const vec &translate = vec(0, 0, 0), const vec &rotate = vec(0, 0, 0), int anim = -1, int basetime = 0, modelattach *attached = NULL)
     {
         if(parts.empty()) return false;
-        return parts[0]->link(p, tag, translate, anim, basetime, attached);
+        return parts[0]->link(p, tag, translate, rotate, anim, basetime, attached);
     }
 
     bool unlink(part *p)
@@ -1163,6 +1183,11 @@ struct animmodel : model
         loopv(parts) parts[i]->preloadshaders();
     }
 
+    void preloadmeshes()
+    {
+        loopv(parts) parts[i]->preloadmeshes();
+    }
+
     void setshader(Shader *shader)
     {
         if(parts.empty()) loaddefaultparts();
@@ -1196,10 +1221,16 @@ struct animmodel : model
         loopv(parts) loopvj(parts[i]->skins) parts[i]->skins[j].ambient = ambient;
     }
 
-    void setglow(float glow)
+    void setglow(float glow, float delta, float pulse)
     {
         if(parts.empty()) loaddefaultparts();
-        loopv(parts) loopvj(parts[i]->skins) parts[i]->skins[j].glow = glow;
+        loopv(parts) loopvj(parts[i]->skins)
+        {
+            skin &s = parts[i]->skins[j];
+            s.glow = glow;
+            s.glowdelta = delta;
+            s.glowpulse = pulse;
+        }
     }
 
     void setglare(float specglare, float glowglare)
@@ -1240,7 +1271,7 @@ struct animmodel : model
     void setmaterial(int material, int material2)
     {
         if(parts.empty()) loaddefaultparts();
-        loopv(parts) loopvj(parts[i]->skins) 
+        loopv(parts) loopvj(parts[i]->skins)
         {
             skin &s = parts[i]->skins[j];
             s.material = material;
@@ -1472,12 +1503,13 @@ template<class MDL, class MESH> struct modelcommands
         loopskins(meshname, s, s.ambient = ambient);
     }
 
-    static void setglow(char *meshname, int *percent)
+    static void setglow(char *meshname, int *percent, int *delta, float *pulse)
     {
-        float glow = 3.0f;
+        float glow = 3.0f, glowdelta = *delta/100.0f, glowpulse = *pulse > 0 ? *pulse/1000.0f : 0;
         if(*percent>0) glow = *percent/100.0f;
         else if(*percent<0) glow = 0.0f;
-        loopskins(meshname, s, s.glow = glow);
+        glowdelta -= glow;
+        loopskins(meshname, s, { s.glow = glow; s.glowdelta = glowdelta; s.glowpulse = glowpulse; });
     }
 
     static void setglare(char *meshname, float *specglare, float *glowglare)
@@ -1539,11 +1571,11 @@ template<class MDL, class MESH> struct modelcommands
         loopskins(meshname, s, { s.material = clamp(*material, 0, int(MAXLIGHTMATERIALS)); s.material2 = clamp(*material2, 0, int(MAXLIGHTMATERIALS)); });
     }
 
-    static void setlink(int *parent, int *child, char *tagname, float *x, float *y, float *z)
+    static void setlink(int *parent, int *child, char *tagname, float *x, float *y, float *z, float *yaw, float *pitch, float *roll)
     {
         if(!MDL::loading) { conoutf("\frnot loading an %s", MDL::formatname()); return; }
         if(!MDL::loading->parts.inrange(*parent) || !MDL::loading->parts.inrange(*child)) { conoutf("\frno models loaded to link"); return; }
-        if(!MDL::loading->parts[*parent]->link(MDL::loading->parts[*child], tagname, vec(*x, *y, *z))) conoutf("\frcould not link model %s", MDL::loading->loadname);
+        if(!MDL::loading->parts[*parent]->link(MDL::loading->parts[*child], tagname, vec(*x, *y, *z), vec(*yaw, *pitch, *roll))) conoutf("\frcould not link model %s", MDL::loading->loadname);
     }
 
     template<class F> void modelcommand(F *fun, const char *suffix, const char *args)
@@ -1560,7 +1592,7 @@ template<class MDL, class MESH> struct modelcommands
             modelcommand(setskin, "skin", "sssff");
             modelcommand(setspec, "spec", "si");
             modelcommand(setambient, "ambient", "si");
-            modelcommand(setglow, "glow", "si");
+            modelcommand(setglow, "glow", "siif");
             modelcommand(setglare, "glare", "sff");
             modelcommand(setalphatest, "alphatest", "sf");
             modelcommand(setalphablend, "alphablend", "si");
@@ -1573,7 +1605,7 @@ template<class MDL, class MESH> struct modelcommands
             modelcommand(setnoclip, "noclip", "si");
             modelcommand(setmaterial, "material", "sii");
         }
-        if(MDL::multiparted()) modelcommand(setlink, "link", "iisfff");
+        if(MDL::multiparted()) modelcommand(setlink, "link", "iisffffff");
     }
 };
 
