@@ -6,7 +6,7 @@ vector<cline> conlines;
 int commandmillis = -1;
 string commandbuf;
 char *commandaction = NULL, *commandicon = NULL;
-enum { CF_COMPLETE = 1<<0, CF_EXECUTE = 1<<1 };
+enum { CF_COMPLETE = 1<<0, CF_EXECUTE = 1<<1, CF_MESSAGE = 1<<2 };
 int commandflags = 0, commandpos = -1, commandcolour = 0;
 
 void conline(int type, const char *sf, int n)
@@ -65,16 +65,6 @@ struct keym
     ~keym() { DELETEA(name); loopi(NUMACTIONS) { DELETEA(actions[i]); persist[i] = false; } }
 };
 
-/*
-extern keym *keypressed;
-extern char *keyaction;
-extern keym *findbind(const char *key);
-extern int findactionkey(const char *action, int which, int num);
-
-extern const char *retbind(const char *key, int which);
-extern const char *retbindaction(const char *action, int which, int num);
-*/
-
 hashtable<int, keym> keyms(128);
 
 void keymap(int *code, char *key)
@@ -257,7 +247,7 @@ void bindkey(char *key, char *action, int state, const char *cmd)
     int len = strlen(action);
     while(len>0 && iscubespace(action[len-1])) len--;
     binding = newstring(action, len);
-    *persist = interactive;
+    *persist = initing != INIT_DEFAULTS;
     changedkeys = totalmillis;
 }
 
@@ -293,7 +283,8 @@ void inputcommand(char *init, char *action = NULL, char *icon = NULL, int colour
     {
         case 'c': commandflags |= CF_COMPLETE; break;
         case 'x': commandflags |= CF_EXECUTE; break;
-        case 's': commandflags |= CF_COMPLETE|CF_EXECUTE; break;
+        case 'm': commandflags |= CF_MESSAGE; break;
+        case 's': commandflags |= CF_COMPLETE|CF_EXECUTE|CF_MESSAGE; break;
     }
     else if(init) commandflags |= CF_COMPLETE|CF_EXECUTE;
 }
@@ -502,6 +493,26 @@ void execbind(keym &k, bool isdown)
     k.pressed = isdown;
 }
 
+void processkey(int code, bool isdown, int cooked)
+{
+    resetcomplete();
+    if(cooked)
+    {
+        size_t len = (int)strlen(commandbuf), maxlen = sizeof(commandbuf);
+        if(commandflags&CF_MESSAGE) maxlen = min((size_t)client::maxmsglen(), maxlen);
+        if(len+1 < maxlen)
+        {
+            if(commandpos<0) commandbuf[len] = cooked;
+            else
+            {
+                memmove(&commandbuf[commandpos+1], &commandbuf[commandpos], len - commandpos);
+                commandbuf[commandpos++] = cooked;
+            }
+            commandbuf[len+1] = '\0';
+        }
+    }
+}
+
 void consolekey(int code, bool isdown, int cooked)
 {
     if(isdown)
@@ -569,41 +580,15 @@ void consolekey(int code, bool isdown, int cooked)
 
             case SDLK_f:
                 if(SDL_GetModState()&MOD_KEYS) cooked = '\f';
-                // fall through
+                else processkey(code, isdown, cooked);
+                break;
 
             case SDLK_v:
-                if(SDL_GetModState()&MOD_KEYS)
-                {
-                    pasteconsole();
-                    break;
-                }
-                // fall through
-
-            case SDLK_F4:
-                if(SDL_GetModState()&MOD_KEYS)
-                {
-                    quit();
-                    break;
-                }
-                // fall through
-
-            default:
-                resetcomplete();
-                if(cooked)
-                {
-                    size_t len = (int)strlen(commandbuf);
-                    if(len+1<sizeof(commandbuf))
-                    {
-                        if(commandpos<0) commandbuf[len] = cooked;
-                        else
-                        {
-                            memmove(&commandbuf[commandpos+1], &commandbuf[commandpos], len - commandpos);
-                            commandbuf[commandpos++] = cooked;
-                        }
-                        commandbuf[len+1] = '\0';
-                    }
-                }
+                if(SDL_GetModState()&MOD_KEYS) pasteconsole();
+                else processkey(code, isdown, cooked);
                 break;
+
+            default: processkey(code, isdown, cooked); break;
         }
     }
     else
@@ -641,8 +626,43 @@ void consolekey(int code, bool isdown, int cooked)
     }
 }
 
+#define keyintercept(name,body) \
+{ \
+    static bool key##name = false; \
+    if(SDL_GetModState()&MOD_ALTS && isdown) \
+    { \
+        if(!key##name) \
+        { \
+            body; \
+            key##name = true; \
+        } \
+        return; \
+    } \
+    if(!isdown && key##name) \
+    { \
+        key##name = false; \
+        return; \
+    } \
+}
 void keypress(int code, bool isdown, int cooked)
 {
+    switch(code)
+    {
+#ifdef __APPLE__
+        case SDLK_q:
+#else
+        case SDLK_F4:
+#endif
+            keyintercept(quit, quit());
+            break;
+        case SDLK_RETURN:
+            keyintercept(fullscreen, setfullscreen(!fullscreen, true));
+            break;
+        case SDLK_TAB:
+            keyintercept(iconify, SDL_WM_IconifyWindow());
+            break;
+        default: break;
+    }
     keym *haskey = keyms.access(code);
     if(haskey && haskey->pressed) execbind(*haskey, isdown); // allow pressed keys to release
     else if(commandmillis > 0) consolekey(code, isdown, cooked);
@@ -676,9 +696,10 @@ void writebinds(stream *f)
         loopv(binds)
         {
             keym &km = *binds[i];
-            if(km.persist[j] && *km.actions[j])
+            if(km.persist[j])
             {
-                if(validateblock(km.actions[j])) f->printf("%s %s [%s]\n", cmds[j], escapestring(km.name), km.actions[j]);
+                if(!*km.actions[j]) f->printf("%s %s []\n", cmds[j], escapestring(km.name));
+                else if(validateblock(km.actions[j])) f->printf("%s %s [%s]\n", cmds[j], escapestring(km.name), km.actions[j]);
                 else f->printf("%s %s %s\n", cmds[j], escapestring(km.name), escapestring(km.actions[j]));
                 found = true;
             }
@@ -736,7 +757,7 @@ static inline uint hthash(const fileskey &k)
 static hashtable<fileskey, filesval *> completefiles;
 static hashtable<char *, filesval *> completions;
 
-int completesize = 0;
+int completeoffset = -1, completesize = 0;
 string lastcomplete;
 
 void resetcomplete() { completesize = 0; }
@@ -756,12 +777,11 @@ void addcomplete(char *command, int type, char *dir, char *ext)
     }
     if(type==FILES_DIR)
     {
-    int dirlen = (int)strlen(dir);
-    while(dirlen > 0 && (dir[dirlen-1] == '/' || dir[dirlen-1] == '\\'))
-        dir[--dirlen] = '\0';
+        int dirlen = (int)strlen(dir);
+        while(dirlen > 0 && (dir[dirlen-1] == '/' || dir[dirlen-1] == '\\')) dir[--dirlen] = '\0';
         if(ext)
         {
-    if(strchr(ext, '*')) ext[0] = '\0';
+            if(strchr(ext, '*')) ext[0] = '\0';
             if(!ext[0]) ext = NULL;
         }
     }
@@ -794,51 +814,67 @@ COMMANDN(0, listcomplete, addlistcomplete, "ss");
 
 void complete(char *s, const char *cmdprefix)
 {
-    int cmdlen = 0;
+    char *start = s;
     if(cmdprefix)
     {
-        cmdlen = strlen(cmdprefix);
+        int cmdlen = strlen(cmdprefix);
         if(strncmp(s, cmdprefix, cmdlen))
         {
-            defformatstring(cmd)("%s%s", cmdprefix, s);
+            defformatstring(cmd)("%s%s", cmdprefix, start);
             copystring(s, cmd);
         }
+        start = &s[cmdlen];
+    }
+    const char chrlist[7] = { ';', '(', ')', '[', ']', '\"', '$', };
+    bool variable = false;
+    loopi(7)
+    {
+        char *semi = strrchr(start, chrlist[i]);
+        if(semi)
+        {
+            start = semi+1;
+            if(chrlist[i] == '$') variable = true;
+        }
+    }
+    while(*start == ' ') start++;
+    if(!start[0]) return;
+    if(start-s != completeoffset || !completesize)
+    {
+        completeoffset = start-s;
+        completesize = (int)strlen(start);
+        lastcomplete[0] = '\0';
     }
-    if(!s[cmdlen]) return;
-    if(!completesize) { completesize = (int)strlen(&s[cmdlen]); lastcomplete[0] = '\0'; }
-
     filesval *f = NULL;
     if(completesize)
     {
-        char *end = strchr(&s[cmdlen], ' ');
+        char *end = strchr(start, ' ');
         if(end)
         {
             string command;
-            copystring(command, &s[cmdlen], min(size_t(end-&s[cmdlen]+1), sizeof(command)));
+            copystring(command, start, min(size_t(end-start+1), sizeof(command)));
             filesval **hasfiles = completions.access(command);
             if(hasfiles) f = *hasfiles;
         }
     }
-
     const char *nextcomplete = NULL;
     string prefix;
     if(f) // complete using filenames
     {
-        int commandsize = strchr(&s[cmdlen], ' ')+1-s;
-        copystring(prefix, s, min(size_t(commandsize+1), sizeof(prefix)));
+        int commandsize = strchr(start, ' ')+1-start;
+        copystring(prefix, s, min(size_t(commandsize+1+(start-s)), sizeof(prefix)));
         f->update();
         loopv(f->files)
         {
-            if(strncmp(f->files[i], &s[commandsize], completesize+cmdlen-commandsize)==0 &&
+            if(strncmp(f->files[i], &start[commandsize], completesize-commandsize)==0 &&
                 strcmp(f->files[i], lastcomplete) > 0 && (!nextcomplete || strcmp(f->files[i], nextcomplete) < 0))
                 nextcomplete = f->files[i];
         }
     }
     else // complete using command names
     {
-        if(cmdprefix) copystring(prefix, cmdprefix); else prefix[0] = '\0';
+        copystring(prefix, s, min(size_t(1+(start-s)), sizeof(prefix)));
         enumerate(idents, ident, id,
-            if(id.flags&IDF_COMPLETE && strncmp(id.name, &s[cmdlen], completesize)==0 &&
+            if((variable ? id.type == ID_VAR || id.type == ID_SVAR || id.type == ID_FVAR || id.type == ID_ALIAS: id.flags&IDF_COMPLETE) && strncmp(id.name, start, completesize)==0 &&
                strcmp(id.name, lastcomplete) > 0 && (!nextcomplete || strcmp(id.name, nextcomplete) < 0))
                 nextcomplete = id.name;
         );
@@ -848,24 +884,29 @@ void complete(char *s, const char *cmdprefix)
         formatstring(s)("%s%s", prefix, nextcomplete);
         copystring(lastcomplete, nextcomplete);
     }
-    else lastcomplete[0] = '\0';
+    else
+    {
+        if((int)strlen(start) > completesize) start[completesize] = '\0';
+        completesize = 0;
+    }
 }
 
 void setidflag(const char *s, const char *v, int flag, const char *msg, bool alias)
 {
     ident *id = idents.access(s);
-    if(!id || (alias && id->type != ID_ALIAS)) conoutf("\fradding %s of %s failed as it is not available", msg, s);
-    else
+    if(!id || (alias && id->type != ID_ALIAS))
     {
-        bool on = false;
-        if(isnumeric(*v)) on = atoi(v) != 0;
-        else if(!strcasecmp("false", v)) on = false;
-        else if(!strcasecmp("true", v)) on = true;
-        else if(!strcasecmp("on", v)) on = true;
-        else if(!strcasecmp("off", v)) on = false;
-        if(on && !(id->flags&flag)) id->flags |= flag;
-        else if(!on && id->flags&flag) id->flags &= ~flag;
+        if(verbose) conoutf("\fradding %s of %s failed as it is not available", msg, s);
+        return;
     }
+    bool on = false;
+    if(isnumeric(*v)) on = atoi(v) != 0;
+    else if(!strcasecmp("false", v)) on = false;
+    else if(!strcasecmp("true", v)) on = true;
+    else if(!strcasecmp("on", v)) on = true;
+    else if(!strcasecmp("off", v)) on = false;
+    if(on && !(id->flags&flag)) id->flags |= flag;
+    else if(!on && id->flags&flag) id->flags &= ~flag;
 }
 
 ICOMMAND(0, setcomplete, "ss", (char *s, char *t), setidflag(s, t, IDF_COMPLETE, "complete", false));
@@ -874,14 +915,15 @@ ICOMMAND(0, setpersist, "ss", (char *s, char *t), setidflag(s, t, IDF_PERSIST, "
 void setiddesc(const char *s, const char *v, const char *u)
 {
     ident *id = idents.access(s);
-    if(!id) conoutf("\fradding description of %s failed as it is not available", s);
-    else
+    if(!id)
     {
-        DELETEA(id->desc);
-        if(v && *v) id->desc = newstring(v);
-        DELETEA(id->usage);
-        if(u && *u) id->usage = newstring(u);
+        if(verbose) conoutf("\fradding description of %s failed as it is not available", s);
+        return;
     }
+    DELETEA(id->desc);
+    if(v && *v) id->desc = newstring(v);
+    DELETEA(id->usage);
+    if(u && *u) id->usage = newstring(u);
 }
 ICOMMAND(0, setdesc, "sss", (char *s, char *t, char *u), setiddesc(s, t, u));
 
