@@ -3,31 +3,25 @@
 
 #include "cube.h"
 
-#define RE_VER_MAJOR        1
-#define RE_VER_MINOR        3
-#define RE_VER_PATCH        1
-#define RE_VER_STR          "1.3.1"
-#define RE_VER_MAKE(a,b,c)  (((a)<<16) | ((b)<<8) | (c))
-#define RE_VER              RE_VER_MAKE(RE_VER_MAJOR, RE_VER_MINOR, RE_VER_PATCH)
-#define RE_VERSION          (RE_VER_MAJOR*100)+(RE_VER_MINOR*10)+RE_VER_PATCH
+#define LAN_PORT         28799
+#define MASTER_PORT      28800
+#define SERVER_PORT      28801
+
+extern bool versioning;
+extern int versionmajor, versionminor, versionpatch;
+extern char *versionstring, *versionname, *versionuname, *versionrelease, *versionurl, *versionmaster;
+#define CUR_VER_MAKE(a,b,c)  (((a)<<16) | ((b)<<8) | (c))
+#define CUR_VER              CUR_VER_MAKE(versionmajor, versionminor, versionpatch)
+#define CUR_VERSION          (versionmajor*100)+(versionminor*10)+versionpatch
 
-#define RE_NAME             "Red Eclipse"
-#define RE_UNAME            "redeclipse"
-#define RE_RELEASE          "Galactic"
-#define RE_URL              "www.redeclipse.net"
 #ifdef WIN32
-#define RE_PLATFORM         "win"
+#define CUR_PLATFORM         "win"
 #elif defined(__APPLE__)
-#define RE_PLATFORM         "mac"
+#define CUR_PLATFORM         "mac"
 #else
-#define RE_PLATFORM         "nix"
+#define CUR_PLATFORM         "nix"
 #endif
-#define RE_ARCH             (8*sizeof(void *))
-
-#define RE_LAN_PORT         28799
-#define RE_MASTER_PORT      28800
-#define RE_SERVER_PORT      28801
-#define RE_MASTER_HOST      "play.redeclipse.net"
+#define CUR_ARCH             (int(8*sizeof(void *)))
 
 #ifdef STANDALONE
 extern void setupmaster();
@@ -36,6 +30,7 @@ extern void cleanupmaster();
 extern void reloadmaster();
 extern int masterserver, masterport;
 extern char *masterip;
+extern int nextcontrolversion();
 #endif
 
 #include "irc.h"
@@ -44,15 +39,15 @@ extern char *masterip;
 enum { CON_DEBUG = 0, CON_MESG, CON_INFO, CON_SELF, CON_GAMESPECIFIC };
 
 enum { PACKAGEDIR_OCTA = 1<<0 };
-extern const char *disc_reasons[];
+extern const char * const disc_reasons[];
 struct ipinfo
 {
     enum { ALLOW = 0, BAN, MUTE, LIMIT, MAXTYPES };
     enum { TEMPORARY = 0, LOCAL, GLOBAL };
     enet_uint32 ip, mask;
-    int type, flag, time;
+    int type, flag, time, version;
 
-    ipinfo() : ip(0), mask(0), type(-1), flag(TEMPORARY), time(-1) {}
+    ipinfo() : ip(0), mask(0), type(-1), flag(TEMPORARY), time(-1), version(-1) {}
     ~ipinfo() {}
 };
 extern vector<ipinfo> control;
@@ -63,6 +58,8 @@ extern bool checkipinfo(vector<ipinfo> &info, int type, enet_uint32 ip);
 extern void writecfg();
 extern void rehash(bool reload = true);
 
+#define NZT(t) (t != 0 ? t : 1)
+
 #ifndef STANDALONE
 #include "world.h"
 #include "octa.h"
@@ -153,6 +150,20 @@ extern PFNGLGETCOMPRESSEDTEXIMAGEARBPROC   glGetCompressedTexImage_;
 // GL_EXT_fog_coord
 extern PFNGLFOGCOORDPOINTEREXTPROC glFogCoordPointer_;
 
+// GL_ARB_map_buffer_range
+#ifndef GL_ARB_map_buffer_range
+#define GL_MAP_READ_BIT                   0x0001
+#define GL_MAP_WRITE_BIT                  0x0002
+#define GL_MAP_INVALIDATE_RANGE_BIT       0x0004
+#define GL_MAP_INVALIDATE_BUFFER_BIT      0x0008
+#define GL_MAP_FLUSH_EXPLICIT_BIT         0x0010
+#define GL_MAP_UNSYNCHRONIZED_BIT         0x0020
+typedef GLvoid* (APIENTRYP PFNGLMAPBUFFERRANGEPROC) (GLenum target, GLintptr offset, GLsizeiptr length, GLbitfield access);
+typedef void (APIENTRYP PFNGLFLUSHMAPPEDBUFFERRANGEPROC) (GLenum target, GLintptr offset, GLsizeiptr length);
+#endif
+extern PFNGLMAPBUFFERRANGEPROC         glMapBufferRange_;
+extern PFNGLFLUSHMAPPEDBUFFERRANGEPROC glFlushMappedBufferRange_;
+
 #include "varray.h"
 
 extern physent *camera1, camera;
@@ -181,20 +192,21 @@ struct font
     char *name;
     vector<Texture *> texs;
     vector<charinfo> chars;
-    int charoffset, defaultw, defaulth, scale;
+    int charoffset, defaultw, defaulth, maxw, maxh, scale;
 
     font() : name(NULL) {}
     ~font() { DELETEA(name); }
 };
 
-#define FONTH (curfont->scale)
+extern float textscale;
+#define FONTH int(curfont->scale*textscale)
 #define FONTW (FONTH/2)
 #define FONTTAB (4*FONTW)
 
 extern font *curfont;
 
 // texture
-extern int hwtexsize, hwcubetexsize, hwmaxaniso, aniso, envmapradius;
+extern int hwtexsize, hwcubetexsize, hwmaxaniso, maxtexsize, aniso, envmapradius;
 
 extern Texture *textureload(const char *name, int clamp = 0, bool mipit = true, bool msg = true);
 extern int texalign(void *data, int w, int bpp);
@@ -259,10 +271,11 @@ static inline bool pvsoccluded(const ivec &bborigin, int size)
 }
 
 // rendergl
-extern bool hasVBO, hasDRE, hasOQ, hasTR, hasFBO, hasDS, hasTF, hasBE, hasBC, hasCM, hasNP2, hasTC, hasTE, hasMT, hasD3, hasAF, hasVP2, hasVP3, hasPP, hasMDA, hasTE3, hasTE4, hasVP, hasFP, hasGLSL, hasGM, hasNVFB, hasSGIDT, hasSGISH, hasDT, hasSH, hasNVPCF, hasRN, hasPBO, hasFBB, hasUBO, hasBUE, hasFC, hasTEX;
+extern bool hasVBO, hasDRE, hasOQ, hasTR, hasFBO, hasDS, hasTF, hasBE, hasBC, hasCM, hasNP2, hasTC, hasS3TC, hasFXT1, hasTE, hasMT, hasD3, hasAF, hasVP2, hasVP3, hasPP, hasMDA, hasTE3, hasTE4, hasVP, hasFP, hasGLSL, hasGM, hasNVFB, hasSGIDT, hasSGISH, hasDT, hasSH, hasNVPCF, hasRN, hasPBO, hasFBB, hasUBO, hasBUE, hasMBR, hasFC, hasTEX;
 extern int hasstencil;
+extern int glversion, glslversion;
 
-extern bool envmapping, minimapping, renderedgame, playerpreviewing;
+extern bool envmapping, minimapping, renderedgame, modelpreviewing;
 extern const glmatrixf viewmatrix;
 extern glmatrixf mvmatrix, projmatrix, mvpmatrix, invmvmatrix, invmvpmatrix, fogmatrix, invfogmatrix, envmatrix;
 extern bvec fogcolor;
@@ -292,12 +305,14 @@ extern void vecfromcursor(float x, float y, float z, vec &dir);
 extern bool vectocursor(const vec &v, float &x, float &y, float &z, float clampxy = -1);
 extern void findorientation(vec &o, float yaw, float pitch, vec &pos);
 extern void rendergame();
-extern void renderavatar(bool early);
+extern void renderavatar(bool early = false, bool project = false);
+extern void viewproject(float zscale = 1);
 extern void invalidatepostfx();
 extern void drawnoview();
 extern bool hasnoview();
 extern void gl_drawframe(int w, int h);
 extern void drawminimap();
+extern void drawtextures();
 extern void enablepolygonoffset(GLenum type);
 extern void disablepolygonoffset(GLenum type);
 extern void calcspherescissor(const vec &center, float size, float &sx1, float &sy1, float &sx2, float &sy2);
@@ -305,6 +320,12 @@ extern int pushscissor(float sx1, float sy1, float sx2, float sy2);
 extern void popscissor();
 extern void setcolormask(bool r = true, bool g = true, bool b = true);
 
+namespace modelpreview
+{
+    extern void start(bool background = true);
+    extern void end();
+}
+
 // renderextras
 extern void render3dbox(vec &o, float tofloor, float toceil, float xradius, float yradius = 0);
 extern void renderellipse(vec &o, float xradius, float yradius, float yaw);
@@ -316,47 +337,48 @@ extern void setcubeext(cube &c, cubeext *ext);
 extern cubeext *newcubeext(cube &c, int maxverts = 0, bool init = true);
 extern void getcubevector(cube &c, int d, int x, int y, int z, ivec &p);
 extern void setcubevector(cube &c, int d, int x, int y, int z, const ivec &p);
-extern int familysize(cube &c);
+extern int familysize(const cube &c);
 extern void freeocta(cube *c);
 extern void discardchildren(cube &c, bool fixtex = false, int depth = 0);
 extern void optiface(uchar *p, cube &c);
 extern void validatec(cube *c, int size = 0);
-extern bool isvalidcube(cube &c);
+extern bool isvalidcube(const cube &c);
 extern ivec lu;
 extern int lusize;
 extern cube &lookupcube(int tx, int ty, int tz, int tsize = 0, ivec &ro = lu, int &rsize = lusize);
-extern cube *neighbourstack[32];
+extern const cube *neighbourstack[32];
 extern int neighbourdepth;
-extern cube &neighbourcube(cube &c, int orient, int x, int y, int z, int size, ivec &ro = lu, int &rsize = lusize);
+extern const cube &neighbourcube(const cube &c, int orient, int x, int y, int z, int size, ivec &ro = lu, int &rsize = lusize);
 extern int lookupmaterial(const vec &o);
 extern void resetclipplanes();
-extern int getmippedtexture(cube &p, int orient);
+extern int getmippedtexture(const cube &p, int orient);
 extern void forcemip(cube &c, bool fixtex = true);
 extern bool subdividecube(cube &c, bool fullcheck=true, bool brighten=true);
 extern void edgespan2vectorcube(cube &c);
-extern int faceconvexity(ivec v[4]);
-extern int faceconvexity(ivec v[4], int &vis);
-extern int faceconvexity(vertinfo *verts, int numverts, int size);
-extern int faceconvexity(cube &c, int orient);
-extern void calcvert(cube &c, int x, int y, int z, int size, ivec &vert, int i, bool solid = false);
-extern void calcvert(cube &c, int x, int y, int z, int size, vec &vert, int i, bool solid = false);
-extern uint faceedges(cube &c, int orient);
-extern bool collapsedface(cube &c, int orient);
-extern bool touchingface(cube &c, int orient);
-extern bool flataxisface(cube &c, int orient);
-extern bool collideface(cube &c, int orient);
-extern int genclipplane(cube &c, int i, vec *v, plane *clip);
-extern void genclipplanes(cube &c, int x, int y, int z, int size, clipplanes &p);
-extern bool visibleface(cube &c, int orient, int x, int y, int z, int size, uchar mat = MAT_AIR, uchar nmat = MAT_AIR, uchar matmask = MATF_VOLUME);
-extern int visibletris(cube &c, int orient, int x, int y, int z, int size, uchar nmat = MAT_AIR, uchar matmask = MAT_AIR);
-extern int visibleorient(cube &c, int orient);
-extern void genfaceverts(cube &c, int orient, ivec v[4]);
+extern int faceconvexity(const ivec v[4]);
+extern int faceconvexity(const ivec v[4], int &vis);
+extern int faceconvexity(const vertinfo *verts, int numverts, int size);
+extern int faceconvexity(const cube &c, int orient);
+extern void calcvert(const cube &c, int x, int y, int z, int size, ivec &vert, int i, bool solid = false);
+extern void calcvert(const cube &c, int x, int y, int z, int size, vec &vert, int i, bool solid = false);
+extern uint faceedges(const cube &c, int orient);
+extern bool collapsedface(const cube &c, int orient);
+extern bool touchingface(const cube &c, int orient);
+extern bool flataxisface(const cube &c, int orient);
+extern bool collideface(const cube &c, int orient);
+extern int genclipplane(const cube &c, int i, vec *v, plane *clip);
+extern void genclipplanes(const cube &c, int x, int y, int z, int size, clipplanes &p, bool collide = true);
+extern bool visibleface(const cube &c, int orient, int x, int y, int z, int size, ushort mat = MAT_AIR, ushort nmat = MAT_AIR, ushort matmask = MATF_VOLUME);
+extern int classifyface(const cube &c, int orient, int x, int y, int z, int size);
+extern int visibletris(const cube &c, int orient, int x, int y, int z, int size, ushort nmat = MAT_ALPHA, ushort matmask = MAT_ALPHA);
+extern int visibleorient(const cube &c, int orient);
+extern void genfaceverts(const cube &c, int orient, ivec v[4]);
 extern int calcmergedsize(int orient, const ivec &co, int size, const vertinfo *verts, int numverts);
 extern void invalidatemerges(cube &c, const ivec &co, int size, bool msg);
 extern void calcmerges();
 
 extern int mergefaces(int orient, facebounds *m, int sz);
-extern void mincubeface(cube &cu, int orient, const ivec &o, int size, const facebounds &orig, facebounds &cf, uchar nmat = MAT_AIR, uchar matmask = MATF_VOLUME);
+extern void mincubeface(const cube &cu, int orient, const ivec &o, int size, const facebounds &orig, facebounds &cf, ushort nmat = MAT_AIR, ushort matmask = MATF_VOLUME);
 
 static inline uchar octantrectangleoverlap(const ivec &c, int size, const ivec &o, const ivec &s)
 {
@@ -396,7 +418,7 @@ extern void pasteundoents(undoblock *u);
 extern selinfo sel;
 extern int texpaneltimer;
 extern void cancelsel();
-extern void render_texture_panel(int w, int h);
+extern void rendertexturepanel(int w, int h);
 extern void addundo(undoblock *u);
 extern void commitchanges(bool force = false);
 extern editinfo *localedit;
@@ -418,6 +440,7 @@ extern void updatevabb(vtxarray *va, bool force = false);
 extern void updatevabbs(bool force = false);
 
 // renderva
+extern int outline, blankgeom;
 extern void visiblecubes(bool cull = true);
 extern void setvfcP(float z = -1, const vec &bbmin = vec(-1, -1, -1), const vec &bbmax = vec(1, 1, 1));
 extern void savevfcP();
@@ -459,23 +482,42 @@ extern bool getdynlight(int n, vec &o, float &radius, vec &color);
 
 extern int showmat;
 
-extern namemap materials[], textypes[];
 extern const char *findmaterialname(int type);
 extern int findmaterial(const char *name, bool tryint = false);
-extern void genmatsurfs(cube &c, int cx, int cy, int cz, int size, vector<materialsurface> &matsurfs);
+extern const char *getmaterialdesc(int mat, const char *prefix = "");
+extern void genmatsurfs(const cube &c, int cx, int cy, int cz, int size, vector<materialsurface> &matsurfs);
 extern void rendermatsurfs(materialsurface *matbuf, int matsurfs);
 extern void rendermatgrid(materialsurface *matbuf, int matsurfs);
 extern int optimizematsurfs(materialsurface *matbuf, int matsurfs);
 extern void setupmaterials(int start = 0, int len = 0);
 extern void rendermaterials();
-extern int visiblematerial(cube &c, int orient, int x, int y, int z, int size, uchar matmask = MATF_VOLUME);
+extern int visiblematerial(const cube &c, int orient, int x, int y, int z, int size, ushort matmask = MATF_VOLUME);
 
 // water
-extern int refracting;
+extern int refracting, refractfog;
+extern bvec refractcolor;
 extern bool reflecting, fading, fogging;
 extern float reflectz;
-extern int reflectdist, vertwater, waterrefract, waterreflect, waterfade, caustics, waterfallrefract, waterfog, lavafog;
-extern bvec watercol, waterfallcol, lavacol;
+extern int reflectdist, vertwater, waterrefract, waterreflect, waterfade, caustics, waterfallrefract;
+
+#define GETMATIDXVAR(name, var, type) \
+    type get##name##var(int mat) \
+    { \
+        switch(mat&MATF_INDEX) \
+        { \
+            default: case 0: return name##var; \
+            case 1: return name##2##var; \
+            case 2: return name##3##var; \
+            case 3: return name##4##var; \
+        } \
+    }
+
+extern const bvec &getwatercol(int mat);
+extern const bvec &getwaterfallcol(int mat);
+extern int getwaterfog(int mat);
+extern const bvec &getlavacol(int mat);
+extern int getlavafog(int mat);
+extern const bvec &getglasscol(int mat);
 
 extern void cleanreflections();
 extern void queryreflections();
@@ -497,6 +539,7 @@ extern bool depthfxing;
 extern void drawdepthfxtex();
 
 // server
+extern const char *timestr(int dur, int style = 0);
 extern vector<char *> gameargs;
 extern void initgame();
 extern void changeservertype();
@@ -519,8 +562,8 @@ extern char *connectname;
 extern int connectport;
 extern void localservertoclient(int chan, ENetPacket *packet);
 extern bool connected(bool attempt = true, bool local = true);
-extern void connectserv(const char *name = NULL, int port = RE_SERVER_PORT, const char *password = NULL);
-extern void reconnect();
+extern void connectserv(const char *name = NULL, int port = MASTER_PORT, const char *password = NULL);
+extern void reconnect(const char *pass);
 extern void lanconnect();
 extern void abortconnect(bool msg = true);
 extern void clientkeepalive();
@@ -530,8 +573,10 @@ extern ENetPeer *curpeer, *connpeer;
 // console
 #ifdef __APPLE__
     #define MOD_KEYS (KMOD_LMETA|KMOD_RMETA)
+    #define MOD_ALTS MOD_KEYS
 #else
     #define MOD_KEYS (KMOD_LCTRL|KMOD_RCTRL)
+    #define MOD_ALTS (KMOD_LALT|KMOD_RALT)
 #endif
 
 extern void writebinds(stream *f);
@@ -540,7 +585,7 @@ extern const char *addreleaseaction(char *s);
 extern const char *getkeyname(int code);
 extern int findkeycode(char *key);
 
-extern int uimillis, conskip, commandmillis,  commandpos, commandcolour;
+extern int uimillis, commandmillis,  commandpos, commandcolour, completeoffset, completesize;
 extern string commandbuf;
 extern char *commandaction, *commandicon;
 extern bool fullconsole;
@@ -550,10 +595,11 @@ enum
 {
     NOT_INITING = 0,
     INIT_LOAD,
-    INIT_RESET
+    INIT_RESET,
+    INIT_DEFAULTS
 };
-extern int initing;
-
+extern int initing, fullscreen, numcpus;
+void setfullscreen(bool enable, bool force = false);
 extern bool progressing;
 extern float loadprogress, progresspart, progressamt;
 extern char *progresstitle, *progresstext;
@@ -565,12 +611,12 @@ enum
     CHANGE_GFX   = 1<<0,
     CHANGE_SOUND = 1<<1
 };
-extern bool initwarning(const char *desc, int level = INIT_RESET, int type = CHANGE_GFX);
+extern bool initwarning(const char *desc, int level = INIT_RESET, int type = CHANGE_GFX, bool force = false);
 extern void resetcursor(bool warp = true, bool reset = true);
 extern int compresslevel, imageformat;
 
 extern void pushevent(const SDL_Event &e);
-extern bool interceptkey(int sym);
+extern bool interceptkey(int sym, int mod = 0);
 extern void getfps(int &fps, int &bestdiff, int &worstdiff);
 extern void swapbuffers();
 extern int getclockmillis();
@@ -579,13 +625,13 @@ extern int getclockmillis();
 extern float menuscale;
 
 extern void menuprocess();
-extern void addchange(const char *desc, int type);
+extern void addchange(const char *desc, int type, bool force = false);
 extern void clearchanges(int type);
 
 // physics
 extern bool pointincube(const clipplanes &p, const vec &v);
 extern bool overlapsdynent(const vec &o, float radius);
-extern void rotatebb(vec &center, vec &radius, int yaw, int roll = 0);
+extern void rotatebb(vec &center, vec &radius, int yaw, int pitch, int roll = 0);
 extern float shadowray(const vec &o, const vec &ray, float radius, int mode, extentity *t = NULL);
 struct ShadowRayCache;
 extern ShadowRayCache *newshadowraycache();
@@ -601,6 +647,8 @@ extern int getmaprevision();
 
 
 // world
+extern vector<int> outsideents;
+
 extern void entcancel();
 extern void entitiesinoctanodes();
 extern void attachentities();
@@ -623,6 +671,8 @@ extern mapmodelinfo *getmminfo(int i);
 extern void startmodelquery(occludequery *query);
 extern void endmodelquery();
 extern void preloadmodelshaders();
+extern void preloadmodel(const char *name);
+extern void flushpreloadedmodels(bool msg = true);
 extern void preloadusedmapmodels(bool msg = false, bool bih = false);
 
 // renderparticles
@@ -662,12 +712,27 @@ extern void drawskybox(int farplane, bool limited);
 extern bool limitsky();
 
 // gui
+extern int mouseaction[2], guibound[2];
+
 extern void progressmenu();
 extern void mainmenu();
 extern void texturemenu();
 extern bool menuactive();
 extern int cleargui(int n = 0);
 
+#define uipad(parent,count,body) { (parent).space(count); body; (parent).space(count); }
+#define uifont(parent,font,body) { (parent).pushfont(font); body; (parent).popfont(); }
+#define uicenter(parent, body) { (parent).spring(); body; (parent).spring(); }
+#define uilistv(parent,count,body) \
+{ \
+    loop(uilistv##__LINE__, count) (parent).pushlist(); \
+    body; \
+    loop(uilistv##__LINE__, count) (parent).poplist(); \
+}
+#define uilist(parent,body) uilistv(parent, 1, body)
+#define uicenterlistv(parent,count,body) uilistv(parent, count, uicenter(parent, body))
+#define uicenterlist(parent,body) uicenterlistv(parent, 1, body)
+
 // octaedit
 extern void replacetexcube(cube &c, int oldtex, int newtex);
 
@@ -694,13 +759,10 @@ struct cline { char *cref; int type, reftime, outtime; };
 extern vector<cline> conlines;
 extern void conline(int type, const char *sf, int n);
 
-// command
-extern char *gettime(char *format);
-
 // rendergl
 extern int dynentsize, watercolour, lavacolour, fog, fogcolour;
 extern bvec ambientcolor, skylightcolor;
-extern float curfov, fovy, aspect;
+extern float curfov, fovy, aspect, forceaspect;
 
 extern void project(float fovy, float aspect, int farplane, bool flipx = false, bool flipy = false, bool swapxy = false, float zscale = 1);
 extern void transplayer();
@@ -711,7 +773,7 @@ extern void usetexturing(bool on);
 #define renderatopview (glaring)
 #define rendernormally (rendermainview || renderatopview)
 
-extern void renderplayerpreview(int model, int color, int team, int weap, bool background);
+extern void renderplayerpreview(int model, int color, int team, int weap, const char *vanity, bool background, float scale, float blend);
 extern void drawslice(float start, float length, float x, float y, float size);
 extern void drawfadedslice(float start, float length, float x, float y, float size, float alpha, float r = 1.f, float g = 1.f, float b = 1.f, float minsize = 0.25f);
 extern void polyhue(dynent *d, vec &colour, int flags = 0);
@@ -722,6 +784,8 @@ extern void generategrass();
 extern void rendergrass();
 
 // blendmap
+extern int blendpaintmode;
+
 struct BlendMapCache;
 extern BlendMapCache *newblendmapcache();
 extern void freeblendmapcache(BlendMapCache *&cache);
@@ -732,6 +796,8 @@ extern void resetblendmap();
 extern void enlargeblendmap();
 extern void shrinkblendmap(int octant);
 extern void optimizeblendmap();
+extern void stoppaintblendmap();
+extern void trypaintblendmap();
 extern void renderblendbrush(GLuint tex, float x, float y, float w, float h);
 extern void renderblendbrush();
 extern bool loadblendmap(stream *f);
