@@ -11,32 +11,69 @@ ircnet *ircfind(const char *name)
     return NULL;
 }
 
+void ircprintf(ircnet *n, int relay, const char *target, const char *msg, ...)
+{
+    defvformatstring(str, msg, msg);
+    string s;
+    if(target && *target && strcasecmp(target, n->nick))
+    {
+        ircchan *c = ircfindchan(n, target);
+        if(c)
+        {
+            formatstring(s)("\fs\fa[%s:%s]\fS", n->name, c->name);
+            #ifndef STANDALONE
+            c->buffer.addline(str, MAXIRCLINES);
+            c->updated |= IRCUP_MSG;
+            #endif
+            if(n->type == IRCT_RELAY && c->relay >= relay)
+                server::srvmsgf(relay > 1 ? -2 : -3, "\fs\fa[%s]\fS %s", c->friendly, str);
+        }
+        else
+        {
+            formatstring(s)("\fs\fa[%s:%s]\fS", n->name, target);
+            #ifndef STANDALONE
+            n->buffer.addline(str, MAXIRCLINES);
+            n->updated |= IRCUP_MSG;
+            #endif
+        }
+    }
+    else
+    {
+        formatstring(s)("\fs\fa[%s]\fS", n->name);
+        #ifndef STANDALONE
+        n->buffer.addline(newstring(str), MAXIRCLINES);
+        n->updated |= IRCUP_MSG;
+        #endif
+    }
+    console(0, "%s %s", s, str); // console is not used to relay
+}
+
 void ircestablish(ircnet *n)
 {
     if(!n) return;
-    n->lastattempt = totalmillis;
+    n->lastattempt = clocktime;
     if(n->address.host == ENET_HOST_ANY)
     {
-        conoutf("looking up %s:[%d]...", n->serv, n->port);
+        ircprintf(n, 4, NULL, "looking up %s:[%d]...", n->serv, n->port);
         if(!resolverwait(n->serv, &n->address))
         {
-            conoutf("unable to resolve %s:[%d]...", n->serv, n->port);
+            ircprintf(n, 4, NULL, "unable to resolve %s:[%d]...", n->serv, n->port);
             n->state = IRC_DISC;
             return;
         }
     }
 
     ENetAddress address = { ENET_HOST_ANY, enet_uint16(n->port) };
-    if(*n->ip && enet_address_set_host(&address, n->ip) < 0) conoutf("failed to bind address: %s", n->ip);
+    if(*n->ip && enet_address_set_host(&address, n->ip) < 0) ircprintf(n, 4, NULL, "failed to bind address: %s", n->ip);
     n->sock = enet_socket_create(ENET_SOCKET_TYPE_STREAM);
     if(n->sock != ENET_SOCKET_NULL && *n->ip && enet_socket_bind(n->sock, &address) < 0)
     {
-        conoutf("failed to bind connection socket: %s", n->ip);
+        ircprintf(n, 4, NULL, "failed to bind connection socket: %s", n->ip);
         address.host = ENET_HOST_ANY;
     }
     if(n->sock == ENET_SOCKET_NULL || connectwithtimeout(n->sock, n->serv, n->address) < 0)
     {
-        conoutf(n->sock == ENET_SOCKET_NULL ? "could not open socket to %s:[%d]" : "could not connect to %s:[%d]", n->serv, n->port);
+        ircprintf(n, 4, NULL, n->sock == ENET_SOCKET_NULL ? "could not open socket to %s:[%d]" : "could not connect to %s:[%d]", n->serv, n->port);
         if(n->sock != ENET_SOCKET_NULL)
         {
             enet_socket_destroy(n->sock);
@@ -46,7 +83,7 @@ void ircestablish(ircnet *n)
         return;
     }
     n->state = IRC_ATTEMPT;
-    conoutf("connecting to %s:[%d]...", n->serv, n->port);
+    ircprintf(n, 4, NULL, "connecting to %s:[%d]...", n->serv, n->port);
 }
 
 void ircsend(ircnet *n, const char *msg, ...)
@@ -74,7 +111,7 @@ void ircsend(ircnet *n, const char *msg, ...)
     }
 }
 
-VAR(0, ircfilter, 0, 1, 2);
+VAR(0, ircfilter, 0, 2, 2);
 
 void converttext(char *dst, const char *src)
 {
@@ -111,8 +148,7 @@ void converttext(char *dst, const char *src)
                 case 'v': *dst++ = '\f'; *dst++ = '0'; *dst++ = '6'; break; // violet
                 case 'p': *dst++ = '\f'; *dst++ = '0'; *dst++ = '6'; break; // purple
                 case 'n': *dst++ = '\f'; *dst++ = '0'; *dst++ = '5'; break; // brown
-                case 'd': case 'A': *dst++ = '\f'; *dst++ = '0'; *dst++ = '1'; break; // dark grey
-                case 'u': case 'w': case '7': case 'k': case '8': *dst++ = '\f'; *dst++ = '0'; *dst++ = '0'; break;
+                case 'u': case 'w': case '7': case 'k': case '8': case 'd': case 'A': *dst++ = '\f'; *dst++ = '0'; *dst++ = '1'; break; // dark grey
                 default: colorstack[colorpos] = oldcolor; break;
             }
             continue;
@@ -154,7 +190,7 @@ void ircoutf(int relay, const char *msg, ...)
 int ircrecv(ircnet *n)
 {
     if(!n) return -1;
-    if(n->sock == ENET_SOCKET_NULL) return -1;
+    if(n->sock == ENET_SOCKET_NULL) return -2;
     int total = 0;
     enet_uint32 events = ENET_SOCKET_WAIT_RECEIVE;
     while(enet_socket_wait(n->sock, &events, 0) >= 0 && events)
@@ -163,7 +199,8 @@ int ircrecv(ircnet *n)
         buf.data = n->input + n->inputlen;
         buf.dataLength = sizeof(n->input) - n->inputlen;
         int len = enet_socket_receive(n->sock, NULL, &buf, 1);
-        if(len <= 0) return -1;
+        if(!len) break;
+        if(len < 0) return -3;
         loopi(len) switch(n->input[n->inputlen+i])
         {
             case '\x01': n->input[n->inputlen+i] = '\v'; break;
@@ -195,10 +232,13 @@ void ircnewnet(int type, const char *name, const char *serv, int port, const cha
     }
     ircnet &n = *ircnets.add(new ircnet);
     n.type = type;
-    n.state = IRC_DISC;
+    n.state = IRC_NEW;
     n.sock = ENET_SOCKET_NULL;
     n.port = port;
     n.lastattempt = 0;
+    #ifndef STANDALONE
+    n.updated = IRCUP_NEW;
+    #endif
     copystring(n.name, name);
     copystring(n.serv, serv);
     copystring(n.nick, nick);
@@ -207,7 +247,7 @@ void ircnewnet(int type, const char *name, const char *serv, int port, const cha
     n.address.host = ENET_HOST_ANY;
     n.address.port = n.port;
     n.input[0] = n.authname[0] = n.authpass[0] = 0;
-    conoutf("added irc %s %s (%s:%d) [%s]", type == IRCT_RELAY ? "relay" : "client", name, serv, port, nick);
+    ircprintf(&n, 4, NULL, "added %s %s (%s:%d) [%s]", type == IRCT_RELAY ? "relay" : "client", name, serv, port, nick);
 }
 
 ICOMMAND(0, ircaddclient, "ssisss", (const char *n, const char *s, int *p, const char *c, const char *h, const char *z), {
@@ -219,44 +259,44 @@ ICOMMAND(0, ircaddrelay, "ssisss", (const char *n, const char *s, int *p, const
 ICOMMAND(0, ircserv, "ss", (const char *name, const char *s), {
     ircnet *n = ircfind(name);
     if(!n) { conoutf("no such ircnet: %s", name); return; }
-    if(!s || !*s) { conoutf("%s current server is: %s", n->name, n->serv); return; }
+    if(!s || !*s) { ircprintf(n, 4, NULL, "current server is: %s", n->serv); return; }
     copystring(n->serv, s);
 });
 ICOMMAND(0, ircport, "ss", (const char *name, const char *s), {
     ircnet *n = ircfind(name);
     if(!n) { conoutf("no such ircnet: %s", name); return; }
-    if(!s || !*s || !parseint(s)) { conoutf("%s current port is: %d", n->name, n->port); return; }
+    if(!s || !*s || !parseint(s)) { ircprintf(n, 4, NULL, "current port is: %d", n->port); return; }
     n->port = parseint(s);
 });
 ICOMMAND(0, ircnick, "ss", (const char *name, const char *s), {
     ircnet *n = ircfind(name);
     if(!n) { conoutf("no such ircnet: %s", name); return; }
-    if(!s || !*s) { conoutf("%s current nickname is: %s", n->name, n->nick); return; }
+    if(!s || !*s) { ircprintf(n, 4, NULL, "current nickname is: %s", n->nick); return; }
     copystring(n->nick, s);
 });
 ICOMMAND(0, ircbind, "ss", (const char *name, const char *s), {
     ircnet *n = ircfind(name);
     if(!n) { conoutf("no such ircnet: %s", name); return; }
-    if(!s || !*s) { conoutf("%s currently bound to: %s", n->name, n->ip); return; }
+    if(!s || !*s) { ircprintf(n, 4, NULL, "currently bound to: %s", n->ip); return; }
     copystring(n->ip, s);
 });
 ICOMMAND(0, ircpass, "ss", (const char *name, const char *s), {
     ircnet *n = ircfind(name);
     if(!n) { conoutf("no such ircnet: %s", name); return; }
-    if(!s || !*s) { conoutf("%s current password is: %s", n->name, n->passkey && *n->passkey ? "<set>" : "<not set>"); return; }
+    if(!s || !*s) { ircprintf(n, 4, NULL, "current password is: %s", n->passkey && *n->passkey ? "<set>" : "<not set>"); return; }
     copystring(n->passkey, s);
 });
 ICOMMAND(0, ircauth, "sss", (const char *name, const char *s, const char *t), {
     ircnet *n = ircfind(name);
     if(!n) { conoutf("no such ircnet: %s", name); return; }
-    if(!s || !*s || !t || !*t) { conoutf("%s current authority is: %s (%s)", n->name, n->authname, n->authpass && *n->authpass ? "<set>" : "<not set>"); return; }
+    if(!s || !*s || !t || !*t) { ircprintf(n, 4, NULL, "current authority is: %s (%s)", n->authname, n->authpass && *n->authpass ? "<set>" : "<not set>"); return; }
     copystring(n->authname, s);
     copystring(n->authpass, t);
 });
 ICOMMAND(0, ircconnect, "s", (const char *name), {
     ircnet *n = ircfind(name);
     if(!n) { conoutf("no such ircnet: %s", name); return; }
-    if(n->state != IRC_DISC) { conoutf("ircnet %s is already connected", n->name); return; }
+    if(n->state != IRC_DISC) { ircprintf(n, 4, NULL, "network already already active"); return; }
     ircestablish(n);
 });
 
@@ -273,15 +313,15 @@ ircchan *ircfindchan(ircnet *n, const char *name)
 bool ircjoin(ircnet *n, ircchan *c)
 {
     if(!n || !c) return false;
-    if(n->state == IRC_DISC)
+    if(n->state != IRC_ONLINE)
     {
-        conoutf("ircnet %s is not connected", n->name);
+        ircprintf(n, 4, NULL, "cannot join %s until connection is online", c->name);
         return false;
     }
     if(*c->passkey) ircsend(n, "JOIN %s :%s", c->name, c->passkey);
     else ircsend(n, "JOIN %s", c->name);
     c->state = IRCC_JOINING;
-    c->lastjoin = totalmillis;
+    c->lastjoin = clocktime;
     c->lastsync = 0;
     return true;
 }
@@ -292,7 +332,7 @@ bool ircenterchan(ircnet *n, const char *name)
     ircchan *c = ircfindchan(n, name);
     if(!c)
     {
-        conoutf("ircnet %s has no channel called %s ready", n->name, name);
+        ircprintf(n, 4, NULL, "no channel called %s available", name);
         return false;
     }
     return ircjoin(n, c);
@@ -310,7 +350,7 @@ bool ircnewchan(int type, const char *name, const char *channel, const char *fri
     ircchan *c = ircfindchan(n, channel);
     if(c)
     {
-        conoutf("%s already has channel %s", n->name, c->name);
+        ircprintf(n, 4, NULL, "channel %s already exists", c->name);
         return false;
     }
     ircchan &d = n->channels.add();
@@ -321,8 +361,8 @@ bool ircnewchan(int type, const char *name, const char *channel, const char *fri
     copystring(d.name, channel);
     copystring(d.friendly, friendly && *friendly ? friendly : channel);
     copystring(d.passkey, passkey);
-    if(n->state != IRC_DISC) ircjoin(n, &d);
-    conoutf("%s added channel %s", n->name, d.name);
+    if(n->state == IRC_ONLINE) ircjoin(n, &d);
+    ircprintf(n, 4, NULL, "added channel: %s", d.name);
     return true;
 }
 
@@ -336,57 +376,27 @@ ICOMMAND(0, ircpasschan, "sss", (const char *name, const char *chan, const char
     ircnet *n = ircfind(name);
     if(!n) { conoutf("no such ircnet: %s", name); return; }
     ircchan *c = ircfindchan(n, chan);
-    if(!c) { conoutf("no such %s channel: %s", n->name, chan); return; }
-    if(!s || !*s) { conoutf("%s channel %s current password is: %s", n->name, c->name, c->passkey && *c->passkey ? "<set>" : "<not set>"); return; }
+    if(!c) { ircprintf(n, 4, NULL, "no such channel: %s", chan); return; }
+    if(!s || !*s) { ircprintf(n, 4, NULL, "channel %s current password is: %s", c->name, c->passkey && *c->passkey ? "<set>" : "<not set>"); return; }
     copystring(c->passkey, s);
 });
 ICOMMAND(0, ircrelaychan, "sss", (const char *name, const char *chan, const char *s), {
     ircnet *n = ircfind(name);
     if(!n) { conoutf("no such ircnet: %s", name); return; }
     ircchan *c = ircfindchan(n, chan);
-    if(!c) { conoutf("no such %s channel: %s", n->name, chan); return; }
-    if(!s || !*s) { conoutf("%s channel %s current relay level is: %d", n->name, c->name, c->relay); return; }
+    if(!c) { ircprintf(n, 4, NULL, "no such channel: %s", chan); return; }
+    if(!s || !*s) { ircprintf(n, 4, NULL, "channel %s current relay level is: %d", c->name, c->relay); return; }
     c->relay = parseint(s);
 });
 ICOMMAND(0, ircfriendlychan, "sss", (const char *name, const char *chan, const char *s), {
     ircnet *n = ircfind(name);
     if(!n) { conoutf("no such ircnet: %s", name); return; }
     ircchan *c = ircfindchan(n, chan);
-    if(!c) { conoutf("no such %s channel: %s", n->name, chan); return; }
-    if(!s || !*s) { conoutf("%s channel %s current friendly name is: %s", n->name, c->name, c->friendly); return; }
+    if(!c) { ircprintf(n, 4, NULL, "no such channel: %s", chan); return; }
+    if(!s || !*s) { ircprintf(n, 4, NULL, "channel %s current friendly name is: %s", c->name, c->friendly); return; }
     copystring(c->friendly, s);
 });
 
-#define MAXLINES 100
-
-void ircprintf(ircnet *n, int relay, const char *target, const char *msg, ...)
-{
-    defvformatstring(str, msg, msg);
-    string s;
-    if(target && *target && strcasecmp(target, n->nick))
-    {
-        ircchan *c = ircfindchan(n, target);
-        if(c)
-        {
-            formatstring(s)("\fs\fa[%s:%s]\fS", n->name, c->name);
-            c->addline(str, MAXLINES);
-            if(n->type == IRCT_RELAY && c->relay >= relay)
-                server::srvmsgf(relay > 1 ? -2 : -3, "\fs\fa[%s]\fS %s", c->friendly, str);
-        }
-        else
-        {
-            formatstring(s)("\fs\fa[%s:%s]\fS", n->name, target);
-            n->addline(str, MAXLINES);
-        }
-    }
-    else
-    {
-        formatstring(s)("\fs\fa[%s]\fS", n->name);
-        n->addline(newstring(str), MAXLINES);
-    }
-    console(0, "%s %s", s, str); // console is not used to relay
-}
-
 void ircprocess(ircnet *n, char *user[3], int g, int numargs, char *w[])
 {
     if(!strcasecmp(w[g], "NOTICE") || !strcasecmp(w[g], "PRIVMSG"))
@@ -417,7 +427,7 @@ void ircprocess(ircnet *n, char *user[3], int g, int numargs, char *w[])
                             ircprintf(n, 4, g ? w[g+1] : NULL, "\fr%s requests: %s %s", user[0], q, r);
 
                             if(!strcasecmp(q, "VERSION"))
-                                ircsend(n, "NOTICE %s :\vVERSION %s v%s-%s %d bit (%s), %s\v", user[0], RE_NAME, RE_VER_STR, RE_PLATFORM, RE_ARCH, RE_RELEASE, RE_URL);
+                                ircsend(n, "NOTICE %s :\vVERSION %s v%s-%s %d bit (%s), %s\v", user[0], versionname, versionstring, CUR_PLATFORM, CUR_ARCH, versionrelease, versionurl);
                             else if(!strcasecmp(q, "PING")) // eh, echo back
                                 ircsend(n, "NOTICE %s :\vPING %s\v", user[0], r);
                         }
@@ -455,7 +465,10 @@ void ircprocess(ircnet *n, char *user[3], int g, int numargs, char *w[])
             if(c && !strcasecmp(user[0], n->nick))
             {
                 c->state = IRCC_JOINED;
-                c->lastjoin = c->lastsync = totalmillis;
+                c->lastjoin = c->lastsync = clocktime;
+                #ifndef STANDALONE
+                c->updated |= IRCUP_NEW;
+                #endif
             }
             ircprintf(n, 3, w[g+1], "\fg%s (%s@%s) has joined", user[0], user[1], user[2]);
         }
@@ -468,7 +481,7 @@ void ircprocess(ircnet *n, char *user[3], int g, int numargs, char *w[])
             if(c && !strcasecmp(user[0], n->nick))
             {
                 c->state = IRCC_NONE;
-                c->lastjoin = totalmillis;
+                c->lastjoin = clocktime;
                 c->lastsync = 0;
             }
             ircprintf(n, 3, w[g+1], "\fo%s (%s@%s) has left", user[0], user[1], user[2]);
@@ -487,7 +500,7 @@ void ircprocess(ircnet *n, char *user[3], int g, int numargs, char *w[])
             if(c && !strcasecmp(w[g+2], n->nick))
             {
                 c->state = IRCC_KICKED;
-                c->lastjoin = totalmillis;
+                c->lastjoin = clocktime;
                 c->lastsync = 0;
             }
             ircprintf(n, 3, w[g+1], "\fr%s (%s@%s) has kicked %s from %s", user[0], user[1], user[2], w[g+2], w[g+1]);
@@ -517,9 +530,8 @@ void ircprocess(ircnet *n, char *user[3], int g, int numargs, char *w[])
         }
         else
         {
-            int t = time(NULL);
-            ircprintf(n, 4, NULL, "%s PING (empty)", user[0]);
-            ircsend(n, "PONG %d", t);
+            ircprintf(n, 4, NULL, "%s PING", user[0]);
+            ircsend(n, "PONG %d", clocktime);
         }
     }
     else
@@ -573,10 +585,11 @@ void ircprocess(ircnet *n, char *user[3], int g, int numargs, char *w[])
                 if(c)
                 {
                     c->state = IRCC_BANNED;
-                    c->lastjoin = totalmillis;
+                    c->lastjoin = clocktime;
                     c->lastsync = 0;
                     if(c->type == IRCCT_AUTO)
                         ircprintf(n, 4, w[g+2], "\fbwaiting 5 mins to rejoin %s", c->name);
+                    else ircprintf(n, 4, NULL, "\fbbanned from channel: %s", c->name);
                 }
                 break;
             }
@@ -661,14 +674,15 @@ void ircparse(ircnet *n)
     }
 }
 
-void ircdiscon(ircnet *n)
+void ircdiscon(ircnet *n, const char *msg = NULL)
 {
     if(!n) return;
-    ircprintf(n, 4, NULL, "disconnected from %s (%s:[%d])", n->name, n->serv, n->port);
+    if(msg) ircprintf(n, 4, NULL, "disconnected from %s (%s:[%d]): %s", n->name, n->serv, n->port, msg);
+    else ircprintf(n, 4, NULL, "disconnected from %s (%s:[%d])", n->name, n->serv, n->port);
     enet_socket_destroy(n->sock);
     n->state = IRC_DISC;
     n->sock = ENET_SOCKET_NULL;
-    n->lastattempt = totalmillis;
+    n->lastattempt = clocktime;
 }
 
 void irccleanup()
@@ -676,17 +690,35 @@ void irccleanup()
     loopv(ircnets) if(ircnets[i]->sock != ENET_SOCKET_NULL)
     {
         ircnet *n = ircnets[i];
-        ircsend(n, "QUIT :%s, %s", RE_NAME, RE_URL);
-        ircdiscon(n);
+        ircsend(n, "QUIT :%s, %s", versionname, versionurl);
+        ircdiscon(n, "shutdown");
     }
 }
 
 void ircslice()
 {
+    #ifndef STANDALONE
+    loopvrev(ircnets)
+    {
+        ircnet *n = ircnets[i];
+        if((n->sock == ENET_SOCKET_NULL || n->state == IRC_DISC) && n->updated&IRCUP_LEAVE)
+        {
+            delete n;
+            ircnets.remove(i);
+            continue;
+        }
+        loopvjrev(n->channels)
+        {
+            ircchan &c = n->channels[j];
+            if(c.state != IRCC_JOINED && c.state != IRCC_JOINING && (c.type != IRCCT_AUTO || c.updated&IRCUP_LEAVE))
+                n->channels.remove(j);
+        }
+    }
+    #endif
     loopv(ircnets)
     {
         ircnet *n = ircnets[i];
-        if(n->sock != ENET_SOCKET_NULL && n->state != IRC_DISC)
+        if(n->sock != ENET_SOCKET_NULL && n->state > IRC_DISC)
         {
             switch(n->state)
             {
@@ -694,13 +726,13 @@ void ircslice()
                 {
                     if(*n->passkey) ircsend(n, "PASS %s", n->passkey);
                     ircsend(n, "NICK %s", n->nick);
-                    ircsend(n, "USER %s +iw %s :%s", n->nick, n->nick, n->nick);
+                    ircsend(n, "USER %s +iw %s :%s v%s-%s %d bit (%s)", versionuname, versionuname, versionname, versionstring, CUR_PLATFORM, CUR_ARCH, versionrelease);
                     n->state = IRC_CONN;
                     loopvj(n->channels)
                     {
                         ircchan *c = &n->channels[j];
                         c->state = IRCC_NONE;
-                        c->lastjoin = totalmillis;
+                        c->lastjoin = clocktime;
                         c->lastsync = 0;
                     }
                     break;
@@ -710,21 +742,19 @@ void ircslice()
                     loopvj(n->channels)
                     {
                         ircchan *c = &n->channels[j];
-                        if(c->type == IRCCT_AUTO && c->state != IRCC_JOINED && (!c->lastjoin || totalmillis-c->lastjoin >= (c->state != IRCC_BANNED ? 5000 : 300000)))
+                        if(c->type == IRCCT_AUTO && c->state != IRCC_JOINED && (!c->lastjoin || clocktime-c->lastjoin >= (c->state != IRCC_BANNED ? 5 : 300)))
                             ircjoin(n, c);
                     }
                     // fall through
                 }
                 case IRC_CONN:
                 {
-                    if(n->state == IRC_CONN && totalmillis-n->lastattempt >= 60000)
-                    {
-                        ircprintf(n, 4, NULL, "connection attempt timed out");
-                        ircdiscon(n);
-                    }
+                    if(n->state == IRC_CONN && clocktime-n->lastattempt >= 60) ircdiscon(n, "connection attempt timed out");
                     else switch(ircrecv(n))
                     {
-                        case -1: ircdiscon(n); break; // fail
+                        case -3: ircdiscon(n, "read error"); break;
+                        case -2: ircdiscon(n, "connection reset"); break;
+                        case -1: ircdiscon(n, "invalid connection"); break;
                         case 0: break;
                         default: ircparse(n); break;
                     }
@@ -732,12 +762,12 @@ void ircslice()
                 }
                 default:
                 {
-                    ircdiscon(n);
+                    ircdiscon(n, "encountered unknown connection state");
                     break;
                 }
             }
         }
-        else if(!n->lastattempt || totalmillis-n->lastattempt >= 60000) ircestablish(n);
+        else if(!n->lastattempt || clocktime-n->lastattempt >= 60) ircestablish(n);
     }
 }
 #ifndef STANDALONE
@@ -759,7 +789,7 @@ void irccmd(ircnet *n, ircchan *c, char *s)
                     ircsend(n, "PRIVMSG %s :\vACTION %s\v", c->name, r);
                     ircprintf(n, 1, c->name, "\fv* %s %s", n->nick, r);
                 }
-                else ircprintf(n, 4, NULL, "\fcyou are not on a channel");
+                else ircprintf(n, 4, NULL, "\fyyou are not on a channel");
             }
             else if(!strcasecmp(q, "JOIN"))
             {
@@ -770,16 +800,25 @@ void irccmd(ircnet *n, ircchan *c, char *s)
             else if(!strcasecmp(q, "PART"))
             {
                 ircchan *d = ircfindchan(n, r);
-                if(d) ircsend(n, "PART %s", d->name);
-                else if(c) ircsend(n, "PART %s", c->name);
-                else ircsend(n, "PART %s", r);
+                if(!d && c) d = c;
+                if(d)
+                {
+                    ircsend(n, "PART %s", d->name);
+                    d->updated |= IRCUP_LEAVE;
+                }
+                else ircprintf(n, 4, NULL, "\fyyou are not on a channel");
+            }
+            else if(!strcasecmp(q, "QUIT"))
+            {
+                ircsend(n, "QUIT :%s", r);
+                n->updated |= IRCUP_LEAVE;
             }
             else if(*r) ircsend(n, "%s %s", q, r); // send it raw so we support any command
             else ircsend(n, "%s", q);
             DELETEA(q); DELETEA(r);
             return;
         }
-        ircprintf(n, 4, c ? c->name : NULL, "\fcyou are not on a channel");
+        ircprintf(n, 4, c ? c->name : NULL, "\fyyou are not on a channel");
     }
     else if(c)
     {
@@ -793,62 +832,71 @@ void irccmd(ircnet *n, ircchan *c, char *s)
     }
 }
 
-bool ircchangui(guient *g, ircnet *n, ircchan *c, bool tab, bool front)
+bool ircchangui(guient *g, ircnet *n, ircchan *c, bool tab)
 {
-    if(tab) g->tab(c->name, 0xFFFFFF, front);
+    if(tab)
+    {
+        bool front = c->updated&IRCUP_NEW, msg = c->updated&IRCUP_MSG;
+        g->tab(c->name, msg ? 0x00FFFF : 0xFFFFFF, front);
+        if(front) c->updated &= ~IRCUP_NEW;
+        if(msg && g->visible()) c->updated &= ~IRCUP_MSG;
+    }
 
     defformatstring(cwindow)("%s_%s_window", n->name, c->name);
-    if(c->newlines < c->lines.length())
+    if(c->buffer.newlines < c->buffer.lines.length())
     {
         editor *e = UI::geteditor(cwindow, EDITORREADONLY);
-        if(e) while(c->newlines < c->lines.length()) UI::editorline(e, c->lines[c->newlines++], MAXLINES);
+        if(e) while(c->buffer.newlines < c->buffer.lines.length()) UI::editorline(e, c->buffer.lines[c->buffer.newlines++], MAXIRCLINES);
     }
     g->field(cwindow, 0x666666, -100, 25, NULL, EDITORREADONLY);
 
     defformatstring(cinput)("%s_%s_input", n->name, c->name);
-    char *v = g->field(cinput, 0x666666, -100, 0, "", EDITORFOREVER);
+    char *v = g->field(cinput, 0x666666, -100, 0, "", EDITORFOREVER, g->visible(), cwindow);
     if(v && *v)
     {
         irccmd(n, c, v);
-        UI::editoredit(UI::geteditor(cinput, EDITORFOREVER));
+        UI::editoredit(UI::geteditor(cinput, EDITORFOREVER, NULL, cwindow));
     }
     return true;
 }
 
-int lastchanupdate = -1;
 bool ircnetgui(guient *g, ircnet *n, bool tab)
 {
-    if(tab) g->tab(n->name);
+    if(tab)
+    {
+        bool front = n->updated&IRCUP_NEW, msg = n->updated&IRCUP_MSG;
+        g->tab(n->name, msg ? 0x00FFFF : 0xFFFFFF, front);
+        if(front) n->updated &= ~IRCUP_NEW;
+        if(msg && g->visible()) n->updated &= ~IRCUP_MSG;
+    }
 
     defformatstring(window)("%s_window", n->name);
-    if(n->newlines < n->lines.length())
+    if(n->buffer.newlines < n->buffer.lines.length())
     {
         editor *e = UI::geteditor(window, EDITORREADONLY);
-        if(e) while(n->newlines < n->lines.length()) UI::editorline(e, n->lines[n->newlines++], MAXLINES);
+        if(e) while(n->buffer.newlines < n->buffer.lines.length()) UI::editorline(e, n->buffer.lines[n->buffer.newlines++], MAXIRCLINES);
     }
     g->field(window, 0x666666, -100, 25, NULL, EDITORREADONLY);
 
     defformatstring(input)("%s_input", n->name);
-    char *w = g->field(input, 0x666666, -100, 0, "", EDITORFOREVER);
+    char *w = g->field(input, 0x666666, -100, 0, "", EDITORFOREVER, g->visible(), window);
     if(w && *w)
     {
         irccmd(n, NULL, w);
-        UI::editoredit(UI::geteditor(input, EDITORFOREVER));
+        UI::editoredit(UI::geteditor(input, EDITORFOREVER, NULL, window));
     }
 
     loopvj(n->channels) if(n->channels[j].state != IRCC_NONE && n->channels[j].name[0])
     {
         ircchan *c = &n->channels[j];
-        if(!ircchangui(g, n, c, true, c->lastsync && c->lastsync > lastchanupdate)) return false;
+        if(!ircchangui(g, n, c, true)) return false;
     }
-    lastchanupdate = totalmillis;
-
     return true;
 }
 
+static const char *ircstates[IRC_MAX] = { "\froffline", "\foconnecting", "\fynegotiating", "\fgonline" };
 bool ircgui(guient *g, const char *s)
 {
-    g->allowautotab(false);
     g->strut(94);
     if(s && *s)
     {
@@ -865,19 +913,16 @@ bool ircgui(guient *g, const char *s)
         loopv(ircnets) if(ircnets[i]->name[0] && ircnets[i]->sock != ENET_SOCKET_NULL)
         {
             ircnet *n = ircnets[i];
-            g->pushlist();
-            g->buttonf("%s via %s:[%d]", 0xFFFFFF, NULL, 0, true, n->name, n->serv, n->port);
-            g->space(1);
-            const char *ircstates[IRC_MAX] = {
-                    "\froffline", "\foconnecting", "\fynegotiating", "\fgonline"
-            };
-            g->buttonf("\fs%s\fS as %s", 0xFFFFFF, NULL, 0, true, ircstates[n->state], n->nick);
-            g->poplist();
+            uilist(*g, {
+                g->buttonf("%s via %s:[%d]", 0xFFFFFF, NULL, 0, true, n->name, n->serv, n->port);
+                g->space(1);
+                g->buttonf("\fs%s\fS as %s", 0xFFFFFF, NULL, 0, true, ircstates[n->state], n->nick);
+            });
             nets++;
         }
         if(nets)
         {
-            loopv(ircnets) if(ircnets[i]->state != IRC_DISC && ircnets[i]->name[0] && ircnets[i]->sock != ENET_SOCKET_NULL)
+            loopv(ircnets)
             {
                 ircnet *n = ircnets[i];
                 if(!ircnetgui(g, n, true)) return false;
