@@ -44,7 +44,7 @@ struct lightmapinfo
     uchar *colorbuf;
     bvec *raybuf;
     bool packed;
-    int type, w, h, bpp, bufsize, surface;
+    int type, w, h, bpp, bufsize, surface, layers;
 };
 
 struct lightmaptask
@@ -90,10 +90,6 @@ VARF(IDF_HEX|IDF_WORLD, skylight, 0, 0, 0xFFFFFF,
     if(skylight <= 255) skylight |= (skylight<<8) | (skylight<<16);
     skylightcolor = bvec((skylight>>16)&0xFF, (skylight>>8)&0xFF, skylight&0xFF);
 });
-VARN(IDF_WORLD, lmshadows, lmshadows_, 0, 2, 2);
-VARN(IDF_WORLD, lmaa, lmaa_, 0, 3, 3);
-VARN(IDF_WORLD, lerptjoints, lerptjoints_, 0, 1, 1);
-static int lmshadows = 2, lmaa = 3, lerptjoints = 1;
 
 static const surfaceinfo brightsurfaces[6] =
 {
@@ -190,6 +186,10 @@ void setsurface(cube &c, int orient, const surfaceinfo &src, const vertinfo *src
     if(srcverts) memcpy(c.ext->verts() + dstoffset, srcverts, numsrcverts*sizeof(vertinfo));
 }
 
+VARN(IDF_WORLD, lmshadows, lmshadows_, 0, 2, 2);
+VARN(IDF_WORLD, lmaa, lmaa_, 0, 3, 3);
+VARN(IDF_WORLD, lerptjoints, lerptjoints_, 0, 1, 1);
+static int lmshadows = 2, lmaa = 3, lerptjoints = 1;
 static uint lmprog = 0, taskprogress = 0;
 GLuint lmprogtex = 0;
 static int lmprogtexticks = 0, lmprogid = -1;
@@ -608,11 +608,11 @@ static void calcskylight(lightmapworker *w, const vec &o, const vec &normal, flo
     flags |= RAY_SHADOW;
     if(skytexturelight) flags |= RAY_SKIPSKY;
     int hit = 0;
-    if(w) loopi(17) 
+    if(w) loopi(17)
     {
         if(normal.dot(rays[i])>=0 && shadowray(w->shadowraycache, vec(rays[i]).mul(tolerance).add(o), rays[i], 1e16f, flags, t)>1e15f) hit++;
     }
-    else loopi(17) 
+    else loopi(17)
     {
         if(normal.dot(rays[i])>=0 && shadowray(vec(rays[i]).mul(tolerance).add(o), rays[i], 1e16f, flags, t)>1e15f) hit++;
     }
@@ -628,33 +628,38 @@ void calcsunlight(lightmapworker *w, const vec &o, const vec &normal, float tole
     {
         const extentity &light = *sunlights[i];
         if(light.attrs.length() < 5 || (slight[0] >= light.attrs[2] && slight[1] >= light.attrs[3] && slight[2] >= light.attrs[4])) continue;
-        int yaw = light.attrs[0], pitch = light.attrs[1]+90, offset = light.attrs.inrange(5) && light.attrs[5] ? light.attrs[5] : 10, hit = 0;
+        int yaw = light.attrs[0], pitch = light.attrs[1]+90,
+            offset = light.attrs.inrange(5) && light.attrs[5] ? (light.attrs[5] > 0 ? light.attrs[5] : 0) : 10, hit = 0;
         vec dir(yaw*RAD, pitch*RAD);
-        if(normal.dot(dir) >= 0 && 
+        if(normal.dot(dir) >= 0 &&
             (w ? shadowray(w->shadowraycache, vec(dir).mul(tolerance).add(o), dir, 1e16f, flags, t) > 1e15f :
                  shadowray(vec(dir).mul(tolerance).add(o), dir, 1e16f, flags, t) > 1e15f))
             hit++;
-        matrix3x3 rot(90*RAD, dir);
-        vec spoke(yaw*RAD, (pitch + offset)*RAD);
-        spoke.rotate(21*RAD, dir);
-        loopk(4)
+        if(offset)
         {
-            if(normal.dot(spoke) >= 0 && 
-                (w ? shadowray(w->shadowraycache, vec(spoke).mul(tolerance).add(o), spoke, 1e16f, flags, t) > 1e15f :
-                     shadowray(vec(spoke).mul(tolerance).add(o), spoke, 1e16f, flags, t) > 1e15f))
-                hit++;
-            spoke = rot.transform(spoke);
+            matrix3x3 rot(90*RAD, dir);
+            vec spoke(yaw*RAD, (pitch + offset)*RAD);
+            spoke.rotate(21*RAD, dir);
+            loopk(4)
+            {
+                if(normal.dot(spoke) >= 0 &&
+                    (w ? shadowray(w->shadowraycache, vec(spoke).mul(tolerance).add(o), spoke, 1e16f, flags, t) > 1e15f :
+                         shadowray(vec(spoke).mul(tolerance).add(o), spoke, 1e16f, flags, t) > 1e15f))
+                    hit++;
+                spoke = rot.transform(spoke);
+            }
+            spoke = vec(yaw*RAD, (pitch + 0.5f*offset)*RAD).rotate((66-21)*RAD, dir);
+            loopk(4)
+            {
+                if(normal.dot(spoke) >= 0 &&
+                    (w ? shadowray(w->shadowraycache, vec(spoke).mul(tolerance).add(o), spoke, 1e16f, flags, t) > 1e15f :
+                         shadowray(vec(spoke).mul(tolerance).add(o), spoke, 1e16f, flags, t) > 1e15f))
+                    hit++;
+                spoke = rot.transform(spoke);
+            }
+            loopk(3) slight[k] = max(uchar(light.attrs[2+k]*hit/9.f), slight[k]);
         }
-        spoke = vec(yaw*RAD, (pitch + 0.5f*offset)*RAD).rotate((66-21)*RAD, dir);
-        loopk(4)
-        {
-            if(normal.dot(spoke) >= 0 && 
-                (w ? shadowray(w->shadowraycache, vec(spoke).mul(tolerance).add(o), spoke, 1e16f, flags, t) > 1e15f :
-                     shadowray(vec(spoke).mul(tolerance).add(o), spoke, 1e16f, flags, t) > 1e15f))
-                hit++;
-            spoke = rot.transform(spoke);
-        }
-        loopk(3) slight[k] = max(uchar(light.attrs[2+k]*hit/9.f), slight[k]);
+        else if(hit) loopk(3) slight[k] = max(uchar(light.attrs[2+k]), slight[k]);
     }
 }
 
@@ -899,8 +904,8 @@ static int finishlightmap(lightmapworker *w)
                 if(ray->iszero()) dstray[0] = bvec(128, 128, 255);
                 else
                 {
-                    // bias the normals towards the amount of ambient/skylight in the lumel 
-                    // this is necessary to prevent the light values in shaders from dropping too far below the skylight (to the ambient) if N.L is small 
+                    // bias the normals towards the amount of ambient/skylight in the lumel
+                    // this is necessary to prevent the light values in shaders from dropping too far below the skylight (to the ambient) if N.L is small
                     ray->normalize();
                     int l = max(r, max(g, b)), a = max(ar, max(ag, ab));
                     ray->mul(max(l-a, 0));
@@ -992,7 +997,7 @@ static void clearsurfaces(cube *c)
     {
         if(c[i].ext)
         {
-            loopj(6) 
+            loopj(6)
             {
                 surfaceinfo &surf = c[i].ext->surfaces[j];
                 if(!surf.used()) continue;
@@ -1011,7 +1016,7 @@ static void clearsurfaces(cube *c)
                         v.norm = 0;
                     }
                 }
-            } 
+            }
         }
         if(c[i].children) clearsurfaces(c[i].children);
     }
@@ -1037,11 +1042,11 @@ void findsunlights()
     loopi(numents) if(ents[i]->type == ET_SUNLIGHT) sunlights.add(ents[i]);
 }
 
-void clearlightcache(int e)
+void clearlightcache(int id)
 {
-    if(e >= 0)
+    if(id >= 0)
     {
-        const extentity &light = *entities::getents()[e];
+        const extentity &light = *entities::getents()[id];
         if(light.type == ET_LIGHT && light.attrs[0])
         {
             int radius = light.attrs[0];
@@ -1167,7 +1172,7 @@ static int packlightmaps(lightmapworker *w = NULL)
     {
         lightmaptask &t = lightmaptasks[0][packidx];
         if(!t.lightmaps) break;
-        if(t.ext && t.c->ext != t.ext) 
+        if(t.ext && t.c->ext != t.ext)
         {
             lightmapext &e = lightmapexts.add();
             e.c = t.c;
@@ -1181,21 +1186,21 @@ static int packlightmaps(lightmapworker *w = NULL)
         {
             l->packed = true;
             space += l->bufsize;
-            if(l->surface < 0 || !t.ext) continue; 
+            if(l->surface < 0 || !t.ext) continue;
             surfaceinfo &surf = t.ext->surfaces[l->surface];
             layoutinfo layout;
             packlightmap(*l, layout);
             int numverts = surf.numverts&MAXFACEVERTS;
             vertinfo *verts = t.ext->verts() + surf.verts;
-            if(surf.numverts&LAYER_DUP)
+            if(l->layers&LAYER_DUP)
             {
                 if(l->type&LM_ALPHA) surf.lmid[0] = layout.lmid;
                 else { surf.lmid[1] = layout.lmid; verts += numverts; }
             }
             else
             {
-                surf.lmid[0] = surf.numverts&LAYER_TOP ? layout.lmid : LMID_AMBIENT;
-                surf.lmid[1] = surf.numverts&LAYER_BOTTOM ? layout.lmid : LMID_AMBIENT;
+                if(l->layers&LAYER_TOP) surf.lmid[0] = layout.lmid;
+                if(l->layers&LAYER_BOTTOM) surf.lmid[1] = layout.lmid;
             }
             ushort offsetx = layout.x*((USHRT_MAX+1)/LM_PACKW), offsety = layout.y*((USHRT_MAX+1)/LM_PACKH);
             loopk(numverts)
@@ -1294,6 +1299,7 @@ static lightmapinfo *alloclightmap(lightmapworker *w)
     l->packed = false;
     l->bufsize = usedspace;
     l->surface = -1;
+    l->layers = 0;
     if(!w->firstlightmap) w->firstlightmap = l;
     if(w->lastlightmap) w->lastlightmap->next = l;
     w->lastlightmap = l;
@@ -1416,7 +1422,7 @@ static int setupsurface(lightmapworker *w, plane planes[2], int numplanes, const
     loopk(numverts)
     {
         litverts[k].u = ushort(floor(clamp(c[k].x*texscale.x, 0.0f, float(USHRT_MAX))));
-        litverts[k].v = ushort(floor(clamp(c[k].y*texscale.y, 0.0f, float(USHRT_MAX)))); 
+        litverts[k].v = ushort(floor(clamp(c[k].y*texscale.y, 0.0f, float(USHRT_MAX))));
     }
     return surftype;
 }
@@ -1446,7 +1452,7 @@ static lightmapinfo *setupsurfaces(lightmapworker *w, lightmaptask &task)
 
     w->curlightmaps = NULL;
     w->c = &c;
-    
+
     surfaceinfo surfaces[6];
     vertinfo litverts[6*2*MAXFACEVERTS];
     int numlitverts = 0;
@@ -1575,16 +1581,30 @@ static lightmapinfo *setupsurfaces(lightmapworker *w, lightmaptask &task)
                 }
 
                 w->lastlightmap->surface = i;
-                if(surftype==SURFACE_LIGHTMAP_BLEND) surf.numverts |= LAYER_BLEND;
+                w->lastlightmap->layers = (surftype==SURFACE_LIGHTMAP_BOTTOM ? LAYER_BOTTOM : LAYER_TOP);
+                if(surftype==SURFACE_LIGHTMAP_BLEND)
+                {
+                    surf.numverts |= LAYER_BLEND;
+                    w->lastlightmap->layers = LAYER_TOP;
+                    if((shader->type^layer->slot->shader->type)&SHADER_NORMALSLMS ||
+                       (shader->type&SHADER_NORMALSLMS && vslot.rotation!=layer->rotation))
+                        break;
+                    w->lastlightmap->layers |= LAYER_BOTTOM;
+                }
                 else
                 {
-                    surf.numverts |= (surftype==SURFACE_LIGHTMAP_BOTTOM ? LAYER_BOTTOM : LAYER_TOP);
+                    if(surftype==SURFACE_LIGHTMAP_BOTTOM)
+                    {
+                        surf.numverts |= LAYER_BOTTOM;
+                        w->lastlightmap->layers = LAYER_BOTTOM;
+                    }
+                    else
+                    {
+                        surf.numverts |= LAYER_TOP;
+                        w->lastlightmap->layers = LAYER_TOP;
+                    }
                     if(w->type&LM_ALPHA) removelmalpha(w);
                 }
-                if(surftype!=SURFACE_LIGHTMAP_BLEND) continue;
-                if((shader->type^layer->slot->shader->type)&SHADER_NORMALSLMS ||
-                   (shader->type&SHADER_NORMALSLMS && vslot.rotation!=layer->rotation))
-                    break;
                 continue;
             }
 
@@ -1598,7 +1618,9 @@ static lightmapinfo *setupsurfaces(lightmapworker *w, lightmaptask &task)
                 surf.numverts |= LAYER_TOP;
                 continue;
 
-            default: freelightmap(w); continue;
+            default:
+                freelightmap(w);
+                continue;
         }
 
         w->slot = layer->slot;
@@ -1620,6 +1642,7 @@ static lightmapinfo *setupsurfaces(lightmapworker *w, lightmaptask &task)
                 else if(!(surf.numverts&LAYER_DUP))
                 {
                     surf.numverts |= LAYER_DUP;
+                    w->lastlightmap->layers |= LAYER_DUP;
                     loopk(numverts)
                     {
                         vertinfo &src = curlitverts[k];
@@ -1630,6 +1653,7 @@ static lightmapinfo *setupsurfaces(lightmapworker *w, lightmaptask &task)
                     numlitverts += numverts;
                 }
                 surf.numverts |= LAYER_BOTTOM;
+                w->lastlightmap->layers |= LAYER_BOTTOM;
 
                 w->lastlightmap->surface = i;
                 break;
@@ -1676,9 +1700,9 @@ int lightmapworker::work(void *data)
             t.lightmaps = l;
             packlightmaps(w);
         }
-        else 
+        else
         {
-            if(packidx >= lightmaptasks[0].length()) SDL_CondSignal(emptycond);   
+            if(packidx >= lightmaptasks[0].length()) SDL_CondSignal(emptycond);
             SDL_CondWait(fullcond, tasklock);
         }
     }
@@ -1704,7 +1728,7 @@ static bool processtasks(bool finish = false)
             SDL_CondWaitTimeout(emptycond, tasklock, 250);
             CHECK_PROGRESS_LOCKED({ SDL_UnlockMutex(tasklock); return false; }, SDL_UnlockMutex(tasklock), SDL_LockMutex(tasklock));
         }
-        else 
+        else
         {
             while(allocidx < lightmaptasks[0].length())
             {
@@ -2036,7 +2060,7 @@ bool setlightmapquality(int quality)
     return true;
 }
 
-VAR(IDF_PERSIST, lightthreads, 1, 1, 16);
+VAR(IDF_PERSIST, lightthreads, 0, 0, 16);
 
 #define ALLOCLOCK(name, init) { if(lightmapping > 1) name = init(); if(!name) lightmapping = 1; }
 #define FREELOCK(name, destroy) { if(name) { destroy(name); name = NULL; } }
@@ -2049,12 +2073,12 @@ static void cleanuplocks()
     FREELOCK(emptycond, SDL_DestroyCond);
 }
 
-static void setupthreads()
+static void setupthreads(int numthreads)
 {
     loopi(2) lightmaptasks[i].setsize(0);
     lightmapexts.setsize(0);
     packidx = allocidx = 0;
-    lightmapping = lightthreads;
+    lightmapping = numthreads;
     if(lightmapping > 1)
     {
         ALLOCLOCK(lightlock, SDL_CreateMutex);
@@ -2114,9 +2138,10 @@ void calclight(int *quality)
     }
     progress(0, "computing lightmaps...");
     mpremip(true);
-    loadlayermasks();
     optimizeblendmap();
-    if(lightthreads > 1) preloadusedmapmodels(false, true);
+    loadlayermasks();
+    int numthreads = lightthreads > 0 ? lightthreads : numcpus;
+    if(numthreads > 1) preloadusedmapmodels(false, true);
     resetlightmaps(false);
     clearsurfaces(worldroot);
     findsunlights();
@@ -2129,7 +2154,7 @@ void calclight(int *quality)
     Uint32 start = SDL_GetTicks();
     calcnormals(lerptjoints > 0);
     show_calclight_lmprog();
-    setupthreads();
+    setupthreads(numthreads);
     generatelightmaps(worldroot, 0, 0, 0, hdr.worldsize >> 1);
     cleanupthreads();
     clearnormals();
@@ -2172,7 +2197,8 @@ void patchlight(int *quality)
     }
     progress(0, "patching lightmaps...");
     loadlayermasks();
-    if(lightthreads > 1) preloadusedmapmodels(false, true);
+    int numthreads = lightthreads > 0 ? lightthreads : numcpus;
+    if(numthreads > 1) preloadusedmapmodels(false, true);
     cleanuplightmaps();
     findsunlights();
     taskprogress = lmprog = 0;
@@ -2192,7 +2218,7 @@ void patchlight(int *quality)
     Uint32 start = SDL_GetTicks();
     if(patchnormals) calcnormals(lerptjoints > 0);
     show_calclight_lmprog();
-    setupthreads();
+    setupthreads(numthreads);
     generatelightmaps(worldroot, 0, 0, 0, hdr.worldsize >> 1);
     cleanupthreads();
     if(patchnormals) clearnormals();
@@ -2241,7 +2267,7 @@ void setfullbrightlevel(int fullbrightlevel)
     initlights();
 }
 
-VARF(0, fullbright, 0, 0, 1, if(lightmaptexs.length()) initlights());
+VARF(0, fullbright, 0, 0, 1, if(lightmaptexs.length()) { initlights(); lightents(); });
 VARF(IDF_PERSIST, fullbrightlevel, 0, 128, 255, setfullbrightlevel(fullbrightlevel));
 
 vector<LightMapTexture> lightmaptexs;
@@ -2319,7 +2345,7 @@ void fixrotatedlightmaps(cube &c, const ivec &co, int size)
         return;
     }
     if(!c.ext) return;
-    loopi(6) 
+    loopi(6)
     {
         if(c.merged&(1<<i)) continue;
         surfaceinfo &surf = c.ext->surfaces[i];
@@ -2340,7 +2366,7 @@ void fixrotatedlightmaps(cube &c, const ivec &co, int size)
             verts[1].x = verts[2].x; verts[1].y = verts[2].y; verts[1].z = verts[2].z;
             verts[2].x = verts[3].x; verts[2].y = verts[3].y; verts[2].z = verts[3].z;
             verts[3].x = tmp.x; verts[3].y = tmp.y; verts[3].z = tmp.z;
-            if(surf.numverts&LAYER_DUP) loopk(4) 
+            if(surf.numverts&LAYER_DUP) loopk(4)
             {
                 vertinfo &v = verts[k], &b = verts[k+4];
                 b.x = v.x;
@@ -2478,7 +2504,6 @@ void genlightmaptexs(int flagmask, int flagval)
         remaining[LM_BUMPMAP0] = remaining[LM_BUMPMAP1] = 0;
     }
 
-    extern int maxtexsize;
     int sizelimit = (maxtexsize ? min(maxtexsize, hwtexsize) : hwtexsize)/max(LM_PACKW, LM_PACKH);
     sizelimit = min(batchlightmaps, sizelimit*sizelimit);
     while(total)
@@ -2551,7 +2576,7 @@ void genlightmaptexs(int flagmask, int flagval)
     }
 }
 
-bool brightengeom = false;
+bool brightengeom = false, shouldlightents = false;
 
 void clearlights()
 {
@@ -2563,6 +2588,7 @@ void clearlights()
         e.light.color = vec(1, 1, 1);
         e.light.dir = vec(0, 0, 1);
     }
+    shouldlightents = false;
     if(nolights) return;
 
     genlightmaptexs(LM_ALPHA, 0);
@@ -2584,10 +2610,14 @@ void lightent(extentity &e, float height)
     lightreaching(target, e.light.color, e.light.dir, false, &e, amb);
 }
 
-void updateentlighting()
+void lightents(bool force)
 {
+    if(!force && !shouldlightents) return;
+
     const vector<extentity *> &ents = entities::getents();
     loopv(ents) lightent(*ents[i]);
+
+    shouldlightents = false;
 }
 
 void initlights()
@@ -2599,10 +2629,10 @@ void initlights()
     }
 
     clearlightcache();
-    updateentlighting();
     genlightmaptexs(LM_ALPHA, 0);
     genlightmaptexs(LM_ALPHA, LM_ALPHA);
     brightengeom = false;
+    shouldlightents = true; 
 }
 
 static inline void fastskylight(const vec &o, float tolerance, uchar *skylight, int flags = RAY_ALPHAPOLY, extentity *t = NULL, bool fast = false)
@@ -2676,7 +2706,7 @@ void lightreaching(const vec &target, vec &color, vec &dir, bool fast, extentity
         if(mag < 1e-4f) ray = vec(0, 0, -1);
         else
         {
-            ray.div(mag); 
+            ray.div(mag);
             if(shadowray(e.o, ray, mag, RAY_SHADOW | RAY_POLY, t) < mag)
                 continue;
         }
