@@ -39,7 +39,7 @@ void showcursor(bool show)
 void setcaption(const char *text)
 {
     static string caption = "";
-    defformatstring(newcaption)("%s v%s-%s %d bit (%s)%s%s", RE_NAME, RE_VER_STR, RE_PLATFORM, RE_ARCH, RE_RELEASE, text ? ": " : "", text ? text : "");
+    defformatstring(newcaption)("%s v%s-%s %d bit (%s)%s%s", versionname, versionstring, CUR_PLATFORM, CUR_ARCH, versionrelease, text ? ": " : "", text ? text : "");
     if(strcmp(caption, newcaption))
     {
         copystring(caption, newcaption);
@@ -82,10 +82,10 @@ void cleanup()
     cleanupserver();
     setscreensaver(true);
     showcursor(true);
-#ifdef FAKESHOWCURSOR
-    if(scursor) SDL_FreeCursor(scursor);
-    scursor = ncursor = NULL;
-#endif
+//#ifdef FAKESHOWCURSOR
+//    if(scursor) SDL_FreeCursor(scursor);
+//    scursor = ncursor = NULL;
+//#endif
     SDL_WM_GrabInput(SDL_GRAB_OFF);
     cleargamma();
     freeocta(worldroot);
@@ -118,7 +118,9 @@ void fatal(const char *s, ...)    // failure exit
     {
         defvformatstring(msg, s, s);
         if(logfile) logoutf("%s", msg);
+        #ifndef WIN32
         fprintf(stderr, "%s\n", msg);
+        #endif
         if(errors <= 1) // avoid recursion
         {
             setscreensaver(true);
@@ -129,7 +131,7 @@ void fatal(const char *s, ...)    // failure exit
                 cleargamma();
             }
             #ifdef WIN32
-            defformatstring(cap)("%s: Error", RE_NAME);
+            defformatstring(cap)("%s: Error", versionname);
             MessageBox(NULL, msg, cap, MB_OK|MB_SYSTEMMODAL);
             #endif
             SDL_Quit();
@@ -172,11 +174,11 @@ void reloadsignal(int signum)
 
 int initing = NOT_INITING;
 
-bool initwarning(const char *desc, int level, int type)
+bool initwarning(const char *desc, int level, int type, bool force)
 {
     if(initing < level)
     {
-        addchange(desc, type);
+        addchange(desc, type, force);
         return true;
     }
     return false;
@@ -232,18 +234,21 @@ void screenshot(char *sname)
     ImageData image(screen->w, screen->h, 3);
     glPixelStorei(GL_PACK_ALIGNMENT, 1);
     glReadPixels(0, 0, screen->w, screen->h, GL_RGB, GL_UNSIGNED_BYTE, image.data);
-    defformatstring(fname)("%s", sname && *sname ? sname : "screenshot");
-    saveimage(fname, image, imageformat, compresslevel, true, true);
+    string fname;
+    if(sname && *sname) copystring(fname, sname);
+    else formatstring(fname)("screenshots/%s", *filetimeformat ? gettime(clocktime, filetimeformat) : (*mapname ? mapname : "screen"));
+    saveimage(fname, image, imageformat, compresslevel, true);
 }
 
 COMMAND(0, screenshot, "s");
 COMMAND(0, quit, "");
 
-void setfullscreen(bool enable)
+void setfullscreen(bool enable, bool force)
 {
     if(!screen) return;
+    fullscreen = enable ? 1 : 0;
 #if defined(WIN32) || defined(__APPLE__)
-    initwarning(enable ? "fullscreen" : "windowed");
+    initwarning(enable ? "fullscreen" : "windowed", INIT_RESET, CHANGE_GFX, force);
 #else
     if(enable == !(screen->flags&SDL_FULLSCREEN))
     {
@@ -522,21 +527,17 @@ static inline bool pollevent(SDL_Event &event)
     return false;
 }
 
-bool interceptkey(int sym)
+bool interceptkey(int sym, int mod)
 {
     SDL_Event event;
-    while(pollevent(event))
+    while(pollevent(event)) switch(event.type)
     {
-        switch(event.type)
-        {
         case SDL_KEYDOWN:
-            if(event.key.keysym.sym == sym)
+            if(event.key.keysym.sym == sym && (!mod || SDL_GetModState()&mod))
                 return true;
-
         default:
             pushevent(event);
             break;
-        }
     }
     return false;
 }
@@ -665,7 +666,7 @@ VAR(IDF_PERSIST, menufps, 0, 60, 1000);
 
 void limitfps(int &millis, int curmillis)
 {
-    int limit = hasnoview() && menufps ? (maxfps ? min(maxfps, menufps) : menufps) : maxfps;
+    int limit = (hasnoview() || minimized) && menufps ? (maxfps ? min(maxfps, menufps) : menufps) : maxfps;
     if(!limit) return;
     static int fpserror = 0;
     int delay = 1000/limit - (millis-curmillis);
@@ -693,7 +694,7 @@ void stackdumper(unsigned int type, EXCEPTION_POINTERS *ep)
     EXCEPTION_RECORD *er = ep->ExceptionRecord;
     CONTEXT *context = ep->ContextRecord;
     string out, t;
-    formatstring(out)("%s Win32 Exception: 0x%x [0x%x]\n\n", RE_NAME, er->ExceptionCode, er->ExceptionCode==EXCEPTION_ACCESS_VIOLATION ? er->ExceptionInformation[1] : -1);
+    formatstring(out)("%s Win32 Exception: 0x%x [0x%x]\n\n", versionname, er->ExceptionCode, er->ExceptionCode==EXCEPTION_ACCESS_VIOLATION ? er->ExceptionInformation[1] : -1);
     SymInitialize(GetCurrentProcess(), NULL, TRUE);
 #ifdef _AMD64_
     STACKFRAME64 sf = {{context->Rip, 0, AddrModeFlat}, {}, {context->Rbp, 0, AddrModeFlat}, {context->Rsp, 0, AddrModeFlat}, 0};
@@ -801,10 +802,17 @@ SVAR(0, progresstitle, "");
 SVAR(0, progresstext, "");
 FVAR(0, progressamt, 0, 0, 1);
 FVAR(0, progresspart, 0, 0, 1);
+VAR(IDF_PERSIST, progressdelay, 0, 100, VAR_MAX);
+VAR(IDF_PERSIST, progressupdate, 0, 0, 1);
+int lastprogress = 0;
 
 void progress(float bar1, const char *text1, float bar2, const char *text2)
 {
     if(progressing || !inbetweenframes || envmapping) return;
+    int ticks = SDL_GetTicks();
+    if(lastprogress > 0 && ticks < 0) lastprogress = 1-INT_MAX;
+    if((actualvsync || !progressupdate || bar1 > 0) && ticks-lastprogress < progressdelay) return;
+    lastprogress = ticks;
     clientkeepalive();
 
     #ifdef __APPLE__
@@ -819,7 +827,7 @@ void progress(float bar1, const char *text1, float bar2, const char *text2)
     {
         if(text2) conoutf("%s [%.2f%%], %s [%.2f%%]", text1, bar1*100.f, text2, bar2*100.f);
         else if(text1) conoutf("%s [%.2f%%]", text1, bar1*100.f);
-        else conoutf("progressing [%.2f%%]", text1, bar1*100.f, text2, bar2*100.f);
+        else conoutf("progressing [%.2f%%]", bar1*100.f);
     }
 
     progressing = true;
@@ -827,6 +835,8 @@ void progress(float bar1, const char *text1, float bar2, const char *text2)
     progressing = false;
 }
 
+VAR(0, numcpus, 1, 1, 16);
+
 int main(int argc, char **argv)
 {
     #ifdef WIN32
@@ -838,6 +848,8 @@ int main(int argc, char **argv)
     #endif
     #endif
 
+    clocktime = time(NULL); // initialise
+
     setlogfile(NULL);
     setlocations(true);
 
@@ -850,6 +862,7 @@ int main(int argc, char **argv)
             case 'h': serveroption(argv[i]); break;
         }
     }
+    setlogfile("log.txt");
     execfile("init.cfg", false);
     for(int i = 1; i<argc; i++)
     {
@@ -871,10 +884,18 @@ int main(int argc, char **argv)
                     case 's': stencilbits = atoi(&argv[i][3]); break;
                     case 'u':
                     {
-                        extern int useshaders, shaderprecision;
-                        int n = atoi(&argv[i][3]);
-                        useshaders = n > 0 ? 1 : 0;
-                        shaderprecision = clamp(n - 1, 0, 2);
+                        extern int useshaders, shaderprecision, forceglsl;
+                        int sh = -1, prec = shaderprecision;
+                        for(int j = 3; argv[i][j]; j++) switch(argv[i][j])
+                        {
+                            case 'a': case 'A': forceglsl = 0; sh = 1; break;
+                            case 'g': case 'G': forceglsl = 1; sh = 1; break;
+                            case 'f': case 'F': case '0': sh = 0; break;
+                            case '1': case '2': case '3': if(sh < 0) sh = 1; prec = argv[i][j] - '1'; break;
+                            default: break;
+                        }
+                        useshaders = sh > 0 ? 1 : 0;
+                        shaderprecision = prec;
                         break;
                     }
                     default: conoutf("\frunknown display option %c", argv[i][2]); break;
@@ -891,6 +912,8 @@ int main(int argc, char **argv)
 
     initing = NOT_INITING;
 
+    numcpus = clamp(guessnumcpus(), 1, 16);
+
     conoutf("loading enet..");
     if(enet_initialize()<0) fatal("Unable to initialise network module");
     atexit(enet_deinitialize);
@@ -957,7 +980,7 @@ int main(int argc, char **argv)
 
     conoutf("loading defaults..");
 
-    if(!execfile("stdlib.cfg", false)) fatal("cannot find data files");
+    if(!execfile("config/stdlib.cfg", false)) fatal("cannot find data files");
     if(!setfont("default")) fatal("no default font specified");
     inbetweenframes = true;
     progress(0, "please wait..");
@@ -997,7 +1020,6 @@ int main(int argc, char **argv)
     localconnect(false);
     resetfps();
     UI::setup();
-    //game::preload();
 
     for(int frameloops = 0; ; frameloops = frameloops >= INT_MAX-1 ? MAXFPSHISTORY+1 : frameloops+1)
     {
