@@ -45,7 +45,7 @@ struct QuadNode
         }
     }
 
-    void genmatsurf(uchar mat, uchar orient, uchar flags, int x, int y, int z, int size, materialsurface *&matbuf)
+    void genmatsurf(ushort mat, uchar orient, uchar flags, int x, int y, int z, int size, materialsurface *&matbuf)
     {
         materialsurface &m = *matbuf++;
         m.material = mat;
@@ -59,7 +59,7 @@ struct QuadNode
         m.o[dim] = z;
     }
 
-    void genmatsurfs(uchar mat, uchar orient, uchar flags, int z, materialsurface *&matbuf)
+    void genmatsurfs(ushort mat, uchar orient, uchar flags, int z, materialsurface *&matbuf)
     {
         if(filled == 0xF) genmatsurf(mat, orient, flags, x, y, z, size, matbuf);
         else if(filled)
@@ -142,14 +142,14 @@ static void drawmaterial(const materialsurface &m, float offset)
     }
 }
 
-namemap materials[] =
+extern const namemap materials[] =
 {
     {"air", MAT_AIR},
-    {"water", MAT_WATER},
+    {"water", MAT_WATER}, {"water1", MAT_WATER}, {"water2", MAT_WATER+1}, {"water3", MAT_WATER+2}, {"water4", MAT_WATER+3},
+    {"glass", MAT_GLASS}, {"glass1", MAT_GLASS}, {"glass2", MAT_GLASS+1}, {"glass3", MAT_GLASS+2}, {"glass4", MAT_GLASS+3},
+    {"lava", MAT_LAVA}, {"lava1", MAT_LAVA}, {"lava2", MAT_LAVA+1}, {"lava3", MAT_LAVA+2}, {"lava4", MAT_LAVA+3},
     {"clip", MAT_CLIP},
-    {"glass", MAT_GLASS},
     {"noclip", MAT_NOCLIP},
-    {"lava", MAT_LAVA},
     {"aiclip", MAT_AICLIP},
     {"death", MAT_DEATH},
     {"ladder", MAT_LADDER},
@@ -162,16 +162,32 @@ int findmaterial(const char *name, bool tryint)
     return tryint && isnumeric(*name) ? atoi(name) : -1;
 }
 
-
 const char *findmaterialname(int type)
 {
     loopi(sizeof(materials)/sizeof(materials[0])) if(materials[i].id == type) { return materials[i].name; }
     return NULL;
 }
 
-int visiblematerial(cube &c, int orient, int x, int y, int z, int size, uchar matmask)
+const char *getmaterialdesc(int mat, const char *prefix)
 {
-    uchar mat = c.material&matmask;
+    static const ushort matmasks[] = { MATF_VOLUME|MATF_INDEX, MATF_CLIP, MAT_DEATH, MAT_LADDER, MAT_ALPHA };
+    static string desc;
+    desc[0] = '\0';
+    loopi(sizeof(matmasks)/sizeof(matmasks[0])) if(mat&matmasks[i])
+    {               
+        const char *matname = findmaterialname(mat&matmasks[i]);
+        if(matname)     
+        {               
+            concatstring(desc, desc[0] ? ", " : prefix);
+            concatstring(desc, matname);
+        }
+    }
+    return desc;
+}
+
+int visiblematerial(const cube &c, int orient, int x, int y, int z, int size, ushort matmask)
+{
+    ushort mat = c.material&matmask;
     switch(mat)
     {
     case MAT_AIR:
@@ -196,15 +212,15 @@ int visiblematerial(cube &c, int orient, int x, int y, int z, int size, uchar ma
     return MATSURF_NOT_VISIBLE;
 }
 
-void genmatsurfs(cube &c, int cx, int cy, int cz, int size, vector<materialsurface> &matsurfs)
+void genmatsurfs(const cube &c, int cx, int cy, int cz, int size, vector<materialsurface> &matsurfs)
 {
     loopi(6)
     {
-        static uchar matmasks[] = { MATF_VOLUME, MATF_CLIP, MAT_DEATH, MAT_LADDER, MAT_ALPHA };
+        static const ushort matmasks[] = { MATF_VOLUME|MATF_INDEX, MATF_CLIP, MAT_DEATH, MAT_LADDER, MAT_ALPHA };
         loopj(sizeof(matmasks)/sizeof(matmasks[0]))
         {
             int matmask = matmasks[j];
-            int vis = visiblematerial(c, i, cx, cy, cz, size, matmask);
+            int vis = visiblematerial(c, i, cx, cy, cz, size, matmask&~MATF_INDEX);
             if(vis != MATSURF_NOT_VISIBLE)
             {
                 materialsurface m;
@@ -310,7 +326,7 @@ int optimizematsurfs(materialsurface *matbuf, int matsurfs)
                cur->flags == start->flags &&
                cur->o[dim] == start->o[dim])
             ++cur;
-         if(!isliquid(start->material) || start->orient != O_TOP || !vertwater)
+         if(!isliquid(start->material&MATF_VOLUME) || start->orient != O_TOP || !vertwater)
          {
             if(start!=matbuf) memmove(matbuf, start, (cur-start)*sizeof(materialsurface));
             matbuf += mergemats(matbuf, cur-start);
@@ -351,13 +367,14 @@ void setupmaterials(int start, int len)
         loopj(va->matsurfs)
         {
             materialsurface &m = va->matbuf[j];
-            if(m.material==MAT_WATER && m.orient==O_TOP)
+            int matvol = m.material&MATF_VOLUME;
+            if(matvol==MAT_WATER && m.orient==O_TOP)
             {
                 m.index = water.length();
                 loopvk(water)
                 {
                     materialsurface &n = *water[k].m;
-                    if(m.o.z!=n.o.z) continue;
+                    if(m.material!=n.material || m.o.z!=n.o.z) continue;
                     if(n.o.x+n.rsize==m.o.x || m.o.x+m.rsize==n.o.x)
                     {
                         if(n.o.y+n.csize>m.o.y && n.o.y<m.o.y+m.csize) uf.unite(m.index, n.index);
@@ -375,7 +392,7 @@ void setupmaterials(int start, int len)
                 wi.depth = double(depth)*m.rsize*m.csize;
                 wi.area = m.rsize*m.csize;
             }
-            else if(isliquid(m.material) && m.orient!=O_BOTTOM && m.orient!=O_TOP)
+            else if(isliquid(matvol) && m.orient!=O_BOTTOM && m.orient!=O_TOP)
             {
                 m.ends = 0;
                 int dim = dimension(m.orient), coord = dimcoord(m.orient);
@@ -401,9 +418,9 @@ void setupmaterials(int start, int len)
                     o[dim^1] += csize;
                 }
             }
-            else if(m.material==MAT_GLASS)
+            else if(matvol==MAT_GLASS)
             {
-                if(!hasCM) m.envmap = EMID_NONE;
+                if(!hasCM || (renderpath==R_FIXEDFUNCTION && !hasTE)) m.envmap = EMID_NONE;
                 else
                 {
                     int dim = dimension(m.orient);
@@ -413,9 +430,9 @@ void setupmaterials(int start, int len)
                     m.envmap = closestenvmap(center);
                 }
             }
-            if(m.material&MATF_VOLUME) hasmat |= 1<<m.material;
+            if(matvol) hasmat |= 1<<m.material;
             m.skip = 0;
-            if(skip && m.material == skip->material && m.orient == skip->orient && skip->skip < 0xFF)
+            if(skip && m.material == skip->material && m.orient == skip->orient && skip->skip < 0xFFFF)
                 skip->skip++;
             else 
                 skip = &m;
@@ -436,19 +453,19 @@ void setupmaterials(int start, int len)
         water[i].m->light = water[root].m->light;
         water[i].m->depth = (short)(water[root].depth/water[root].area);
     }
-    if(hasmat&(1<<MAT_WATER))
+    if(hasmat&(0xF<<MAT_WATER))
     {
         loadcaustics(true);
         preloadwatershaders(true);
-        lookupmaterialslot(MAT_WATER);
+        loopi(4) if(hasmat&(1<<(MAT_WATER+i))) lookupmaterialslot(MAT_WATER+i);
     }
-    if(hasmat&(1<<MAT_LAVA))
+    if(hasmat&(0xF<<MAT_LAVA))
     {
         useshaderbyname("lava");
         useshaderbyname("lavaglare");
-        lookupmaterialslot(MAT_LAVA);
+        loopi(4) if(hasmat&(1<<(MAT_LAVA+i))) lookupmaterialslot(MAT_LAVA+i);
     }
-    if(hasmat&(1<<MAT_GLASS)) useshaderbyname("glass");
+    if(hasmat&(0xF<<MAT_GLASS)) useshaderbyname("glass");
 }
 
 VAR(IDF_PERSIST, showmat, 0, 0, 1);
@@ -460,6 +477,11 @@ static bool sortedit;
 static inline bool vismatcmp(const materialsurface *xm, const materialsurface *ym)
 {
     const materialsurface &x = *xm, &y = *ym;
+    if(!sortedit)
+    {
+        if((x.material&MATF_VOLUME) == MAT_LAVA) { if((y.material&MATF_VOLUME) != MAT_LAVA) return true; }
+        else if((y.material&MATF_VOLUME) == MAT_LAVA) return false;
+    }
     int xdim = dimension(x.orient), ydim = dimension(y.orient);
     loopi(3)
     {
@@ -508,9 +530,10 @@ void sortmaterials(vector<materialsurface *> &vismats)
             materialsurface &m = va->matbuf[i];
             if(!editmode || !showmat || envmapping)
             {
-                if(m.material==MAT_WATER && (m.orient==O_TOP || (refracting<0 && reflectz>hdr.worldsize))) { i += m.skip; continue; }
+                int matvol = m.material&MATF_VOLUME;
+                if(matvol==MAT_WATER && (m.orient==O_TOP || (refracting<0 && reflectz>hdr.worldsize))) { i += m.skip; continue; }
                 if(m.flags&materialsurface::F_EDIT) { i += m.skip; continue; }
-                if(glaring && m.material!=MAT_LAVA) { i += m.skip; continue; }
+                if(glaring && matvol!=MAT_LAVA) { i += m.skip; continue; }
             }
             else if(glaring) continue;
             vismats.add(&m);
@@ -532,7 +555,7 @@ void rendermatgrid(vector<materialsurface *> &vismats)
         {
             xtraverts += varray::end();
             lastmat = m.material;
-            switch(m.material)
+            switch(m.material&~MATF_INDEX)
             {
                 case MAT_WATER:  glColor3ub( 0,  0,  85); break; // blue
                 case MAT_CLIP:   glColor3ub(85,  0,  0); break; // red
@@ -552,6 +575,22 @@ void rendermatgrid(vector<materialsurface *> &vismats)
     disablepolygonoffset(GL_POLYGON_OFFSET_LINE);
 }
 
+#define GLASSVARS(name) \
+    bvec name##col(0x20, 0x80, 0xC0); \
+    VARF(IDF_HEX|IDF_WORLD, name##colour, 0, 0x2080C0, 0xFFFFFF, \
+    { \
+        if(!name##colour) name##colour = 0x2080FF; \
+        name##col = bvec((name##colour>>16)&0xFF, (name##colour>>8)&0xFF, name##colour&0xFF); \
+    });
+
+GLASSVARS(glass)
+GLASSVARS(glass2)
+GLASSVARS(glass3)
+GLASSVARS(glass4)
+
+GETMATIDXVAR(glass, colour, int)
+GETMATIDXVAR(glass, col, const bvec &)
+
 VAR(IDF_PERSIST, glassenv, 0, 1, 1);
 
 static void drawglass(const materialsurface &m, float offset, const vec *normal = NULL)
@@ -601,12 +640,11 @@ void rendermaterials()
 
     varray::enable();
 
-    MSlot &wslot = lookupmaterialslot(MAT_WATER, false), &lslot = lookupmaterialslot(MAT_LAVA, false);
-    uchar wcol[4] = { watercol[0], watercol[1], watercol[2], 192 }, wfcol[4] = { waterfallcol[0], waterfallcol[1], waterfallcol[2], 192 };
-    if(!wfcol[0] && !wfcol[1] && !wfcol[2]) memcpy(wfcol, wcol, 3);
-    int lastorient = -1, lastmat = -1;
+    MSlot *mslot = NULL;
+    uchar wcol[4] = { 255, 255, 255, 192 }, wfcol[4] = { 255, 255, 255, 192 };
+    int lastorient = -1, lastmat = -1, usedwaterfall = -1;
     GLenum textured = GL_TEXTURE_2D;
-    bool depth = true, blended = false, overbright = false, usedcamera = false, usedwaterfall = false;
+    bool depth = true, blended = false, tint = false, overbright = false, usedcamera = false;
     ushort envmapped = EMID_NONE;
     static const vec normals[6] =
     {
@@ -635,7 +673,7 @@ void rendermaterials()
             if(lastmat!=m.material)
             {
                 xtraverts += varray::end();
-                switch(m.material)
+                switch(m.material&~MATF_INDEX)
                 {
                     case MAT_WATER:    glColor3ub(255, 128,   0); break; // blue
                     case MAT_CLIP:     glColor3ub(  0, 255, 255); break; // red
@@ -656,39 +694,48 @@ void rendermaterials()
     else loopv(vismats)
     {
         const materialsurface &m = *vismats[i];
-        if(lastmat!=m.material || lastorient!=m.orient || (m.material==MAT_GLASS && envmapped && m.envmap != envmapped))
+        int matvol = m.material&~MATF_INDEX;
+        if(lastmat!=m.material || lastorient!=m.orient || (matvol==MAT_GLASS && envmapped && m.envmap != envmapped))
         {
             int fogtype = lastfogtype;
-            switch(m.material)
+            switch(matvol)
             {
                 case MAT_WATER:
-                {
-                    if(!wslot.loaded || m.orient == O_TOP) continue;
-                    else if(lastmat==MAT_WATER) break;
+                    if(m.orient == O_TOP) continue;
+                    if(lastmat == m.material) break;
+                    mslot = &lookupmaterialslot(m.material, false);
+                    if(!mslot->loaded || !mslot->sts.inrange(1)) continue;
                     else
                     {
-                        if(!wslot.sts.inrange(1)) continue;
                         xtraverts += varray::end();
-                        glBindTexture(GL_TEXTURE_2D, wslot.sts[1].t->id);
+                        glBindTexture(GL_TEXTURE_2D, mslot->sts[1].t->id);
                         float angle = fmod(float(lastmillis/(renderpath!=R_FIXEDFUNCTION ? 600.0f : 300.0f)/(2*M_PI)), 1.0f), 
                               s = angle - int(angle) - 0.5f;
                         s *= 8 - fabs(s)*16;
                         wfwave = vertwater ? WATER_AMPLITUDE*s-WATER_OFFSET : -WATER_OFFSET;
                         wfscroll = 16.0f*lastmillis/1000.0f;
-                        wfxscale = TEX_SCALE/(wslot.sts[1].t->xs*wslot.scale);
-                        wfyscale = TEX_SCALE/(wslot.sts[1].t->ys*wslot.scale);
-                    }
-                    if(lastmat!=MAT_WATER || (lastorient==O_TOP)!=(m.orient==O_TOP))
-                    {
-                        if(overbright) { resettmu(0); overbright = false; }
-                        if(m.orient==O_TOP)
+                        wfxscale = TEX_SCALE/(mslot->sts[1].t->xs*mslot->scale);
+                        wfyscale = TEX_SCALE/(mslot->sts[1].t->ys*mslot->scale);
+
+                        memcpy(wcol, getwatercol(m.material).v, 3);
+                        memcpy(wfcol, getwaterfallcol(m.material).v, 3);
+                        if(!wfcol[0] && !wfcol[1] && !wfcol[2]) memcpy(wfcol, wcol, 3);
+                        int wfog = getwaterfog(m.material);
+
+                        if(overbright || tint) { resettmu(0); overbright = tint = false; }
+                        if(!wfog && (renderpath==R_FIXEDFUNCTION || !hasCM || !waterfallenv))
                         {
-                            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-                            glColor4ubv(wcol);
-                            foggedshader->set();
+                            glColor3ubv(wfcol);
+                            foggednotextureshader->set();
                             fogtype = 1;
-                            if(!blended) { glEnable(GL_BLEND); blended = true; }
-                            if(depth) { glDepthMask(GL_FALSE); depth = false; }
+                            if(blended) { glDisable(GL_BLEND); blended = false; }
+                            if(!depth) { glDepthMask(GL_TRUE); depth = true; }
+                            if(textured)
+                            {
+                                glDisable(textured);
+                                textured = 0;
+                            }
+                            break;
                         }
                         else if(renderpath==R_FIXEDFUNCTION || ((!waterfallrefract || reflecting || refracting) && (!hasCM || !waterfallenv)))
                         {
@@ -710,51 +757,52 @@ void rendermaterials()
                                 usedcamera = true;
                             }
 
-                            #define SETWATERFALLSHADER(name) \
-                            do { \
-                                static Shader *name##shader = NULL; \
-                                if(!name##shader) name##shader = lookupshaderbyname(#name); \
-                                name##shader->set(); \
-                            } while(0)
-
-                            if(waterfallrefract && !reflecting && !refracting)
+                            if(waterfallrefract && wfog && !reflecting && !refracting)
                             {
-                                if(hasCM && waterfallenv) SETWATERFALLSHADER(waterfallenvrefract);
-                                else SETWATERFALLSHADER(waterfallrefract);
+                                if(hasCM && waterfallenv) SETSHADER(waterfallenvrefract);
+                                else SETSHADER(waterfallrefract);
                                 if(blended) { glDisable(GL_BLEND); blended = false; }
                                 if(!depth) { glDepthMask(GL_TRUE); depth = true; }
                             }
                             else
                             {
-                                SETWATERFALLSHADER(waterfallenv);
+                                SETSHADER(waterfallenv);
                                 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-                                if(!blended) { glEnable(GL_BLEND); blended = true; }
-                                if(depth) { glDepthMask(GL_FALSE); depth = false; }
+                                if(wfog)
+                                {
+                                    if(!blended) { glEnable(GL_BLEND); blended = true; }
+                                    if(depth) { glDepthMask(GL_FALSE); depth = false; }
+                                }
+                                else
+                                {
+                                    if(blended) { glDisable(GL_BLEND); blended = false; }
+                                    if(!depth) { glDepthMask(GL_TRUE); depth = true; }
+                                }
                             }
 
-                            if(!usedwaterfall)
+                            if(usedwaterfall != m.material)
                             {
-                                Texture *dudv = wslot.sts.inrange(5) ? wslot.sts[5].t : notexture;
-                                float scale = 8.0f/(dudv->ys*wslot.scale);
+                                Texture *dudv = mslot->sts.inrange(5) ? mslot->sts[5].t : notexture;
+                                float scale = 8.0f/(dudv->ys*mslot->scale);
                                 setlocalparamf("dudvoffset", SHPARAM_PIXEL, 1, 0, scale*16*lastmillis/1000.0f);
 
                                 glActiveTexture_(GL_TEXTURE1_ARB);
-                                glBindTexture(GL_TEXTURE_2D, wslot.sts.inrange(4) ? wslot.sts[4].t->id : notexture->id);
+                                glBindTexture(GL_TEXTURE_2D, mslot->sts.inrange(4) ? mslot->sts[4].t->id : notexture->id);
                                 glActiveTexture_(GL_TEXTURE2_ARB);
-                                glBindTexture(GL_TEXTURE_2D, wslot.sts.inrange(5) ? wslot.sts[5].t->id : notexture->id);
+                                glBindTexture(GL_TEXTURE_2D, mslot->sts.inrange(5) ? mslot->sts[5].t->id : notexture->id);
                                 if(hasCM && waterfallenv)
                                 {
                                     glActiveTexture_(GL_TEXTURE3_ARB);
-                                    glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, lookupenvmap(wslot));
+                                    glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, lookupenvmap(*mslot));
                                 }
-                                if(waterfallrefract && (!reflecting || !refracting))
+                                if(waterfallrefract && (!reflecting || !refracting) && usedwaterfall < 0)
                                 {
                                     extern void setupwaterfallrefract(GLenum tmu1, GLenum tmu2);
                                     setupwaterfallrefract(GL_TEXTURE4_ARB, GL_TEXTURE0_ARB);
                                 }
                                 else glActiveTexture_(GL_TEXTURE0_ARB);
 
-                                usedwaterfall = true;
+                                usedwaterfall = m.material;
                             }
                         }
                     }
@@ -765,17 +813,17 @@ void rendermaterials()
                         textured = GL_TEXTURE_2D;
                     }
                     break;
-                }
+
                 case MAT_LAVA:
-                {
-                    if(!lslot.loaded) continue;
-                    if(lastmat==MAT_LAVA && lastorient!=O_TOP && m.orient!=O_TOP) break;
+                    if(lastmat==m.material && lastorient!=O_TOP && m.orient!=O_TOP) break;
+                    mslot = &lookupmaterialslot(m.material, false);
+                    if(!mslot->loaded) continue;
                     else
                     {
                         int subslot = m.orient==O_TOP ? 0 : 1;
-                        if(!lslot.sts.inrange(subslot)) continue;
+                        if(!mslot->sts.inrange(subslot)) continue;
                         xtraverts += varray::end();
-                        glBindTexture(GL_TEXTURE_2D, lslot.sts[subslot].t->id);
+                        glBindTexture(GL_TEXTURE_2D, mslot->sts[subslot].t->id);
                     }
                     if(m.orient!=O_TOP)
                     {
@@ -784,14 +832,14 @@ void rendermaterials()
                         s *= 8 - fabs(s)*16;
                         wfwave = vertwater ? WATER_AMPLITUDE*s-WATER_OFFSET : -WATER_OFFSET;
                         wfscroll = 16.0f*lastmillis/3000.0f;
-                        wfxscale = TEX_SCALE/(lslot.sts[1].t->xs*lslot.scale);
-                        wfyscale = TEX_SCALE/(lslot.sts[1].t->ys*lslot.scale);
+                        wfxscale = TEX_SCALE/(mslot->sts[1].t->xs*mslot->scale);
+                        wfyscale = TEX_SCALE/(mslot->sts[1].t->ys*mslot->scale);
                     }
-                    if(lastmat!=MAT_LAVA)
+                    if(lastmat!=m.material)
                     {
                         if(!depth) { glDepthMask(GL_TRUE); depth = true; }
                         if(blended) { glDisable(GL_BLEND); blended = false; }
-                        if(renderpath==R_FIXEDFUNCTION && !overbright) { setuptmu(0, "C * T x 2"); overbright = true; }
+                        if(renderpath==R_FIXEDFUNCTION && !overbright) { setuptmu(0, "C * T x 2"); overbright = true; tint = false; }
                         float t = lastmillis/2000.0f;
                         t -= floor(t);
                         t = 1.0f - 2*fabs(t-0.5f);
@@ -799,10 +847,7 @@ void rendermaterials()
                         if(renderpath!=R_FIXEDFUNCTION && glare) t = 0.625f + 0.075f*t;
                         else t = 0.5f + 0.5f*t;
                         glColor3f(t, t, t);
-                        static Shader *lavashader = NULL, *lavaglareshader = NULL;
-                        if(!lavashader) lavashader = lookupshaderbyname("lava");
-                        if(!lavaglareshader) lavaglareshader = lookupshaderbyname("lavaglare");
-                        (glaring ? lavaglareshader : lavashader)->set();
+                        if(glaring) SETSHADER(lavaglare); else SETSHADER(lava);
                         fogtype = 1;
                     }
                     if(textured!=GL_TEXTURE_2D)
@@ -812,9 +857,9 @@ void rendermaterials()
                         textured = GL_TEXTURE_2D;
                     }
                     break;
-                }
+
                 case MAT_GLASS:
-                    if((m.envmap==EMID_NONE || !glassenv || (envmapped==m.envmap && textured==GL_TEXTURE_CUBE_MAP_ARB)) && lastmat==MAT_GLASS) break;
+                    if((m.envmap==EMID_NONE || !glassenv || (envmapped==m.envmap && textured==GL_TEXTURE_CUBE_MAP_ARB)) && lastmat==m.material) break;
                     xtraverts += varray::end();
                     if(m.envmap!=EMID_NONE && glassenv)
                     {
@@ -835,39 +880,40 @@ void rendermaterials()
                             envmapped = m.envmap;
                         }
                     }
-                    if(lastmat!=MAT_GLASS)
+                    if(lastmat!=m.material)
                     {
                         if(!blended) { glEnable(GL_BLEND); blended = true; }
-                        if(overbright) { resettmu(0); overbright = false; }
                         if(depth) { glDepthMask(GL_FALSE); depth = false; }
+                        const bvec &gcol = getglasscol(m.material);
                         if(m.envmap!=EMID_NONE && glassenv)
                         {
                             if(renderpath==R_FIXEDFUNCTION)
                             {
+                                if(!tint) { colortmu(0, 0.8f, 0.8f, 0.8f, 0.8f); setuptmu(0, "T , C @ Ka", "= Ca"); tint = true; overbright = false; }
                                 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-                                glColor4f(0.8f, 0.9f, 1.0f, 0.5f);
+                                glColor4f(gcol.x/255.0f, gcol.y/255.0f, gcol.z/255.0f, 0.25f);
                                 fogtype = 1;
                             }
                             else
                             {
+                                if(overbright || tint) { resettmu(0); overbright = tint = false; }
                                 glBlendFunc(GL_ONE, GL_SRC_ALPHA);
-                                glColor3f(0, 0.5f, 1.0f);
+                                glColor3ubv(gcol.v);
                             }
-                            static Shader *glassshader = NULL;
-                            if(!glassshader) glassshader = lookupshaderbyname("glass");
-                            glassshader->set();
+                            SETSHADER(glass);
                         }
                         else
                         {
+                            if(overbright || tint) { resettmu(0); overbright = tint = false; }
                             if(textured)
                             {
                                 glDisable(textured);
                                 textured = 0;
                             }
-                            glBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_COLOR);
-                            glColor3f(0.5f, 0.25f, 0.0f);
+                            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+                            glColor4f(gcol.x/255.0f, gcol.y/255.0f, gcol.z/255.0f, 0.15f);
                             foggednotextureshader->set();
-                            fogtype = 0;
+                            fogtype = 1;
                         }
                     }
                     break;
@@ -882,14 +928,14 @@ void rendermaterials()
                 lastfogtype = fogtype;
             }
         }
-        switch(m.material)
+        switch(matvol)
         {
             case MAT_WATER:
                 renderwaterfall(m, 0.1f, renderpath!=R_FIXEDFUNCTION && hasCM && waterfallenv ? &normals[m.orient] : NULL);
                 break;
 
             case MAT_LAVA:
-                if(m.orient==O_TOP) renderlava(m, lslot.sts[0].t, lslot.scale);
+                if(m.orient==O_TOP) renderlava(m, mslot->sts[0].t, mslot->scale);
                 else renderwaterfall(m, 0.1f);
                 break;
 
@@ -904,7 +950,7 @@ void rendermaterials()
 
     if(!depth) glDepthMask(GL_TRUE);
     if(blended) glDisable(GL_BLEND);
-    if(overbright) resettmu(0);
+    if(overbright || tint) resettmu(0);
     if(!lastfogtype) glFogfv(GL_FOG_COLOR, oldfogc);
     if(editmode && showmat && !envmapping)
     {
