@@ -207,14 +207,6 @@ void guistayopen(uint *contents)
     shouldclearmenu = oldclearmenu;
 }
 
-void guinoautotab(uint *contents)
-{
-    if(!cgui) return;
-    cgui->allowautotab(false);
-    execute(contents);
-    cgui->allowautotab(true);
-}
-
 void guinohitfx(uint *contents)
 {
     if(!cgui) return;
@@ -308,9 +300,9 @@ void guislice(char *path, char *action, float *scale, float *start, float *end,
     }
 }
 
-void guitext(char *name, char *icon, int *colour)
+void guitext(char *name, char *icon, int *colour, int *icolour)
 {
-    if(cgui) cgui->text(name, *colour ? *colour : 0xFFFFFF, icon[0] ? icon : NULL);
+    if(cgui) cgui->text(name, *colour ? *colour : 0xFFFFFF, icon[0] ? icon : NULL, *icolour ? *icolour : 0xFFFFFF);
 }
 
 void guititle(char *name)
@@ -413,8 +405,8 @@ void guiprogress(float *percent, float *scale)
 
 void guislider(char *var, int *min, int *max, char *onchange, int *reverse, int *scroll, int *colour)
 {
-    if(!cgui) return;
-    int oldval = getval(var), val = oldval, vmin = *max ? *min : getvarmin(var), vmax = *max ? *max : getvarmax(var);
+    if(!cgui || !var || !*var) return;
+    int oldval = getval(var), val = oldval, vmin = *max > INT_MIN ? *min : getvarmin(var), vmax = *max > INT_MIN ? *max : getvarmax(var);
     if(vmax >= INT_MAX-1)
     { // not a sane value for a slider..
         int vdef = getvardef(var);
@@ -491,28 +483,28 @@ void guibitfield(char *name, char *var, int *mask, char *onchange, int *colour)
 }
 
 //-ve length indicates a wrapped text field of any (approx 260 chars) length, |length| is the field width
-void guifield(char *var, int *maxlength, char *onchange, int *colour)
+void guifield(char *var, int *maxlength, char *onchange, int *colour, int *focus, char *parent)
 {
     if(!cgui) return;
     const char *initval = getsval(var);
-    char *result = cgui->field(var, *colour ? *colour : 0xFFFFFF, *maxlength ? *maxlength : 12, 0, initval);
+    char *result = cgui->field(var, *colour ? *colour : 0xFFFFFF, *maxlength ? *maxlength : 12, 0, initval, EDITORFOCUSED, *focus!=0, parent);
     if(result) updateval(var, result, onchange);
 }
 
 //-ve maxlength indicates a wrapped text field of any (approx 260 chars) length, |maxlength| is the field width
-void guieditor(char *name, int *maxlength, int *height, int *mode, int *colour)
+void guieditor(char *name, int *maxlength, int *height, int *mode, int *colour, int *focus, char *parent)
 {
     if(!cgui) return;
-    cgui->field(name, *colour ? *colour : 0xFFFFFF, *maxlength ? *maxlength : 12, *height, NULL, *mode<=0 ? EDITORFOREVER : *mode);
+    cgui->field(name, *colour ? *colour : 0xFFFFFF, *maxlength ? *maxlength : 12, *height, NULL, *mode<=0 ? EDITORFOREVER : *mode, *focus!=0, parent);
     //returns a non-NULL pointer (the currentline) when the user commits, could then manipulate via text* commands
 }
 
 //-ve length indicates a wrapped text field of any (approx 260 chars) length, |length| is the field width
-void guikeyfield(char *var, int *maxlength, char *onchange, int *colour)
+void guikeyfield(char *var, int *maxlength, char *onchange, int *colour, int *focus, char *parent)
 {
     if(!cgui) return;
     const char *initval = getsval(var);
-    char *result = cgui->keyfield(var, *colour ? *colour : 0xFFFFFF, *maxlength ? *maxlength : -8, 0, initval);
+    char *result = cgui->keyfield(var, *colour ? *colour : 0xFFFFFF, *maxlength ? *maxlength : -8, 0, initval, EDITORFOCUSED, *focus!=0, parent);
     if(result) updateval(var, result, onchange);
 }
 
@@ -583,12 +575,11 @@ COMMAND(0, newgui, "sss");
 COMMAND(0, guiheader, "s");
 COMMAND(0, guimodify, "ss");
 COMMAND(0, guibutton, "ssssi");
-COMMAND(0, guitext, "ssi");
+COMMAND(0, guitext, "ssii");
 COMMANDN(0, cleargui, cleargui_, "i");
 ICOMMAND(0, showgui, "si", (const char *s, int *n), showgui(s, *n));
 COMMAND(0, guishowtitle, "i");
 COMMAND(0, guistayopen, "e");
-COMMAND(0, guinoautotab, "e");
 COMMAND(0, guinohitfx, "e");
 
 ICOMMAND(0, guicount, "", (), intret(menustack.length()));
@@ -604,21 +595,21 @@ COMMAND(0, guifont,"se");
 COMMAND(0, guiimage,"ssfiss");
 COMMAND(0, guislice,"ssfffsss");
 COMMAND(0, guiprogress,"ff");
-COMMAND(0, guislider,"siisiii");
+COMMAND(0, guislider,"sbbsiii");
 COMMAND(0, guilistslider, "sssiii");
 COMMAND(0, guinameslider, "ssssiii");
 COMMAND(0, guiradio,"ssfsi");
 COMMAND(0, guibitfield, "ssisi");
 COMMAND(0, guicheckbox, "ssffsi");
 COMMAND(0, guitab, "s");
-COMMAND(0, guifield, "sisi");
-COMMAND(0, guikeyfield, "sisi");
-COMMAND(0, guieditor, "siiii");
+COMMAND(0, guifield, "sisiis");
+COMMAND(0, guikeyfield, "sisiis");
+COMMAND(0, guieditor, "siiiiis");
 
-void guiplayerpreview(int *model, int *color, int *team, int *weap, char *action, float *scale, int *overlaid, char *altact)
+void guiplayerpreview(int *model, int *color, int *team, int *weap, char *vanity, char *action, float *scale, int *overlaid, float *size, float *blend, char *altact)
 {
     if(!cgui) return;
-    int ret = cgui->playerpreview(*model, *color, *team, *weap, *scale, *overlaid!=0);
+    int ret = cgui->playerpreview(*model, *color, *team, *weap, vanity, *scale, *overlaid!=0, *size!=0 ? *size : 1.f, *blend!=0 ? *blend : 1.f);
     if(ret&GUI_UP)
     {
         char *act = NULL;
@@ -631,7 +622,41 @@ void guiplayerpreview(int *model, int *color, int *team, int *weap, char *action
         }
     }
 }
-COMMAND(0, guiplayerpreview, "iiiisfis");
+COMMAND(0, guiplayerpreview, "iiiissfiffs");
+
+void guimodelpreview(char *model, char *animspec, char *action, float *scale, int *overlaid, float *size, float *blend, char *altact)
+{
+    if(!cgui) return;
+    int anim = ANIM_ALL;
+    if(animspec[0])
+    {
+        if(isdigit(animspec[0]))
+        {
+            anim = parseint(animspec);
+            if(anim >= 0) anim %= ANIM_INDEX;
+            else anim = ANIM_ALL;
+        }
+        else
+        {
+            vector<int> anims;
+            game::findanims(animspec, anims);
+            if(anims.length()) anim = anims[0];
+        }
+    }
+    int ret = cgui->modelpreview(model, anim|ANIM_LOOP, *scale, *overlaid!=0, *size!=0 ? *size : 1.f, *blend!=0 ? *blend : 1.f);
+    if(ret&GUI_UP)
+    {
+        char *act = NULL;
+        if(altact[0] && ret&GUI_ALT) act = altact;
+        else if(action[0]) act = action;
+        if(act)
+        {
+            updatelater.add().schedule(act);
+            if(shouldclearmenu) clearlater = true;
+        }
+    }
+}
+COMMAND(0, guimodelpreview, "sssfiffs");
 
 struct change
 {
@@ -648,7 +673,7 @@ static struct applymenu : menu
     void gui(guient &g, bool firstpass)
     {
         if(menustack.empty()) return;
-        g.start(menustart, menuscale, &menutab, true);
+        g.start(menustart, menuscale, NULL, true);
         g.text("the following settings have changed:", 0xFFFFFF, "info");
         loopv(needsapply) g.text(needsapply[i].desc, 0xFFFFFF, "info");
         g.separator();
@@ -674,13 +699,26 @@ static struct applymenu : menu
 
 VAR(IDF_PERSIST, applydialog, 0, 1, 1);
 
-void addchange(const char *desc, int type)
+void addchange(const char *desc, int type, bool force)
 {
-    if(!applydialog) return;
-    loopv(needsapply) if(!strcmp(needsapply[i].desc, desc)) return;
-    needsapply.add(change(type, desc));
-    if(needsapply.length() && menustack.find(&applymenu) < 0)
-        pushgui(&applymenu, max(menustack.length()-1, 0));
+    if(!applydialog || force)
+    {
+        int changetypes = type;
+        if(menustack.find(&applymenu) >= 0)
+        {
+            loopv(needsapply) changetypes |= needsapply[i].type;
+            clearlater = true;
+        }
+        if(changetypes&CHANGE_GFX) updatelater.add().schedule("resetgl");
+        if(changetypes&CHANGE_SOUND) updatelater.add().schedule("resetsound");
+    }
+    else
+    {
+        loopv(needsapply) if(!strcmp(needsapply[i].desc, desc)) return;
+        needsapply.add(change(type, desc));
+        if(needsapply.length() && menustack.find(&applymenu) < 0)
+            pushgui(&applymenu, max(menustack.length()-1, 0));
+    }
 }
 
 void clearchanges(int type)
