@@ -57,7 +57,6 @@ cube *newcubes(uint face, int mat)
         c->children = NULL;
         c->ext = NULL;
         c->visible = 0;
-        c->collide = 0;
         c->merged = 0;
         setfaces(*c, face);
         loopl(6) c->texture[l] = DEFAULT_GEOM;
@@ -68,7 +67,7 @@ cube *newcubes(uint face, int mat)
     return c-8;
 }
 
-int familysize(cube &c)
+int familysize(const cube &c)
 {
     int size = 1;
     if(c.children) loopi(8) size += familysize(c.children[i]);
@@ -96,7 +95,6 @@ void discardchildren(cube &c, bool fixtex, int depth)
 {
     c.material = MAT_AIR;
     c.visible = 0;
-    c.collide = 0;
     c.merged = 0;
     if(c.ext)
     {
@@ -171,7 +169,7 @@ void printcube()
 
 COMMAND(0, printcube, "");
 
-bool isvalidcube(cube &c)
+bool isvalidcube(const cube &c)
 {
     clipplanes p;
     genclipplanes(c, 0, 0, 0, 256, p);
@@ -261,10 +259,10 @@ int lookupmaterial(const vec &v)
     return c->material;
 }
 
-cube *neighbourstack[32];
+const cube *neighbourstack[32];
 int neighbourdepth = -1;
 
-cube &neighbourcube(cube &c, int orient, int x, int y, int z, int size, ivec &ro, int &rsize)
+const cube &neighbourcube(const cube &c, int orient, int x, int y, int z, int size, ivec &ro, int &rsize)
 {
     ivec n(x, y, z);
     int dim = dimension(orient);
@@ -273,7 +271,7 @@ cube &neighbourcube(cube &c, int orient, int x, int y, int z, int size, ivec &ro
     diff ^= n[dim];
     if(diff >= uint(hdr.worldsize)) { ro = n; rsize = size; return c; }
     int scale = worldscale;
-    cube *nc = worldroot;
+    const cube *nc = worldroot;
     if(neighbourdepth >= 0)
     {
         scale -= neighbourdepth + 1;
@@ -295,7 +293,7 @@ cube &neighbourcube(cube &c, int orient, int x, int y, int z, int size, ivec &ro
 
 ////////// (re)mip //////////
 
-int getmippedtexture(cube &p, int orient)
+int getmippedtexture(const cube &p, int orient)
 {
     cube *c = p.children;
     int d = dimension(orient), dc = dimcoord(orient), texs[4] = { -1, -1, -1, -1 }, numtexs = 0;
@@ -444,7 +442,7 @@ bool subdividecube(cube &c, bool fullcheck, bool brighten)
 
 bool crushededge(uchar e, int dc) { return dc ? e==0 : e==0x88; }
 
-int visibleorient(cube &c, int orient)
+int visibleorient(const cube &c, int orient)
 {
     loopi(2)
     {
@@ -489,7 +487,7 @@ bool remip(cube &c, int x, int y, int z, int size)
     if(!perfect) return false;
     if(size<<1 > 0x1000) return false;
 
-    uchar mat = MAT_AIR;
+    ushort mat = MAT_AIR;
     loopi(8)
     {
         mat = ch[i].material;
@@ -547,7 +545,8 @@ bool remip(cube &c, int x, int y, int z, int size)
     discardchildren(c);
     loopi(3) c.faces[i] = n.faces[i];
     c.material = mat;
-    loopi(6) if(vis[i]) { c.visible |= 1<<i; c.collide |= 1<<i; }
+    loopi(6) if(vis[i]) c.visible |= 1<<i;
+    if(c.visible) c.visible |= 0x40;
     brightencube(c);
     return true;
 }
@@ -656,7 +655,7 @@ const ivec cubecoords[8] = // verts of bounding cube
 };
 
 template<class T>
-static inline void gencubevert(cube &c, int i, T &v)
+static inline void gencubevert(const cube &c, int i, T &v)
 {
     switch(i)
     {
@@ -671,7 +670,7 @@ static inline void gencubevert(cube &c, int i, T &v)
     }
 }
 
-void genfaceverts(cube &c, int orient, ivec v[4])
+void genfaceverts(const cube &c, int orient, ivec v[4])
 {
 
     switch(orient)
@@ -731,14 +730,14 @@ const uchar faceedgesidx[6][4] = // ordered edges surrounding each orient
     { 2,  3,  5, 7 },
 };
 
-bool flataxisface(cube &c, int orient)
+bool flataxisface(const cube &c, int orient)
 {
     uint face = c.faces[dimension(orient)];
     if(dimcoord(orient)) face >>= 4;
     return (face&0x0F0F0F0F) == 0x01010101*(face&0x0F);
 }
 
-bool collideface(cube &c, int orient)
+bool collideface(const cube &c, int orient)
 {
     if(flataxisface(c, orient))
     {
@@ -750,19 +749,19 @@ bool collideface(cube &c, int orient)
     return true;
 }
 
-bool touchingface(cube &c, int orient)
+bool touchingface(const cube &c, int orient)
 {
     uint face = c.faces[dimension(orient)];
     return dimcoord(orient) ? (face&0xF0F0F0F0)==0x80808080 : (face&0x0F0F0F0F)==0;
 }
 
-bool notouchingface(cube &c, int orient)
+bool notouchingface(const cube &c, int orient)
 {
     uint face = c.faces[dimension(orient)];
     return dimcoord(orient) ? (face&0x80808080)==0 : ((0x88888888-face)&0x08080808) == 0;
 }   
 
-int faceconvexity(ivec v[4])
+int faceconvexity(const ivec v[4])
 {
     ivec n;
     n.cross(ivec(v[1]).sub(v[0]), ivec(v[2]).sub(v[0]));
@@ -770,7 +769,7 @@ int faceconvexity(ivec v[4])
     // 1 if convex, -1 if concave, 0 if flat
 }
 
-int faceconvexity(vertinfo *verts, int numverts, int size)
+int faceconvexity(const vertinfo *verts, int numverts, int size)
 {
     if(numverts < 4) return 0;
     ivec v0 = verts[0].getxyz(),
@@ -786,7 +785,7 @@ int faceconvexity(vertinfo *verts, int numverts, int size)
     return verts[3].getxyz().sub(v0).dot(n);
 }
 
-int faceconvexity(ivec v[4], int &vis)
+int faceconvexity(const ivec v[4], int &vis)
 {
     ivec e1, e2, e3, n;
     n.cross((e1 = v[1]).sub(v[0]), (e2 = v[2]).sub(v[0]));
@@ -800,7 +799,7 @@ int faceconvexity(ivec v[4], int &vis)
     return convex;
 } 
 
-int faceconvexity(cube &c, int orient)
+int faceconvexity(const cube &c, int orient)
 {
     if(flataxisface(c, orient)) return 0;
     ivec v[4];
@@ -808,7 +807,7 @@ int faceconvexity(cube &c, int orient)
     return faceconvexity(v);
 }
 
-int faceorder(cube &c, int orient) // gets above 'fv' so that each face is convex
+int faceorder(const cube &c, int orient) // gets above 'fv' so that each face is convex
 {
     return faceconvexity(c, orient)<0 ? 1 : 0;
 }
@@ -818,7 +817,7 @@ static inline void faceedges(const cube &c, int orient, uchar edges[4])
     loopk(4) edges[k] = c.edges[faceedgesidx[orient][k]];
 }
 
-uint faceedges(cube &c, int orient)
+uint faceedges(const cube &c, int orient)
 {
     union { uchar edges[4]; uint face; } u;
     faceedges(c, orient, u.edges);
@@ -836,7 +835,7 @@ struct facevec
     bool operator!=(const facevec &f) const { return x != f.x || y != f.y; }
 };
 
-static inline int genfacevecs(cube &cu, int orient, const ivec &pos, int size, bool solid, facevec *fvecs, const ivec *v = NULL)
+static inline int genfacevecs(const cube &cu, int orient, const ivec &pos, int size, bool solid, facevec *fvecs, const ivec *v = NULL)
 {
     int i = 0;
     if(solid)
@@ -973,7 +972,7 @@ static inline int clipfacevecs(const facevec *o, int numo, int cx, int cy, int s
     return r;
 }
 
-bool collapsedface(cube &c, int orient)
+bool collapsedface(const cube &c, int orient)
 {
     int e0 = c.edges[faceedgesidx[orient][0]], e1 = c.edges[faceedgesidx[orient][1]],
         e2 = c.edges[faceedgesidx[orient][2]], e3 = c.edges[faceedgesidx[orient][3]],
@@ -990,7 +989,7 @@ bool collapsedface(cube &c, int orient)
            ivec().cross(v2, v3.sub(v0)).iszero();
 }
 
-static inline bool occludesface(cube &c, int orient, const ivec &o, int size, const ivec &vo, int vsize, uchar vmat, uchar nmat, uchar matmask, const facevec *vf, int numv)
+static inline bool occludesface(const cube &c, int orient, const ivec &o, int size, const ivec &vo, int vsize, ushort vmat, ushort nmat, ushort matmask, const facevec *vf, int numv)
 {
     int dim = dimension(orient);
     if(!c.children)
@@ -1022,7 +1021,7 @@ static inline bool occludesface(cube &c, int orient, const ivec &o, int size, co
     return true;
 }
 
-bool visibleface(cube &c, int orient, int x, int y, int z, int size, uchar mat, uchar nmat, uchar matmask)
+bool visibleface(const cube &c, int orient, int x, int y, int z, int size, ushort mat, ushort nmat, ushort matmask)
 {
     if(mat != MAT_AIR)
     {
@@ -1036,7 +1035,7 @@ bool visibleface(cube &c, int orient, int x, int y, int z, int size, uchar mat,
 
     ivec no;
     int nsize;
-    cube &o = neighbourcube(c, orient, x, y, z, size, no, nsize);
+    const cube &o = neighbourcube(c, orient, x, y, z, size, no, nsize);
     if(&o==&c) return false;
 
     int opp = opposite(orient);
@@ -1065,8 +1064,48 @@ bool visibleface(cube &c, int orient, int x, int y, int z, int size, uchar mat,
     return !occludesface(o, opp, no, nsize, vo, size, mat, nmat, matmask, cf, numc);
 }
 
+int classifyface(const cube &c, int orient, int x, int y, int z, int size)
+{
+    if(collapsedface(c, orient)) return 0;
+    int vismask = (c.material&MATF_CLIP) == MAT_NOCLIP ? 1 : 3;
+    if(!touchingface(c, orient)) return vismask;
+
+    ivec no;
+    int nsize;
+    const cube &o = neighbourcube(c, orient, x, y, z, size, no, nsize);
+    if(&o==&c) return 0;
+
+    int vis = 0, opp = opposite(orient);
+    if(nsize > size || (nsize == size && !o.children))
+    {
+        if((~c.material & o.material) & MAT_ALPHA) vis |= 1;
+        if((o.material&MATF_CLIP) == MAT_NOCLIP) vis |= vismask&2;
+        if(vis == vismask || isentirelysolid(o)) return vis;
+        if(isempty(o) || notouchingface(o, opp)) return vismask;
+        if(touchingface(o, opp) && faceedges(o, opp) == F_SOLID) return vis;
+
+        ivec vo(x, y, z);
+        vo.mask(0xFFF);
+        no.mask(0xFFF);
+        facevec cf[4], of[4];
+        int numc = genfacevecs(c, orient, vo, size, false, cf),
+            numo = genfacevecs(o, opp, no, nsize, false, of);
+        if(numo < 3 || !insideface(cf, numc, of, numo)) return vismask;
+        return vis;
+    }
+
+    ivec vo(x, y, z);
+    vo.mask(0xFFF);
+    no.mask(0xFFF);
+    facevec cf[4];
+    int numc = genfacevecs(c, orient, vo, size, false, cf);
+    if(!occludesface(o, opp, no, nsize, vo, size, MAT_AIR, (c.material&MAT_ALPHA)^MAT_ALPHA, MAT_ALPHA, cf, numc)) vis |= 1;
+    if(vismask&2 && !occludesface(o, opp, no, nsize, vo, size, MAT_AIR, MAT_NOCLIP, MATF_CLIP, cf, numc)) vis |= 2;
+    return vis; 
+}
+
 // more expensive version that checks both triangles of a face independently
-int visibletris(cube &c, int orient, int x, int y, int z, int size, uchar nmat, uchar matmask)
+int visibletris(const cube &c, int orient, int x, int y, int z, int size, ushort nmat, ushort matmask)
 {
     int vis = 3, touching = 0xF;
     ivec v[4], e1, e2, e3, n;
@@ -1096,14 +1135,10 @@ int visibletris(cube &c, int orient, int x, int y, int z, int size, uchar nmat,
 
     ivec no;
     int nsize;
-    cube &o = neighbourcube(c, orient, x, y, z, size, no, nsize);
+    const cube &o = neighbourcube(c, orient, x, y, z, size, no, nsize);
     if(&o==&c) return 0;
     
-    if(matmask == MAT_AIR)
-    {
-        nmat = c.material&MAT_ALPHA ? MAT_AIR : MAT_ALPHA;
-        matmask = MAT_ALPHA;
-    }
+    if((c.material&matmask) == nmat) nmat = MAT_AIR;
 
     ivec vo(x, y, z);
     vo.mask(0xFFF);
@@ -1156,7 +1191,7 @@ int visibletris(cube &c, int orient, int x, int y, int z, int size, uchar nmat,
     return 3;
 }
 
-void calcvert(cube &c, int x, int y, int z, int size, ivec &v, int i, bool solid)
+void calcvert(const cube &c, int x, int y, int z, int size, ivec &v, int i, bool solid)
 {
     if(solid) v = cubecoords[i]; else gencubevert(c, i, v);
     // avoid overflow
@@ -1165,13 +1200,13 @@ void calcvert(cube &c, int x, int y, int z, int size, ivec &v, int i, bool solid
     v.add(ivec(x, y, z).shl(3));
 }
 
-void calcvert(cube &c, int x, int y, int z, int size, vec &v, int i, bool solid)
+void calcvert(const cube &c, int x, int y, int z, int size, vec &v, int i, bool solid)
 {
     if(solid) v = cubecoords[i].tovec(); else gencubevert(c, i, v);
     v.mul(size/8.0f).add(vec(x, y, z));
 }
 
-int genclipplane(cube &c, int orient, vec *v, plane *clip)
+int genclipplane(const cube &c, int orient, vec *v, plane *clip)
 {
     int planes = 0, convex = faceconvexity(c, orient), order = convex < 0 ? 1 : 0;
     const vec &v0 = v[fv[orient][order]], &v1 = v[fv[orient][order+1]], &v2 = v[fv[orient][order+2]], &v3 = v[fv[orient][(order+3)&3]];
@@ -1181,7 +1216,7 @@ int genclipplane(cube &c, int orient, vec *v, plane *clip)
     return planes;
 }
 
-void genclipplanes(cube &c, int x, int y, int z, int size, clipplanes &p)
+void genclipplanes(const cube &c, int x, int y, int z, int size, clipplanes &p, bool collide)
 {
     // generate tight bounding box
     calcvert(c, x, y, z, size, p.v[0], 0);
@@ -1198,16 +1233,34 @@ void genclipplanes(cube &c, int x, int y, int z, int size, clipplanes &p)
 
     p.size = 0;
     p.visible = 0;
-    loopi(6) if(c.collide&(1<<i))
+    if(collide || (c.visible&0xC0) == 0x40) 
+    {
+        loopi(6) if(c.visible&(1<<i))
+        {
+            int vis;
+            if(flataxisface(c, i)) p.visible |= 1<<i;
+            else if((vis = visibletris(c, i, x, y, z, size, MAT_NOCLIP, MATF_CLIP)))
+            {
+                int convex = faceconvexity(c, i), order = vis&4 || convex < 0 ? 1 : 0;
+                const vec &v0 = p.v[fv[i][order]], &v1 = p.v[fv[i][order+1]], &v2 = p.v[fv[i][order+2]], &v3 = p.v[fv[i][(order+3)&3]];
+                if(vis&1) { p.side[p.size] = i; p.p[p.size++].toplane(v0, v1, v2); }
+                if(vis&2 && (!(vis&1) || convex)) { p.side[p.size] = i; p.p[p.size++].toplane(v0, v2, v3); }
+            }
+        }
+    }
+    else if(c.visible&0x80)
     {
         int vis;
-        if(flataxisface(c, i)) p.visible |= 1<<i;
-        else if((vis = visibletris(c, i, x, y, z, size, MAT_NOCLIP, MATF_CLIP)))
+        loopi(6) if((vis = visibletris(c, i, x, y, z, size))) 
         {
-            int convex = faceconvexity(c, i), order = vis&4 || convex < 0 ? 1 : 0;
-            const vec &v0 = p.v[fv[i][order]], &v1 = p.v[fv[i][order+1]], &v2 = p.v[fv[i][order+2]], &v3 = p.v[fv[i][(order+3)&3]];
-            if(vis&1) { p.side[p.size] = i; p.p[p.size++].toplane(v0, v1, v2); }
-            if(vis&2 && (!(vis&1) || convex)) { p.side[p.size] = i; p.p[p.size++].toplane(v0, v2, v3); }
+            if(flataxisface(c, i)) p.visible |= 1<<i;
+            else
+            {
+                int convex = faceconvexity(c, i), order = vis&4 || convex < 0 ? 1 : 0;
+                const vec &v0 = p.v[fv[i][order]], &v1 = p.v[fv[i][order+1]], &v2 = p.v[fv[i][order+2]], &v3 = p.v[fv[i][(order+3)&3]];
+                if(vis&1) { p.side[p.size] = i; p.p[p.size++].toplane(v0, v1, v2); }
+                if(vis&2 && (!(vis&1) || convex)) { p.side[p.size] = i; p.p[p.size++].toplane(v0, v2, v3); }
+            }
         }
     }
 }
@@ -1273,8 +1326,8 @@ int mergefaces(int orient, facebounds *m, int sz)
 
 struct cfkey
 {
-    uchar orient, material;
-    ushort tex;
+    uchar orient;
+    ushort material, tex;
     ivec n;
     int offset;
 };
@@ -1289,7 +1342,7 @@ static inline uint hthash(const cfkey &k)
     return hthash(k.n)^k.offset^k.tex^k.orient^k.material;
 }
 
-void mincubeface(cube &cu, int orient, const ivec &o, int size, const facebounds &orig, facebounds &cf, uchar nmat, uchar matmask)
+void mincubeface(const cube &cu, int orient, const ivec &o, int size, const facebounds &orig, facebounds &cf, ushort nmat, ushort matmask)
 {
     int dim = dimension(orient);
     if(cu.children)
@@ -1336,11 +1389,11 @@ void mincubeface(cube &cu, int orient, const ivec &o, int size, const facebounds
     cf.v2 = max(cf.v2, vc2);
 }
 
-bool mincubeface(cube &cu, int orient, const ivec &co, int size, facebounds &orig)
+bool mincubeface(const cube &cu, int orient, const ivec &co, int size, facebounds &orig)
 {
     ivec no;
     int nsize;
-    cube &nc = neighbourcube(cu, orient, co.x, co.y, co.z, size, no, nsize);
+    const cube &nc = neighbourcube(cu, orient, co.x, co.y, co.z, size, no, nsize);
     facebounds mincf;
     mincf.u1 = orig.u2;
     mincf.u2 = orig.u1;
@@ -1619,7 +1672,6 @@ bool mergepolys(int orient, hashset<plink> &links, vector<plink *> &queue, int o
 void addmerge(cube &cu, int orient, const ivec &co, const ivec &n, int offset, poly &p)
 {
     cu.merged |= 1<<orient;
-    cu.visible &= ~(1<<orient);
     if(!p.numverts)
     {
         if(cu.ext) cu.ext->surfaces[orient] = ambientsurface;
@@ -1667,7 +1719,6 @@ static inline void clearmerge(cube &c, int orient)
     if(c.merged&(1<<orient))
     {
         c.merged &= ~(1<<orient);
-        c.visible |= 1<<orient;
         if(c.ext) c.ext->surfaces[orient] = brightsurface;
     }
 }
@@ -1809,7 +1860,6 @@ static void invalidatemerges(cube &c)
     if(c.merged)
     {
         brightencube(c);
-        c.visible |= c.merged;
         c.merged = 0;
     }
     if(c.ext)
