@@ -1,6 +1,5 @@
 #include "engine.h"
 
-extern int outline, blankgeom;
 editinfo *localedit = NULL;
 
 VAR(0, showpastegrid, 0, 0, 1);
@@ -193,12 +192,12 @@ cube &blockcube(int x, int y, int z, const block3 &b, int rgrid) // looks up a w
 
 #define loopxy(b)       loop(y,(b).s[C[dimension((b).orient)]]) loop(x,(b).s[R[dimension((b).orient)]])
 #define loopxyz(b, r, f) { loop(z,(b).s[D[dimension((b).orient)]]) loopxy((b)) { cube &c = blockcube(x,y,z,b,r); f; } }
-#define loopselxyz(f)   { makeundo(); loopxyz(sel, sel.grid, f); changed(sel); }
+#define loopselxyz(f)   { if(local) makeundo(); loopxyz(sel, sel.grid, f); changed(sel); }
 #define selcube(x, y, z) blockcube(x, y, z, sel, sel.grid)
 
 ////////////// cursor ///////////////
 
-int selchildcount=0;
+int selchildcount = 0, selchildmat = -1;
 
 ICOMMAND(0, havesel, "", (), intret(havesel ? selchildcount : 0));
 
@@ -210,7 +209,15 @@ void countselchild(cube *c, const ivec &cor, int size)
     {
         ivec o(i, cor.x, cor.y, cor.z, size);
         if(c[i].children) countselchild(c[i].children, o, size/2);
-        else selchildcount++;
+        else
+        {
+            selchildcount++;
+            if(c[i].material != MAT_AIR && selchildmat != MAT_AIR)
+            {
+                if(selchildmat < 0) selchildmat = c[i].material;
+                else if(selchildmat != c[i].material) selchildmat = MAT_AIR;
+            }
+        }
     }
 }
 
@@ -266,7 +273,7 @@ inline bool isheightmap(int orient, int d, bool empty, cube *c);
 extern void entdrag(const vec &ray);
 extern bool hoveringonent(int ent, int orient);
 extern void renderentselection(const vec &o, const vec &ray, bool entmoving);
-extern float rayent(const vec &o, const vec &ray, vec &hitpos, float radius, int mode, int size, int &orient, int &ent);
+extern float rayent(const vec &o, const vec &ray, float radius, int mode, int size, int &orient, int &ent);
 
 VAR(0, gridlookup, 0, 0, 1);
 VAR(0, passthroughcube, 0, 1, 1);
@@ -309,12 +316,12 @@ void rendereditcursor()
     }
     else
     {
-        vec v;
         ivec w;
         float sdist = 0, wdist = 0, t;
         int entorient = 0, ent = -1;
 
-        wdist = rayent(player->o, ray, v, 0, (editmode && showmat ? RAY_EDITMAT : 0)    // select cubes first
+        wdist = rayent(player->o, ray, 1e16f,
+                       (editmode && showmat ? RAY_EDITMAT : 0)   // select cubes first
                                             | (!dragging && entediting ? RAY_ENTS : 0)
                                             | RAY_SKIPFIRST
                                             | (passthroughcube==1 ? RAY_PASS : 0), gridsize, entorient, ent);
@@ -334,22 +341,30 @@ void rendereditcursor()
             if(!havesel)
             {
                 selchildcount = 0;
+                selchildmat = -1;
                 sel.s = ivec(0, 0, 0);
             }
         }
         else
         {
-            v = ray;
-            v.mul(wdist+0.1f);
-            v.add(player->o);
-            w = v;
-            cube *c = &lookupcube(w.x, w.y, w.z);
+            vec w = vec(ray).mul(wdist+0.05f).add(player->o);
+            if(!insideworld(w))
+            {
+                loopi(3) wdist = min(wdist, ((ray[i] > 0 ? hdr.worldsize : 0) - player->o[i]) / ray[i]);
+                w = vec(ray).mul(wdist-0.05f).add(player->o);
+                if(!insideworld(w))
+                {
+                    wdist = 0;
+                    loopi(3) w[i] = clamp(player->o[i], 0.0f, float(hdr.worldsize));
+                }
+            }
+            cube *c = &lookupcube(int(w.x), int(w.y), int(w.z));
             if(gridlookup && !dragging && !moving && !havesel && hmapedit!=1) gridsize = lusize;
             int mag = gridsize && lusize ? lusize / gridsize : 0;
-            normalizelookupcube(w.x, w.y, w.z);
+            normalizelookupcube(int(w.x), int(w.y), int(w.z));
             if(sdist == 0 || sdist > wdist) rayrectintersect(lu.tovec(), vec(gridsize), player->o, ray, t=0, orient); // just getting orient
             cur = lu;
-            cor = vec(v).mul(2).div(gridsize);
+            cor = vec(w).mul(2).div(gridsize);
             od = dimension(orient);
             d = dimension(sel.orient);
 
@@ -400,8 +415,13 @@ void rendereditcursor()
 
             sel.corner = (cor[R[d]]-(lu[R[d]]*2)/gridsize)+(cor[C[d]]-(lu[C[d]]*2)/gridsize)*2;
             selchildcount = 0;
+            selchildmat = -1;
             countselchild(worldroot, ivec(0, 0, 0), hdr.worldsize/2);
-            if(mag>1 && selchildcount==1) selchildcount = -mag;
+            if(mag>=1 && selchildcount==1)
+            {
+                selchildmat = c->material;
+                if(mag>1) selchildcount = -mag;
+            }
         }
     }
 
@@ -555,7 +575,6 @@ static inline void copycube(const cube &src, cube &dst)
 {
     dst = src;
     dst.visible = 0;
-    dst.collide = 0;
     dst.merged = 0;
     dst.ext = NULL; // src cube is responsible for va destruction
     if(src.children)
@@ -794,9 +813,10 @@ static void packcube(cube &c, B &buf)
     }
     else
     {
-        buf.put(c.material);
         cube data = c;
         lilswap(data.texture, 6);
+        buf.put(c.material&0xFF);
+        buf.put(c.material>>8);
         buf.put(data.edges, sizeof(data.edges));
         buf.put((uchar *)data.texture, sizeof(data.texture));
     }
@@ -828,7 +848,7 @@ static void unpackcube(cube &c, B &buf)
     }
     else
     {
-        c.material = mat;
+        c.material = mat | (buf.get()<<8);
         buf.get(c.edges, sizeof(c.edges));
         buf.get((uchar *)c.texture, sizeof(c.texture));
         lilswap(c.texture, 6);
@@ -970,7 +990,7 @@ void savebrush(char *name)
 }
 COMMAND(0, savebrush, "s");
 
-void pasteblock(block3 &b, selinfo &sel)
+void pasteblock(block3 &b, selinfo &sel, bool local)
 {
     sel.s = b.s;
     int o = sel.orient;
@@ -1002,7 +1022,7 @@ void pastebrush(char *name)
         b->name = newstring(name);
         b->copy = copy;
     }
-    pasteblock(*b->copy, sel);
+    pasteblock(*b->copy, sel, true);
 }
 COMMAND(0, pastebrush, "s");
 
@@ -1020,7 +1040,7 @@ void mppaste(editinfo *&e, selinfo &sel, bool local)
 {
     if(e==NULL) return;
     if(local) client::edittrigger(sel, EDIT_PASTE);
-    if(e->copy) pasteblock(*e->copy, sel);
+    if(e->copy) pasteblock(*e->copy, sel, local);
 }
 
 void copy()
@@ -1219,7 +1239,6 @@ namespace hmap
 
     void pushside(cube &c, int d, int x, int y, int z)
     {
-        if(d < 0 || d > 2) return;
         ivec a;
         getcubevector(c, d, x, y, z, a);
         a[R[d]] = 8 - a[R[d]];
@@ -1230,7 +1249,7 @@ namespace hmap
     {
         if(!(flags[x][y] & MAPPED))
           map[x][y] = v + (z*8);
-      flags[x][y] |= MAPPED;
+        flags[x][y] |= MAPPED;
     }
 
     void select(int x, int y, int z)
@@ -1251,10 +1270,8 @@ namespace hmap
         {   // try up
             c[2] = c[1];
             c[1] = getcube(t, 1);
-            if(!c[1] || isempty(*c[1])) {
-                c[0] = c[1], c[1] = c[2], c[2] = NULL;
-            }else
-                z++, t[d]+=fg;
+            if(!c[1] || isempty(*c[1])) { c[0] = c[1]; c[1] = c[2]; c[2] = NULL; }
+            else { z++; t[d]+=fg; }
         }
         else // drop down
         {
@@ -1328,7 +1345,7 @@ namespace hmap
         if(biasup)
             pullhmap(0, >, <, 1, 0, -);
         else
-            pullhmap(hdr.worldsize, <, >, 0, 8, +);
+            pullhmap(hdr.worldsize*8, <, >, 0, 8, +);
 
         cube **c  = cmap[x][y];
         int e[2][2];
@@ -1355,7 +1372,7 @@ namespace hmap
                     edgeset(cubeedge(*c[k], d, i, j), dc, dc ? f : 8-f);
                 }
             }
-            else if(c[k])
+            else
                 emptyfaces(*c[k]);
         }
 
@@ -1474,7 +1491,7 @@ namespace hmap
 }
 
 void edithmap(int dir, int mode) {
-    if(multiplayer() || !hmapsel || gridsize < 8) return;
+    if(multiplayer() || !hmapsel) return;
     hmap::run(dir, mode);
 }
 
@@ -1503,11 +1520,11 @@ void linkedpush(cube &c, int d, int x, int y, int dc, int dir)
     }
 }
 
-static uchar getmaterial(cube &c)
+static ushort getmaterial(cube &c)
 {
     if(c.children)
     {
-        uchar mat = getmaterial(c.children[7]);
+        ushort mat = getmaterial(c.children[7]);
         loopi(7) if(mat != getmaterial(c.children[i])) return MAT_AIR;
         return mat;
     }
@@ -1538,7 +1555,7 @@ void mpeditface(int dir, int mode, selinfo &sel, bool local)
 
     loopselxyz(
         if(c.children) solidfaces(c);
-        uchar mat = getmaterial(c);
+        ushort mat = getmaterial(c);
         discardchildren(c, true);
         c.material = mat;
         if(mode==1) // fill command
@@ -1842,7 +1859,7 @@ void vcolor(float *r, float *g, float *b)
     if(noedit() || multiplayer()) return;
     VSlot ds;
     ds.changed = 1<<VSLOT_COLOR;
-    ds.colorscale = vec(clamp(*r, 0.0f, 1.0f), clamp(*g, 0.0f, 1.0f), clamp(*b, 0.0f, 1.0f));
+    ds.colorscale = vec(max(*r, 0.0f), max(*g, 0.0f), max(*b, 0.0f));
     mpeditvslot(ds, allfaces, sel, true);
 }
 COMMAND(0, vcolor, "fff");
@@ -1946,7 +1963,7 @@ void compactmruvslots()
 void edittex(int i, bool save = true, bool edit = true)
 {
     lasttex = i;
-    lasttexmillis = lastmillis;
+    lasttexmillis = totalmillis;
     if(save)
     {
         loopvj(texmru) if(texmru[j]==lasttex) { curtexindex = j; break; }
@@ -1955,11 +1972,12 @@ void edittex(int i, bool save = true, bool edit = true)
 }
 ICOMMAND(0, settex, "i", (int *slot), edittex(*slot));
 
+VAR(IDF_PERSIST, texpaneltime, 0, 5000, VAR_MAX);
 void edittex_(int *dir)
 {
     if(noedit()) return;
     filltexlist();
-    texpaneltimer = 5000;
+    texpaneltimer = texpaneltime;
     if(!(lastsel==sel)) tofronttex();
     curtexindex = clamp(curtexindex<0 ? 0 : curtexindex+*dir, 0, texmru.length()-1);
     edittex(texmru[curtexindex], false);
@@ -2009,6 +2027,7 @@ COMMANDN(0, edittex, edittex_, "i");
 COMMAND(0, gettex, "");
 COMMAND(0, getcurtex, "");
 COMMAND(0, getseltex, "");
+ICOMMAND(0, getreptex, "", (), { if(!noedit()) intret(vslots.inrange(reptex) ? reptex : -1); });
 COMMAND(0, gettexname, "ii");
 
 void replacetexcube(cube &c, int oldtex, int newtex)
@@ -2017,24 +2036,38 @@ void replacetexcube(cube &c, int oldtex, int newtex)
     if(c.children) loopi(8) replacetexcube(c.children[i], oldtex, newtex);
 }
 
-void mpreplacetex(int oldtex, int newtex, selinfo &sel, bool local)
+void mpreplacetex(int oldtex, int newtex, bool insel, selinfo &sel, bool local)
 {
-    if(local) client::edittrigger(sel, EDIT_REPLACE, oldtex, newtex);
-    loopi(8) replacetexcube(worldroot[i], oldtex, newtex);
+    if(local) client::edittrigger(sel, EDIT_REPLACE, oldtex, newtex, insel ? 1 : 0);
+    if(insel)
+    {
+        loopselxyz(replacetexcube(c, oldtex, newtex));
+    }
+    else
+    {
+        loopi(8) replacetexcube(worldroot[i], oldtex, newtex);
+    }
     allchanged();
 }
 
-void replacetex(int texnum = -1)
+void replacetex(bool insel, int texnum = -1)
 {
     if(noedit()) return;
-    mpreplacetex(texnum, lasttex, sel, true);
+    mpreplacetex(texnum, lasttex, insel, sel, true);
 }
 
-ICOMMAND(0, replace, "", (void), {
-    if(reptex < 0) { conoutf("\frcan only replace after a texture edit"); return; }
-    replacetex(reptex);
+ICOMMAND(0, replace, "iN", (int *t, int *numargs), {
+    int tex = *numargs >= 1 ? *t : reptex;
+    if(tex < 0) { conoutf("\frcan only replace after a texture edit"); return; }
+    replacetex(false, tex);
 });
-ICOMMAND(0, replaceall, "", (void), replacetex(););
+ICOMMAND(0, replacesel, "iN", (int *t, int *numargs), {
+    int tex = *numargs >= 1 ? *t : reptex;
+    if(tex < 0) { conoutf("\frcan only replace after a texture edit"); return; }
+    replacetex(true, tex);
+});
+ICOMMAND(0, replaceall, "", (void), replacetex(false));
+ICOMMAND(0, replaceallsel, "", (void), replacetex(true));
 
 void resettexmru()
 {
@@ -2097,9 +2130,12 @@ void rotatecube(cube &c, int d) // rotates cube clockwise. see pics in cvs for h
 
 void mpflip(selinfo &sel, bool local)
 {
-    if(local) client::edittrigger(sel, EDIT_FLIP);
+    if(local)
+    {
+        client::edittrigger(sel, EDIT_FLIP);
+        makeundo();
+    }
     int zs = sel.s[dimension(sel.orient)];
-    makeundo();
     loopxy(sel)
     {
         loop(z,zs) flipcube(selcube(x, y, z), dimension(sel.orient));
@@ -2121,12 +2157,15 @@ void flip()
 
 void mprotate(int cw, selinfo &sel, bool local)
 {
-    if(local) client::edittrigger(sel, EDIT_ROTATE, cw);
+    if(local)
+    {
+        client::edittrigger(sel, EDIT_ROTATE, cw);
+        makeundo();
+    }
     int d = dimension(sel.orient);
     if(!dimcoord(sel.orient)) cw = -cw;
     int m = sel.s[C[d]] < sel.s[R[d]] ? C[d] : R[d];
     int ss = sel.s[m] = max(sel.s[R[d]], sel.s[C[d]]);
-    makeundo();
     loop(z,sel.s[D[d]]) loopi(cw>0 ? 1 : 3)
     {
         loopxy(sel) rotatecube(selcube(x,y,z), d);
@@ -2150,21 +2189,29 @@ void rotate(int *cw)
 COMMAND(0, flip, "");
 COMMAND(0, rotate, "i");
 
-void setmat(cube &c, uchar mat, uchar matmask, int style)
+enum { EDITMATF_EMPTY = 0x10000, EDITMATF_NOTEMPTY = 0x20000, EDITMATF_SOLID = 0x30000, EDITMATF_NOTSOLID = 0x40000 };
+static const struct { const char *name; int filter; } editmatfilters[] =
+{
+    { "empty", EDITMATF_EMPTY },
+    { "notempty", EDITMATF_NOTEMPTY },
+    { "solid", EDITMATF_SOLID },
+    { "notsolid", EDITMATF_NOTSOLID }
+};
+
+void setmat(cube &c, ushort mat, ushort matmask, ushort filtermat, ushort filtermask, int filtergeom)
 {
     if(c.children)
-        loopi(8) setmat(c.children[i], mat, matmask, style);
-    else
+        loopi(8) setmat(c.children[i], mat, matmask, filtermat, filtermask, filtergeom);
+    else if((c.material&filtermask) == filtermat)
     {
-        switch(style)
+        switch(filtergeom)
         {
-            case 1: if(isempty(c)) return; break;
-            case 2: if(!isempty(c)) return; break;
-            case 3: if(isentirelysolid(c)) return; break;
-            case 4: if(!isentirelysolid(c)) return; break;
-            case 0: default: break;
+            case EDITMATF_EMPTY: if(isempty(c)) break; return;
+            case EDITMATF_NOTEMPTY: if(!isempty(c)) break; return;
+            case EDITMATF_SOLID: if(isentirelysolid(c)) break; return;
+            case EDITMATF_NOTSOLID: if(!isentirelysolid(c)) break; return;
         }
-        if(mat != MAT_AIR)
+        if(mat!=MAT_AIR)
         {
             c.material &= matmask;
             c.material |= mat;
@@ -2173,27 +2220,61 @@ void setmat(cube &c, uchar mat, uchar matmask, int style)
     }
 }
 
-void mpeditmat(int matid, int style, selinfo &sel, bool local)
+void mpeditmat(int matid, int filter, int style, selinfo &sel, bool local)
 {
-    if(local) client::edittrigger(sel, EDIT_MAT, matid, style);
+    if(local) client::edittrigger(sel, EDIT_MAT, matid, filter, style);
 
-    uchar matmask = matid&MATF_VOLUME ? 0 : (matid&MATF_CLIP ? ~MATF_CLIP : 0xFF);
-    if(isclipped(matid&MATF_VOLUME)) matid |= MAT_CLIP;
-    if(isdeadly(matid&MATF_VOLUME)) matid |= MAT_DEATH;
-    if(local && (matid&MATF_VOLUME) == MAT_GLASS)
-        conoutft(CON_SELF, "\fzoyWARNING\fw - quin dislikes glass material: if seeing through this is unimportant, think about using alpha instead");
-    loopselxyz(setmat(c, matid, matmask, style));
+    ushort filtermat = 0, filtermask = 0, matmask;
+    int filtergeom = 0;
+    if(filter >= 0)
+    {
+        filtermat = filter&0xFFFF;
+        filtermask = filtermat&(MATF_VOLUME|MATF_INDEX) ? MATF_VOLUME|MATF_INDEX : (filtermat&MATF_CLIP ? MATF_CLIP : filtermat);
+        filtergeom = filter&~0xFFFF;
+    }
+    switch(style)
+    {
+        case 1: filtergeom = EDITMATF_NOTEMPTY; break;
+        case 2: filtergeom = EDITMATF_EMPTY; break;
+        case 3: filtergeom = EDITMATF_NOTSOLID; break;
+        case 4: filtergeom = EDITMATF_SOLID; break;
+    }
+    if(matid < 0)
+    {
+        matid = 0;
+        matmask = filtermask;
+        if(isclipped(filtermat&MATF_VOLUME)) matmask &= ~MATF_CLIP;
+        if(isdeadly(filtermat&MATF_VOLUME)) matmask &= ~MAT_DEATH;
+    }
+    else
+    {
+        matmask = matid&(MATF_VOLUME|MATF_INDEX) ? 0 : (matid&MATF_CLIP ? ~MATF_CLIP : ~matid);
+        if(isclipped(matid&MATF_VOLUME)) matid |= MAT_CLIP;
+        if(isdeadly(matid&MATF_VOLUME)) matid |= MAT_DEATH;
+    }
+    loopselxyz(setmat(c, matid, matmask, filtermat, filtermask, filtergeom));
 }
 
-void editmat(char *name, int *style)
+void editmat(char *name, char *filtername, int *style)
 {
     if(noedit()) return;
-    int id = findmaterial(name, true);
-    if(id<0) { conoutf("\frunknown material \"%s\"", name); return; }
-    mpeditmat(id, *style, sel, true);
+    int filter = -1;
+    if(filtername[0])
+    {
+        loopi(sizeof(editmatfilters)/sizeof(editmatfilters[0])) if(!strcmp(editmatfilters[i].name, filtername)) { filter = editmatfilters[i].filter; break; }
+        if(filter < 0) filter = findmaterial(filtername, true);
+        if(filter < 0) { conoutf("\frunknown material \"%s\"", filtername); return; }
+    }
+    int id = -1;
+    if(name[0] || filter < 0)
+    {
+        id = findmaterial(name, true);
+        if(id<0) { conoutf("\frunknown material \"%s\"", name); return; }
+    }
+    mpeditmat(id, filter, *style, sel, true);
 }
 
-COMMAND(0, editmat, "si");
+COMMAND(0, editmat, "ssi");
 
 VAR(IDF_PERSIST, autoapplytexgui, 0, 1, 1);
 VAR(IDF_PERSIST, autopreviewtexgui, 0, 1, 1);
@@ -2222,100 +2303,94 @@ struct texturegui : guicb
         if(menupage > numpages) menupage = numpages;
         else if(menupage < 0) menupage = 0;
         g.start(menustart, menuscale, NULL, true);
-        g.pushlist();
-        g.space(2);
-        if(g.button("\fgauto apply", 0xFFFFFF, autoapplytexgui ? "checkboxon" : "checkbox", 0xFFFFFF, autoapplytexgui ? false : true)&GUI_UP)
-            autoapplytexgui = autoapplytexgui ? 0 : 1;
-        g.space(2);
-        if(g.button("\fgauto preview", 0xFFFFFF, autopreviewtexgui ? "checkboxon" : "checkbox", 0xFFFFFF, autopreviewtexgui ? false : true)&GUI_UP)
-            autopreviewtexgui = autopreviewtexgui ? 0 : 1;
-        g.space(2);
-        if(g.button("\fgauto close", 0xFFFFFF, autoclosetexgui ? (autoclosetexgui > 1 ? "checkboxtwo" : "checkboxon") : "checkbox", 0xFFFFFF, autoclosetexgui ? false : true)&GUI_UP)
-            autoclosetexgui = autoclosetexgui ? (autoclosetexgui > 1 ? 0 : 2) : 1;
-        g.space(2);
-        if(g.button("\foreset mru", 0xFFFFFF, NULL)&GUI_UP)
-        {
-            int old = texmru.inrange(menutex) ? texmru[menutex] : -1;
-            resettexmru();
-            curtex = menutex = rolltex = texmru.find(old);
-        }
-        g.poplist();
-        g.space(1);
-        g.pushlist();
-        if(texmru.inrange(curtex))
-        {
-            VSlot &v = lookupvslot(texmru[curtex], false);
-            if(v.slot->sts.empty()) g.texture(dummyvslot, thumbheight*thumbsize, false);
-            else if(!v.slot->loaded && !v.slot->thumbnail)
+        uilist(g, {
+            g.space(2);
+            if(g.button("\fgauto apply", 0xFFFFFF, autoapplytexgui ? "checkboxon" : "checkbox", 0xFFFFFF, autoapplytexgui ? false : true)&GUI_UP)
+                autoapplytexgui = autoapplytexgui ? 0 : 1;
+            g.space(2);
+            if(g.button("\fgauto preview", 0xFFFFFF, autopreviewtexgui ? "checkboxon" : "checkbox", 0xFFFFFF, autopreviewtexgui ? false : true)&GUI_UP)
+                autopreviewtexgui = autopreviewtexgui ? 0 : 1;
+            g.space(2);
+            if(g.button("\fgauto close", 0xFFFFFF, autoclosetexgui ? (autoclosetexgui > 1 ? "checkboxtwo" : "checkboxon") : "checkbox", 0xFFFFFF, autoclosetexgui ? false : true)&GUI_UP)
+                autoclosetexgui = autoclosetexgui ? (autoclosetexgui > 1 ? 0 : 2) : 1;
+            g.space(2);
+            if(g.button("\foreset order", 0xFFFFFF, NULL)&GUI_UP)
             {
-                if(totalmillis-lastthumbnail<thumbtime)
-                    g.texture(dummyvslot, thumbheight*thumbsize, false); //create an empty space
-                loadthumbnail(*v.slot);
-                lastthumbnail = totalmillis;
+                int old = texmru.inrange(menutex) ? texmru[menutex] : -1;
+                resettexmru();
+                curtex = menutex = rolltex = texmru.find(old);
             }
-            int ret = g.texture(v, thumbheight*thumbsize, true);
-            if(ret&GUI_UP)
-            {
-                edittex(texmru[curtex]);
-                if(autoclosetexgui) menuon = false;
-            }
-        }
-        else g.image(textureload("textures/nothumb", 3), thumbheight*thumbsize, true);
+        });
         g.space(1);
-        g.pushlist();
-        g.pushlist();
-        g.pushlist();
-        loop(h, thumbheight)
-        {
-            g.pushlist();
-            loop(w, thumbwidth)
+        uilist(g, {
+            if(texmru.inrange(curtex))
             {
-                int ti = (menupage*thumbheight+h)*thumbwidth+w;
-                if(ti<texmru.length())
+                VSlot &v = lookupvslot(texmru[curtex], false);
+                if(v.slot->sts.empty()) g.texture(dummyvslot, thumbheight*thumbsize, false);
+                else if(!v.slot->loaded && !v.slot->thumbnail)
                 {
-                    VSlot &v = lookupvslot(texmru[ti], false);
-                    if(v.slot->sts.empty()) continue;
-                    else if(!v.slot->loaded && !v.slot->thumbnail)
-                    {
-                        if(totalmillis-lastthumbnail<thumbtime)
-                        {
-                            g.texture(dummyvslot, thumbsize, false); //create an empty space
-                            continue;
-                        }
-                        loadthumbnail(*v.slot);
-                        lastthumbnail = totalmillis;
-                    }
-                    int ret = g.texture(v, thumbsize, true);
-                    if(ret&GUI_UP)
-                    {
-                        if(autoapplytexgui && (v.slot->loaded || v.slot->thumbnail!=notexture))
-                        {
-                            menutex = ti;
-                            edittex(texmru[ti]);
-                            if(autoclosetexgui > 1) menuon = false;
-                        }
-                    }
-                    else if(ret&GUI_ROLLOVER) rolltex = ti;
+                    if(totalmillis-lastthumbnail<thumbtime)
+                        g.texture(dummyvslot, thumbheight*thumbsize, false); //create an empty space
+                    loadthumbnail(*v.slot);
+                    lastthumbnail = totalmillis;
+                }
+                int ret = g.texture(v, thumbheight*thumbsize, true);
+                if(ret&GUI_UP)
+                {
+                    edittex(texmru[curtex]);
+                    if(autoclosetexgui) menuon = false;
                 }
-                else g.texture(dummyvslot, thumbsize, false); //create an empty space
             }
-            g.poplist();
-        }
-        g.poplist();
-        g.slider(menupage, 0, numpages, 0xFFFFFF, NULL, true, true);
-        g.poplist();
-        g.poplist();
-        g.poplist();
+            else g.image(textureload("textures/nothumb", 3), thumbheight*thumbsize, true);
+            g.space(1);
+            uilistv(g, 2, {
+                uilist(g, loop(h, thumbheight)
+                {
+                    uilist(g,  loop(w, thumbwidth)
+                    {
+                        int ti = (menupage*thumbheight+h)*thumbwidth+w;
+                        if(ti<texmru.length())
+                        {
+                            VSlot &v = lookupvslot(texmru[ti], false);
+                            if(v.slot->sts.empty()) continue;
+                            else if(!v.slot->loaded && !v.slot->thumbnail)
+                            {
+                                if(totalmillis-lastthumbnail<thumbtime)
+                                {
+                                    g.texture(dummyvslot, thumbsize, false); //create an empty space
+                                    continue;
+                                }
+                                loadthumbnail(*v.slot);
+                                lastthumbnail = totalmillis;
+                            }
+                            int ret = g.texture(v, thumbsize, true);
+                            if(ret&GUI_UP)
+                            {
+                                if(autoapplytexgui && (v.slot->loaded || v.slot->thumbnail!=notexture))
+                                {
+                                    menutex = ti;
+                                    edittex(texmru[ti]);
+                                    if(autoclosetexgui > 1) menuon = false;
+                                }
+                            }
+                            else if(ret&GUI_ROLLOVER) rolltex = ti;
+                        }
+                        else g.texture(dummyvslot, thumbsize, false); //create an empty space
+                    });
+                });
+                g.slider(menupage, 0, numpages, 0xFFFFFF, NULL, true, true);
+            });
+        });
         g.space(1);
-        g.pushlist();
-        g.space(1);
-        if(texmru.inrange(curtex))
-        {
-            VSlot &v = lookupvslot(texmru[curtex]);
-            g.textf("#%-3d \fa%s", 0xFFFFFF, NULL, 0, texmru[curtex], v.slot->sts.empty() ? "<unknown texture>" : v.slot->sts[0].name);
-        }
-        else g.textf("no texture selected", 0x888888);
-        g.poplist();
+        uilist(g, {
+            g.space(1);
+            if(texmru.inrange(curtex))
+            {
+                VSlot &v = lookupvslot(texmru[curtex]);
+                g.textf("#%-3d \fa%s", 0xFFFFFF, NULL, 0, texmru[curtex], v.slot->sts.empty() ? "<unknown texture>" : v.slot->sts[0].name);
+            }
+            else g.textf("no texture selected", 0x888888);
+        });
         g.end();
     }
 
@@ -2356,7 +2431,7 @@ void showtexgui(int *n)
 
 COMMAND(0, showtexgui, "i"); // 0/noargs = toggle, 1 = on, other = off - will autoclose when exiting editmode
 
-void render_texture_panel(int w, int h)
+void rendertexturepanel(int w, int h)
 {
     if((texpaneltimer -= curtime)>0 && editmode)
     {
@@ -2365,10 +2440,7 @@ void render_texture_panel(int w, int h)
         glOrtho(0, width, 1800, 0, -1, 1);
         int y = 50, gap = 10;
 
-        static Shader *rgbonlyshader = NULL;
-        if(!rgbonlyshader) rgbonlyshader = lookupshaderbyname("rgbonly");
-
-        rgbonlyshader->set();
+        SETSHADER(rgbonly);
 
         loopi(7)
         {
