@@ -8,23 +8,28 @@
 
 const int MAXCLIPPLANES = 1024;
 static clipplanes clipcache[MAXCLIPPLANES];
-static int clipcacheversion = 0;
+static int clipcacheversion = -2;
 
-static inline clipplanes &getclipplanes(cube &c, const ivec &o, int size)
+static inline clipplanes &getclipplanes(const cube &c, const ivec &o, int size, bool collide = true, int offset = 0)
 {
     clipplanes &p = clipcache[int(&c - worldroot)&(MAXCLIPPLANES-1)];
-    if(p.owner != &c || p.version != clipcacheversion)
+    if(p.owner != &c || p.version != clipcacheversion+offset)
     {
         p.owner = &c;
-        p.version = clipcacheversion;
-        genclipplanes(c, o.x, o.y, o.z, size, p);
+        p.version = clipcacheversion+offset;
+        genclipplanes(c, o.x, o.y, o.z, size, p, collide);
     }
     return p;
 }
 
 void resetclipplanes()
 {
-    if(!clipcacheversion++) memset(clipcache, 0, sizeof(clipcache));
+    clipcacheversion += 2;
+    if(!clipcacheversion)
+    {
+        memset(clipcache, 0, sizeof(clipcache));
+        clipcacheversion = 2;
+    }
 }
 
 /////////////////////////  ray - cube collision ///////////////////////////////////////////////
@@ -96,7 +101,7 @@ bool pointincube(const clipplanes &p, const vec &v)
 
 vec hitsurface;
 
-static inline bool raycubeintersect(clipplanes &p, const cube &c, const vec &v, const vec &ray, const vec &invray, float maxdist, float &dist)
+static inline bool raycubeintersect(const clipplanes &p, const cube &c, const vec &v, const vec &ray, const vec &invray, float maxdist, float &dist)
 {
     int entry = -1, bbentry = -1;
     INTERSECTPLANES(entry = i, return false);
@@ -118,10 +123,10 @@ int hitent, hitorient;
 
 #define mapmodelskip \
     { \
-            if(e.attrs[5]&MMT_NOCLIP) continue; \
+            if(e.attrs[6]&MMT_NOCLIP) continue; \
             if(e.lastemit) \
             { \
-                if(e.attrs[5]&MMT_HIDE) \
+                if(e.attrs[6]&MMT_HIDE) \
                 { \
                     if(e.spawned) continue; \
                 } \
@@ -185,6 +190,28 @@ static float disttoent(octaentities *oc, octaentities *last, const vec &o, const
     return dist;
 }
 
+static float disttooutsideent(const vec &o, const vec &ray, float radius, int mode, extentity *t)
+{
+    vec eo, es;
+    int orient;
+    float dist = 1e16f, f = 0.0f;
+    const vector<extentity *> &ents = entities::getents();
+    loopv(outsideents)
+    {
+        extentity &e = *ents[outsideents[i]];
+        if(!e.inoctanode || &e == t) continue;
+        entselectionbox(e, eo, es);
+        if(!rayrectintersect(eo, es, o, ray, f, orient)) continue;
+        if(f<dist && f>0)
+        {
+            hitentdist = dist = f;
+            hitent = outsideents[i];
+            hitorient = orient;
+        }
+    }
+    return dist;
+}
+
 // optimized shadow version
 static float shadowent(octaentities *oc, octaentities *last, const vec &o, const vec &ray, float radius, int mode, extentity *t)
 {
@@ -195,7 +222,7 @@ static float shadowent(octaentities *oc, octaentities *last, const vec &o, const
     {
         extentity &e = *ents[oc->mapmodels[i]];
         if(!e.inoctanode || &e==t) continue;
-        if(e.attrs[5]&MMT_NOSHADOW) continue;
+        if(e.attrs[6]&MMT_NOSHADOW) continue;
         if(!mmintersect(e, o, ray, radius, mode, f)) continue;
         if(f>0 && f<dist) dist = f;
     }
@@ -322,7 +349,7 @@ float raycube(const vec &o, const vec &ray, float radius, int mode, int size, ex
 
         if(!isempty(c))
         {
-            clipplanes &p = getclipplanes(c, lo, lsize);
+            const clipplanes &p = getclipplanes(c, lo, lsize, false, 1);
             float f = 0;
             if(raycubeintersect(p, c, v, ray, invray, dent-dist, f) && (dist+f>0 || !(mode&RAY_SKIPFIRST)) && (!(mode&RAY_CLIPMAT) || (c.material&MATF_CLIP)!=MAT_NOCLIP))
                 return min(dent, dist+f);
@@ -353,7 +380,7 @@ float shadowray(const vec &o, const vec &ray, float radius, int mode, extentity
         if(!isempty(c) && !(c.material&MAT_ALPHA))
         {
             if(isentirelysolid(c)) return c.texture[side]==DEFAULT_SKY && mode&RAY_SKIPSKY ? radius : dist;
-            clipplanes &p = getclipplanes(c, lo, 1<<lshift);
+            const clipplanes &p = getclipplanes(c, lo, 1<<lshift, false, 1);
             INTERSECTPLANES(side = p.side[i], goto nextcube);
             INTERSECTBOX(side = (i<<1) + 1 - lsizemask[i], goto nextcube);
             if(exitdist >= 0) return c.texture[side]==DEFAULT_SKY && mode&RAY_SKIPSKY ? radius : dist+max(enterdist+0.1f, 0.0f);
@@ -375,7 +402,7 @@ struct ShadowRayCache
     clipplanes clipcache[MAXCLIPPLANES];
     int version;
 
-    ShadowRayCache() : version(0) {}
+    ShadowRayCache() : version(-1) {}
 };
 
 ShadowRayCache *newshadowraycache() { return new ShadowRayCache; }
@@ -384,7 +411,12 @@ void freeshadowraycache(ShadowRayCache *&cache) { delete cache; cache = NULL; }
 
 void resetshadowraycache(ShadowRayCache *cache)
 {
-    if(!cache->version++) memset(cache->clipcache, 0, sizeof(cache->clipcache));
+    cache->version++;
+    if(!cache->version)
+    {
+        memset(cache->clipcache, 0, sizeof(cache->clipcache));
+        cache->version = 1;
+    }
 }
 
 float shadowray(ShadowRayCache *cache, const vec &o, const vec &ray, float radius, int mode, extentity *t)
@@ -404,7 +436,7 @@ float shadowray(ShadowRayCache *cache, const vec &o, const vec &ray, float radiu
         {
             if(isentirelysolid(c)) return c.texture[side]==DEFAULT_SKY && mode&RAY_SKIPSKY ? radius : dist;
             clipplanes &p = cache->clipcache[int(&c - worldroot)&(MAXCLIPPLANES-1)];
-            if(p.owner != &c || p.version != cache->version) { p.owner = &c; p.version = cache->version; genclipplanes(c, lo.x, lo.y, lo.z, 1<<lshift, p); }
+            if(p.owner != &c || p.version != cache->version) { p.owner = &c; p.version = cache->version; genclipplanes(c, lo.x, lo.y, lo.z, 1<<lshift, p, false); }
             INTERSECTPLANES(side = p.side[i], goto nextcube);
             INTERSECTBOX(side = (i<<1) + 1 - lsizemask[i], goto nextcube);
             if(exitdist >= 0) return c.texture[side]==DEFAULT_SKY && mode&RAY_SKIPSKY ? radius : dist+max(enterdist+0.1f, 0.0f);
@@ -419,13 +451,19 @@ float shadowray(ShadowRayCache *cache, const vec &o, const vec &ray, float radiu
     }
 }
 
-float rayent(const vec &o, const vec &ray, vec &hitpos, float radius, int mode, int size, int &orient, int &ent)
+float rayent(const vec &o, const vec &ray, float radius, int mode, int size, int &orient, int &ent)
 {
     hitent = -1;
-    float d = raycubepos(o, ray, hitpos, hitentdist = radius, mode, size);
+    hitentdist = radius;
+    float dist = raycube(o, ray, radius, mode, size);
+    if((mode&RAY_ENTS) == RAY_ENTS)
+    {
+        float dent = disttooutsideent(o, ray, dist < 0 ? 1e16f : dist, mode, NULL);
+        if(dent < 1e15f && (dist < 0 || dent < dist)) dist = dent;
+    }
     orient = hitorient;
-    ent = (hitentdist == d) ? hitent : -1;
-    return d;
+    ent = hitentdist == dist ? hitent : -1;
+    return dist;
 }
 
 float raycubepos(const vec &o, const vec &ray, vec &hitpos, float radius, int mode, int size)
@@ -714,8 +752,19 @@ bool plcollide(physent *d, const vec &dir)  // collide with player or monster
     return true;
 }
 
-void rotatebb(vec &center, vec &radius, int yaw, int roll)
+void rotatebb(vec &center, vec &radius, int yaw, int pitch, int roll)
 {
+    if(pitch)
+    {
+        if(pitch < 0) pitch = 360 + pitch%360;
+        else if(pitch >= 360) pitch %= 360;
+        const vec2 &rot = sincos360[pitch];
+        vec2 oldcenter(center.x, center.z), oldradius(radius.x, radius.z);
+        center.x = oldcenter.x*rot.x - oldcenter.y*rot.y;
+        center.z = oldcenter.y*rot.x + oldcenter.x*rot.y;
+        radius.x = fabs(oldradius.x*rot.x) + fabs(oldradius.y*rot.y);
+        radius.z = fabs(oldradius.y*rot.x) + fabs(oldradius.x*rot.y);
+    }
     if(roll)
     {
         if(roll < 0) roll = 360 + roll%360;
@@ -768,12 +817,12 @@ bool mmcollide(physent *d, const vec &dir, octaentities &oc)               // co
         if(!m || !m->collide) continue;
         vec center, radius;
         m->collisionbox(0, center, radius);
-        if(e.attrs[4]) { float scale = max(e.attrs[4]/100.f, 1e-3f); center.mul(scale); radius.mul(scale); }
-        int yaw = e.attrs[1], roll = e.attrs[2];
+        if(e.attrs[5]) { float scale = max(e.attrs[5]/100.f, 1e-3f); center.mul(scale); radius.mul(scale); }
+        int yaw = e.attrs[1], pitch = e.attrs[2], roll = e.attrs[3];
         switch(d->collidetype)
         {
             case COLLIDE_ELLIPSE:
-                if(roll) rotatebb(center, radius, 0, roll);
+                if(pitch || roll) rotatebb(center, radius, 0, pitch, roll);
                 if(m->ellipsecollide)
                 {
                     if(!ellipsecollide(d, dir, e.o, center, yaw, radius.x, radius.y, radius.z, radius.z)) return false;
@@ -789,7 +838,7 @@ bool mmcollide(physent *d, const vec &dir, octaentities &oc)               // co
                 break;
             case COLLIDE_AABB:
             default:
-                rotatebb(center, radius, yaw, roll);
+                rotatebb(center, radius, yaw, pitch, roll);
                 if(!rectcollide(d, dir, center.add(e.o), radius.x, radius.y, radius.z, radius.z)) return false;
                 break;
         }
@@ -798,7 +847,7 @@ bool mmcollide(physent *d, const vec &dir, octaentities &oc)               // co
 }
 
 template<class E>
-static bool fuzzycollidesolid(physent *d, const vec &dir, float cutoff, cube &c, const ivec &co, int size) // collide with solid cube geometry
+static bool fuzzycollidesolid(physent *d, const vec &dir, float cutoff, const cube &c, const ivec &co, int size) // collide with solid cube geometry
 {
     int crad = size/2;
     if(fabs(d->o.x - co.x - crad) > d->radius + crad || fabs(d->o.y - co.y - crad) > d->radius + crad ||
@@ -808,7 +857,7 @@ static bool fuzzycollidesolid(physent *d, const vec &dir, float cutoff, cube &c,
     E entvol(d);
     wall = vec(0, 0, 0);
     float bestdist = -1e10f;
-    int visible = isentirelysolid(c) ? c.collide : 0xFF;
+    int visible = isentirelysolid(c) ? c.visible : 0xFF;
     loopi(6) if(visible&(1<<i))
     {
         int dim = dimension(i), dc = dimcoord(i), dimdir = 2*dc - 1;
@@ -866,9 +915,9 @@ static inline bool clampcollide(const clipplanes &p, const E &entvol, const plan
 }
 
 template<class E>
-static bool fuzzycollideplanes(physent *d, const vec &dir, float cutoff, cube &c, const ivec &co, int size) // collide with deformed cube geometry
+static bool fuzzycollideplanes(physent *d, const vec &dir, float cutoff, const cube &c, const ivec &co, int size) // collide with deformed cube geometry
 {
-    clipplanes &p = getclipplanes(c, co, size);
+    const clipplanes &p = getclipplanes(c, co, size);
 
     if(fabs(d->o.x - p.o.x) > p.r.x + d->radius || fabs(d->o.y - p.o.y) > p.r.y + d->radius ||
        d->o.z + d->aboveeye < p.o.z - p.r.z || d->o.z - d->height > p.o.z + p.r.z)
@@ -901,7 +950,7 @@ static bool fuzzycollideplanes(physent *d, const vec &dir, float cutoff, cube &c
     int bestplane = -1;
     loopi(p.size)
     {
-        plane &w = p.p[i];
+        const plane &w = p.p[i];
         vec pw = entvol.supportpoint(vec(w).neg());
         float dist = w.dist(pw);
         if(dist >= 0) return true;
@@ -930,7 +979,7 @@ static bool fuzzycollideplanes(physent *d, const vec &dir, float cutoff, cube &c
 }
 
 template<class E>
-static bool cubecollidesolid(physent *d, const vec &dir, float cutoff, cube &c, const ivec &co, int size) // collide with solid cube geometry
+static bool cubecollidesolid(physent *d, const vec &dir, float cutoff, const cube &c, const ivec &co, int size) // collide with solid cube geometry
 {
     int crad = size/2;
     if(fabs(d->o.x - co.x - crad) > d->radius + crad || fabs(d->o.y - co.y - crad) > d->radius + crad ||
@@ -943,7 +992,7 @@ static bool cubecollidesolid(physent *d, const vec &dir, float cutoff, cube &c,
 
     wall = vec(0, 0, 0);
     float bestdist = -1e10f;
-    int visible = isentirelysolid(c) ? c.collide : 0xFF;
+    int visible = isentirelysolid(c) ? c.visible : 0xFF;
     loopi(6) if(visible&(1<<i))
     {
         int dim = dimension(i), dc = dimcoord(i), dimdir = 2*dc - 1;
@@ -973,9 +1022,9 @@ static bool cubecollidesolid(physent *d, const vec &dir, float cutoff, cube &c,
 }
 
 template<class E>
-static bool cubecollideplanes(physent *d, const vec &dir, float cutoff, cube &c, const ivec &co, int size) // collide with deformed cube geometry
+static bool cubecollideplanes(physent *d, const vec &dir, float cutoff, const cube &c, const ivec &co, int size) // collide with deformed cube geometry
 {
-    clipplanes &p = getclipplanes(c, co, size);
+    const clipplanes &p = getclipplanes(c, co, size);
 
     if(fabs(d->o.x - p.o.x) > p.r.x + d->radius || fabs(d->o.y - p.o.y) > p.r.y + d->radius ||
        d->o.z + d->aboveeye < p.o.z - p.r.z || d->o.z - d->height > p.o.z + p.r.z)
@@ -1010,7 +1059,7 @@ static bool cubecollideplanes(physent *d, const vec &dir, float cutoff, cube &c,
     int bestplane = -1;
     loopi(p.size)
     {
-        plane &w = p.p[i];
+        const plane &w = p.p[i];
         vec pw = entvol.supportpoint(vec(w).neg());
         float dist = w.dist(pw);
         if(dist <= bestdist) continue;
@@ -1037,7 +1086,7 @@ static bool cubecollideplanes(physent *d, const vec &dir, float cutoff, cube &c,
     return false;
 }
 
-static inline bool cubecollide(physent *d, const vec &dir, float cutoff, cube &c, const ivec &co, int size, bool solid)
+static inline bool cubecollide(physent *d, const vec &dir, float cutoff, const cube &c, const ivec &co, int size, bool solid)
 {
     switch(d->collidetype)
     {
@@ -1047,7 +1096,7 @@ static inline bool cubecollide(physent *d, const vec &dir, float cutoff, cube &c
             if(cutoff <= 0)
             {
                 int crad = size/2;
-                return rectcollide(d, dir, vec(co.x + crad, co.y + crad, co.z), crad, crad, size, 0, isentirelysolid(c) ? c.collide : 0xFF);
+                return rectcollide(d, dir, vec(co.x + crad, co.y + crad, co.z), crad, crad, size, 0, isentirelysolid(c) ? c.visible : 0xFF);
             }
 #if 0
             else return cubecollidesolid<mpr::EntAABB>(d, dir, cutoff, c, co, size);
@@ -1060,7 +1109,7 @@ static inline bool cubecollide(physent *d, const vec &dir, float cutoff, cube &c
 #if 0
             if(cutoff <= 0)
             {
-                clipplanes &p = getclipplanes(c, co, size);
+                const clipplanes &p = getclipplanes(c, co, size);
                 if(!p.size) return rectcollide(d, dir, p.o, p.r.x, p.r.y, p.r.z, p.r.z, p.visible);
             }
             return cubecollideplanes<mpr::EntAABB>(d, dir, cutoff, c, co, size);
@@ -1083,7 +1132,7 @@ static inline bool cubecollide(physent *d, const vec &dir, float cutoff, cube &c
     }
 }
 
-static inline bool octacollide(physent *d, const vec &dir, float cutoff, const ivec &bo, const ivec &bs, cube *c, const ivec &cor, int size) // collide with octants
+static inline bool octacollide(physent *d, const vec &dir, float cutoff, const ivec &bo, const ivec &bs, const cube *c, const ivec &cor, int size) // collide with octants
 {
     loopoctabox(cor, size, bo, bs)
     {
@@ -1114,7 +1163,7 @@ static inline bool octacollide(physent *d, const vec &dir, float cutoff, const i
         scale = worldscale-1;
     if(diff&~((1<<scale)-1) || uint(bo.x|bo.y|bo.z|(bo.x+bs.x)|(bo.y+bs.y)|(bo.z+bs.z)) >= uint(hdr.worldsize))
        return octacollide(d, dir, cutoff, bo, bs, worldroot, ivec(0, 0, 0), hdr.worldsize>>1);
-    cube *c = &worldroot[octastep(bo.x, bo.y, bo.z, scale)];
+    const cube *c = &worldroot[octastep(bo.x, bo.y, bo.z, scale)];
     if(c->ext && c->ext->ents && !mmcollide(d, dir, *c->ext->ents)) return false;
     scale--;
     while(c->children && !(diff&(1<<scale)))
@@ -1192,8 +1241,7 @@ void phystest()
 {
     static const char *states[] = {"float", "fall", "slide", "slope", "floor", "step up", "step down", "bounce"};
     physent *player = (physent *)game::focusedent();
-    conoutf("PHYS(pl): %s, air %d, floor: (%f, %f, %f), vel: (%f, %f, %f), g: (%f, %f, %f)", states[player->physstate], player->timeinair, player->floor.x, player->floor.y, player->floor.z, player->vel.x, player->vel.y, player->vel.z, player->falling.x, player->falling.y, player->falling.z);
-    //conoutf("PHYS(cam): %s, air %d, floor: (%f, %f, %f), vel: (%f, %f, %f), g: (%f, %f, %f)", states[camera1->physstate], camera1->timeinair, camera1->floor.x, camera1->floor.y, camera1->floor.z, camera1->vel.x, camera1->vel.y, camera1->vel.z, camera1->falling.x, camera1->falling.y, camera1->falling.z);
+    conoutf("PHYS(pl): %s, air %d, mat: %d, ladder: %s, floor: (%f, %f, %f), vel: (%f, %f, %f), g: (%f, %f, %f)", states[player->physstate], lastmillis-player->airmillis, player->inmaterial, player->onladder ? "yes" : "no", player->floor.x, player->floor.y, player->floor.z, player->vel.x, player->vel.y, player->vel.z, player->falling.x, player->falling.y, player->falling.z);
 }
 
 COMMAND(0, phystest, "");
