@@ -2,7 +2,7 @@
 
 #include "engine.h"
 
-bool hasVBO = false, hasDRE = false, hasOQ = false, hasTR = false, hasFBO = false, hasDS = false, hasTF = false, hasBE = false, hasBC = false, hasCM = false, hasNP2 = false, hasTC = false, hasTE = false, hasMT = false, hasD3 = false, hasAF = false, hasVP2 = false, hasVP3 = false, hasPP = false, hasMDA = false, hasTE3 = false, hasTE4 = false, hasVP = false, hasFP = false, hasGLSL = false, hasGM = false, hasNVFB = false, hasSGIDT = false, hasSGISH = false, hasDT = false, hasSH = false, hasNVPCF = false, hasRN = false, hasPBO = false, hasFBB = false, hasUBO = false, hasBUE = false, hasFC = false, hasTEX = false;
+bool hasVBO = false, hasDRE = false, hasOQ = false, hasTR = false, hasFBO = false, hasDS = false, hasTF = false, hasBE = false, hasBC = false, hasCM = false, hasNP2 = false, hasTC = false, hasS3TC = false, hasFXT1 = false, hasTE = false, hasMT = false, hasD3 = false, hasAF = false, hasVP2 = false, hasVP3 = false, hasPP = false, hasMDA = false, hasTE3 = false, hasTE4 = false, hasVP = false, hasFP = false, hasGLSL = false, hasGM = false, hasNVFB = false, hasSGIDT = false, hasSGISH = false, hasDT = false, hasSH = false, hasNVPCF = false, hasRN = false, hasPBO = false, hasFBB = false, hasUBO = false, hasBUE = false, hasMBR = false, hasFC = false, hasTEX = false;
 int hasstencil = 0;
 
 VAR(0, renderpath, 1, 0, 0);
@@ -94,6 +94,13 @@ PFNGLGETACTIVEUNIFORMPROC         glGetActiveUniform_         = NULL;
 PFNGLENABLEVERTEXATTRIBARRAYPROC  glEnableVertexAttribArray_  = NULL;
 PFNGLDISABLEVERTEXATTRIBARRAYPROC glDisableVertexAttribArray_ = NULL;
 PFNGLVERTEXATTRIBPOINTERPROC      glVertexAttribPointer_      = NULL;
+
+PFNGLUNIFORMMATRIX2X3FVPROC       glUniformMatrix2x3fv_       = NULL;
+PFNGLUNIFORMMATRIX3X2FVPROC       glUniformMatrix3x2fv_       = NULL;
+PFNGLUNIFORMMATRIX2X4FVPROC       glUniformMatrix2x4fv_       = NULL;
+PFNGLUNIFORMMATRIX4X2FVPROC       glUniformMatrix4x2fv_       = NULL;
+PFNGLUNIFORMMATRIX3X4FVPROC       glUniformMatrix3x4fv_       = NULL;
+PFNGLUNIFORMMATRIX4X3FVPROC       glUniformMatrix4x3fv_       = NULL;
 #endif
 
 // GL_EXT_draw_range_elements
@@ -135,6 +142,10 @@ PFNGLGETUNIFORMOFFSETEXTPROC     glGetUniformOffset_     = NULL;
 // GL_EXT_fog_coord
 PFNGLFOGCOORDPOINTEREXTPROC glFogCoordPointer_ = NULL;
 
+// GL_ARB_map_buffer_range
+PFNGLMAPBUFFERRANGEPROC         glMapBufferRange_         = NULL;
+PFNGLFLUSHMAPPEDBUFFERRANGEPROC glFlushMappedBufferRange_ = NULL;
+
 void *getprocaddress(const char *name)
 {
     return SDL_GL_GetProcAddress(name);
@@ -148,12 +159,14 @@ VAR(0, ati_line_bug, 0, 0, 1);
 VAR(0, ati_cubemap_bug, 0, 0, 1);
 VAR(0, ati_ubo_bug, 0, 0, 1);
 VAR(0, nvidia_scissor_bug, 0, 0, 1);
+VAR(0, intel_immediate_bug, 0, 0, 1);
+VAR(0, intel_vertexarray_bug, 0, 0, 1);
 VAR(0, apple_glsldepth_bug, 0, 0, 1);
 VAR(0, apple_ff_bug, 0, 0, 1);
 VAR(0, apple_vp_bug, 0, 0, 1);
 VAR(0, sdl_backingstore_bug, -1, 0, 1);
-VAR(0, mesa_program_bug, 0, 0, 1);
 VAR(0, avoidshaders, 1, 0, 0);
+VAR(0, preferglsl, 1, 0, 0);
 VAR(0, minimizetcusage, 1, 0, 0);
 VAR(0, emulatefog, 1, 0, 0);
 VAR(0, usevp2, 1, 0, 0);
@@ -162,6 +175,7 @@ VAR(0, usetexrect, 1, 0, 0);
 VAR(0, hasglsl, 1, 0, 0);
 VAR(0, useubo, 1, 0, 0);
 VAR(0, usebue, 1, 0, 0);
+VAR(0, usetexcompress, 1, 0, 0);
 VAR(0, rtscissor, 0, 1, 1);
 VAR(0, blurtile, 0, 1, 1);
 VAR(0, rtsharefb, 0, 1, 1);
@@ -198,13 +212,16 @@ void gl_checkextensions()
 
 #ifdef __APPLE__
     extern int mac_osversion();
-    int osversion = mac_osversion();  /* 0x1050 = 10.5 (Leopard) */
+    int osversion = mac_osversion();  /* 0x0A0500 = 10.5 (Leopard) */
     sdl_backingstore_bug = -1;
 #endif
 
     bool mesa = false, intel = false, ati = false, nvidia = false;
     if(strstr(renderer, "Mesa") || strstr(version, "Mesa"))
+    {
         mesa = true;
+        if(strstr(renderer, "Intel")) intel = true;
+    }
     else if(strstr(vendor, "NVIDIA"))
         nvidia = true;
     else if(strstr(vendor, "ATI") || strstr(vendor, "Advanced Micro Devices"))
@@ -212,10 +229,6 @@ void gl_checkextensions()
     else if(strstr(vendor, "Intel"))
         intel = true;
 
-    GLint val;
-    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &val);
-    hwtexsize = val;
-
     uint glmajorversion, glminorversion;
     if(sscanf(version, " %u.%u", &glmajorversion, &glminorversion) != 2) glversion = 100;
     else glversion = glmajorversion*100 + glminorversion*10;
@@ -225,6 +238,10 @@ void gl_checkextensions()
     // char *weakcards[] = { "GeForce FX", "Quadro FX", "6200", "9500", "9550", "9600", "9700", "9800", "X300", "X600", "FireGL", "Intel", "Chrome", NULL }
     // if(shaderprecision==2) for(char **wc = weakcards; *wc; wc++) if(strstr(renderer, *wc)) shaderprecision = 1;
 
+    GLint val;
+    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &val);
+    hwtexsize = val;
+
     if(hasext(exts, "GL_EXT_texture_env_combine") || hasext(exts, "GL_ARB_texture_env_combine"))
     {
         hasTE = true;
@@ -257,7 +274,7 @@ void gl_checkextensions()
 #ifdef __APPLE__
     /* VBOs over 256KB seem to destroy performance on 10.5, but not in 10.6 */
     extern int maxvbosize;
-    if(osversion < 0x1060) maxvbosize = min(maxvbosize, 8192);
+    if(osversion < 0x0A0600) maxvbosize = min(maxvbosize, 8192);
 #endif
 
     if(hasext(exts, "GL_ARB_pixel_buffer_object"))
@@ -295,7 +312,7 @@ void gl_checkextensions()
 
 #ifdef __APPLE__
     // floating point FBOs not fully supported until 10.5
-    if(osversion>=0x1050)
+    if(osversion>=0x0A0500)
 #endif
     if(hasext(exts, "GL_ARB_texture_float") || hasext(exts, "GL_ATI_texture_float"))
     {
@@ -336,10 +353,6 @@ void gl_checkextensions()
     }
     else conoutf("\frWARNING: No framebuffer object support. (reflective water may be slow)");
 
-#ifdef __APPLE__
-    // Intel HD3000 broke occlusion queries - either causing software fallback, or returning wrong results
-    if(!intel)
-#endif
     if(hasext(exts, "GL_ARB_occlusion_query"))
     {
         GLint bits;
@@ -356,7 +369,7 @@ void gl_checkextensions()
             hasOQ = true;
             if(dbgexts) conoutf("\frUsing GL_ARB_occlusion_query extension.");
 #if defined(__APPLE__) && SDL_BYTEORDER == SDL_BIG_ENDIAN
-            if(ati && (osversion<0x1050)) ati_oq_bug = 1;
+            if(ati && (osversion<0x0A0500)) ati_oq_bug = 1;
 #endif
             //if(ati_oq_bug) conoutf("\frWARNING: Using ATI occlusion query bug workaround. (use \"/ati_oq_bug 0\" to disable if unnecessary)");
         }
@@ -386,17 +399,14 @@ void gl_checkextensions()
 #endif
 
         if(ati) ati_dph_bug = ati_line_bug = 1;
-        else if(mesa) mesa_program_bug = 1;
 
 #ifdef __APPLE__
-        if(osversion>=0x1050) // fixed in 1055 for some hardware.. but not all.
+        if(osversion>=0x0A0500) // fixed in 1055 for some hardware.. but not all.
         {
             apple_ff_bug = 1;
             //conoutf("\frWARNING: Using Leopard ARB_position_invariant bug workaround. (use \"/apple_ff_bug 0\" to disable if unnecessary)");
         }
 #endif
-
-        if(!avoidshaders) setvar("matskel", 0, false, true);
     }
 
     if(glversion >= 200)
@@ -430,6 +440,16 @@ void gl_checkextensions()
         glEnableVertexAttribArray_ =  (PFNGLENABLEVERTEXATTRIBARRAYPROC)  getprocaddress("glEnableVertexAttribArray");
         glDisableVertexAttribArray_ = (PFNGLDISABLEVERTEXATTRIBARRAYPROC) getprocaddress("glDisableVertexAttribArray");
         glVertexAttribPointer_ =      (PFNGLVERTEXATTRIBPOINTERPROC)      getprocaddress("glVertexAttribPointer");
+
+        if(glversion >= 210)
+        {
+            glUniformMatrix2x3fv_ =   (PFNGLUNIFORMMATRIX2X3FVPROC)       getprocaddress("glUniformMatrix2x3fv");
+            glUniformMatrix3x2fv_ =   (PFNGLUNIFORMMATRIX3X2FVPROC)       getprocaddress("glUniformMatrix3x2fv");
+            glUniformMatrix2x4fv_ =   (PFNGLUNIFORMMATRIX2X4FVPROC)       getprocaddress("glUniformMatrix2x4fv");
+            glUniformMatrix4x2fv_ =   (PFNGLUNIFORMMATRIX4X2FVPROC)       getprocaddress("glUniformMatrix4x2fv");
+            glUniformMatrix3x4fv_ =   (PFNGLUNIFORMMATRIX3X4FVPROC)       getprocaddress("glUniformMatrix3x4fv");
+            glUniformMatrix4x3fv_ =   (PFNGLUNIFORMMATRIX4X3FVPROC)       getprocaddress("glUniformMatrix4x3fv");
+        }
 #endif
 
         extern bool checkglslsupport();
@@ -438,7 +458,7 @@ void gl_checkextensions()
             hasGLSL = true;
             hasglsl = 1;
 #ifdef __APPLE__
-            //if(osversion<0x1050) ??
+            //if(osversion<0x0A0500) ??
             if(hasVP && hasFP) apple_glsldepth_bug = 1;
 #endif
             //if(apple_glsldepth_bug) conoutf("\frWARNING: Using Apple GLSL depth bug workaround. (use \"/apple_glsldepth_bug 0\" to disable if unnecessary");
@@ -447,6 +467,11 @@ void gl_checkextensions()
             uint majorversion, minorversion;
             if(!str || sscanf(str, " %u.%u", &majorversion, &minorversion) != 2) glslversion = 100;
             else glslversion = majorversion*100 + minorversion;
+#ifdef __APPLE__
+            if(osversion >= 0x0A0600) { if(glslversion >= 120) preferglsl = 1; }
+            else
+#endif
+            if(glslversion >= 130) preferglsl = 1;
         }
     }
 
@@ -477,10 +502,15 @@ void gl_checkextensions()
         {
 #ifdef __APPLE__
             apple_vp_bug = 1;
+            intel_immediate_bug = 1;
+#endif
+#ifdef WIN32
+            intel_immediate_bug = 1;
+            intel_vertexarray_bug = 1;
 #endif
         }
 
-        if(!hasGLSL || glslversion < 130)
+        if(!hasGLSL || !preferglsl)
         {
             avoidshaders = 1;
             if(hwtexsize < 4096)
@@ -516,6 +546,14 @@ void gl_checkextensions()
         if(dbgexts) conoutf("\frUsing GL_EXT_gpu_program_parameters extension.");
     }
 
+    if(hasext(exts, "GL_ARB_map_buffer_range"))
+    {
+        glMapBufferRange_         = (PFNGLMAPBUFFERRANGEPROC)        getprocaddress("glMapBufferRange");
+        glFlushMappedBufferRange_ = (PFNGLFLUSHMAPPEDBUFFERRANGEPROC)getprocaddress("glFlushMappedBufferRange");
+        hasMBR = true;
+        if(dbgexts) conoutf("\frUsing GL_ARB_map_buffer_range.");
+    }
+
     if(hasext(exts, "GL_ARB_uniform_buffer_object"))
     {
         glGetUniformIndices_       = (PFNGLGETUNIFORMINDICESPROC)      getprocaddress("glGetUniformIndices");
@@ -599,7 +637,7 @@ void gl_checkextensions()
     }
     else if(usenp2) conoutf("\frWARNING: Non-power-of-two textures not supported");
 
-    if(hasext(exts, "GL_ARB_texture_compression") && hasext(exts, "GL_EXT_texture_compression_s3tc"))
+    if(hasext(exts, "GL_ARB_texture_compression"))
     {
         glCompressedTexImage3D_ =    (PFNGLCOMPRESSEDTEXIMAGE3DARBPROC)   getprocaddress("glCompressedTexImage3DARB");
         glCompressedTexImage2D_ =    (PFNGLCOMPRESSEDTEXIMAGE2DARBPROC)   getprocaddress("glCompressedTexImage2DARB");
@@ -610,7 +648,29 @@ void gl_checkextensions()
         glGetCompressedTexImage_ =   (PFNGLGETCOMPRESSEDTEXIMAGEARBPROC)  getprocaddress("glGetCompressedTexImageARB");
 
         hasTC = true;
-        if(dbgexts) conoutf("\frUsing GL_EXT_texture_compression_s3tc extension.");
+        if(dbgexts) conoutf("\frUsing GL_ARB_texture_compression.");
+
+        if(hasext(exts, "GL_EXT_texture_compression_s3tc"))
+        {
+            hasS3TC = true;
+#ifdef __APPLE__
+            usetexcompress = 1;
+#else
+            if(!mesa) usetexcompress = 2;
+#endif
+            if(dbgexts) conoutf("\frUsing GL_EXT_texture_compression_s3tc extension.");
+        }
+        else if(hasext(exts, "GL_EXT_texture_compression_dxt1") && hasext(exts, "GL_ANGLE_texture_compression_dxt3") && hasext(exts, "GL_ANGLE_texture_compression_dxt5"))
+        {
+            hasS3TC = true;
+            if(dbgexts) conoutf("\frUsing GL_EXT_texture_compression_dxt1 extension.");
+        }
+        if(hasext(exts, "GL_3DFX_texture_compression_FXT1"))
+        {
+            hasFXT1 = true;
+            if(mesa) usetexcompress = max(usetexcompress, 1);
+            if(dbgexts) conoutf("\frUsing GL_3DFX_texture_compression_FXT1.");
+        }
     }
 
     if(hasext(exts, "GL_EXT_texture_filter_anisotropic"))
@@ -729,10 +789,13 @@ void gl_init(int w, int h, int bpp, int depth, int fsaa)
     if(!useshaders || (useshaders<0 && avoidshaders) || !hasMT || !hasshaders)
     {
         if(!hasMT || !hasshaders) conoutf("\frWARNING: No shader support! Using fixed-function fallback. (no fancy visuals for you)");
-        else if(useshaders<0 && !hasTF) conoutf("\frWARNING: Disabling shaders for extra performance. (use \"/shaders 1\" to enable shaders if desired)");
+        else if(useshaders<0 && avoidshaders) conoutf("\frWARNING: Disabling shaders for extra performance. (use \"/shaders 1\" to enable shaders if desired)");
         renderpath = R_FIXEDFUNCTION;
     }
-    else renderpath = hasGLSL ? ((forceglsl && (forceglsl > 0 || glslversion >= 130)) || !hasVP || !hasFP ? (forceglsl ? R_GLSLANG : R_FIXEDFUNCTION) : R_ASMGLSLANG) : R_ASMSHADER;
+    else renderpath = hasGLSL ? ((forceglsl && (forceglsl > 0 || preferglsl)) || !hasVP || !hasFP ? (forceglsl ? R_GLSLANG : R_FIXEDFUNCTION) : R_ASMGLSLANG) : R_ASMSHADER;
+
+    extern void setupshaders();
+    setupshaders();
 
     static const char * const rpnames[4] = { "fixed-function", "assembly shader", "GLSL shader", "assembly/GLSL shader" };
     conoutf("Rendering using the OpenGL %s path.", rpnames[renderpath]);
@@ -788,6 +851,8 @@ void transplayer()
 }
 
 float curfov = 100, fovy, aspect;
+FVARN(IDF_PERSIST, aspect, forceaspect, 0, 0, 1e3f);
+
 int farplane, xtraverts, xtravertsva;
 
 VAR(IDF_WORLD, fog, 16, 4000, VAR_MAX);
@@ -843,9 +908,13 @@ void readmatrices()
     mvpmatrix.mul(projmatrix, mvmatrix);
     invmvmatrix.invert(mvmatrix);
     invmvpmatrix.invert(mvpmatrix);
+
+    mvmatrix.transposedtransformnormal(vec(viewmatrix.getcolumn(1)), camdir);
+    mvmatrix.transposedtransformnormal(vec(viewmatrix.getcolumn(0)).neg(), camright);
+    mvmatrix.transposedtransformnormal(vec(viewmatrix.getcolumn(2)), camup);
 }
 
-FVAR(0, nearplane, 1e-3f, 0.54f, 1e3f);
+FVAR(0, nearplane, 0.01f, 0.54f, 2.0f);
 
 void project(float fovy, float aspect, int farplane, bool flipx, bool flipy, bool swapxy, float zscale)
 {
@@ -1047,6 +1116,7 @@ void setenvmatrix()
 
 static float findsurface(int fogmat, const vec &v, int &abovemat)
 {
+    fogmat &= MATF_VOLUME;
     ivec o(v), co;
     int csize;
     do
@@ -1055,7 +1125,7 @@ static float findsurface(int fogmat, const vec &v, int &abovemat)
         int mat = c.material&MATF_VOLUME;
         if(mat != fogmat)
         {
-            abovemat = isliquid(mat) ? mat : MAT_AIR;
+            abovemat = isliquid(mat) ? c.material : MAT_AIR;
             return o.z;
         }
         o.z = co.z + csize;
@@ -1067,17 +1137,25 @@ static float findsurface(int fogmat, const vec &v, int &abovemat)
 
 static void blendfog(int fogmat, float blend, float logblend, float &start, float &end, float *fogc)
 {
-    switch(fogmat)
+    switch(fogmat&MATF_VOLUME)
     {
         case MAT_WATER:
-            loopk(3) fogc[k] += blend*watercol[k]/255.0f;
-            end += logblend*min(fog, max(waterfog*4, 32));
+        {
+            const bvec &wcol = getwatercol(fogmat);
+            int wfog = getwaterfog(fogmat);
+            loopk(3) fogc[k] += blend*wcol[k]/255.0f;
+            end += logblend*min(fog, max(wfog*4, 32));
             break;
+        }
 
         case MAT_LAVA:
-            loopk(3) fogc[k] += blend*lavacol[k]/255.0f;
-            end += logblend*min(fog, max(lavafog*4, 32));
+        {
+            const bvec &lcol = getlavacol(fogmat);
+            int lfog = getlavafog(fogmat);
+            loopk(3) fogc[k] += blend*lcol[k]/255.0f;
+            end += logblend*min(fog, max(lfog*4, 32));
             break;
+        }
 
         default:
             loopk(3) fogc[k] += blend*fogcolor[k]/255.0f;
@@ -1102,6 +1180,17 @@ static void setfog(int fogmat, float below = 1, int abovemat = MAT_AIR)
     glClearColor(fogc[0], fogc[1], fogc[2], 1.0f);
 }
 
+bool deferdrawtextures = false;
+
+void drawtextures()
+{
+    extern bool minimized;
+    if(minimized) { deferdrawtextures = true; return; }
+    deferdrawtextures = false;
+    genenvmaps();
+    drawminimap();
+}
+
 GLuint motiontex = 0;
 int motionw = 0, motionh = 0, lastmotion = 0;
 
@@ -1192,17 +1281,23 @@ void invalidatepostfx()
 static void blendfogoverlay(int fogmat, float blend, float *overlay)
 {
     float maxc;
-    switch(fogmat)
+    switch(fogmat&MATF_VOLUME)
     {
         case MAT_WATER:
-            maxc = max(watercol[0], max(watercol[1], watercol[2]));
-            loopk(3) overlay[k] += blend*max(0.4f, watercol[k]/min(32.0f + maxc*7.0f/8.0f, 255.0f));
+        {
+            const bvec &wcol = getwatercol(fogmat);
+            maxc = max(wcol[0], max(wcol[1], wcol[2]));
+            loopk(3) overlay[k] += blend*max(0.4f, wcol[k]/min(32.0f + maxc*7.0f/8.0f, 255.0f));
             break;
+        }
 
         case MAT_LAVA:
-            maxc = max(lavacol[0], max(lavacol[1], lavacol[2]));
-            loopk(3) overlay[k] += blend*max(0.4f, lavacol[k]/min(32.0f + maxc*7.0f/8.0f, 255.0f));
+        {
+            const bvec &lcol = getlavacol(fogmat);
+            maxc = max(lcol[0], max(lcol[1], lcol[2]));
+            loopk(3) overlay[k] += blend*max(0.4f, lcol[k]/min(32.0f + maxc*7.0f/8.0f, 255.0f));
             break;
+        }
 
         default:
             loopk(3) overlay[k] += blend;
@@ -1256,15 +1351,12 @@ void rendergame()
     if(!shadowmapping) renderedgame = true;
 }
 
-void renderavatar(bool early)
+void renderavatar(bool early, bool project)
 {
-    game::renderavatar(early);
+    game::renderavatar(early, project);
 }
 
-extern void viewproject(float zscale = 1);
-
 VAR(IDF_PERSIST, skyboxglare, 0, 1, 1);
-FVAR(0, firstpersondepth, 0, 0.5f, 1);
 
 void drawglare()
 {
@@ -1297,14 +1389,7 @@ void drawglare()
     rendermaterials();
     renderalphageom();
     renderparticles();
-
-    if(game::thirdpersonview()) renderavatar(false);
-    else
-    {
-        viewproject(firstpersondepth);
-        renderavatar(false);
-        viewproject();
-    }
+    renderavatar(false, true);
 
     glFogf(GL_FOG_START, oldfogstart);
     glFogf(GL_FOG_END, oldfogend);
@@ -1319,13 +1404,15 @@ VAR(IDF_WORLD, refractsky, 0, 0, 1);
 
 glmatrixf fogmatrix, invfogmatrix;
 
-void drawreflection(float z, bool refract)
+void drawreflection(float z, bool refract, int fogdepth, const bvec &col)
 {
     reflectz = z < 0 ? 1e16f : z;
     reflecting = !refract;
     refracting = refract ? (z < 0 || camera1->o.z >= z ? -1 : 1) : 0;
     fading = renderpath!=R_FIXEDFUNCTION && waterrefract && waterfade && hasFBO && z>=0;
     fogging = refracting<0 && z>=0;
+    refractfog = fogdepth;
+    refractcolor = fogging ? col : fogcolor;
 
     float oldfogstart, oldfogend, oldfogcolor[4];
     glGetFloatv(GL_FOG_START, &oldfogstart);
@@ -1335,7 +1422,7 @@ void drawreflection(float z, bool refract)
     if(fogging)
     {
         glFogf(GL_FOG_START, camera1->o.z - z);
-        glFogf(GL_FOG_END, camera1->o.z - (z-max(waterfog, 1)));
+        glFogf(GL_FOG_END, camera1->o.z - (z-max(refractfog, 1)));
         GLfloat m[16] =
         {
              1,   0,  0, 0,
@@ -1348,7 +1435,7 @@ void drawreflection(float z, bool refract)
         pushprojection();
         glPushMatrix();
         glLoadMatrixf(fogmatrix.v);
-        float fogc[4] = { watercol.x/255.0f, watercol.y/255.0f, watercol.z/255.0f, 1.0f };
+        float fogc[4] = { col.x/255.0f, col.y/255.0f, col.z/255.0f, 1.0f };
         glFogfv(GL_FOG_COLOR, fogc);
     }
     else
@@ -1430,7 +1517,7 @@ void drawreflection(float z, bool refract)
     renderalphageom(fogging);
     renderparticles();
 
-    if(game::thirdpersonview() || reflecting) renderavatar(false);
+    if(game::thirdpersonview() || reflecting) renderavatar();
 
     if(fading) glColorMask(COLORMASK, GL_TRUE);
 
@@ -1482,14 +1569,14 @@ void drawcubemap(int size, int level, const vec &o, float yaw, float pitch, bool
 
     updatedynlights();
 
-    int fogmat = lookupmaterial(camera1->o)&MATF_VOLUME, abovemat = MAT_AIR;
+    int fogmat = lookupmaterial(camera1->o)&(MATF_VOLUME|MATF_INDEX), abovemat = MAT_AIR;
     float fogblend = 1.0f, causticspass = 0.0f;
-    if(fogmat==MAT_WATER || fogmat==MAT_LAVA)
+    if(isliquid(fogmat&MATF_VOLUME))
     {
         float z = findsurface(fogmat, camera1->o, abovemat) - WATER_OFFSET;
         if(camera1->o.z < z + 1) fogblend = min(z + 1 - camera1->o.z, 1.0f);
         else fogmat = abovemat;
-        if(level > 1 && caustics && fogmat==MAT_WATER && camera1->o.z < z)
+        if(level > 1 && caustics && (fogmat&MATF_VOLUME)==MAT_WATER && camera1->o.z < z)
             causticspass = renderpath==R_FIXEDFUNCTION ? 1.0f : min(z - camera1->o.z, 1.0f);
     }
     else
@@ -1572,69 +1659,76 @@ void drawcubemap(int size, int level, const vec &o, float yaw, float pitch, bool
     envmapping = false;
 }
 
-bool playerpreviewing = false;
+bool modelpreviewing = false;
 
-void renderplayerpreview(int model, int color, int team, int weap, bool background)
+namespace modelpreview
 {
-    float fovy = 90.f, aspect = 1.f;
-    envmapping = playerpreviewing = true;
-
-    physent *oldcamera = camera1;
-    static physent pcamera;
-    pcamera = *camera1;
-    pcamera.reset();
-    pcamera.type = ENT_CAMERA;
-    pcamera.o = vec(0, 0, 0);
-    pcamera.yaw = 0;
-    pcamera.pitch = 0;
-    pcamera.roll = 0;
-    camera1 = &pcamera;
-
+    physent *oldcamera;
     float oldfogstart, oldfogend, oldfogcolor[4];
-    glGetFloatv(GL_FOG_START, &oldfogstart);
-    glGetFloatv(GL_FOG_END, &oldfogend);
-    glGetFloatv(GL_FOG_COLOR, oldfogcolor);
 
-    GLfloat fogc[4] = { 0, 0, 0, 1 };
-    glFogf(GL_FOG_START, 0);
-    glFogf(GL_FOG_END, 1000000);
-    glFogfv(GL_FOG_COLOR, fogc);
-    glClearColor(fogc[0], fogc[1], fogc[2], fogc[3]);
+    physent camera;
 
-    glClear((background ? GL_COLOR_BUFFER_BIT : 0) | GL_DEPTH_BUFFER_BIT);
+    void start(bool background)
+    {
+        float fovy = 90.f, aspect = 1.f;
+        envmapping = modelpreviewing = true;
 
-    glMatrixMode(GL_PROJECTION);
-    glPushMatrix();
-    glMatrixMode(GL_MODELVIEW);
-    glPushMatrix();
+        oldcamera = camera1;
+        camera = *camera1;
+        camera.reset();
+        camera.type = ENT_CAMERA;
+        camera.o = vec(0, 0, 0);
+        camera.yaw = 0;
+        camera.pitch = 0;
+        camera.roll = 0;
+        camera1 = &camera;
 
-    project(fovy, aspect, 1024);
-    transplayer();
-    readmatrices();
-    setenvmatrix();
+        glGetFloatv(GL_FOG_START, &oldfogstart);
+        glGetFloatv(GL_FOG_END, &oldfogend);
+        glGetFloatv(GL_FOG_COLOR, oldfogcolor);
 
-    glEnable(GL_CULL_FACE);
-    glEnable(GL_DEPTH_TEST);
+        GLfloat fogc[4] = { 0, 0, 0, 1 };
+        glFogf(GL_FOG_START, 0);
+        glFogf(GL_FOG_END, 1000000);
+        glFogfv(GL_FOG_COLOR, fogc);
+        glClearColor(fogc[0], fogc[1], fogc[2], fogc[3]);
 
-    game::renderplayerpreview(model, color, team, weap);
+        glClear((background ? GL_COLOR_BUFFER_BIT : 0) | GL_DEPTH_BUFFER_BIT);
 
-    glDisable(GL_CULL_FACE);
-    glDisable(GL_DEPTH_TEST);
+        glMatrixMode(GL_PROJECTION);
+        glPushMatrix();
+        glMatrixMode(GL_MODELVIEW);
+        glPushMatrix();
 
-    defaultshader->set();
+        project(fovy, aspect, 1024);
+        transplayer();
+        readmatrices();
+        setenvmatrix();
 
-    glMatrixMode(GL_PROJECTION);
-    glPopMatrix();
-    glMatrixMode(GL_MODELVIEW);
-    glPopMatrix();
+        glEnable(GL_CULL_FACE);
+        glEnable(GL_DEPTH_TEST);
+    }
 
-    glFogf(GL_FOG_START, oldfogstart);
-    glFogf(GL_FOG_END, oldfogend);
-    glFogfv(GL_FOG_COLOR, oldfogcolor);
-    glClearColor(oldfogcolor[0], oldfogcolor[1], oldfogcolor[2], oldfogcolor[3]);
+    void end()
+    {
+        glDisable(GL_CULL_FACE);
+        glDisable(GL_DEPTH_TEST);
 
-    camera1 = oldcamera;
-    envmapping = playerpreviewing = false;
+        defaultshader->set();
+
+        glMatrixMode(GL_PROJECTION);
+        glPopMatrix();
+        glMatrixMode(GL_MODELVIEW);
+        glPopMatrix();
+
+        glFogf(GL_FOG_START, oldfogstart);
+        glFogf(GL_FOG_END, oldfogend);
+        glFogfv(GL_FOG_COLOR, oldfogcolor);
+        glClearColor(oldfogcolor[0], oldfogcolor[1], oldfogcolor[2], oldfogcolor[3]);
+
+        camera1 = oldcamera;
+        envmapping = modelpreviewing = false;
+    }
 }
 
 bool minimapping = false;
@@ -2070,6 +2164,7 @@ void drawnoviewtype(int targtype)
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
     int w = screen->w, h = screen->h;
+    if(forceaspect) w = int(ceil(h*forceaspect));
     gettextres(w, h);
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
@@ -2236,7 +2331,6 @@ void drawviewtype(int targtype)
     if(limitsky()) drawskybox(farplane, true);
 
     rendergeom(causticspass);
-    extern int outline, blankgeom;
     if(!wireframe && editmode && (outline || (fullbright && blankgeom))) renderoutline();
 
     queryreflections();
@@ -2273,13 +2367,7 @@ void drawviewtype(int targtype)
 
     if(wireframe && editmode) glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
 
-    if(game::thirdpersonview()) renderavatar(false);
-    else
-    {
-        viewproject(0.5f);
-        renderavatar(false);
-        viewproject();
-    }
+    renderavatar(false, true);
 
     if(wireframe && editmode) glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 
@@ -2289,7 +2377,7 @@ void drawviewtype(int targtype)
 
     addmotionblur();
     addglare();
-    if(fogmat==MAT_WATER || fogmat==MAT_LAVA) drawfogoverlay(fogmat, fogblend, abovemat);
+    if(isliquid(fogmat&MATF_VOLUME)) drawfogoverlay(fogmat, fogblend, abovemat);
     renderpostfx();
 
     glDisable(GL_TEXTURE_2D);
@@ -2309,6 +2397,7 @@ void drawviewtype(int targtype)
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
     int w = screen->w, h = screen->h;
+    if(forceaspect) w = int(ceil(h*forceaspect));
     gettextres(w, h);
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
@@ -2339,7 +2428,7 @@ void drawviewtype(int targtype)
     glEnable(GL_TEXTURE_2D);
     defaultshader->set();
     hud::drawhud();
-    render_texture_panel(w, h);
+    rendertexturepanel(w, h);
     hud::drawlast();
     glDisable(GL_TEXTURE_2D);
 
@@ -2365,14 +2454,16 @@ void gl_drawframe(int w, int h)
     if(hasnoview()) drawnoview();
     else
     {
-        fogmat = lookupmaterial(camera1->o)&MATF_VOLUME;
+        if(deferdrawtextures) drawtextures();
+
+        fogmat = lookupmaterial(camera1->o)&(MATF_VOLUME|MATF_INDEX);
         causticspass = 0.f;
-        if(fogmat == MAT_WATER || fogmat == MAT_LAVA)
+        if(isliquid(fogmat&MATF_VOLUME))
         {
             float z = findsurface(fogmat, camera1->o, abovemat) - WATER_OFFSET;
             if(camera1->o.z < z + 1) fogblend = min(z + 1 - camera1->o.z, 1.0f);
             else fogmat = abovemat;
-            if(caustics && fogmat == MAT_WATER && camera1->o.z < z)
+            if(caustics && (fogmat&MATF_VOLUME) == MAT_WATER && camera1->o.z < z)
                 causticspass = renderpath==R_FIXEDFUNCTION ? 1.0f : min(z - camera1->o.z, 1.0f);
 
             float blend = abovemat == MAT_AIR ? fogblend : 1.0f;
