@@ -127,16 +127,17 @@ void mdldepthoffset(int *offset)
 
 COMMAND(0, mdldepthoffset, "i");
 
-void mdlglow(int *percent)
+void mdlglow(int *percent, int *delta, float *pulse)
 {
     checkmdl;
-    float glow = 3.0f;
+    float glow = 3.0f, glowdelta = *delta/100.0f, glowpulse = *pulse > 0 ? *pulse/1000.0f : 0;
     if(*percent>0) glow = *percent/100.0f;
     else if(*percent<0) glow = 0.0f;
-    loadingmodel->setglow(glow);
+    glowdelta -= glow;
+    loadingmodel->setglow(glow, glowdelta, glowpulse);
 }
 
-COMMAND(0, mdlglow, "i");
+COMMAND(0, mdlglow, "iif");
 
 void mdlglare(float *specglare, float *glowglare)
 {
@@ -170,13 +171,15 @@ void mdlshader(const char *shader)
 
 COMMAND(0, mdlshader, "s");
 
-void mdlspin(float *rate)
+void mdlspin(float *yaw, float *pitch, float *roll)
 {
     checkmdl;
-    loadingmodel->spin = *rate;
+    loadingmodel->spinyaw = *yaw;
+    loadingmodel->spinpitch = *pitch;
+    loadingmodel->spinroll = *roll;
 }
 
-COMMAND(0, mdlspin, "f");
+COMMAND(0, mdlspin, "fff");
 
 void mdlscale(int *percent)
 {
@@ -300,18 +303,18 @@ void rdtri(int *v1, int *v2, int *v3)
 }
 COMMAND(0, rdtri, "iii");
 
-void rdjoint(int *n, int *t, const char *v1, const char *v2, const char *v3)
+void rdjoint(int *n, int *t, int *v1, int *v2, int *v3)
 {
     checkragdoll;
     if(*n < 0 || *n >= skel->numbones) return;
     ragdollskel::joint &j = ragdoll->joints.add();
     j.bone = *n;
     j.tri = *t;
-    j.vert[0] = v1[0] ? parseint(v1) : -1;
-    j.vert[1] = v2[0] ? parseint(v2) : -1;
-    j.vert[2] = v3[0] ? parseint(v3) : -1;
+    j.vert[0] = *v1;
+    j.vert[1] = *v2;
+    j.vert[2] = *v3;
 }
-COMMAND(0, rdjoint, "iisss");
+COMMAND(0, rdjoint, "iibbb");
 
 void rdlimitdist(int *v1, int *v2, float *mindist, float *maxdist)
 {
@@ -377,6 +380,29 @@ ICOMMAND(0, mapmodelindex, "s", (char *a), {
 // model registry
 
 hashtable<const char *, model *> mdllookup;
+vector<const char *> preloadmodels;
+
+void preloadmodel(const char *name)
+{
+    if(!name || !name[0] || mdllookup.access(name)) return;
+    preloadmodels.add(newstring(name));
+}
+
+void flushpreloadedmodels(bool msg)
+{
+    loopv(preloadmodels)
+    {
+        loadprogress = float(i+1)/preloadmodels.length();
+        model *m = loadmodel(preloadmodels[i], -1, msg);
+        if(!m) { if(msg) conoutf("\frcould not load model: %s", preloadmodels[i]); }
+        else
+        {
+            m->preloadmeshes();
+        }
+    }
+    preloadmodels.deletearrays();
+    loadprogress = 0;
+}
 
 void preloadusedmapmodels(bool msg, bool bih)
 {
@@ -396,10 +422,13 @@ void preloadusedmapmodels(bool msg, bool bih)
         loadprogress = float(i+1)/mapmodels.length();
         int mmindex = mapmodels[i];
         mapmodelinfo *mmi = getmminfo(mmindex);
-        if(!mmi) conoutf("\frcould not find map model: %d", mmindex);
-        else if(!loadmodel(NULL, mmindex, true))
-            conoutf("\frcould not load model: %s", mmi->name);
-        else if(mmi->m && bih) mmi->m->preloadBIH();
+        if(!mmi) { if(msg) conoutf("\frcould not find map model: %d", mmindex); }
+        else if(!loadmodel(NULL, mmindex, true)) { if(msg) conoutf("\frcould not load model: %s", mmi->name); }
+        else if(mmi->m)
+        {
+            if(bih) mmi->m->preloadBIH();
+            mmi->m->preloadmeshes();
+        }
     }
     loadprogress = 0;
 }
@@ -418,7 +447,7 @@ model *loadmodel(const char *name, int i, bool msg)
     if(mm) m = *mm;
     else
     {
-        if(lightmapping > 1) return NULL;
+        if(loadingmodel || lightmapping > 1) return NULL;
         if(msg)
         {
             defformatstring(str)("models/%s", name);
@@ -435,6 +464,7 @@ model *loadmodel(const char *name, int i, bool msg)
         loadingmodel = NULL;
         if(!m) return NULL;
         mdllookup.access(m->name(), m);
+        m->preloadshaders();
     }
     if(mapmodels.inrange(i) && !mapmodels[i].m) mapmodels[i].m = m;
     return m;
@@ -737,9 +767,36 @@ void rendermodelquery(model *m, dynent *d, const vec &center, float radius)
 
 extern int oqfrags;
 
+void renderradius(const vec &o, float radius)
+{
+    glColor3f(0.5f, 0.5f, 0.5f);
+    loopk(3)
+    {
+        glBegin(GL_LINE_LOOP);
+        loopi(16)
+        {
+            vec d = o;
+            switch(k)
+            {
+                case 0:
+                    d.add(vec(radius*cosf(2*M_PI*i/16.0f), radius*sinf(2*M_PI*i/16.0f), 0).rotate_around_x(90*RAD));
+                    break;
+                case 1:
+                    d.add(vec(radius*cosf(2*M_PI*i/16.0f), radius*sinf(2*M_PI*i/16.0f), 0).rotate_around_y(-90*RAD));
+                    break;
+                case 2: default:
+                    d.add(vec(radius*cosf(2*M_PI*i/16.0f), radius*sinf(2*M_PI*i/16.0f), 0).rotate_around_z(90*RAD));
+                    break;
+            }
+            glVertex3fv(d.v);
+        }
+        glEnd();
+    }
+}
+
 void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, float yaw, float pitch, float roll, int flags, dynent *d, modelattach *a, int basetime, int basetime2, float trans, float size)
 {
-    if(shadowmapping && !(flags&(MDL_SHADOW|MDL_DYNSHADOW))) return;
+    if((shadowmapping && !(flags&(MDL_SHADOW|MDL_DYNSHADOW))) || trans <= 0 || size <= 0) return;
     model *m = loadmodel(mdl);
     if(!m) return;
     vec center(0, 0, 0), bbradius(0, 0, 0);
@@ -774,7 +831,7 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
                 }
                 else
                 {
-                    if(fogging && center.z+radius<reflectz-waterfog) return;
+                    if(fogging && center.z+radius<reflectz-refractfog) return;
                     if(!shadow && center.z-radius>=reflectz) return;
                 }
                 if(center.dist(camera1->o)-radius>reflectdist) return;
@@ -821,7 +878,22 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
         glEnable(GL_BLEND);
         glBlendFunc(GL_ONE, GL_ONE);
 
-        if(d && showboundingbox==1) physics::complexboundbox(d);
+        if(d && showboundingbox==1)
+        {
+            if(d->ragdoll && (d->state == CS_DEAD || d->state == CS_WAITING))
+            {
+                loopv(d->ragdoll->skel->verts) renderradius(d->ragdoll->verts[i].pos, d->ragdoll->skel->verts[i].radius);
+                loopv(d->ragdoll->skel->tris)
+                {
+                    ragdollskel::tri &t = d->ragdoll->skel->tris[i];
+                    glColor3f(0.5f, 1, 1);
+                    glBegin(GL_LINE_LOOP);
+                    loopk(3) glVertex3fv(d->ragdoll->verts[t.vert[k]].pos.v);
+                    glEnd();
+                }
+            }
+            else physics::complexboundbox(d);
+        }
         else
         {
             vec center, radius;
@@ -832,6 +904,7 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
             glTranslatef(o.x, o.y, o.z);
             glRotatef(yaw, 0, 0, 1);
             glRotatef(-roll, 1, 0, 0);
+            glRotatef(-pitch, 0, 1, 0);
             render3dbox(center, radius.z, radius.z, radius.x, radius.y);
         }
 
