@@ -16,7 +16,7 @@ VAR(0, debugparticles, 0, 0, 1);
 
 // Check emit_particles() to limit the rate that paricles can be emitted for models/sparklies
 // Automatically stops particles being emitted when paused or in reflective drawing
-VAR(IDF_PERSIST, emitmillis, 0, 15, VAR_MAX);
+VAR(IDF_PERSIST, emitmillis, 1, 15, VAR_MAX);
 static int lastemitframe = 0;
 static bool emit = false;
 
@@ -104,9 +104,16 @@ struct partrenderer
                 if(ts > p->fade) ts = p->fade;
                 float secs = curtime/1000.f;
                 vec v = vec(p->d).mul(secs);
-                static physent dummy;
-                dummy.weight = weight;
-                v.z -= physics::gravityvel(&dummy)*secs;
+                static struct particleent : physent
+                {
+                    particleent()
+                    {
+                        physent::reset();
+                        type = ENT_DUMMY;
+                    }
+                } d;
+                d.weight = weight;
+                v.z -= physics::gravityvel(&d)*secs;
                 p->o.add(v);
             }
             if(p->collide && p->o.z < p->val && lastpass)
@@ -896,7 +903,7 @@ struct loopprimitiverenderer : listrenderer<loopprimitive>
                     v = vec(p->value.x*cosf(2*M_PI*i/16.0f), p->value.z*sinf(2*M_PI*i/16.0f), 0).rotate_around_x(90*RAD);
                     break;
                 case 1:
-                    v = vec(p->value.z*cosf(2*M_PI*i/16.0f), p->value.y*sinf(2*M_PI*i/16.0f), 0).rotate_around_y(90*RAD);
+                    v = vec(p->value.z*cosf(2*M_PI*i/16.0f), p->value.y*sinf(2*M_PI*i/16.0f), 0).rotate_around_y(-90*RAD);
                     break;
                 case 2: default:
                     v = vec(p->value.x*cosf(2*M_PI*i/16.0f), p->value.y*sinf(2*M_PI*i/16.0f), 0).rotate_around_z(90*RAD);
@@ -1612,41 +1619,6 @@ void regularflame(int type, const vec &p, float radius, float height, int color,
     }
 }
 
-#if 0
-vector<mapparticle> mapparts;
-
-int addmapparticle(const char *name, int part, int type, int colour, int grav, int fade, float radius, float height, int attr, float size, float blend, float vel)
-{
-    if(!name || !*name || part < 0 || part >= PART_MAX) return -1;
-    int idx = -1;
-    loopv(mapparts) if(!strcasecmp(name, mapparts[i].name)) return i;
-    idx = mapparts.length();
-    mapparticle &p = mapparts.add();
-    p.name = newstring(name);
-    p.part = part;
-    p.type = type;
-    p.colour = colour;
-    p.grav = grav;
-    p.fade = fade;
-    p.radius = radius;
-    p.height = height;
-    p.attr = attr;
-    p.size = size;
-    p.blend = blend;
-    p.vel = vel;
-}
-ICOMMAND(0, mapparticle, "siiiiiffifff", (const char *name, int *part, int *type, int *colour, int *grav, int *fade, float *radius, float *height, int *attr, float *size, float *blend, float *vel),
-    intret(addmapparticle(name, *part, *type, *colour, *grav, *fade, *radius, *height, *attr, *size, *blend, *vel)));
-
-void defaultparticles()
-{
-    if(mapparts.length()) mapparts.setsize(0);
-    addmapparticle("fire", PART_FLAME, PARTTYPE_FIRE, 0x903020, -10, 500, 1.5f, 0.5f, 0, 2.f, 1, 200);
-    addmapparticle("smoke", PART_SMOKE, PARTTYPE_SPLASH, 0x897661, -20, 200, 2.f, 0.f, 3, 2.4f, 0.f, 1, 1);
-    addmapparticle("water", PART_SPARK, PARTTYPE_SPLASH, (int(watercol[0])<<16) | (int(watercol[1])<<8) | int(watercol[2]), 10, 500, 10.f, 0.f, 20, 0.6f, 1, 4);
-}
-#endif
-
 static int partcolour(int c, int p, int x)
 {
     if(p || x)
@@ -1679,7 +1651,14 @@ void makeparticle(const vec &o, attrvector &attr)
             break;
         case 2: //water fountain - <dir>
         {
-            int color = (int(watercol[0])<<16) | (int(watercol[1])<<8) | int(watercol[2]);
+            int mat = MAT_WATER + clamp(-attr[2], 0, 3);
+            const bvec &wfcol = getwaterfallcol(mat);
+            int color = (int(wfcol[0])<<16) | (int(wfcol[1])<<8) | int(wfcol[2]);
+            if(!color)
+            {
+                const bvec &wcol = getwatercol(mat);
+                color = (int(wcol[0])<<16) | (int(wcol[1])<<8) | int(wcol[2]);
+            }
             regularsplash(PART_SPARK, color, 10, 4, 200, offsetvec(o, attr[1], rnd(10)), 0.6f, 1, 20);
             break;
         }
@@ -1740,7 +1719,7 @@ void makeparticle(const vec &o, attrvector &attr)
         case 33:
         case 34:
         case 35:
-            flares.addflare(o, attr[1], attr[2], attr[3], (attr[0]&0x02)!=0, (attr[0]&0x01)!=0);
+            flares.addflare(o, attr[1], attr[2], attr[3], (attr[0]&2)!=0, false, (attr[0]&1)!=0 ? ((attr[0]&2)!=0 ? 1 : 2) : 0);
             break;
         default:
             defformatstring(ds)("%d?", attr[0]);
