@@ -1,6 +1,9 @@
 #include "engine.h"
 
-VAR(IDF_PERSIST, blinkingtext, 0, 250, VAR_MAX);
+VAR(IDF_PERSIST, textblinking, 0, 350, VAR_MAX);
+FVARF(IDF_PERSIST, textscale, FVAR_NONZERO, 1, FVAR_MAX, UI::setup());
+VAR(IDF_PERSIST, textfaded, 0, 1, 1);
+VAR(IDF_PERSIST, textminintensity, 0, 32, 255);
 
 static inline bool htcmp(const char *key, const font &f) { return !strcmp(key, f.name); }
 
@@ -19,9 +22,8 @@ void newfont(char *name, char *tex, int *defaultw, int *defaulth)
     f->texs.add(textureload(tex));
     f->chars.shrink(0);
     f->charoffset = '!';
-    f->defaultw = *defaultw;
-    f->defaulth = *defaulth;
-    f->scale = f->defaulth;
+    f->maxw = f->defaultw = *defaultw;
+    f->maxh = f->defaulth = f->scale = *defaulth;
 
     fontdef = f;
     fontdeftex = 0;
@@ -62,6 +64,8 @@ void fontchar(int *x, int *y, int *w, int *h, int *offsetx, int *offsety, int *a
     c.h = *h ? *h : fontdef->defaulth;
     c.offsetx = *offsetx;
     c.offsety = *offsety;
+    if(c.offsetx+c.w > fontdef->maxw) fontdef->maxw = c.offsetx+c.w;
+    if(c.offsety+c.h > fontdef->maxh) fontdef->maxh = c.offsety+c.h;
     c.advance = *advance ? *advance : c.offsetx + c.w;
     c.tex = fontdeftex;
 }
@@ -106,6 +110,8 @@ void fontalias(const char *dst, const char *src)
     d->defaultw = s->defaultw;
     d->defaulth = s->defaulth;
     d->scale = s->scale;
+    d->maxw = s->maxw;
+    d->maxh = s->maxh;
 
     fontdef = d;
     fontdeftex = d->texs.length()-1;
@@ -128,8 +134,24 @@ float text_widthf(const char *str, int flags)
     return width;
 }
 ICOMMAND(0, textwidth, "si", (char *s, int *f), floatret(text_widthf(s, *f)));
-ICOMMAND(0, fontwidth, "", (), intret(FONTW));
-ICOMMAND(0, fontheight, "", (), intret(FONTH));
+
+int text_fontw(const char *s)
+{
+    if(s && *s) pushfont(s);
+    int w = FONTW;
+    if(s && *s) popfont();
+    return w;
+}
+ICOMMAND(0, fontwidth, "s", (char *s), intret(text_fontw(s)));
+
+int text_fonth(const char *s)
+{
+    if(s && *s) pushfont(s);
+    int h = FONTH;
+    if(s && *s) popfont();
+    return h;
+}
+ICOMMAND(0, fontheight, "s", (char *s), intret(text_fonth(s)));
 
 #define TEXTTAB(x) (max((int((x)/FONTTAB)+1.0f)*FONTTAB, (x) + FONTW))
 
@@ -180,52 +202,55 @@ static float draw_char(Texture *&tex, int c, float x, float y, float scale)
     return scale*info.advance;
 }
 
-static void text_color(char c, char *stack, int size, int &sp, bvec &color, int r, int g, int b, int a)
+#define TVECA(ci, ca) (cvec::from24c(ci).mul(255-textminintensity).div(255).add(textminintensity).min(255).alpha(ca))
+#define TVECX(cr, cg, cb, ca) (cvec(cr, cg, cb).mul(255-textminintensity).div(255).add(textminintensity).min(255).alpha(ca))
+
+static void text_color(char c, cvec *stack, int size, int &sp, cvec &color, int r, int g, int b, int a)
 {
-    char d = c;
-    if(d=='s') // save color
+    int alpha = stack[sp].a;
+    switch(c)
     {
-        d = stack[sp];
-        if(sp<size-1) stack[++sp] = d;
-    }
-    else
-    {
-        if(d=='S') d = stack[(sp > 0) ? --sp : sp]; // restore color
-        else stack[sp] = d;
+        case 'g': case '0': stack[sp] = color = TVECX( 64, 255,  64, alpha); break; // green
+        case 'b': case '1': stack[sp] = color = TVECX(86,  92,  255, alpha); break; // blue
+        case 'y': case '2': stack[sp] = color = TVECX(255, 255,   0, alpha); break; // yellow
+        case 'r': case '3': stack[sp] = color = TVECX(255,  64,  64, alpha); break; // red
+        case 'a': case '4': stack[sp] = color = TVECX(192, 192, 192, alpha); break; // grey
+        case 'm': case '5': stack[sp] = color = TVECX(255, 186, 255, alpha); break; // magenta
+        case 'o': case '6': stack[sp] = color = TVECX(255,  64,   0, alpha); break; // orange
+        case 'w': case '7': stack[sp] = color = TVECX(255, 255, 255, alpha); break; // white
+        case 'k': case '8': stack[sp] = color = TVECX(0,     0,   0, alpha); break; // black
+        case 'c': case '9': stack[sp] = color = TVECX(64,  255, 255, alpha); break; // cyan
+        case 'v': stack[sp] = color = TVECX(192,  96, 255, alpha); break; // violet
+        case 'p': stack[sp] = color = TVECX(224,  64, 224, alpha); break; // purple
+        case 'n': stack[sp] = color = TVECX(164,  72,  56, alpha); break; // brown
+        case 'G': stack[sp] = color = TVECX( 86, 164,  56, alpha); break; // dark green
+        case 'B': stack[sp] = color = TVECX( 56,  64, 172, alpha); break; // dark blue
+        case 'Y': stack[sp] = color = TVECX(172, 172,   0, alpha); break; // dark yellow
+        case 'R': stack[sp] = color = TVECX(172,  56,  56, alpha); break; // dark red
+        case 'M': stack[sp] = color = TVECX(172,  72, 172, alpha); break; // dark magenta
+        case 'O': stack[sp] = color = TVECX(172,  56,   0, alpha); break; // dark orange
+        case 'C': stack[sp] = color = TVECX(48,  172, 172, alpha); break; // dark cyan
+        case 'A': case 'd': stack[sp] = color = TVECX(102, 102, 102, alpha); break; // dark grey
+        case 'P': stack[sp] = color = TVECX(255, 128, 128, alpha); break; // pink
+        case 'e': case 'E': (stack[sp] = color = stack[sp]).alpha(c != 'E' ? a/2 : a/4); break;
+        case 'u': case 'Z': stack[sp] = color = TVECX(r, g, b, a); break; // default colour
+        case 's': // save
+        {
+            if(sp < size-1) stack[++sp] = color;
+            break;
+        }
+        case 'S': // restore
+        {
+            color = stack[sp > 0 ? --sp : sp];
+            break;
+        }
+        default: color = stack[sp]; break; // everything else
     }
     xtraverts += varray::end();
-    int f = a;
-    switch(d)
-    {
-        case 'g': case '0': color = bvec( 64, 255,  64); break; // green
-        case 'b': case '1': color = bvec(86,  92,  255); break; // blue
-        case 'y': case '2': color = bvec(255, 255,   0); break; // yellow
-        case 'r': case '3': color = bvec(255,  64,  64); break; // red
-        case 'a': case '4': color = bvec(192, 192, 192); break; // grey
-        case 'm': case '5': color = bvec(255, 186, 255); break; // magenta
-        case 'o': case '6': color = bvec(255,  64,   0); break; // orange
-        case 'w': case '7': color = bvec(255, 255, 255); break; // white
-        case 'k': case '8': color = bvec(0,     0,   0); break; // black
-        case 'c': case '9': color = bvec(64,  255, 255); break; // cyan
-        case 'v': color = bvec(192,  96, 255); break; // violet
-        case 'p': color = bvec(224,  64, 224); break; // purple
-        case 'n': color = bvec(164,  72,  56); break; // brown
-        case 'G': color = bvec( 86, 164,  56); break; // dark green
-        case 'B': color = bvec( 56,  64, 172); break; // dark blue
-        case 'Y': color = bvec(172, 172,   0); break; // dark yellow
-        case 'R': color = bvec(172,  56,  56); break; // dark red
-        case 'M': color = bvec(172,  72, 172); break; // dark magenta
-        case 'O': color = bvec(172,  56,   0); break; // dark orange
-        case 'C': color = bvec(48,  172, 172); break; // dark cyan
-        case 'A': case 'd': color = bvec(102, 102, 102); break; // dark grey
-        case 'e': case 'E': f -= d!='E' ? f/2 : f/4; break;
-        case 'u': color = bvec(r, g, b); break; // user colour
-        case 'Z': default: break; // everything else
-    }
-    glColor4ub(color.x, color.y, color.z, f);
+    glColor4ub((uchar)color.r, (uchar)color.g, (uchar)color.b, (uchar)color.a);
 }
 
-#define FONTX ((FONTH*9)/8)
+#define FONTX int(curfont->maxh*textscale)
 static const char *gettexvar(const char *var)
 {
     ident *id = getident(var);
@@ -242,8 +267,10 @@ static const char *gettexvar(const char *var)
 static float draw_icon(Texture *&tex, const char *name, float x, float y, float scale)
 {
     if(!name && !*name) return 0;
-    if(*name == '$') name = gettexvar(++name);
-    Texture *t = textureload(name, 3);
+    const char *file = name;
+    if(*file == '$') file = gettexvar(++file);
+    if(!*file) { conoutf("invalid texture: %s (%s)", name, file); return 0; }
+    Texture *t = textureload(file, 3);
     if(!t) return 0;
     if(tex != t)
     {
@@ -261,70 +288,76 @@ static float draw_icon(Texture *&tex, const char *name, float x, float y, float
 
 static float icon_width(const char *name, float scale)
 {
-    if(*name == '$') name = gettexvar(++name);
-    Texture *t = textureload(name, 3);
-    if(t) return (t->w*scale*FONTX)/t->h;
-    return 0;
+    if(!name && !*name) return 0;
+    const char *file = name;
+    if(*file == '$') file = gettexvar(++file);
+    if(!*file) { conoutf("invalid texture: %s (%s)", name, file); return 0; }
+    Texture *t = textureload(file, 3);
+    if(!t) return 0;
+    return (t->w*scale*FONTX)/t->h;
 }
 
-#define TEXTCOLORIZE(g,h) \
+#define TEXTCOLORIZE(h,s) \
 { \
-    if(g[h] == 'z' && g[h+1]) \
+    if(str[h] == 'z' && str[h+1]) \
     { \
         h++; \
-        bool alt = blinkingtext && totalmillis%(blinkingtext*2) > blinkingtext; \
-        TEXTCOLOR(h); \
-        if(g[h+1]) \
+        bool alt = textblinking && totalmillis%(textblinking*2) > textblinking; \
+        if(s) TEXTCOLOR(h); \
+        if(str[h+1]) \
         { \
             h++; \
-            if(alt) TEXTCOLOR(h); \
+            if(s && alt) TEXTCOLOR(h); \
         } \
     } \
-    else if(g[h] == '[') \
+    else if(str[h] == '[') \
     { \
         h++; \
-        const char *start = &g[h]; \
+        const char *start = &str[h]; \
         const char *end = strchr(start, ']'); \
         if(end) \
         { \
-            if(end > start) { TEXTHEXCOLOR(bvec(parseint(start))); } \
+            if(s && end > start) { TEXTHEXCOLOR(parseint(start)); } \
             h += end-start; \
         } \
         else break; \
     } \
-    else if(g[h] == '(') \
+    else if(str[h] == '(') \
     { \
         h++; \
-        const char *start = &g[h]; \
+        const char *start = &str[h]; \
         const char *end = strchr(start, ')'); \
         if(end) \
         { \
-            if(end > start) \
+            if(s && end > start) \
             { \
-                string value; copystring(value, start, min(size_t(end - start + 1), sizeof(value))); \
+                string value; \
+                copystring(value, start, min(size_t(end - start + 1), sizeof(value))); \
                 TEXTICON(value); \
             } \
             h += end-start; \
         } \
         else break; \
     } \
-    else TEXTCOLOR(h); \
+    else if(s) TEXTCOLOR(h); \
 }
 #define TEXTALIGN \
+{ \
     x = (!(flags&TEXT_RIGHT_JUSTIFY) && !(flags&TEXT_NO_INDENT) ? FONTTAB : 0); \
     if(!y && (flags&TEXT_RIGHT_JUSTIFY) && !(flags&TEXT_NO_INDENT)) maxwidth -= FONTTAB; \
-    y += FONTH;
+    y += FONTH; \
+}
 #define TEXTSKELETON \
-    float y = 0, x = 0, scale = curfont->scale/float(curfont->defaulth);\
-    int i;\
+    float y = 0, x = 0, scale = curfont->scale/float(curfont->defaulth)*textscale;\
+    int i = 0;\
     for(i = 0; str[i]; i++)\
     {\
         int c = uchar(str[i]);\
         TEXTINDEX(i)\
-        if(c=='\t')      { x = TEXTTAB(x); TEXTWHITE(i) }\
-        else if(c==' ')  { x += scale*curfont->defaultw; TEXTWHITE(i) }\
-        else if(c=='\n') { TEXTLINE(i) TEXTALIGN }\
-        else if(c=='\f') { if(str[i+1]) { i++; TEXTCOLORIZE(str, i); } }\
+        if(c == '\t')      { x = TEXTTAB(x); TEXTWHITE(i) }\
+        else if(c == ' ')  { x += scale*curfont->defaultw*textscale; TEXTWHITE(i) }\
+        else if(c == '\n') { TEXTLINE(i) TEXTALIGN }\
+        else if(c == '\f') { if(str[i+1]) { i++; TEXTCOLORIZE(i, true); } }\
         else if(curfont->chars.inrange(c-curfont->charoffset))\
         {\
             float cw = scale*(curfont->chars[c-curfont->charoffset].advance);\
@@ -336,14 +369,14 @@ static float icon_width(const char *name, float scale)
                 for(; str[i+1]; i++)\
                 {\
                     int c = uchar(str[i+1]);\
-                    if(c=='\f') { if(str[i+2]) i++; continue; }\
+                    if(c == '\f') { if(str[i+2]) { i += 2; TEXTCOLORIZE(i, false); } }\
                     if(i-j > 16) break;\
                     if(!curfont->chars.inrange(c-curfont->charoffset)) break;\
-                    float cw = scale*curfont->chars[c-curfont->charoffset].advance;\
-                    if(cw <= 0 || w + cw > maxwidth) break; \
+                    cw = scale*curfont->chars[c-curfont->charoffset].advance;\
+                    if(cw <= 0 || w+cw > maxwidth) break; \
                     w += cw;\
                 }\
-                if(x + w > maxwidth && j!=0) { TEXTLINE(j-1) TEXTALIGN }\
+                if(x+w > maxwidth && j != 0) { TEXTLINE(j-1) TEXTALIGN }\
                 TEXTWORD\
             }\
             else { TEXTCHAR(i) }\
@@ -352,13 +385,13 @@ static float icon_width(const char *name, float scale)
 
 //all the chars are guaranteed to be either drawable or color commands
 #define TEXTWORDSKELETON \
-                for(; j <= i; j++)\
-                {\
-                    TEXTINDEX(j)\
-                    int c = uchar(str[j]);\
-                    if(c=='\f') { if(str[j+1]) { j++; TEXTCOLORIZE(str, j); } }\
-                    else { float cw = scale*curfont->chars[c-curfont->charoffset].advance; TEXTCHAR(j) }\
-                }
+    for(; j <= i; j++) \
+    {\
+        TEXTINDEX(j)\
+        int c = uchar(str[j]);\
+        if(c == '\f') { if(str[j+1]) { j++; TEXTCOLORIZE(j, true); } }\
+        else { float cw = scale*curfont->chars[c-curfont->charoffset].advance; TEXTCHAR(j) }\
+    }
 
 #define TEXTEND(cursor) if(cursor >= i) { do { TEXTINDEX(cursor); } while(0); }
 
@@ -392,7 +425,7 @@ void text_posf(const char *str, int cursor, float &cx, float &cy, int maxwidth,
     #define TEXTLINE(idx)
     #define TEXTCOLOR(idx)
     #define TEXTHEXCOLOR(ret)
-    #define TEXTICON(ret) x += icon_width(ret, scale);
+    #define TEXTICON(ret) x += icon_width(ret, scale)+2;
     #define TEXTCHAR(idx) x += cw;
     #define TEXTWORD TEXTWORDSKELETON if(i >= cursor) break;
     cx = cy = 0;
@@ -415,7 +448,7 @@ void text_boundsf(const char *str, float &width, float &height, int maxwidth, in
     #define TEXTLINE(idx) if(x > width) width = x;
     #define TEXTCOLOR(idx)
     #define TEXTHEXCOLOR(ret)
-    #define TEXTICON(ret) x += icon_width(ret, scale);
+    #define TEXTICON(ret) x += icon_width(ret, scale)+2;
     #define TEXTCHAR(idx) x += cw;
     #define TEXTWORD TEXTWORDSKELETON
     width = 0;
@@ -434,12 +467,30 @@ void text_boundsf(const char *str, float &width, float &height, int maxwidth, in
 
 int draw_text(const char *str, int rleft, int rtop, int r, int g, int b, int a, int flags, int cursor, int maxwidth)
 {
-    #define TEXTINDEX(idx) if(cursor == idx) { cx = x; cy = y; }
+    #define TEXTINDEX(idx) \
+        if(cursor >= 0 && idx == cursor) \
+        { \
+            cx = x; \
+            cy = y; \
+            if(!hasfade && usecolor && textfaded) \
+            { \
+                text_color('e', colorstack, sizeof(colorstack), colorpos, color, r, g, b, fade); \
+                hasfade = true; \
+            } \
+        }
     #define TEXTWHITE(idx)
     #define TEXTLINE(idx) ly += FONTH;
-    #define TEXTCOLOR(idx) if(usecolor) text_color(str[idx], colorstack, sizeof(colorstack), colorpos, color, r, g, b, a);
-    #define TEXTHEXCOLOR(ret) if(usecolor) { xtraverts += varray::end(); color = ret; glColor4ub(color.x, color.y, color.z, a); }
-    #define TEXTICON(ret) { x += draw_icon(tex, ret, left+x, top+y, scale); }
+    #define TEXTCOLOR(idx) if(usecolor) { text_color(str[idx], colorstack, sizeof(colorstack), colorpos, color, r, g, b, fade); }
+    #define TEXTHEXCOLOR(ret) \
+        if(usecolor) \
+        { \
+            int alpha = colorstack[colorpos].a; \
+            color = TVECA(ret, alpha); \
+            colorstack[colorpos] = color; \
+            xtraverts += varray::end(); \
+            glColor4ub((uchar)color.r, (uchar)color.g, (uchar)color.b, (char)color.a); \
+        }
+    #define TEXTICON(ret) x += draw_icon(tex, ret, left+x, top+y, scale);
     #define TEXTCHAR(idx) { draw_char(tex, c, left+x, top+y, scale); x += cw; }
     #define TEXTWORD TEXTWORDSKELETON
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -449,14 +500,14 @@ int draw_text(const char *str, int rleft, int rtop, int r, int g, int b, int a,
     varray::defattrib(varray::ATTRIB_VERTEX, 2, GL_FLOAT);
     varray::defattrib(varray::ATTRIB_TEXCOORD0, 2, GL_FLOAT);
     varray::begin(GL_QUADS);
-    bool usecolor = true;
-    if(a < 0) { usecolor = false; a = -a; }
-    int colorpos = 0, ly = 0, left = rleft, top = rtop;
+    int fade = a;
+    bool usecolor = true, hasfade = false;
+    if(fade < 0) { usecolor = false; fade = -a; }
+    int colorpos = 1, ly = 0, left = rleft, top = rtop;
     float cx = -FONTW, cy = 0;
-    char colorstack[10];
-    memset(colorstack, 'u', sizeof(colorstack)); //indicate user color
-    bvec color(r, g, b);
-    glColor4ub(color.x, color.y, color.z, a);
+    cvec colorstack[16], color = TVECX(r, g, b, fade);
+    loopi(16) colorstack[i] = color;
+    glColor4ub((uchar)color.r, (uchar)color.g, (uchar)color.b, (uchar)color.a);
     TEXTSKELETON
     TEXTEND(cursor)
     xtraverts += varray::end();
