@@ -283,6 +283,7 @@ void clearqueries()
 }
 
 VAR(0, oqfrags, 0, 8, 64);
+VAR(0, oqwait, 0, 1, 1);
 
 bool checkquery(occludequery *query, bool nowait)
 {
@@ -290,7 +291,7 @@ bool checkquery(occludequery *query, bool nowait)
     if(query->fragments >= 0) fragments = query->fragments;
     else
     {
-        if(nowait)
+        if(nowait || !oqwait)
         {
             GLint avail;
             glGetQueryObjectiv_(query->id, GL_QUERY_RESULT_AVAILABLE, &avail);
@@ -315,7 +316,7 @@ void drawbb(const ivec &bo, const ivec &br, const vec &camera)
         else if(camera[dim] > bo[dim]) continue; \
         v0 v1 v2 v3 \
         xtraverts += 4; \
-    } while(0); 
+    } while(0);
     #define GENFACEVERT(orient, vert, ox,oy,oz, rx,ry,rz) \
         glVertex3f(ox rx, oy ry, oz rz);
     GENFACEVERTS(bo.x, bo.x + br.x, bo.y, bo.y + br.y, bo.z, bo.z + br.z, , , , , , )
@@ -360,7 +361,7 @@ void findvisiblemms(const vector<extentity *> &ents)
                 loopv(oe->mapmodels)
                 {
                     extentity &e = *ents[oe->mapmodels[i]];
-                    if(e.lastemit && e.spawned && e.attrs[5]&MMT_HIDE) continue;
+                    if(e.lastemit && e.spawned && e.attrs[6]&MMT_HIDE) continue;
                     e.visible = true;
                     ++visible;
                 }
@@ -394,12 +395,12 @@ void rendermapmodel(extentity &e)
     int anim = ANIM_MAPMODEL|ANIM_LOOP, basetime = 0, flags = MDL_CULL_VFC|MDL_CULL_DIST|MDL_DYNLIGHT;
     if(e.lastemit)
     {
-        if(e.attrs[5]&MMT_HIDE && e.spawned) return;
+        if(e.attrs[6]&MMT_HIDE && e.spawned) return;
         anim = e.spawned ? ANIM_TRIGGER_ON : ANIM_TRIGGER_OFF;
         if(e.lastemit > 0 && lastmillis-e.lastemit < entities::triggertime(e)) basetime = e.lastemit;
         else anim |= ANIM_END;
     }
-    if(e.attrs[5]&MMT_NOSHADOW && !(e.attrs[5]&MMT_NODYNSHADOW)) flags |= e.lastemit ? MDL_DYNSHADOW : MDL_SHADOW;
+    if(e.attrs[6]&MMT_NOSHADOW && !(e.attrs[6]&MMT_NODYNSHADOW)) flags |= e.lastemit ? MDL_DYNSHADOW : MDL_SHADOW;
     if(mmanimoverride)
     {
         anim = (mmanimoverride<0 ? ANIM_ALL : mmanimoverride)|ANIM_LOOP;
@@ -408,14 +409,14 @@ void rendermapmodel(extentity &e)
     mapmodelinfo *mmi = getmminfo(e.attrs[0]);
     if(mmi)
     {
-        if(e.attrs[7] || e.attrs[8])
+        if(e.attrs[8] || e.attrs[9])
         {
-            vec r = game::getpalette(e.attrs[7], e.attrs[8]);
-            if(e.attrs[6]) r.mul(vec::hexcolor(e.attrs[6]));
+            vec r = game::getpalette(e.attrs[8], e.attrs[9]);
+            if(e.attrs[7]) r.mul(vec::hexcolor(e.attrs[7]));
             e.light.material[0] = bvec::fromcolor(r);
         }
-        else e.light.material[0] = e.attrs[6] ? bvec(e.attrs[6]) : bvec(255, 255, 255);
-        rendermodel(&e.light, mmi->name, anim, e.o, (float)(e.attrs[1]%360), 0, (float)(e.attrs[2]%360), flags, NULL, NULL, basetime, 0, e.attrs[3] ? min(e.attrs[3]/100.f, 1.f) : 1.f, e.attrs[4] ? max(e.attrs[4]/100.f, 1e-3f) : 1.f);
+        else e.light.material[0] = e.attrs[7] ? bvec(e.attrs[7]) : bvec(255, 255, 255);
+        rendermodel(&e.light, mmi->name, anim, e.o, e.attrs[1], e.attrs[2], e.attrs[3], flags, NULL, NULL, basetime, 0, e.attrs[4] ? min(e.attrs[4]/100.f, 1.f) : 1.f, e.attrs[5] ? max(e.attrs[5]/100.f, 1e-3f) : 1.f);
     }
 }
 
@@ -443,14 +444,14 @@ void renderreflectedmapmodels()
         }
         *lastmms = NULL;
     }
-    for(octaentities *oe = mms; oe; oe = reflecting ? oe->rnext : oe->next) if(reflecting || oe->distance >= 0) 
+    for(octaentities *oe = mms; oe; oe = reflecting ? oe->rnext : oe->next) if(reflecting || oe->distance >= 0)
     {
         if(reflecting || refracting>0 ? oe->bbmax.z <= reflectz : oe->bbmin.z >= reflectz) continue;
         if(isfoggedcube(oe->o, oe->size)) continue;
         loopv(oe->mapmodels)
         {
            extentity &e = *ents[oe->mapmodels[i]];
-           if(e.visible || (e.lastemit && e.spawned && e.attrs[4]&MMT_HIDE)) continue;
+           if(e.visible || (e.lastemit && e.spawned && e.attrs[6]&MMT_HIDE)) continue;
            e.visible = true;
         }
     }
@@ -661,9 +662,7 @@ VAR(IDF_HEX, blendbrushcolor, 0, 0x0000C0, 0xFFFFFF);
 
 void renderblendbrush(GLuint tex, float x, float y, float w, float h)
 {
-    static Shader *blendbrushshader = NULL;
-    if(!blendbrushshader) blendbrushshader = lookupshaderbyname("blendbrush");
-    blendbrushshader->set();
+    SETSHADER(blendbrush);
 
     glEnableClientState(GL_VERTEX_ARRAY);
 
@@ -732,9 +731,7 @@ void rendershadowmapreceivers()
 {
     if(!hasBE) return;
 
-    static Shader *shadowmapshader = NULL;
-    if(!shadowmapshader) shadowmapshader = lookupshaderbyname("shadowmapreceiver");
-    shadowmapshader->set();
+    SETSHADER(shadowmapreceiver);
 
     glDisable(GL_TEXTURE_2D);
     glEnableClientState(GL_VERTEX_ARRAY);
@@ -906,6 +903,9 @@ void renderquery(renderstate &cur, occludequery *query, vtxarray *va, bool full
     else drawbb(va->geommin, ivec(va->geommax).sub(va->geommin), camera);
 
     endquery(query);
+
+    extern int intel_immediate_bug;
+    if(intel_immediate_bug && cur.vbuf) cur.vbuf = 0;
 }
 
 enum
@@ -1446,13 +1446,16 @@ static void changecolor(renderstate &cur, int pass, Slot &slot, VSlot &vslot, co
             glActiveTexture_(GL_TEXTURE0_ARB+cur.diffusetmu);
         }
         else if(pass==RENDERPASS_LIGHTMAP) setuptmu(cur.diffusetmu, "C * T");
-        if(cur.mtglow && !cur.envscale.x && cur.glowcolor != vec(1, 1, 1))
+        if(cur.glowcolor != vec(1, 1, 1))
         {
-            cur.glowcolor = vec(-1, -1, -1);
-            cur.mtglow = false;
-            glActiveTexture_(GL_TEXTURE0_ARB+cur.glowtmu);
-            glDisable(GL_TEXTURE_2D);
-            glActiveTexture_(GL_TEXTURE0_ARB+cur.diffusetmu);
+            cur.glowcolor.x = -1;
+            if(cur.mtglow && !cur.envscale.x)
+            {
+                cur.mtglow = false;
+                glActiveTexture_(GL_TEXTURE0_ARB+cur.glowtmu);
+                glDisable(GL_TEXTURE_2D);
+                glActiveTexture_(GL_TEXTURE0_ARB+cur.diffusetmu);
+            }
         }
     }
     cur.colorscale = colorscale;
@@ -1463,7 +1466,7 @@ static void changeslottmus(renderstate &cur, int pass, Slot &slot, VSlot &vslot)
 {
     if(pass==RENDERPASS_LIGHTMAP || pass==RENDERPASS_COLOR || pass==RENDERPASS_ENVMAP || pass==RENDERPASS_DYNLIGHT)
     {
-        GLuint diffusetex = blankgeom && editmode ? blanktexture->id : (slot.sts.empty() ? notexture->id : slot.sts[0].t->id);
+        GLuint diffusetex = blankgeom ? blanktexture->id : (slot.sts.empty() ? notexture->id : slot.sts[0].t->id);
         if(cur.textures[cur.diffusetmu]!=diffusetex)
             glBindTexture(GL_TEXTURE_2D, cur.textures[cur.diffusetmu] = diffusetex);
     }
@@ -1556,23 +1559,24 @@ static void changeslottmus(renderstate &cur, int pass, Slot &slot, VSlot &vslot)
         loopvj(slot.sts)
         {
             Slot::Tex &t = slot.sts[j];
-            Texture *u = slot.sts[j].t;
             if(t.type==TEX_DIFFUSE || t.combined>=0) continue;
             if(t.type==TEX_ENVMAP)
             {
-                if(envmaptmu>=0 && cur.textures[envmaptmu]!=u->id)
+                if(envmaptmu>=0 && t.t && cur.textures[envmaptmu]!=t.t->id)
                 {
                     glActiveTexture_(GL_TEXTURE0_ARB+envmaptmu);
-                    glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, cur.textures[envmaptmu] = u->id);
+                    glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, cur.textures[envmaptmu] = t.t->id);
                 }
-                continue;
             }
-            else if(cur.textures[tmu]!=u->id)
+            else
             {
-                glActiveTexture_(GL_TEXTURE0_ARB+tmu);
-                glBindTexture(GL_TEXTURE_2D, cur.textures[tmu] = u->id);
+                if(cur.textures[tmu]!=t.t->id)
+                {
+                    glActiveTexture_(GL_TEXTURE0_ARB+tmu);
+                    glBindTexture(GL_TEXTURE_2D, cur.textures[tmu] = t.t->id);
+                }
+                if(++tmu >= 8) break;
             }
-            tmu++;
         }
         glActiveTexture_(GL_TEXTURE0_ARB+cur.diffusetmu);
     }
@@ -1756,7 +1760,6 @@ void renderzpass(renderstate &cur, vtxarray *va)
     if(cur.vbuf!=va->vbuf) changevbuf(cur, RENDERPASS_Z, va);
     if(!cur.depthmask) { cur.depthmask = true; glDepthMask(GL_TRUE); }
     if(cur.colormask) { cur.colormask = false; glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE); }
-
     extern int apple_glsldepth_bug;
     int firsttex = 0, numtexs = va->texs, numtris = va->tris;
     ushort *edata = va->edata;
@@ -1776,8 +1779,6 @@ void renderzpass(renderstate &cur, vtxarray *va)
     }
     else
     {
-        static Shader *nocolorglslshader = NULL;
-        if(!nocolorglslshader) nocolorglslshader = lookupshaderbyname("nocolorglsl");
         int lastflags = 0, lastdraw = 0, offset = 0;
         for(int i = firsttex; i < firsttex + numtexs; i++)
         {
@@ -1829,7 +1830,7 @@ void renderfoggedvas(renderstate &cur, bool doquery = false)
 
     glDisable(GL_TEXTURE_2D);
 
-    glColor3ubv(watercol.v);
+    glColor3ubv(fogging ? refractcolor.v : fogcolor.v);
 
     loopv(foggedvas)
     {
@@ -1889,9 +1890,9 @@ void renderva(renderstate &cur, vtxarray *va, int pass = RENDERPASS_LIGHTMAP, bo
             if(!cur.alphaing) vverts += va->verts;
             va->shadowed = false;
             va->dynlightmask = 0;
-            if(fogpass ? va->geommax.z<=reflectz-waterfog || !waterfog : va->curvfc==VFC_FOGGED)
+            if(fogpass ? va->geommax.z<=reflectz-refractfog || !refractfog : va->curvfc==VFC_FOGGED)
             {
-                foggedvas.add(va);
+                if(!cur.alphaing && !cur.blending) foggedvas.add(va);
                 break;
             }
             if(renderpath!=R_FIXEDFUNCTION && !envmapping && !glaring && !cur.alphaing)
@@ -2045,9 +2046,7 @@ void setupcaustics(int tmu, float blend, GLfloat *color = NULL)
     }
     if(renderpath!=R_FIXEDFUNCTION)
     {
-        static Shader *causticshader = NULL;
-        if(!causticshader) causticshader = lookupshaderbyname("caustic");
-        causticshader->set();
+        SETSHADER(caustic);
         setlocalparamfv("texgenS", SHPARAM_VERTEX, 0, s);
         setlocalparamfv("texgenT", SHPARAM_VERTEX, 1, t);
         setlocalparamf("frameoffset", SHPARAM_PIXEL, 0, blend*(1-frac), blend*frac, blend);
@@ -2206,7 +2205,7 @@ static void rendergeommultipass(renderstate &cur, int pass, bool fogpass)
             if(va->geommax.z <= reflectz) continue;
         }
         else if(va->occluded >= OCCLUDE_GEOM) continue;
-        if(fogpass ? va->geommax.z <= reflectz-waterfog || !waterfog : va->curvfc==VFC_FOGGED) continue;
+        if(fogpass ? va->geommax.z <= reflectz-refractfog || !refractfog : va->curvfc==VFC_FOGGED) continue;
         renderva(cur, va, pass, fogpass);
     }
     if(geombatches.length()) renderbatches(cur, pass);
@@ -2362,7 +2361,7 @@ void rendergeom(float causticspass, bool fogpass)
         glFlush();
         if(shadowmap && hasFBO && mainpass)
         {
-            glDisableClientState(GL_VERTEX_ARRAY);
+			glDisableClientState(GL_VERTEX_ARRAY);
             if(hasVBO)
             {
                 glBindBuffer_(GL_ARRAY_BUFFER_ARB, 0);
@@ -2436,7 +2435,7 @@ void rendergeom(float causticspass, bool fogpass)
                 if(va->geommax.z <= reflectz) continue;
             }
             else if(va->occluded >= OCCLUDE_GEOM) continue;
-            if(fogpass ? va->geommax.z <= reflectz-waterfog : va->curvfc==VFC_FOGGED) continue;
+            if(fogpass ? va->geommax.z <= reflectz-refractfog || !refractfog : va->curvfc==VFC_FOGGED) continue;
             renderva(cur, va, RENDERPASS_LIGHTMAP_BLEND, fogpass);
         }
         if(geombatches.length()) renderbatches(cur, RENDERPASS_LIGHTMAP);
@@ -2609,7 +2608,7 @@ void renderalphageom(bool fogpass)
             if(va->occluded >= OCCLUDE_BB) continue;
             if(va->occluded >= OCCLUDE_GEOM && pvsoccluded(va->geommin, va->geommax)) continue;
         }
-        if(fogpass ? va->geommax.z <= reflectz-waterfog : va->curvfc==VFC_FOGGED) continue;
+        if(fogpass ? va->geommax.z <= reflectz-refractfog || !refractfog : va->curvfc==VFC_FOGGED) continue;
         alphavas.add(va);
         if(va->alphabacktris) hasback = true;
     }
