@@ -6,26 +6,69 @@
 #include <shlobj.h>
 #endif
 
-VAR(0, version, 1, RE_VERSION, -1); // for scripts
-VAR(IDF_PERSIST, kidmode, 0, 0, 1); // kid protections
+bool versioning = false;
+VAR(0, version, 1, 0, -1);
+VAR(0, versionmajor, 0, 0, VAR_MAX);
+VAR(0, versionminor, 0, 0, VAR_MAX);
+VAR(0, versionpatch, 0, 0, VAR_MAX);
+SVAR(0, versionstring, "0.0");
+SVAR(0, versionname, "");
+SVAR(0, versionuname, "");
+SVAR(0, versionrelease, "");
+SVAR(0, versionurl, "");
 
-const char *disc_reasons[] = { "normal", "end of packet", "client num", "user was kicked", "tag type error", "address is banned", "server is in private mode", "server is full", "connection timed out", "packet overflow", "server shutting down" };
+VAR(0, rehashing, 1, 0, -1);
 
-SVAR(IDF_PERSIST, consoletimefmt, "%c");
-char *gettime(char *format)
+extern const char * const disc_reasons[] = { "normal", "end of packet", "client num", "user was kicked", "message error", "address is banned", "server is in private mode", "server is full", "connection timed out", "packet overflow", "server shutting down" };
+
+SVAR(IDF_PERSIST, logtimeformat, "%c");
+SVAR(IDF_PERSIST, filetimeformat, "%Y%m%d%H%M%S");
+const char *gettime(time_t ctime, const char *format)
 {
-    time_t ltime;
-    struct tm *t;
-
-    ltime = time (NULL);
-    t = localtime (&ltime);
-
     static string buf;
-    strftime (buf, sizeof (buf) - 1, format, t);
-
+    if(!ctime) ctime = clocktime;
+    struct tm *t = localtime(&ctime);
+    if(!strftime(buf, sizeof(buf), format && *format ? format : logtimeformat, t)) buf[0] = '\0';
     return buf;
 }
-ICOMMAND(0, gettime, "s", (char *a), result(gettime(a)));
+ICOMMAND(0, gettime, "is", (int *n, char *a), result(gettime(*n, a)));
+
+const char *timestr(int dur, int style)
+{
+    static string buf; buf[0] = 0;
+    int tm = dur, ms = 0, ss = 0, mn = 0;
+    if(style < 2 && tm > 0)
+    {
+        ms = tm%1000;
+        tm = (tm-ms)/1000;
+    }
+    if(style < 0 && tm > 0) ss = tm;
+    else if(style < 4 && tm > 0)
+    {
+        ss = tm%60;
+        tm = (tm-ss)/60;
+        if(tm > 0) mn = tm;
+    }
+    switch(style)
+    {
+        case -1: formatstring(buf)("%d.%d", ss, ms/100); break;
+        case 0: formatstring(buf)("%d:%02d.%03d", mn, ss, ms); break;
+        case 1: formatstring(buf)("%d:%02d.%d", mn, ss, ms/100); break;
+        case 2: formatstring(buf)("%d:%02d", mn, ss); break;
+        case 3:
+        {
+            if(mn > 0)
+            {
+                formatstring(buf)("%dm%ds", mn, ss);
+                break;
+            }
+            formatstring(buf)("%ds", ss);
+            break;
+        }
+    }
+    return buf;
+}
+ICOMMAND(0, timestr, "ii", (int *d, int *s), result(timestr(*d, *s)));
 
 vector<ipinfo> control;
 void addipinfo(vector<ipinfo> &info, int type, const char *name)
@@ -48,6 +91,9 @@ void addipinfo(vector<ipinfo> &info, int type, const char *name)
     p.type = clamp(type, 0, int(ipinfo::MAXTYPES)-1);
     p.flag = ipinfo::LOCAL;
     p.time = totalmillis ? totalmillis : 1;
+#ifdef STANDALONE
+    p.version = nextcontrolversion();
+#endif
     server::updatecontrols = true;
 }
 ICOMMAND(0, addallow, "s", (char *name), addipinfo(control, ipinfo::ALLOW, name));
@@ -93,6 +139,15 @@ void closelogfile()
     }
 }
 
+FILE *getlogfile()
+{
+#ifdef WIN32
+    return logfile;
+#else
+    return logfile ? logfile : stdout;
+#endif
+}
+
 void setlogfile(const char *fname)
 {
     closelogfile();
@@ -102,7 +157,8 @@ void setlogfile(const char *fname)
         if(fname[0] != PATHDIV && (fname[0] != '.' || fname[1] != '.')) fname = findfile(fname, "w");
         if(fname) logfile = fopen(fname, "w");
     }
-    setvbuf(logfile ? logfile : stdout, NULL, _IOLBF, BUFSIZ);
+    FILE *f = getlogfile();
+    if(f) setvbuf(f, NULL, _IOLBF, BUFSIZ);
 }
 
 void logoutf(const char *fmt, ...)
@@ -116,10 +172,10 @@ void logoutf(const char *fmt, ...)
 void console(int type, const char *s, ...)
 {
     defvformatstring(sf, s, s);
-    string osf, fmt;
-    formatstring(fmt)(consoletimefmt);
+    string osf;
     filtertext(osf, sf);
-    logoutf("%s", osf);
+    if(*logtimeformat) logoutf("%s %s", gettime(clocktime, logtimeformat), osf);
+    else logoutf("%s", osf);
 #ifndef STANDALONE
     conline(type, sf, 0);
 #endif
@@ -140,20 +196,25 @@ void conoutf(const char *s, ...)
 
 VAR(0, verbose, 0, 0, 6);
 
-static void writelog(FILE *file, const char *fmt, va_list args)
+static void writelog(FILE *file, const char *buf)
 {
-    static char buf[LOGSTRLEN];
     static uchar ubuf[512];
-    vformatstring(buf, fmt, args, sizeof(buf));
     int len = strlen(buf), carry = 0;
     while(carry < len)
     {
-        int numu = encodeutf8(ubuf, sizeof(ubuf)-1, &((uchar *)buf)[carry], len - carry, &carry);
+        int numu = encodeutf8(ubuf, sizeof(ubuf)-1, &((const uchar *)buf)[carry], len - carry, &carry);
         if(carry >= len) ubuf[numu++] = '\n';
         fwrite(ubuf, 1, numu, file);
     }
 }
 
+static void writelogv(FILE *file, const char *fmt, va_list args)
+{
+    static char buf[LOGSTRLEN];
+    vformatstring(buf, fmt, args, sizeof(buf));
+    writelog(file, buf);
+}
+
 #ifdef STANDALONE
 void localservertoclient(int chan, ENetPacket *packet) {}
 VAR(0, servertype, 1, 3, 3); // 1: private, 2: public, 3: dedicated
@@ -161,117 +222,18 @@ VAR(0, servertype, 1, 3, 3); // 1: private, 2: public, 3: dedicated
 VARF(0, servertype, 0, 0, 3, changeservertype()); // 0: local only, 1: private, 2: public, 3: dedicated
 #endif
 VAR(0, serveruprate, 0, 0, VAR_MAX);
-VAR(0, serverport, 1, RE_SERVER_PORT, VAR_MAX);
+VAR(0, serverport, 1, SERVER_PORT, VAR_MAX);
 SVAR(0, serverip, "");
 
 int curtime = 0, totalmillis = 1, lastmillis = 1, timescale = 100, paused = 0, timeerr = 0;
+time_t clocktime = 0;
 uint totalsecs = 0;
 const char *load = NULL;
 vector<char *> gameargs;
 
-// all network traffic is in 32bit ints, which are then compressed using the following simple scheme (assumes that most values are small).
-
-template<class T>
-static inline void putint_(T &p, int n)
-{
-    if(n<128 && n>-127) p.put(n);
-    else if(n<0x8000 && n>=-0x8000) { p.put(0x80); p.put(n); p.put(n>>8); }
-    else { p.put(0x81); p.put(n); p.put(n>>8); p.put(n>>16); p.put(n>>24); }
-}
-void putint(ucharbuf &p, int n) { putint_(p, n); }
-void putint(packetbuf &p, int n) { putint_(p, n); }
-void putint(vector<uchar> &p, int n) { putint_(p, n); }
-
-int getint(ucharbuf &p)
-{
-    int c = (char)p.get();
-    if(c==-128) { int n = p.get(); n |= char(p.get())<<8; return n; }
-    else if(c==-127) { int n = p.get(); n |= p.get()<<8; n |= p.get()<<16; return n|(p.get()<<24); }
-    else return c;
-}
-
-// much smaller encoding for unsigned integers up to 28 bits, but can handle signed
-template<class T>
-static inline void putuint_(T &p, int n)
-{
-    if(n < 0 || n >= (1<<21))
-    {
-        p.put(0x80 | (n & 0x7F));
-        p.put(0x80 | ((n >> 7) & 0x7F));
-        p.put(0x80 | ((n >> 14) & 0x7F));
-        p.put(n >> 21);
-    }
-    else if(n < (1<<7)) p.put(n);
-    else if(n < (1<<14))
-    {
-        p.put(0x80 | (n & 0x7F));
-        p.put(n >> 7);
-    }
-    else
-    {
-        p.put(0x80 | (n & 0x7F));
-        p.put(0x80 | ((n >> 7) & 0x7F));
-        p.put(n >> 14);
-    }
-}
-void putuint(ucharbuf &p, int n) { putuint_(p, n); }
-void putuint(packetbuf &p, int n) { putuint_(p, n); }
-void putuint(vector<uchar> &p, int n) { putuint_(p, n); }
-
-int getuint(ucharbuf &p)
-{
-    int n = p.get();
-    if(n & 0x80)
-    {
-        n += (p.get() << 7) - 0x80;
-        if(n & (1<<14)) n += (p.get() << 14) - (1<<14);
-        if(n & (1<<21)) n += (p.get() << 21) - (1<<21);
-        if(n & (1<<28)) n |= -1<<28;
-    }
-    return n;
-}
-
-template<class T>
-static inline void putfloat_(T &p, float f)
-{
-    lilswap(&f, 1);
-    p.put((uchar *)&f, sizeof(float));
-}
-void putfloat(ucharbuf &p, float f) { putfloat_(p, f); }
-void putfloat(packetbuf &p, float f) { putfloat_(p, f); }
-void putfloat(vector<uchar> &p, float f) { putfloat_(p, f); }
-
-float getfloat(ucharbuf &p)
-{
-    float f;
-    p.get((uchar *)&f, sizeof(float));
-    return lilswap(f);
-}
-
-template<class T>
-static inline void sendstring_(const char *t, T &p)
-{
-    while(*t) putint(p, *t++);
-    putint(p, 0);
-}
-void sendstring(const char *t, ucharbuf &p) { sendstring_(t, p); }
-void sendstring(const char *t, packetbuf &p) { sendstring_(t, p); }
-void sendstring(const char *t, vector<uchar> &p) { sendstring_(t, p); }
-
-void getstring(char *text, ucharbuf &p, int len)
-{
-    char *t = text;
-    do
-    {
-        if(t>=&text[len]) { text[len-1] = 0; return; }
-        if(!p.remaining()) { *t = 0; return; }
-        *t = getint(p);
-    }
-    while(*t++);
-}
-
-void filtertext(char *dst, const char *src, bool newline, bool colour, bool whitespace, int len)
+bool filtertext(char *dst, const char *src, bool newline, bool colour, bool whitespace, int len)
 {
+    bool filtered = false;
     for(int c = uchar(*src); c; c = uchar(*++src))
     {
         if(newline && (c=='\n' || c=='\r')) c = ' ';
@@ -280,6 +242,7 @@ void filtertext(char *dst, const char *src, bool newline, bool colour, bool whit
             if(!colour) *dst++ = c;
             else
             {
+                filtered = true;
                 c = *++src;
                 if(!c) break;
                 else if(c=='z')
@@ -302,8 +265,10 @@ void filtertext(char *dst, const char *src, bool newline, bool colour, bool whit
             *dst++ = c;
             if(!--len) break;
         }
+        else filtered = true;
     }
     *dst = '\0';
+    return filtered;
 }
 ICOMMAND(0, filter, "siiiN", (char *s, int *a, int *b, int *c, int *numargs),
 {
@@ -386,10 +351,12 @@ void cleanupserver()
 void reloadserver()
 {
     loopvrev(control) if(control[i].flag == ipinfo::LOCAL) control.remove(i);
+    server::reload();
 }
 
 void process(ENetPacket *packet, int sender, int chan);
 
+int getservermtu() { return serverhost ? serverhost->mtu : -1; }
 void *getinfo(int i)            { return !clients.inrange(i) || clients[i]->type==ST_EMPTY ? NULL : clients[i]->info; }
 const char *gethostname(int i)  { int o = server::peerowner(i); return !clients.inrange(o) || clients[o]->type==ST_EMPTY ? "unknown" : clients[o]->hostname; }
 int getnumclients()             { return clients.length(); }
@@ -399,8 +366,8 @@ void sendpacket(int n, int chan, ENetPacket *packet, int exclude)
 {
     if(n < 0)
     {
-        server::recordpacket(chan, packet->data, packet->dataLength);
-        loopv(clients) if(i != server::peerowner(exclude) && server::allowbroadcast(i)) sendpacket(i, chan, packet, exclude);
+        server::recordpacket(abs(chan), packet->data, packet->dataLength);
+        if(chan >= 0) loopv(clients) if(i != server::peerowner(exclude) && server::allowbroadcast(i)) sendpacket(i, chan, packet, exclude);
         return;
     }
     switch(clients[n]->type)
@@ -601,7 +568,7 @@ void disconnectmaster()
         server::disconnectedmaster();
         enet_socket_destroy(mastersock);
         mastersock = ENET_SOCKET_NULL;
-        conoutf("disconnected from master server");
+        if(servertype >= 2) conoutf("disconnected from master server");
     }
 
     masterout.setsize(0);
@@ -612,8 +579,8 @@ void disconnectmaster()
     masteraddress.port = ENET_PORT_ANY;
 }
 
-VARF(0, servermasterport, 1, RE_MASTER_PORT, INT_MAX-1, disconnectmaster());
-SVARF(0, servermaster, RE_MASTER_HOST, disconnectmaster());
+VARF(0, servermasterport, 1, MASTER_PORT, INT_MAX-1, disconnectmaster());
+SVARF(0, servermaster, "", disconnectmaster());
 
 ENetSocket connectmaster(bool reuse)
 {
@@ -622,7 +589,7 @@ ENetSocket connectmaster(bool reuse)
 
     if(masteraddress.host == ENET_HOST_ANY)
     {
-        conoutf("\falooking up %s:[%d]...", servermaster, servermasterport);
+        if(servertype >= 2) conoutf("\falooking up %s:[%d]...", servermaster, servermasterport);
         masteraddress.port = servermasterport;
         if(!resolverwait(servermaster, &masteraddress))
         {
@@ -651,12 +618,12 @@ bool connectedmaster() { return mastersock != ENET_SOCKET_NULL; }
 
 bool requestmaster(const char *req)
 {
-    if(mastersock == ENET_SOCKET_NULL)
-    {
-        mastersock = connectmaster();
+    // RE master server need an init message
+    //if(mastersock == ENET_SOCKET_NULL)
+    //{
+    //    mastersock = connectmaster();
         if(mastersock == ENET_SOCKET_NULL) return false;
-    }
-
+    //}
     masterout.put(req, strlen(req));
     return true;
 }
@@ -762,22 +729,24 @@ void checkserversockets()        // reply all server info requests
     CHECKSOCKET(lansock);
     if(maxsock == ENET_SOCKET_NULL || enet_socketset_select(maxsock, &sockset, NULL, 0) <= 0) return;
 
-    ENetBuffer buf;
-    uchar pong[MAXTRANS];
-    loopi(2)
+    if(serverhost)
     {
-        ENetSocket sock = i ? lansock : pongsock;
-        if(sock == ENET_SOCKET_NULL || !ENET_SOCKETSET_CHECK(sockset, sock)) continue;
+        ENetBuffer buf;
+        uchar pong[MAXTRANS];
+        loopi(2)
+        {
+            ENetSocket sock = i ? lansock : pongsock;
+            if(sock == ENET_SOCKET_NULL || !ENET_SOCKETSET_CHECK(sockset, sock)) continue;
 
-        buf.data = pong;
-        buf.dataLength = sizeof(pong);
-        int len = enet_socket_receive(sock, &pongaddr, &buf, 1);
-        if(len < 0) return;
-        ucharbuf req(pong, len), p(pong, sizeof(pong));
-        p.len += len;
-        server::queryreply(req, p);
+            buf.data = pong;
+            buf.dataLength = sizeof(pong);
+            int len = enet_socket_receive(sock, &pongaddr, &buf, 1);
+            if(len < 0) return;
+            ucharbuf req(pong, len), p(pong, sizeof(pong));
+            p.len += len;
+            server::queryreply(req, p);
+        }
     }
-
     if(mastersock != ENET_SOCKET_NULL && ENET_SOCKETSET_CHECK(sockset, mastersock)) flushmasterinput();
 }
 
@@ -785,15 +754,15 @@ void serverslice(uint timeout)  // main server update, called from main loop in
 {
     server::serverupdate();
 
+    flushmasteroutput();
+    checkserversockets();
+
     if(!serverhost)
     {
         server::sendpackets();
         return;
     }
 
-    flushmasteroutput();
-    checkserversockets();
-
     if(servertype >= 2 && totalmillis-laststatus >= 60000)  // display bandwidth stats, useful for server ops
     {
         laststatus = totalmillis;
@@ -873,6 +842,7 @@ int getclockmillis()
 
 int updatetimer(bool limit)
 {
+    clocktime = time(NULL);
     int millis = getclockmillis();
 #ifndef STANDALONE
     if(limit) limitfps(millis, totalmillis);
@@ -1155,23 +1125,25 @@ int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR szCmdLine, int sw)
 
 void logoutfv(const char *fmt, va_list args)
 {
-    if(logfile) writelog(logfile, fmt, args);
     if(appwindow)
     {
         logline &line = loglines.add();
         vformatstring(line.buf, fmt, args, sizeof(line.buf));
+        if(logfile) writelog(logfile, line.buf);
         line.len = min(strlen(line.buf), sizeof(line.buf)-2);
         line.buf[line.len++] = '\n';
         line.buf[line.len] = '\0';
         if(outhandle) writeline(line);
     }
+    else if(logfile) writelogv(logfile, fmt, args);
 }
 
 #else
 
 void logoutfv(const char *fmt, va_list args)
 {
-    writelog(logfile ? logfile : stdout, fmt, args);
+    FILE *f = getlogfile();
+    if(f) writelogv(f, fmt, args);
 }
 
 #endif
@@ -1179,7 +1151,7 @@ void logoutfv(const char *fmt, va_list args)
 void serverloop()
 {
 #ifdef WIN32
-    defformatstring(cap)("%s server", RE_NAME);
+    defformatstring(cap)("%s server", versionname);
     setupwindow(cap);
     SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);
 #endif
@@ -1252,7 +1224,7 @@ int setupserversockets()
         }
         enet_socket_set_option(pongsock, ENET_SOCKOPT_NONBLOCK, 1);
 
-        address.port = RE_LAN_PORT;
+        address.port = LAN_PORT;
         lansock = enet_socket_create(ENET_SOCKET_TYPE_DATAGRAM);
         if(lansock != ENET_SOCKET_NULL && (enet_socket_set_option(lansock, ENET_SOCKOPT_REUSEADDR, 1) < 0 || enet_socket_bind(lansock, &address) < 0))
         {
@@ -1293,9 +1265,6 @@ void setupserver()
 void initgame()
 {
     server::start();
-#ifndef STANDALONE
-    game::start();
-#endif
     loopv(gameargs)
     {
 #ifndef STANDALONE
@@ -1311,18 +1280,26 @@ void initgame()
 }
 
 VAR(0, hasoctapaks, 1, 0, 0); // mega hack; try to find Cube 2, done after our own data so as to not clobber stuff
-SVAR(IDF_PERSIST, octadir, "");
-//, if(!hasoctapaks) trytofindocta(false););
+
+void octadirchanged()
+{
+#ifdef STANDALONE
+    if(rehashing) trytofindocta();
+#else
+    if(initing == NOT_INITING) trytofindocta();
+#endif
+}
+
+SVARF(IDF_PERSIST, octadir, "", octadirchanged());
 
 bool serveroption(char *opt)
 {
     switch(opt[1])
     {
-        case 'k': kidmode = atoi(opt+2); return true;
-        case 'h': logoutf("set home directory: %s", &opt[2]); sethomedir(&opt[2]); return true;
+        case 'h': sethomedir(&opt[2]); logoutf("set home directory: %s", &opt[2]); return true;
         case 'o': setsvar("octadir", &opt[2]); return true;
-        case 'p': logoutf("add package directory: %s", &opt[2]); addpackagedir(&opt[2]); return true;
-        case 'g': logoutf("setting log file: %s", &opt[2]); setlogfile(&opt[2]); return true;
+        case 'p': addpackagedir(&opt[2]); logoutf("add package directory: %s", &opt[2]); return true;
+        case 'g': setlogfile(&opt[2]); logoutf("set log file: %s", opt[2] ? &opt[2] : "<stdout>"); return true;
         case 'v': setvar("verbose", atoi(opt+2)); return true;
         case 's':
         {
@@ -1369,18 +1346,27 @@ bool findoctadir(const char *name, bool fallback)
         addpackagedir(s, PACKAGEDIR_OCTA);
         addpackagedir(octadata, PACKAGEDIR_OCTA);
         addpackagedir(octapaks, PACKAGEDIR_OCTA);
+        maskpackagedirs(~PACKAGEDIR_OCTA);
         hasoctapaks = fallback ? 1 : 2;
         return true;
     }
+    hasoctapaks = 0;
     return false;
 }
+void octapaks(uint *contents)
+{
+    int mask = maskpackagedirs(~0);
+    execute(contents);
+    maskpackagedirs(mask);
+}
+COMMAND(0, octapaks, "e");
 
 void trytofindocta(bool fallback)
 {
     if(!octadir || !*octadir)
     {
         const char *dir = getenv("OCTA_DIR");
-        if(dir && *dir) setsvar("octadir", dir);
+        if(dir && *dir) setsvar("octadir", dir, false);
     }
     if((!octadir || !*octadir || !findoctadir(octadir, false)) && fallback)
     { // user hasn't specifically set it, try some common locations alongside our folder
@@ -1413,44 +1399,49 @@ void trytofindocta(bool fallback)
 void setlocations(bool wanthome)
 {
 #if defined(__APPLE__)
-    extern const char *mac_resourcedir(const char *what);
-    const char *dir = mac_resourcedir("data"); // ./blah.app/Contents/Resources
-    conoutf("attempting to use resources in: %s", dir);
-    defformatstring(resource)("%s/defaults.cfg", dir);
-    if(fileexists(findfile(resource, "r"), "r")) chdir(dir);
+    extern const char *mac_resourcedir();
+    const char *dir = mac_resourcedir(); // ./blah.app/Contents/Resources
+    if(dir && *dir)
+    {
+        conoutf("attempting to use resources in: %s", dir);
+        chdir(dir);
+    }
 #endif
-    if(!fileexists(findfile("data/defaults.cfg", "r"), "r"))
-        if(fileexists(findfile("../data/defaults.cfg", "r"), "r")) chdir("..");
+    loopi(3) if(!fileexists(findfile("data/config/keymap.cfg", "r"), "r"))
+    {
+        if(i != 2) chdir("..");
+        else fatal("could not find data directory");
+    }
     addpackagedir("data");
+    defformatstring(gamedata)("game/%s", server::gameid());
+    addpackagedir(gamedata);
+    execfile("version.cfg", false, EXEC_VERSION);
     if(wanthome)
     {
 #if defined(WIN32)
         mkstring(dir);
         if(SHGetFolderPath(NULL, CSIDL_PERSONAL, NULL, 0, dir) == S_OK)
         {
-            defformatstring(s)("%s\\My Games\\%s", dir, RE_NAME);
+            defformatstring(s)("%s\\My Games\\%s", dir, versionname);
             sethomedir(s);
-            return;
         }
 #elif defined(__APPLE__)
         extern const char *mac_personaldir();
         const char *dir = mac_personaldir(); // typically  /Users/<name>/Application Support/
         if(dir && *dir)
         {
-            defformatstring(s)("%s/%s", dir, RE_UNAME);
+            defformatstring(s)("%s/%s", dir, versionuname);
             sethomedir(s);
-            return;
         }
 #else
         const char *dir = getenv("HOME");
         if(dir && *dir)
         {
-            defformatstring(s)("%s/.%s", dir, RE_UNAME);
+            defformatstring(s)("%s/.%s", dir, versionuname);
             sethomedir(s);
-            return;
         }
 #endif
-        sethomedir("home");
+        else sethomedir("home");
     }
 }
 
@@ -1459,7 +1450,6 @@ void writecfg()
 #ifndef STANDALONE
     stream *f = openutf8file("config.cfg", "w");
     if(!f) return;
-    client::writeclientinfo(f);
     vector<ident *> ids;
     enumerate(idents, ident, id, ids.add(&id));
     ids.sort(ident::compare);
@@ -1513,9 +1503,8 @@ void writecfg()
 #endif
 }
 
-COMMAND(0, writecfg, "");
+ICOMMAND(0, writecfg, "", (void), if(!(identflags&IDF_WORLD)) writecfg());
 
-VAR(0, rehashing, 1, 0, -1);
 void rehash(bool reload)
 {
     if(reload)
@@ -1526,10 +1515,11 @@ void rehash(bool reload)
     reloadserver();
 #ifdef STANDALONE
     reloadmaster();
-    execfile("servinit.cfg", false);
+    execfile("servinit.cfg", false, EXEC_VERSION);
 #else
-    execfile("localinit.cfg", false);
-    execfile("defaults.cfg");
+    execfile("localinit.cfg", false, EXEC_VERSION);
+    initing = INIT_DEFAULTS;
+    execfile("config/defaults.cfg");
     initing = INIT_LOAD;
     interactive = true;
     execfile("servers.cfg", false);
@@ -1541,7 +1531,7 @@ void rehash(bool reload)
     conoutf("\fwconfiguration reloaded");
     rehashing = 0;
 }
-ICOMMAND(0, rehash, "i", (int *nosave), rehash(*nosave ? false : true));
+ICOMMAND(0, rehash, "i", (int *nosave), if(!(identflags&IDF_WORLD)) rehash(*nosave ? false : true));
 
 #ifdef STANDALONE
 #include <signal.h>
@@ -1551,8 +1541,10 @@ void fatal(const char *s, ...)    // failure exit
     if(++errors <= 2) // print up to one extra recursive error
     {
         defvformatstring(msg, s, s);
-        if(logfile) logoutf(msg);
+        if(logfile) logoutf("%s", msg);
+#ifndef WIN32
         fprintf(stderr, "Exiting: %s\n", msg);
+#endif
         if(errors <= 1) // avoid recursion
         {
             cleanupserver();
@@ -1561,7 +1553,7 @@ void fatal(const char *s, ...)    // failure exit
 #endif
             enet_deinitialize();
 #ifdef WIN32
-            defformatstring(cap)("%s: Error", RE_NAME);
+            defformatstring(cap)("%s: Error", versionname);
             MessageBox(NULL, msg, cap, MB_OK|MB_SYSTEMMODAL);
 #endif
         }
@@ -1586,8 +1578,11 @@ void reloadsignal(int signum)
 
 int main(int argc, char **argv)
 {
+    clocktime = time(NULL); // initialise
+
     setlogfile(NULL);
     setlocations(true);
+
     char *initscript = NULL;
     for(int i = 1; i<argc; i++)
     {
@@ -1598,7 +1593,9 @@ int main(int argc, char **argv)
         }
         else gameargs.add(argv[i]);
     }
+
     if(enet_initialize()<0) fatal("Unable to initialise network module");
+
     signal(SIGINT, fatalsignal);
     signal(SIGILL, fatalsignal);
     signal(SIGABRT, fatalsignal);
