@@ -11,13 +11,13 @@ struct GlobalShaderParamState : ShaderParamState
 
 Shader *Shader::lastshader = NULL;
 
-Shader *defaultshader = NULL, *rectshader = NULL, *cubemapshader = NULL, *notextureshader = NULL, *nocolorshader = NULL, *foggedshader = NULL, *foggednotextureshader = NULL, *stdworldshader = NULL, *lineshader = NULL, *foggedlineshader = NULL;
+Shader *defaultshader = NULL, *rectshader = NULL, *cubemapshader = NULL, *notextureshader = NULL, *nocolorshader = NULL, *nocolorglslshader = NULL, *foggedshader = NULL, *foggednotextureshader = NULL, *stdworldshader = NULL, *lineshader = NULL, *foggedlineshader = NULL;
 
 static hashtable<const char *, Shader> shaders;
 static Shader *curshader = NULL;
 static vector<ShaderParam> curparams;
 static GlobalShaderParamState vertexparamstate[RESERVEDSHADERPARAMS + MAXSHADERPARAMS], pixelparamstate[RESERVEDSHADERPARAMS + MAXSHADERPARAMS];
-static bool dirtyenvparams = false, standardshader = false, initshaders = false, forceshaders = true;
+static bool dirtyenvparams = false, standardshader = false, forceshaders = true;
 static uint paramversion = 0;
 
 VAR(0, reservevpparams, 1, 16, 0);
@@ -33,40 +33,9 @@ VAR(0, dbgshader, 0, 0, 2);
 
 void loadshaders()
 {
-    if(renderpath==R_ASMSHADER || renderpath==R_ASMGLSLANG)
-    {
-        GLint val;
-        glGetProgramivARB_(GL_VERTEX_PROGRAM_ARB, GL_MAX_PROGRAM_ENV_PARAMETERS_ARB, &val);
-        maxvpenvparams = val;
-        glGetProgramivARB_(GL_VERTEX_PROGRAM_ARB, GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB, &val);
-        maxvplocalparams = val;
-        glGetProgramivARB_(GL_FRAGMENT_PROGRAM_ARB, GL_MAX_PROGRAM_ENV_PARAMETERS_ARB, &val);
-        maxfpenvparams = val;
-        glGetProgramivARB_(GL_FRAGMENT_PROGRAM_ARB, GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB, &val);
-        maxfplocalparams = val;
-    }
-    if(renderpath==R_GLSLANG || renderpath==R_ASMGLSLANG)
-    {
-        GLint val;
-        glGetIntegerv(GL_MAX_VERTEX_UNIFORM_COMPONENTS, &val);
-        maxvsuniforms = val/4;
-        glGetIntegerv(GL_MAX_FRAGMENT_UNIFORM_COMPONENTS, &val);
-        maxfsuniforms = val/4;
-        glGetIntegerv(GL_MAX_VARYING_FLOATS, &val);
-        maxvaryings = val;
-    }
-    if(renderpath != R_FIXEDFUNCTION)
-    {
-        GLint val;
-        glGetIntegerv(GL_MAX_TEXTURE_COORDS_ARB, &val);
-        maxtexcoords = val;
-    }
-
-    initshaders = true;
     standardshader = true;
-    execfile(renderpath==R_GLSLANG ? "glsl.cfg" : "stdshader.cfg");
+    execfile(renderpath==R_GLSLANG ? "config/glsl.cfg" : "config/stdshader.cfg");
     standardshader = false;
-    initshaders = false;
 
     defaultshader = lookupshaderbyname("default");
     stdworldshader = lookupshaderbyname("stdworld");
@@ -80,17 +49,12 @@ void loadshaders()
     cubemapshader = lookupshaderbyname("cubemap");
     notextureshader = lookupshaderbyname("notexture");
     nocolorshader = lookupshaderbyname("nocolor");
+    nocolorglslshader = lookupshaderbyname("nocolorglsl");
     foggedshader = lookupshaderbyname("fogged");
     foggednotextureshader = lookupshaderbyname("foggednotexture");
     lineshader = lookupshaderbyname(ati_line_bug && renderpath == R_ASMGLSLANG ? "notextureglsl" : "notexture");
     foggedlineshader = lookupshaderbyname(ati_line_bug && renderpath == R_ASMGLSLANG ? "foggednotextureglsl" : "foggednotexture");
 
-    if(renderpath==R_ASMSHADER || renderpath==R_ASMGLSLANG)
-    {
-        glEnable(GL_VERTEX_PROGRAM_ARB);
-        glEnable(GL_FRAGMENT_PROGRAM_ARB);
-    }
-
     defaultshader->set();
 }
 
@@ -116,9 +80,14 @@ static bool compileasmshader(GLenum type, GLuint &idx, const char *def, const ch
         conoutf("\frCOMPILE ERROR (%s:%s) - %s", tname, name, glGetString(GL_PROGRAM_ERROR_STRING_ARB));
         if(err>=0 && err<(int)strlen(def))
         {
-            loopi(err) putchar(*def++);
-            puts(" <<HERE>> ");
-            while(*def) putchar(*def++);
+            FILE *l = getlogfile();
+            if(l)
+            {
+                fwrite(def, 1, err, l);
+                def += err;
+                fputs(" <<HERE>> \n", l);
+                fputs(def, l);
+            }
         }
     }
     else if(msg && !native) conoutf("\fr%s:%s EXCEEDED NATIVE LIMITS", tname, name);
@@ -138,20 +107,24 @@ static void showglslinfo(GLenum type, GLuint obj, const char *name, const char *
     else glGetProgramiv_(obj, GL_INFO_LOG_LENGTH, &length);
     if(length > 1)
     {
-        GLchar *log = new GLchar[length];
-        if(type) glGetShaderInfoLog_(obj, length, &length, log);
-        else glGetProgramInfoLog_(obj, length, &length, log);
         conoutf("\frGLSL ERROR (%s:%s)", type == GL_VERTEX_SHADER ? "VS" : (type == GL_FRAGMENT_SHADER ? "FS" : "PROG"), name);
-        puts(log);
-        if(source) loopi(1000)
+        FILE *l = getlogfile();
+        if(l)
         {
-            const char *next = strchr(source, '\n');
-            printf("%d: ", i+1);
-            fwrite(source, 1, next ? next - source + 1 : strlen(source), stdout);
-            if(!next) { putchar('\n'); break; }
-            source = next + 1;
+            GLchar *log = new GLchar[length];
+            if(type) glGetShaderInfoLog_(obj, length, &length, log);
+            else glGetProgramInfoLog_(obj, length, &length, log);
+            fprintf(l, "%s\n", log);
+            if(source) loopi(1000)
+            {
+                const char *next = strchr(source, '\n');
+                fprintf(l, "%d: ", i+1);
+                fwrite(source, 1, next ? next - source + 1 : strlen(source), l);
+                if(!next) { fputc('\n', l); break; }
+                source = next + 1;
+            }
+            delete[] log;
         }
-        delete[] log;
     }
 }
 
@@ -266,10 +239,14 @@ static void linkglslprogram(Shader &s, bool msg = true)
 
 bool checkglslsupport()
 {
+    const GLchar *vsstr =
+        "void main(void) {\n"
+        "    gl_Position = ftransform();\n"
+        "}\n";
 #if 0
     /* check if GLSL profile supports loops
      */
-    const GLchar *source =
+    const GLchar *psstr =
         "uniform int N;\n"
         "uniform vec4 delta;\n"
         "void main(void) {\n"
@@ -278,33 +255,36 @@ bool checkglslsupport()
         "   gl_FragColor = test;\n"
         "}\n";
 #else
-    const GLchar *source =
+    const GLchar *psstr =
         "void main(void) {\n"
         "   gl_FragColor = vec4(0.0);\n"
         "}\n";
 #endif
-    GLuint obj = glCreateShader_(GL_FRAGMENT_SHADER);
-    if(!obj) return false;
-    glShaderSource_(obj, 1, &source, NULL);
-    glCompileShader_(obj);
-    GLint success;
-    glGetShaderiv_(obj, GL_COMPILE_STATUS, &success);
-    if(!success)
-    {
-        glDeleteShader_(obj);
-        return false;
-    }
+    GLuint vsobj = glCreateShader_(GL_VERTEX_SHADER), psobj = glCreateShader_(GL_FRAGMENT_SHADER);
     GLuint program = glCreateProgram_();
-    if(!program)
+    GLint success = 0;
+    if(vsobj && psobj && program)
     {
-        glDeleteShader_(obj);
-        return false;
+        glShaderSource_(vsobj, 1, &vsstr, NULL);
+        glCompileShader_(vsobj);
+        glGetShaderiv_(vsobj, GL_COMPILE_STATUS, &success);
+        if(success)
+        {
+            glShaderSource_(psobj, 1, &psstr, NULL);
+            glCompileShader_(psobj);
+            glGetShaderiv_(psobj, GL_COMPILE_STATUS, &success);
+            if(success)
+            {
+                glAttachShader_(program, vsobj);
+                glAttachShader_(program, psobj);
+                glLinkProgram_(program);
+                glGetProgramiv_(program, GL_LINK_STATUS, &success);
+            }
+        }
     }
-    glAttachShader_(program, obj);
-    glLinkProgram_(program);
-    glGetProgramiv_(program, GL_LINK_STATUS, &success);
-    glDeleteShader_(obj);
-    glDeleteProgram_(program);
+    if(vsobj) glDeleteShader_(vsobj);
+    if(psobj) glDeleteShader_(psobj);
+    if(program) glDeleteProgram_(program);
     return success!=0;
 }
 
@@ -702,7 +682,7 @@ static inline void setasmslotparam(const ShaderParam &p, LocalShaderParamState &
             float palval[4] = { p.val[0]*palcol.x, p.val[1]*palcol.y, p.val[2]*palcol.z, p.val[3] };
             if(memcmp(val.val, palval, sizeof(val.val))) memcpy(val.val, palval, sizeof(val.val));
             else if(val.dirty==ShaderParamState::CLEAN) return;
-        } 
+        }
         else if(memcmp(val.val, p.val, sizeof(val.val))) memcpy(val.val, p.val, sizeof(val.val));
         else if(val.dirty==ShaderParamState::CLEAN) return;
         glProgramEnvParameter4fvARB_(l.type==SHPARAM_VERTEX ? GL_VERTEX_PROGRAM_ARB : GL_FRAGMENT_PROGRAM_ARB, RESERVEDSHADERPARAMS+l.index, val.val);
@@ -791,10 +771,10 @@ void Shader::cleanup(bool invalid)
     detailshader = NULL;
     used = false;
     native = true;
-    if(vs) { if(reusevs) glDeleteProgramsARB_(1, &vs); vs = 0; }
-    if(ps) { if(reuseps) glDeleteProgramsARB_(1, &ps); ps = 0; }
-    if(vsobj) { if(reusevs) glDeleteShader_(vsobj); vsobj = 0; }
-    if(psobj) { if(reuseps) glDeleteShader_(psobj); psobj = 0; }
+    if(vs) { if(!reusevs) glDeleteProgramsARB_(1, &vs); vs = 0; }
+    if(ps) { if(!reuseps) glDeleteProgramsARB_(1, &ps); ps = 0; }
+    if(vsobj) { if(!reusevs) glDeleteShader_(vsobj); vsobj = 0; }
+    if(psobj) { if(!reuseps) glDeleteShader_(psobj); psobj = 0; }
     if(program) { glDeleteProgram_(program); program = 0; }
     numextparams = 0;
     DELETEA(extparams);
@@ -824,7 +804,7 @@ static void genattriblocs(Shader &s, const char *vs, const char *ps)
     int loc;
     while((vs = strstr(vs, "#pragma CUBE2_attrib")))
     {
-        if(sscanf(vs, "#pragma CUBE2_attrib %s %d", name, &loc) == 2)
+        if(sscanf(vs, "#pragma CUBE2_attrib %100s %d", name, &loc) == 2)
             s.attriblocs.add(AttribLoc(getshaderparamname(name), loc));
         vs += len;
     }
@@ -837,7 +817,7 @@ static void genuniformlocs(Shader &s, const char *vs, const char *ps)
     int binding, stride;
     while((vs = strstr(vs, "#pragma CUBE2_uniform")))
     {
-        int numargs = sscanf(vs, "#pragma CUBE2_uniform %s %s %d %d", name, blockname, &binding, &stride);
+        int numargs = sscanf(vs, "#pragma CUBE2_uniform %100s %100s %d %d", name, blockname, &binding, &stride);
         if(numargs >= 3) s.uniformlocs.add(UniformLoc(getshaderparamname(name), getshaderparamname(blockname), binding, numargs >= 4 ? stride : 0));
         else if(numargs >= 1) s.uniformlocs.add(UniformLoc(getshaderparamname(name)));
         vs += len;
@@ -904,6 +884,92 @@ Shader *newshader(int type, const char *name, const char *vs, const char *ps, Sh
     return &s;
 }
 
+void setupshaders()
+{
+    if(renderpath==R_ASMSHADER || renderpath==R_ASMGLSLANG)
+    {
+        GLint val;
+        glGetProgramivARB_(GL_VERTEX_PROGRAM_ARB, GL_MAX_PROGRAM_ENV_PARAMETERS_ARB, &val);
+        maxvpenvparams = val;
+        glGetProgramivARB_(GL_VERTEX_PROGRAM_ARB, GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB, &val);
+        maxvplocalparams = val;
+        glGetProgramivARB_(GL_FRAGMENT_PROGRAM_ARB, GL_MAX_PROGRAM_ENV_PARAMETERS_ARB, &val);
+        maxfpenvparams = val;
+        glGetProgramivARB_(GL_FRAGMENT_PROGRAM_ARB, GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB, &val);
+        maxfplocalparams = val;
+    }
+    if(renderpath==R_GLSLANG || renderpath==R_ASMGLSLANG)
+    {
+        GLint val;
+        glGetIntegerv(GL_MAX_VERTEX_UNIFORM_COMPONENTS, &val);
+        maxvsuniforms = val/4;
+        glGetIntegerv(GL_MAX_FRAGMENT_UNIFORM_COMPONENTS, &val);
+        maxfsuniforms = val/4;
+        glGetIntegerv(GL_MAX_VARYING_FLOATS, &val);
+        maxvaryings = val;
+    }
+    if(renderpath != R_FIXEDFUNCTION)
+    {
+        GLint val;
+        glGetIntegerv(GL_MAX_TEXTURE_COORDS_ARB, &val);
+        maxtexcoords = val;
+    }
+
+    standardshader = true;
+    if(renderpath == R_GLSLANG)
+    {
+        defaultshader = newshader(SHADER_GLSLANG, "<init>default",
+            "void main(void) {\n"
+            "    gl_Position = ftransform();\n"
+            "    gl_TexCoord[0] = gl_MultiTexCoord0;\n"
+            "    gl_FrontColor = gl_Color;\n"
+            "}\n",
+            "uniform sampler2D tex0;\n"
+            "void main(void) {\n"
+            "    gl_FragColor = gl_Color * texture2D(tex0, gl_TexCoord[0].xy);\n"
+            "}\n");
+        notextureshader = newshader(SHADER_GLSLANG, "<init>notexture",
+            "void main(void) {\n"
+            "    gl_Position = ftransform();\n"
+            "    gl_FrontColor = gl_Color;\n"
+            "}\n",
+            "void main(void) {\n"
+            "    gl_FragColor = gl_Color;\n"
+            "}\n");
+    }
+    else
+    {
+        if(renderpath==R_ASMSHADER || renderpath==R_ASMGLSLANG)
+        {
+            glEnable(GL_VERTEX_PROGRAM_ARB);
+            glEnable(GL_FRAGMENT_PROGRAM_ARB);
+        }
+
+        defaultshader = newshader(0, "<init>default",
+            "!!ARBvp1.0\n"
+            "OPTION ARB_position_invariant;\n"
+            "MOV result.texcoord[0], vertex.texcoord[0];\n"
+            "MOV result.color, vertex.color;\n"
+            "END\n",
+            "!!ARBfp1.0\n"
+            "TEMP color;\n"
+            "TEX color, fragment.texcoord[0], texture[0], 2D;\n"
+            "MUL result.color, fragment.color, color;\n"
+            "END\n");
+        notextureshader = newshader(0, "<init>notexture",
+            "!!ARBvp1.0\n"
+            "OPTION ARB_position_invariant;\n"
+            "MOV result.color, vertex.color;\n"
+            "END\n",
+            "!!ARBfp1.0\n"
+            "TEX result.color, fragment.texcoord[0], texture[0], 2D;\n"
+            "END\n");
+    }
+    standardshader = false;
+
+    if(!defaultshader || !notextureshader) fatal("failed to setup shaders");
+}
+
 static const char *findglslmain(const char *s)
 {
     const char *main = strstr(s, "main");
@@ -1031,7 +1097,8 @@ static void gengenericvariant(Shader &s, const char *sname, const char *vs, cons
             memset(vspragma, ' ', olen);
             vspragma += olen;
             char *end = vspragma + strcspn(vspragma, "\n\r");
-            int endlen = strspn(end, "\n\r");
+            end += strspn(end, "\n\r");
+            int endlen = strcspn(end, "\n\r");
             memset(end, ' ', endlen);
         }
     }
@@ -1046,7 +1113,8 @@ static void gengenericvariant(Shader &s, const char *sname, const char *vs, cons
             memset(pspragma, ' ', olen);
             pspragma += olen;
             char *end = pspragma + strcspn(pspragma, "\n\r");
-            int endlen = strspn(end, "\n\r");
+            end += strspn(end, "\n\r");
+            int endlen = strcspn(end, "\n\r");
             memset(end, ' ', endlen);
         }
     }
@@ -1147,7 +1215,7 @@ static void gendynlightvariant(Shader &s, const char *sname, const char *vs, con
     vspragma += strcspn(vspragma, "\n");
     if(*vspragma) vspragma++;
 
-    if(sscanf(pspragma, "#pragma CUBE2_dynlight %s", pslight)!=1) return;
+    if(sscanf(pspragma, "#pragma CUBE2_dynlight %100s", pslight)!=1) return;
 
     pspragma += strcspn(pspragma, "\n");
     if(*pspragma) pspragma++;
@@ -1278,7 +1346,7 @@ static void genshadowmapvariant(Shader &s, const char *sname, const char *vs, co
     vspragma += strcspn(vspragma, "\n");
     if(*vspragma) vspragma++;
 
-    if(sscanf(pspragma, "#pragma CUBE2_shadowmap %s", pslight)!=1) return;
+    if(sscanf(pspragma, "#pragma CUBE2_shadowmap %100s", pslight)!=1) return;
 
     pspragma += strcspn(pspragma, "\n");
     if(*pspragma) pspragma++;
@@ -1554,17 +1622,11 @@ void shader(int *type, char *name, char *vs, char *ps)
         return;
     }
 
-    extern int mesa_program_bug;
     if(renderpath!=R_FIXEDFUNCTION)
     {
         defformatstring(info)("shader %s", name);
         progress(loadprogress, info);
     }
-    if((renderpath==R_ASMSHADER || renderpath==R_ASMGLSLANG) && mesa_program_bug && initshaders && !(*type & SHADER_GLSLANG))
-    {
-        glEnable(GL_VERTEX_PROGRAM_ARB);
-        glEnable(GL_FRAGMENT_PROGRAM_ARB);
-    }
     vector<char> vsbuf, psbuf, vsbak, psbak;
 #define GENSHADER(cond, body) \
     if(cond) \
@@ -1591,11 +1653,6 @@ void shader(int *type, char *name, char *vs, char *ps)
         if(strstr(vs, "#pragma CUBE2_shadowmap")) genshadowmapvariant(*s, s->name, vs, ps);
         if(strstr(vs, "#pragma CUBE2_dynlight")) gendynlightvariant(*s, s->name, vs, ps);
     }
-    if((renderpath==R_ASMSHADER || renderpath==R_ASMGLSLANG) && mesa_program_bug && initshaders && !(*type & SHADER_GLSLANG))
-    {
-        glDisable(GL_VERTEX_PROGRAM_ARB);
-        glDisable(GL_FRAGMENT_PROGRAM_ARB);
-    }
     curparams.shrink(0);
 }
 
@@ -1615,12 +1672,6 @@ void variantshader(int *type, char *name, int *row, char *vs, char *ps)
     defformatstring(varname)("<variant:%d,%d>%s", s->variants[*row].length(), *row, name);
     //defformatstring(info)("shader %s", varname);
     //progress(loadprogress, info);
-    extern int mesa_program_bug;
-    if((renderpath==R_ASMSHADER || renderpath==R_ASMGLSLANG) && mesa_program_bug && initshaders && !(*type & SHADER_GLSLANG))
-    {
-        glEnable(GL_VERTEX_PROGRAM_ARB);
-        glEnable(GL_FRAGMENT_PROGRAM_ARB);
-    }
     vector<char> vsbuf, psbuf, vsbak, psbak;
     if(renderpath!=R_FIXEDFUNCTION)
     {
@@ -1637,11 +1688,6 @@ void variantshader(int *type, char *name, int *row, char *vs, char *ps)
         if(strstr(vs, "#pragma CUBE2_dynlight")) gendynlightvariant(*s, varname, vs, ps, *row);
         if(strstr(ps, "#pragma CUBE2_variant") || strstr(vs, "#pragma CUBE2_variant")) gengenericvariant(*s, varname, vs, ps, *row);
     }
-    if((renderpath==R_ASMSHADER || renderpath==R_ASMGLSLANG) && mesa_program_bug && initshaders && !(*type & SHADER_GLSLANG))
-    {
-        glDisable(GL_VERTEX_PROGRAM_ARB);
-        glDisable(GL_FRAGMENT_PROGRAM_ARB);
-    }
 }
 
 void setshader(char *name)
@@ -1740,7 +1786,7 @@ void linkvslotshader(VSlot &s, bool load)
     if(s.slot->texmask&(1<<TEX_GLOW))
     {
         ShaderParam *cparam = findshaderparam(s, "glowcolor");
-        s.glowcolor = cparam; 
+        s.glowcolor = cparam;
         //if(cparam) loopk(3) s.glowcolor[k] = clamp(cparam->val[k], 0.0f, 1.0f);
         ShaderParam *pulseparam = findshaderparam(s, "pulseglowcolor"),
                     *speedparam = findshaderparam(s, "pulseglowspeed");
