@@ -1,6 +1,9 @@
 VAR(IDF_PERSIST, gpuskel, 0, 1, 1);
 VAR(IDF_PERSIST, matskel, 0, 1, 1);
 
+VAR(0, maxskelanimdata, 1, 192, 0);
+VAR(0, testtags, 0, 0, 1);
+
 #define BONEMASK_NOT  0x8000
 #define BONEMASK_END  0xFFFF
 #define BONEMASK_BONE 0x7FFF
@@ -145,10 +148,9 @@ struct skelmodel : animmodel
         dualquat *bdata;
         matrix3x4 *mdata;
         int version;
-        GLuint ubuf;
         bool dirty;
 
-        skelcacheentry() : bdata(NULL), mdata(NULL), version(-1), ubuf(0), dirty(false) {}
+        skelcacheentry() : bdata(NULL), mdata(NULL), version(-1), dirty(false) {}
 
         void nextversion()
         {
@@ -176,7 +178,7 @@ struct skelmodel : animmodel
         int voffset, eoffset, elen;
         ushort minvert, maxvert;
 
-        skelmesh() : verts(NULL), bumpverts(NULL), tris(0), numverts(0), numtris(0), maxweights(0)
+        skelmesh() : verts(NULL), bumpverts(NULL), tris(NULL), numverts(0), numtris(0), maxweights(0)
         {
         }
 
@@ -412,7 +414,7 @@ struct skelmodel : animmodel
         template<class M>
         void interpverts(const M * RESTRICT mdata1, const M * RESTRICT mdata2, bool norms, bool tangents, void * RESTRICT vdata, skin &s)
         {
-            const int blendoffset = ((skelmeshgroup *)group)->skel->numinterpbones;
+            const int blendoffset = ((skelmeshgroup *)group)->skel->numgpubones;
             mdata2 -= blendoffset;
 
             #define IPLOOP(type, dosetup, dotransform) \
@@ -621,8 +623,9 @@ struct skelmodel : animmodel
 
         bool usegpuskel, usematskel;
         vector<skelcacheentry> skelcache;
+        hashtable<GLuint, int> blendoffsets;
 
-        skeleton() : name(NULL), shared(0), bones(NULL), numbones(0), numinterpbones(0), numgpubones(0), numframes(0), framebones(NULL), ragdoll(NULL), usegpuskel(false), usematskel(false)
+        skeleton() : name(NULL), shared(0), bones(NULL), numbones(0), numinterpbones(0), numgpubones(0), numframes(0), framebones(NULL), ragdoll(NULL), usegpuskel(false), usematskel(false), blendoffsets(32)
         {
         }
 
@@ -636,7 +639,6 @@ struct skelmodel : animmodel
             {
                 DELETEA(skelcache[i].bdata);
                 DELETEA(skelcache[i].mdata);
-                if(skelcache[i].ubuf) glDeleteBuffers_(1, &skelcache[i].ubuf);
             }
         }
 
@@ -679,11 +681,21 @@ struct skelmodel : animmodel
 
         bool addtag(const char *name, int bone, const matrix3x4 &matrix)
         {
-            if(findtag(name) >= 0) return false;
-            tag &t = tags.add();
-            t.name = newstring(name);
-            t.bone = bone;
-            t.matrix = matrix;
+            int idx = findtag(name);
+            if(idx >= 0)
+            {
+                if(!testtags) return false;
+                tag &t = tags[idx];
+                t.bone = bone;
+                t.matrix = matrix;
+            }
+            else
+            {
+                tag &t = tags.add();
+                t.name = newstring(name);
+                t.bone = bone;
+                t.matrix = matrix;
+            }
             return true;
         }
 
@@ -809,7 +821,7 @@ struct skelmodel : animmodel
             {
                 int pos = pitchdeps.length();
                 loopvj(pitchdeps) if(bone <= pitchdeps[j].bone)
-                { 
+                {
                     if(bone == pitchdeps[j].bone) goto nextbone;
                     pos = j;
                     break;
@@ -851,7 +863,7 @@ struct skelmodel : animmodel
             {
                 pitchdep &d = pitchdeps[i];
                 int parent = bones[d.bone].parent;
-                if(parent >= 0) 
+                if(parent >= 0)
                 {
                     int j = findpitchdep(parent);
                     if(j >= 0)
@@ -869,7 +881,7 @@ struct skelmodel : animmodel
                 {
                     t.deps = j;
                     t.pose = pitchdeps[j].pose;
-                }    
+                }
                 t.corrects = -1;
                 for(int parent = t.bone; parent >= 0; parent = bones[parent].parent)
                 {
@@ -946,7 +958,7 @@ struct skelmodel : animmodel
                 default: return 0;
             }
         }
-        int availgpubones() const { return (min(maxgpuparams() - reservevpparams, 256) - 10) / (matskel ? 3 : 2); }
+        int availgpubones() const { return min(maxgpuparams() - reservevpparams - 10, maxskelanimdata) / (matskel ? 3 : 2); }
         bool gpuaccelerate() const { return renderpath!=R_FIXEDFUNCTION && numframes && gpuskel && numgpubones<=availgpubones(); }
 
         float calcdeviation(const vec &axis, const vec &forward, const dualquat &pose1, const dualquat &pose2)
@@ -982,8 +994,8 @@ struct skelmodel : animmodel
                 {
                     pitchcorrect &c = pitchcorrects[i];
                     if(c.target != j) continue;
-                    float total = c.parent >= 0 ? pitchcorrects[c.parent].pitchtotal : 0, 
-                          avail = tpitch - total, 
+                    float total = c.parent >= 0 ? pitchcorrects[c.parent].pitchtotal : 0,
+                          avail = tpitch - total,
                           used = tpitch*c.pitchscale;
                     if(c.pitchmin || c.pitchmax)
                     {
@@ -1191,7 +1203,7 @@ struct skelmodel : animmodel
 
         void calctags(part *p, modelattach *attached, skelcacheentry *sc = NULL)
         {
-            loopv(p->links) 
+            loopv(p->links)
             {
                 linkedpart &l = p->links[i];
                 tag &t = tags[l.tag];
@@ -1218,13 +1230,25 @@ struct skelmodel : animmodel
                 loopj(MAXANIMPARTS) sc.as[j].cur.fr1 = -1;
                 DELETEA(sc.bdata);
                 DELETEA(sc.mdata);
-                if(sc.ubuf) { glDeleteBuffers_(1, &sc.ubuf); sc.ubuf = 0; }
             }
             skelcache.setsize(0);
+            blendoffsets.clear();
             lastsdata = lastbdata = NULL;
             if(full) loopv(users) users[i]->cleanup();
         }
 
+        bool canpreload() { return !numframes || gpuaccelerate(); }
+
+        void preload()
+        {
+            if(!numframes) return;
+            if(skelcache.empty())
+            {
+                usegpuskel = gpuaccelerate();
+                usematskel = matskel!=0;
+            }
+        }
+
         skelcacheentry &checkskelcache(part *p, const animstate *as, float pitch, const vec &axis, const vec &forward, ragdolldata *rdata)
         {
             if(skelcache.empty())
@@ -1275,78 +1299,55 @@ struct skelmodel : animmodel
             if(!offset) count = numgpubones;
             if(hasPP)
             {
-                if(usematskel) glProgramEnvParameters4fv_(GL_VERTEX_PROGRAM_ARB, 10 + 3*offset, 3*count, sc.mdata[offset].a.v);
-                else glProgramEnvParameters4fv_(GL_VERTEX_PROGRAM_ARB, 10 + 2*offset, 2*count, sc.bdata[offset].real.v);
+                if(usematskel) glProgramEnvParameters4fv_(GL_VERTEX_PROGRAM_ARB, 10 + 3*offset, 3*count, sc.mdata[0].a.v);
+                else glProgramEnvParameters4fv_(GL_VERTEX_PROGRAM_ARB, 10 + 2*offset, 2*count, sc.bdata[0].real.v);
             }
             else if(usematskel) loopi(count)
             {
-                glProgramEnvParameter4fvARB_(GL_VERTEX_PROGRAM_ARB, 10 + 3*(offset+i), sc.mdata[offset+i].a.v);
-                glProgramEnvParameter4fvARB_(GL_VERTEX_PROGRAM_ARB, 11 + 3*(offset+i), sc.mdata[offset+i].b.v);
-                glProgramEnvParameter4fvARB_(GL_VERTEX_PROGRAM_ARB, 12 + 3*(offset+i), sc.mdata[offset+i].c.v);
+                glProgramEnvParameter4fvARB_(GL_VERTEX_PROGRAM_ARB, 10 + 3*(offset+i), sc.mdata[i].a.v);
+                glProgramEnvParameter4fvARB_(GL_VERTEX_PROGRAM_ARB, 11 + 3*(offset+i), sc.mdata[i].b.v);
+                glProgramEnvParameter4fvARB_(GL_VERTEX_PROGRAM_ARB, 12 + 3*(offset+i), sc.mdata[i].c.v);
             }
             else loopi(count)
             {
-                glProgramEnvParameter4fvARB_(GL_VERTEX_PROGRAM_ARB, 10 + 2*(offset+i), sc.bdata[offset+i].real.v);
-                glProgramEnvParameter4fvARB_(GL_VERTEX_PROGRAM_ARB, 11 + 2*(offset+i), sc.bdata[offset+i].dual.v);
+                glProgramEnvParameter4fvARB_(GL_VERTEX_PROGRAM_ARB, 10 + 2*(offset+i), sc.bdata[i].real.v);
+                glProgramEnvParameter4fvARB_(GL_VERTEX_PROGRAM_ARB, 11 + 2*(offset+i), sc.bdata[i].dual.v);
             }
             if(offset) lastbdata = usematskel ? (void *)sc.mdata : (void *)sc.bdata;
             else lastsdata = usematskel ? (void *)sc.mdata : (void *)sc.bdata;
         }
 
-        void bindubo(UniformLoc &u, skelcacheentry &sc, skelcacheentry &bc, int count)
+        int getblendoffset(UniformLoc &u)
         {
-            if(hasUBO)
+            int &offset = blendoffsets.access(Shader::lastshader->program, -1);
+            if(offset < 0)
             {
-                if(!lastsdata && lastbdata == &bc.ubuf && !bc.dirty) return;
-            }
-            else if(u.version == bc.version && u.data == &bc.ubuf) return;
-            if(!bc.ubuf) { glGenBuffers_(1, &bc.ubuf); bc.dirty = true; }
-            if(bc.dirty)
-            {
-                GLenum target = hasUBO ? GL_UNIFORM_BUFFER : GL_UNIFORM_BUFFER_EXT;
-                glBindBuffer_(target, bc.ubuf);
-                glBufferData_(target, u.size, NULL, GL_STREAM_DRAW_ARB);
-                int bsize = usematskel ? sizeof(matrix3x4) : sizeof(dualquat), boffset = numgpubones*bsize;
-                glBufferSubData_(target, u.offset, boffset, usematskel ? (void *)sc.mdata : (void *)sc.bdata);
-                if(count > 0) glBufferSubData_(target, u.offset + boffset, count*bsize, usematskel ? (void *)&bc.mdata[numgpubones] : (void *)&bc.bdata[numgpubones]);
-                glBindBuffer_(target, 0);
-                bc.dirty = false;
-            }
-            if(hasUBO)
-            {
-                glBindBufferBase_(GL_UNIFORM_BUFFER, u.binding, bc.ubuf);
-                lastsdata = NULL;
-                lastbdata = &bc.ubuf;
-            }
-            else
-            {
-                glUniformBuffer_(Shader::lastshader->program, u.loc, bc.ubuf);
-                u.version = bc.version;
-                u.data = &bc.ubuf;
+                defformatstring(offsetname)("%s[%d]", u.name, (usematskel ? 3 : 2)*numgpubones);
+                offset = glGetUniformLocation_(Shader::lastshader->program, offsetname);
             }
+            return offset;
         }
 
         void setglslbones(UniformLoc &u, skelcacheentry &sc, skelcacheentry &bc, int count)
         {
             if(u.version == bc.version && u.data == (usematskel ? (void *)bc.mdata : (void *)bc.bdata)) return;
-            count += numgpubones;
             if(usematskel)
             {
-                if(count > numgpubones && bc.dirty)
+                glUniform4fv_(u.loc, 3*numgpubones, sc.mdata[0].a.v);
+                if(count > 0)
                 {
-                    memcpy(bc.mdata, sc.mdata, numgpubones*sizeof(matrix3x4));
-                    bc.dirty = false;
+                    int offset = getblendoffset(u);
+                    if(offset >= 0) glUniform4fv_(offset, 3*count, bc.mdata[0].a.v);
                 }
-                glUniform4fv_(u.loc, 3*count, bc.mdata[0].a.v);
             }
             else
             {
-                if(count > numgpubones && bc.dirty)
+                glUniform4fv_(u.loc, 2*numgpubones, sc.bdata[0].real.v);
+                if(count > 0)
                 {
-                    memcpy(bc.bdata, sc.bdata, numgpubones*sizeof(dualquat));
-                    bc.dirty = false;
+                    int offset = getblendoffset(u);
+                    if(offset >= 0) glUniform4fv_(offset, 2*count, bc.bdata[0].real.v);
                 }
-                glUniform4fv_(u.loc, 2*count, bc.bdata[0].real.v);
             }
             u.version = bc.version;
             u.data = usematskel ? (void *)bc.mdata : (void *)bc.bdata;
@@ -1359,8 +1360,7 @@ struct skelmodel : animmodel
             {
                 if(Shader::lastshader->uniformlocs.length() < 1) return;
                 UniformLoc &u = Shader::lastshader->uniformlocs[0];
-                if(u.size > 0 && (hasUBO || hasBUE)) bindubo(u, sc, bc ? *bc : sc, count);
-                else setglslbones(u, sc, bc ? *bc : sc, count);
+                setglslbones(u, sc, bc ? *bc : sc, count);
             }
             else
             {
@@ -1394,7 +1394,7 @@ struct skelmodel : animmodel
         int vlen, vertsize, vblends, vweights;
         uchar *vdata;
 
-        skelmeshgroup() : skel(NULL), edata(NULL), ebuf(0), vdata(NULL)
+        skelmeshgroup() : skel(NULL), edata(NULL), ebuf(0), vnorms(false), vtangents(false), vlen(0), vertsize(0), vblends(0), vweights(0), vdata(NULL)
         {
             memset(numblends, 0, sizeof(numblends));
         }
@@ -1411,7 +1411,6 @@ struct skelmodel : animmodel
             {
                 DELETEA(blendcache[i].bdata);
                 DELETEA(blendcache[i].mdata);
-                if(blendcache[i].ubuf) glDeleteBuffers_(1, &blendcache[i].ubuf);
             }
             loopi(MAXVBOCACHE)
             {
@@ -1430,7 +1429,7 @@ struct skelmodel : animmodel
                 return;
             }
 
-            static hashtable<char *, skeleton *> skeletons;
+            static hashtable<const char *, skeleton *> skeletons;
             if(skeletons.access(name)) skel = skeletons[name];
             else
             {
@@ -1447,6 +1446,7 @@ struct skelmodel : animmodel
             return skel->findtag(name);
         }
 
+        void *animkey() { return skel; }
         int totalframes() const { return max(skel->numframes, 1); }
 
         virtual skelanimspec *loadanim(const char *filename) { return NULL; }
@@ -1488,7 +1488,7 @@ struct skelmodel : animmodel
                 loopv(blendcombos)
                 {
                     blendcombo &c = blendcombos[i];
-                    c.interpindex = c.weights[1] ? skel->numinterpbones + vblends++ : -1;
+                    c.interpindex = c.weights[1] ? skel->numgpubones + vblends++ : -1;
                 }
 
                 vertsize = tangents ? sizeof(vvertbump) : (norms ? sizeof(vvertn) : sizeof(vvert));
@@ -1543,7 +1543,7 @@ struct skelmodel : animmodel
                     int numverts = 0, htlen = 128;
                     loopv(meshes) numverts += ((skelmesh *)meshes[i])->numverts;
                     while(htlen < numverts) htlen *= 2;
-                    if(numverts*4 > htlen*3) htlen *= 2;  
+                    if(numverts*4 > htlen*3) htlen *= 2;
                     int *htdata = new int[htlen];
                     memset(htdata, -1, htlen*sizeof(int));
                     if(tangents) GENVBOSTAT(vvertbump);
@@ -1695,9 +1695,9 @@ struct skelmodel : animmodel
         void blendmatbones(const skelcacheentry &sc, blendcacheentry &bc)
         {
             bc.nextversion();
-            if(!bc.mdata) bc.mdata = new matrix3x4[(skel->usegpuskel ? skel->numgpubones : 0) + vblends];
+            if(!bc.mdata) bc.mdata = new matrix3x4[vblends];
             if(lastbdata == bc.mdata) lastbdata = NULL;
-            matrix3x4 *dst = bc.mdata - (skel->usegpuskel ? 0 : skel->numinterpbones);
+            matrix3x4 *dst = bc.mdata - skel->numgpubones;
             loopv(blendcombos)
             {
                 const blendcombo &c = blendcombos[i];
@@ -1709,9 +1709,9 @@ struct skelmodel : animmodel
         void blendbones(const skelcacheentry &sc, blendcacheentry &bc)
         {
             bc.nextversion();
-            if(!bc.bdata) bc.bdata = new dualquat[(skel->usegpuskel ? skel->numgpubones : 0) + vblends];
+            if(!bc.bdata) bc.bdata = new dualquat[vblends];
             if(lastbdata == bc.bdata) lastbdata = NULL;
-            dualquat *dst = bc.bdata - (skel->usegpuskel ? 0 : skel->numinterpbones);
+            dualquat *dst = bc.bdata - skel->numgpubones;
             bool normalize = !skel->usegpuskel || vweights<=1;
             loopv(blendcombos)
             {
@@ -1730,7 +1730,6 @@ struct skelmodel : animmodel
                 blendcacheentry &c = blendcache[i];
                 DELETEA(c.bdata);
                 DELETEA(c.mdata);
-                if(c.ubuf) { glDeleteBuffers_(1, &c.ubuf); c.ubuf = 0; }
                 c.owner = -1;
             }
             loopi(MAXVBOCACHE)
@@ -1774,6 +1773,21 @@ struct skelmodel : animmodel
             SEARCHCACHE(MAXBLENDCACHE, blendcacheentry, blendcache, )
         }
 
+        void preload(part *p)
+        {
+            if(!skel->canpreload()) return;
+            bool norms = false, tangents = false;
+            loopv(p->skins)
+            {
+                if(p->skins[i].normals()) norms = true;
+                if(p->skins[i].tangents()) tangents = true;
+            }
+            if(skel->shouldcleanup()) skel->cleanup();
+            else if(norms!=vnorms || tangents!=vtangents) cleanup();
+            skel->preload();
+            if(hasVBO ? !vbocache->vbuf : !vbocache->vdata) genvbo(norms, tangents, *vbocache);
+        }
+
         void render(const animstate *as, float pitch, const vec &axis, const vec &forward, dynent *d, part *p, modelattach *attached)
         {
             bool norms = false, tangents = false;
@@ -1943,7 +1957,7 @@ struct skelmodel : animmodel
                 LINK_REUSE :
                 LINK_TAG;
     }
-    
+
     bool skeletal() const { return true; }
 };
 
@@ -2001,7 +2015,7 @@ template<class MDL> struct skelcommands : modelcommands<MDL, struct MDL::skelmes
             mdl.initskins();
         }
     }
-   
+
     static void settag(char *name, char *tagname, float *tx, float *ty, float *tz, float *rx, float *ry, float *rz)
     {
         if(!MDL::loading || MDL::loading->parts.empty()) { conoutf("\frnot loading an %s", MDL::formatname()); return; }
@@ -2024,7 +2038,7 @@ template<class MDL> struct skelcommands : modelcommands<MDL, struct MDL::skelmes
     {
         if(!MDL::loading || MDL::loading->parts.empty()) { conoutf("\frnot loading an %s", MDL::formatname()); return; }
         part &mdl = *(part *)MDL::loading->parts.last();
-    
+
         if(name[0])
         {
             int i = mdl.meshes ? ((meshgroup *)mdl.meshes)->skel->findbone(name) : -1;
@@ -2048,7 +2062,7 @@ template<class MDL> struct skelcommands : modelcommands<MDL, struct MDL::skelmes
             conoutf("\frcould not find bone %s to pitch", name);
             return;
         }
-    
+
         mdl.pitchscale = *pitchscale;
         mdl.pitchoffset = *pitchoffset;
         if(*pitchmin || *pitchmax)
@@ -2114,13 +2128,13 @@ template<class MDL> struct skelcommands : modelcommands<MDL, struct MDL::skelmes
         c.pitchscale = *scale;
         int pos = skel->pitchcorrects.length();
         loopv(skel->pitchcorrects) if(bone <= skel->pitchcorrects[i].bone) { pos = i; break; break; }
-        skel->pitchcorrects.insert(pos, c); 
+        skel->pitchcorrects.insert(pos, c);
     }
 
     static void setanim(char *anim, char *animfile, float *speed, int *priority, int *startoffset, int *endoffset)
     {
         if(!MDL::loading || MDL::loading->parts.empty()) { conoutf("\frnot loading an %s", MDL::formatname()); return; }
-    
+
         vector<int> anims;
         game::findanims(anim, anims);
         if(anims.empty()) conoutf("\frcould not find animation %s", anim);
@@ -2138,18 +2152,18 @@ template<class MDL> struct skelcommands : modelcommands<MDL, struct MDL::skelmes
                 else if(*startoffset < 0) start += max(end + *startoffset, 0);
                 end -= start - sa->frame;
                 if(*endoffset > 0) end = min(end, *endoffset);
-                else if(*endoffset < 0) end = max(end + *endoffset, 1); 
+                else if(*endoffset < 0) end = max(end + *endoffset, 1);
                 MDL::loading->parts.last()->setanim(p->numanimparts-1, anims[i], start, end, *speed, *priority);
             }
         }
     }
-    
+
     static void setanimpart(char *maskstr)
     {
         if(!MDL::loading || MDL::loading->parts.empty()) { conoutf("\frnot loading an %s", MDL::formatname()); return; }
-    
+
         part *p = (part *)MDL::loading->parts.last();
-    
+
         vector<char *> bonestrs;
         explodelist(maskstr, bonestrs);
         vector<ushort> bonemask;
@@ -2163,7 +2177,7 @@ template<class MDL> struct skelcommands : modelcommands<MDL, struct MDL::skelmes
         bonestrs.deletearrays();
         bonemask.sort();
         if(bonemask.length()) bonemask.add(BONEMASK_END);
-    
+
         if(!p->addanimpart(bonemask.getbuf())) conoutf("\frtoo many animation parts");
     }
 
@@ -2178,7 +2192,7 @@ template<class MDL> struct skelcommands : modelcommands<MDL, struct MDL::skelmes
         while(!MDL::adjustments.inrange(i)) MDL::adjustments.add(skeladjustment(0, 0, 0, vec(0, 0, 0)));
         MDL::adjustments[i] = skeladjustment(*yaw, *pitch, *roll, vec(*tx/4, *ty/4, *tz/4));
     }
-    
+
     skelcommands()
     {
         if(MDL::multiparted()) this->modelcommand(loadpart, "load", "ssf");
