@@ -52,7 +52,13 @@ VAR(IDF_PERSIST, soundvol, 0, 255, 255);
 VARF(0, soundmono, 0, 0, 1, initwarning("sound configuration", INIT_RESET, CHANGE_SOUND));
 VARF(0, soundchans, 1, 32, 128, initwarning("sound configuration", INIT_RESET, CHANGE_SOUND));
 VARF(0, soundfreq, 0, 44100, 48000, initwarning("sound configuration", INIT_RESET, CHANGE_SOUND));
-VARF(0, soundbufferlen, 128, 1024, INT_MAX-1, initwarning("sound configuration", INIT_RESET, CHANGE_SOUND));
+VARF(0, soundbufferlen, 128, 1024, VAR_MAX, initwarning("sound configuration", INIT_RESET, CHANGE_SOUND));
+VAR(IDF_PERSIST, soundmaxrad, 0, 512, VAR_MAX);
+VAR(IDF_PERSIST, soundminrad, 0, 0, VAR_MAX);
+FVAR(IDF_PERSIST, soundevtvol, 0, 1, FVAR_MAX);
+FVAR(IDF_PERSIST, soundevtscale, 0, 1, FVAR_MAX);
+FVAR(IDF_PERSIST, soundenvvol, 0, 1, FVAR_MAX);
+FVAR(IDF_PERSIST, soundenvscale, 0, 1, FVAR_MAX);
 
 VARF(IDF_PERSIST, musicvol, 0, 64, 255, changedvol = true);
 VAR(IDF_PERSIST, musicfadein, 0, 1000, VAR_MAX);
@@ -331,7 +337,8 @@ void calcvol(int flags, int vol, int slotvol, int maxrad, int minrad, const vec
         }
         if(!(flags&SND_NODIST))
         {
-            float mrad = maxrad > 0 ? maxrad : 256, nrad = minrad > 0 ? (minrad <= mrad ? minrad : mrad) : 0;
+            float mrad = int((maxrad > 0 ? maxrad : soundmaxrad)*(flags&SND_MAP ? soundenvscale : soundevtscale)),
+                  nrad = minrad > 0 ? (minrad <= mrad ? minrad : mrad) : soundminrad;
             if(dist > nrad)
             {
                 if(dist <= mrad) svol = int(svol*(1.f-((dist-nrad)/max(mrad-nrad,1e-16f))));
@@ -341,7 +348,7 @@ void calcvol(int flags, int vol, int slotvol, int maxrad, int minrad, const vec
     }
     if(!(flags&SND_NOQUIET) && svol > 0 && liquid) svol = int(svol*0.65f);
     if(flags&SND_CLAMPED) svol = max(svol, clamp(vol, 0, 255));
-    *curvol = clamp(int((mastervol/255.f)*(soundvol/255.f)*(slotvol/255.f)*(svol/255.f)*MIX_MAX_VOLUME), 0, MIX_MAX_VOLUME);
+    *curvol = clamp(int((mastervol/255.f)*(soundvol/255.f)*(slotvol/255.f)*(svol/255.f)*MIX_MAX_VOLUME*(flags&SND_MAP ? soundenvvol : soundevtvol)), 0, MIX_MAX_VOLUME);
     *curpan = span;
 }
 
@@ -423,8 +430,8 @@ int playsound(int n, const vec &pos, physent *d, int flags, int vol, int maxrad,
             oldhook = NULL;
 
         int cvol = 0, cpan = 0, v = vol > 0 && vol < 256 ? vol : (flags&SND_CLAMPED ? 64 : 255),
-            x = maxrad > 0 ? maxrad : (flags&SND_CLAMPED ? getworldsize() : (slot->maxrad > 0 ? slot->maxrad : 256)),
-            y = minrad >= 0 ? minrad : (flags&SND_CLAMPED ? 32 : (slot->minrad >= 0 ? slot->minrad : 0)),
+            x = maxrad > 0 ? maxrad : (flags&SND_CLAMPED ? getworldsize() : (slot->maxrad > 0 ? slot->maxrad : soundmaxrad)),
+            y = minrad >= 0 ? minrad : (flags&SND_CLAMPED ? 32 : (slot->minrad >= 0 ? slot->minrad : soundminrad)),
             mat = lookupmaterial(pos);
 
         bool liquid = isliquid(lookupmaterial(camera1->o)&MATF_VOLUME);
@@ -611,7 +618,7 @@ void initmumble()
         if(mumblelink)
         {
             mumbleinfo = (MumbleInfo *)MapViewOfFile(mumblelink, FILE_MAP_ALL_ACCESS, 0, 0, sizeof(MumbleInfo));
-            if(mumbleinfo) wcsncpy(mumbleinfo->name, L"redeclipse", 256);
+            if(mumbleinfo) wcsncpy(mumbleinfo->name, (const wchar_t *)versionuname, 256);
         }
     #elif defined(_POSIX_SHARED_MEMORY_OBJECTS)
         defformatstring(shmname)("/MumbleLink.%d", getuid());
@@ -619,7 +626,7 @@ void initmumble()
         if(mumblelink >= 0)
         {
             mumbleinfo = (MumbleInfo *)mmap(NULL, sizeof(MumbleInfo), PROT_READ|PROT_WRITE, MAP_SHARED, mumblelink, 0);
-            if(mumbleinfo != (MumbleInfo *)-1) wcsncpy(mumbleinfo->name, L"redeclipse", 256);
+            if(mumbleinfo != (MumbleInfo *)-1) wcsncpy(mumbleinfo->name, (const wchar_t *)versionuname, 256);
         }
     #endif
     if(!VALID_MUMBLELINK) closemumble();
