@@ -4,7 +4,7 @@ int uimillis = -1;
 
 bool layoutpass = false;
 static bool actionon = false;
-static int mouseaction[2] = {0}, guibound[2] = {0};
+int mouseaction[2] = {0}, guibound[2] = {0};
 
 static float firstx, firsty;
 
@@ -13,11 +13,13 @@ static int fieldmode = FIELDSHOW;
 static bool fieldsactive = false;
 
 VAR(IDF_PERSIST, guishadow, 0, 2, 8);
-VAR(IDF_PERSIST, guiautotab, 6, 16, 40);
 VAR(IDF_PERSIST, guiclicktab, 0, 1, 1);
 VAR(IDF_PERSIST, guiblend, 1, 255, 255);
 VAR(IDF_PERSIST, guilinesize, 1, 36, 128);
 VAR(IDF_PERSIST, guisepsize, 1, 10, 128);
+VAR(IDF_PERSIST, guiscaletime, 0, 250, VAR_MAX);
+VAR(IDF_PERSIST|IDF_HEX, guibgcolour, -1, 0x888888, 0xFFFFFF);
+VAR(IDF_PERSIST|IDF_HEX, guibordercolour, -1, 0x181818, 0xFFFFFF);
 
 static bool needsinput = false, hastitle = true;
 
@@ -30,55 +32,13 @@ struct gui : guient
     static vector<list> lists;
     static float hitx, hity;
     static int curdepth, curlist, xsize, ysize, curx, cury, fontdepth, mergelist, mergedepth;
-    static bool shouldautotab, hitfx;
+    static bool hitfx;
 
     static void reset() { lists.shrink(0); mergelist = mergedepth = -1; }
 
     static int ty, tx, tpos, *tcurrent, tcolor; //tracking tab size and position since uses different layout method...
 
-    void allowautotab(bool on) { shouldautotab = on; }
     void allowhitfx(bool on) { hitfx = on; }
-
-    void autotab()
-    {
-        if(tcurrent)
-        {
-            if(layoutpass && !tpos) tcurrent = NULL; //disable tabs because you didn't start with one
-            if(shouldautotab && !curdepth && (layoutpass ? 0 : cury) + ysize > guiautotab*guibound[1]) tab(NULL, tcolor, false);
-        }
-    }
-
-    bool shouldtab()
-    {
-        if(tcurrent && shouldautotab)
-        {
-            if(layoutpass)
-            {
-                int space = guiautotab*guibound[1] - ysize;
-                if(space < 0) return true;
-                int l = lists[curlist].parent;
-                while(l >= 0)
-                {
-                    space -= lists[l].h;
-                    if(space < 0) return true;
-                    l = lists[l].parent;
-                }
-            }
-            else
-            {
-                int space = guiautotab*guibound[1] - cury;
-                if(ysize > space) return true;
-                int l = lists[curlist].parent;
-                while(l >= 0)
-                {
-                    if(lists[l].h > space) return true;
-                    l = lists[l].parent;
-                }
-            }
-        }
-        return false;
-    }
-
     static inline bool visibletab() { return !tcurrent || tpos == *tcurrent; }
     bool visible() { return !layoutpass && visibletab(); }
 
@@ -221,11 +181,11 @@ struct gui : guient
         return 0;
     }
 
-    int text  (const char *text, int color, const char *icon, int icolor) { autotab(); return button_(text, color, icon, icolor, false, false); }
-    int button(const char *text, int color, const char *icon, int icolor, bool faded) { autotab(); return button_(text, color, icon, icolor, true, faded); }
-    int title (const char *text, int color, const char *icon, int icolor) { autotab(); return button_(text, color, icon, icolor, false, false, "emphasis"); }
+    int text  (const char *text, int color, const char *icon, int icolor) { return button_(text, color, icon, icolor, false, false); }
+    int button(const char *text, int color, const char *icon, int icolor, bool faded) { return button_(text, color, icon, icolor, true, faded); }
+    int title (const char *text, int color, const char *icon, int icolor) { return button_(text, color, icon, icolor, false, false, "emphasis"); }
 
-    void separator() { autotab(); line_(guisepsize); }
+    void separator() { line_(guisepsize); }
 
     //use to set min size (useful when you have progress bars)
     void strut(float size) { layout(isvertical() ? int(size*guibound[0]) : 0, isvertical() ? 0 : int(size*guibound[1])); }
@@ -291,7 +251,6 @@ struct gui : guient
 
     int image(Texture *t, float scale, bool overlaid, int icolor)
     {
-        autotab();
         if(scale == 0) scale = 1;
         int size = (int)(scale*2*guibound[1])-guishadow;
         if(visible()) icon_(t, overlaid, curx, cury, size, ishit(size+guishadow, size+guishadow), icolor);
@@ -300,16 +259,14 @@ struct gui : guient
 
     int texture(VSlot &vslot, float scale, bool overlaid)
     {
-        autotab();
         if(scale==0) scale = 1;
         int size = (int)(scale*2*guibound[1])-guishadow;
         if(visible()) previewslot(vslot, overlaid, curx, cury, size, ishit(size+guishadow, size+guishadow));
         return layout(size+guishadow, size+guishadow);
     }
 
-    int playerpreview(int model, int color, int team, int weap, float sizescale, bool overlaid)
+    int playerpreview(int model, int color, int team, int weap, const char *vanity, float sizescale, bool overlaid, float scale, float blend)
     {
-        autotab();
         if(sizescale==0) sizescale = 1;
         int size = (int)(sizescale*2*guibound[1])-guishadow;
         if(visible())
@@ -325,12 +282,71 @@ struct gui : guient
                 xs -= 2*xpad;
                 ys -= 2*ypad;
             }
-            int x1 = int(floor(screen->w*(xi*scale.x+origin.x))), y1 = int(floor(screen->h*(1 - ((yi+ys)*scale.y+origin.y)))),
-                x2 = int(ceil(screen->w*((xi+xs)*scale.x+origin.x))), y2 = int(ceil(screen->h*(1 - (yi*scale.y+origin.y))));
+            int x1 = int(floor(screen->w*(xi*uiscale.x+uiorigin.x))), y1 = int(floor(screen->h*(1 - ((yi+ys)*uiscale.y+uiorigin.y)))),
+                x2 = int(ceil(screen->w*((xi+xs)*uiscale.x+uiorigin.x))), y2 = int(ceil(screen->h*(1 - (yi*uiscale.y+uiorigin.y))));
             glViewport(x1, y1, x2-x1, y2-y1);
             glScissor(x1, y1, x2-x1, y2-y1);
             glEnable(GL_SCISSOR_TEST);
-            renderplayerpreview(model, color, team, weap, overlaid);
+            glDisable(GL_BLEND);
+            modelpreview::start(overlaid);
+            game::renderplayerpreview(model, color, team, weap, vanity, scale, blend);
+            modelpreview::end();
+            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+            glEnable(GL_BLEND);
+            glDisable(GL_SCISSOR_TEST);
+            glViewport(0, 0, screen->w, screen->h);
+            if(overlaid)
+            {
+                if(!overlaytex) overlaytex = textureload(guioverlaytex, 3, true, false);
+                const vec &ocolor = hit && hitfx ? vec(1, 0.25f, 0.25f) : vec(1, 1, 1);
+                glColor3fv(ocolor.v);
+                glBindTexture(GL_TEXTURE_2D, overlaytex->id);
+                rect_(xi - xpad, yi - ypad, xs + 2*xpad, ys + 2*ypad, 0);
+            }
+        }
+        return layout(size+guishadow, size+guishadow);
+    }
+
+    int modelpreview(const char *name, int anim, float sizescale, bool overlaid, float scale, float blend)
+    {
+        if(sizescale==0) sizescale = 1;
+        int size = (int)(sizescale*2*guibound[1])-guishadow;
+        if(visible())
+        {
+            bool hit = ishit(size+guishadow, size+guishadow);
+            float xs = size, ys = size, xi = curx, yi = cury, xpad = 0, ypad = 0;
+            if(overlaid)
+            {
+                xpad = xs/32;
+                ypad = ys/32;
+                xi += xpad;
+                yi += ypad;
+                xs -= 2*xpad;
+                ys -= 2*ypad;
+            }
+            int x1 = int(floor(screen->w*(xi*uiscale.x+uiorigin.x))), y1 = int(floor(screen->h*(1 - ((yi+ys)*uiscale.y+uiorigin.y)))),
+                x2 = int(ceil(screen->w*((xi+xs)*uiscale.x+uiorigin.x))), y2 = int(ceil(screen->h*(1 - (yi*uiscale.y+uiorigin.y))));
+            glViewport(x1, y1, x2-x1, y2-y1);
+            glScissor(x1, y1, x2-x1, y2-y1);
+            glEnable(GL_SCISSOR_TEST);
+            glDisable(GL_BLEND);
+            modelpreview::start(overlaid);
+            model *m = loadmodel(name);
+            if(m)
+            {
+                entitylight light;
+                light.color = vec(1, 1, 1);
+                light.dir = vec(0, -1, 2).normalize();
+                vec center, radius;
+                m->boundbox(0, center, radius);
+                float dist =  2.0f*max(radius.magnitude2(), 1.1f*radius.z),
+                      yaw = fmod(lastmillis/10000.0f*360.0f, 360.0f);
+                vec o(-center.x, dist - center.y, -0.1f*dist - center.z);
+                rendermodel(&light, name, anim, o, yaw, 0, 0, NULL, NULL, 0, scale, blend);
+            }
+            modelpreview::end();
+            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+            glEnable(GL_BLEND);
             glDisable(GL_SCISSOR_TEST);
             glViewport(0, 0, screen->w, screen->h);
             if(overlaid)
@@ -347,7 +363,6 @@ struct gui : guient
 
     int slice(Texture *t, float scale, float start, float end, const char *text)
     {
-        autotab();
         if(scale == 0) scale = 1;
         int size = (int)(scale*2*guibound[1]);
         if(t!=notexture && visible()) slice_(t, curx, cury, size, start, end, text);
@@ -356,7 +371,6 @@ struct gui : guient
 
     void progress(float percent, float scale)
     {
-        autotab();
         Texture *t = textureload(hud::progresstex, 3, true, false);
         if(scale == 0) scale = 1;
         int size = (int)(scale*2*guibound[1]), part = size*2/3;
@@ -368,7 +382,6 @@ struct gui : guient
 
     void slider(int &val, int vmin, int vmax, int color, const char *label, bool reverse, bool scroll)
     {
-        autotab();
         int x = curx;
         int y = cury;
         int space = line_(guilinesize, 1.0f, ishorizontal() ? guibound[0]*3 : guibound[1]);
@@ -426,20 +439,20 @@ struct gui : guient
         }
     }
 
-    char *field(const char *name, int color, int length, int height, const char *initval, int initmode)
+    char *field(const char *name, int color, int length, int height, const char *initval, int initmode, bool focus, const char *parent)
     {
-        return field_(name, color, length, height, initval, initmode, FIELDEDIT, "console");
+        return field_(name, color, length, height, initval, initmode, FIELDEDIT, focus, parent, "console");
     }
 
-    char *keyfield(const char *name, int color, int length, int height, const char *initval, int initmode)
+    char *keyfield(const char *name, int color, int length, int height, const char *initval, int initmode, bool focus, const char *parent)
     {
-        return field_(name, color, length, height, initval, initmode, FIELDKEY, "console");
+        return field_(name, color, length, height, initval, initmode, FIELDKEY, focus, parent, "console");
     }
 
-    char *field_(const char *name, int color, int length, int height, const char *initval, int initmode, int fieldtype = FIELDEDIT, const char *font = "")
+    char *field_(const char *name, int color, int length, int height, const char *initval, int initmode, int fieldtype = FIELDEDIT, bool focus = false, const char *parent = NULL, const char *font = "")
     {
         if(font && *font) gui::pushfont(font);
-        editor *e = useeditor(name, initmode, false, initval); // generate a new editor if necessary
+        editor *e = useeditor(name, initmode, false, initval, parent); // generate a new editor if necessary
         if(layoutpass)
         {
             if(initval && e->mode==EDITORFOCUSED && (e!=currentfocus() || fieldmode == FIELDSHOW))
@@ -471,7 +484,7 @@ struct gui : guient
         if(visible())
         {
             e->rendered = true;
-
+            if(focus && e->unfocus) focus = false;
             bool hit = ishit(w, h) && e->mode!=EDITORREADONLY;
             bool editing = (fieldmode != FIELDSHOW) && e==currentfocus() && e->mode!=EDITORREADONLY;
             if(mouseaction[0]&GUI_UP && mergedepth >= 0 && hit) mouseaction[0] &= ~GUI_UP;
@@ -479,16 +492,17 @@ struct gui : guient
             {
                 if(hit)
                 {
-                    if(fieldtype==FIELDKEY) e->clear();
-                    useeditor(e->name, initmode, true);
-                    e->mark(false);
-                    fieldmode = fieldtype;
-                }
-                else if(editing)
-                {
-                    fieldmode = FIELDCOMMIT;
-                    //e->mode = EDITORFOCUSED;
+                    focus = true;
+                    if(e->unfocus) e->unfocus = false;
                 }
+                else if(editing) fieldmode = FIELDCOMMIT;
+            }
+            if(focus)
+            {
+                if(fieldtype == FIELDKEY) e->clear();
+                useeditor(e->name, initmode, true, initval, parent);
+                e->mark(false);
+                fieldmode = fieldtype;
             }
             if(hit && editing && (mouseaction[0]&GUI_PRESSED)!=0 && fieldtype==FIELDEDIT)
                 e->hit(int(floor(hitx-(curx+wpad/2))), int(floor(hity-(cury+hpad/2))), (mouseaction[0]&GUI_DRAGGED)!=0); //mouse request position
@@ -512,6 +526,7 @@ struct gui : guient
             glEnable(GL_BLEND);
             defaultshader->set();
         }
+        else if(e->unfocus) e->unfocus = false;
         layout(w, h);
         if(e->maxy != 1)
         {
@@ -688,9 +703,7 @@ struct gui : guient
             xs -= 2*xpad;
             ys -= 2*ypad;
         }
-        static Shader *rgbonlyshader = NULL;
-        if(!rgbonlyshader) rgbonlyshader = lookupshaderbyname("rgbonly");
-        rgbonlyshader->set();
+        SETSHADER(rgbonly);
         const vec &color = hit && hitfx && !overlaid ? vec(1.25f, 1.25f, 1.25f) : vec(1, 1, 1);
         float tc[4][2] = { { 0, 0 }, { 1, 0 }, { 1, 1 }, { 0, 1 } };
         int xoff = vslot.xoffset, yoff = vslot.yoffset;
@@ -774,7 +787,6 @@ struct gui : guient
         if(visible())
         {
             if(!slidertex) slidertex = textureload(guislidertex, 3, true, false);
-            glEnable(GL_TEXTURE_2D);
             glBindTexture(GL_TEXTURE_2D, slidertex->id);
             if(percent < 0.99f)
             {
@@ -822,7 +834,7 @@ struct gui : guient
 
     static Texture *overlaytex, *slidertex;
 
-    vec origin, scale;
+    vec uiorigin, uiscale;
     guicb *cb;
 
     static float basescale, maxscale;
@@ -830,19 +842,21 @@ struct gui : guient
 
     void adjustscale()
     {
-        int w = xsize + guibound[0]*2, h = ysize + guibound[1]*2;
-        float aspect = float(screen->h)/float(screen->w), fit = 1.0f;
+        int w = xsize + guibound[0]*4, h = ysize + guibound[1]*2;
+        float aspect = forceaspect ? 1.0f/forceaspect : float(screen->h)/float(screen->w), fit = 1.0f;
         if(w*aspect*basescale>1.0f) fit = 1.0f/(w*aspect*basescale);
         if(h*basescale*fit>maxscale) fit *= maxscale/(h*basescale*fit);
-        origin = vec(0.5f-((w-xsize)/2 - guibound[0])*aspect*scale.x*fit, 0.5f + (0.5f*h-guibound[1])*scale.y*fit, 0);
-        scale = vec(aspect*scale.x*fit, scale.y*fit, 1);
+        uiscale = vec(aspect*uiscale.x*fit, uiscale.y*fit, 1);
+        uiorigin = vec(0.5f - ((w-xsize)/2 - guibound[0])*uiscale.x, 0.5f + (0.5f*h-guibound[1])*uiscale.y, 0);
+        //uiorigin = vec(0.5f - (guibound[0]*2)*uiscale.x, 0.5f + (h-guibound[1]*2)*uiscale.y, 0);
     }
 
     void start(int starttime, float initscale, int *tab, bool allowinput, bool wantstitle)
     {
         initscale *= 0.025f;
         basescale = initscale;
-        if(layoutpass) scale.x = scale.y = scale.z = basescale; //min(basescale*(totalmillis-starttime)/300.0f, basescale);
+        if(layoutpass)
+            uiscale.x = uiscale.y = uiscale.z = guiscaletime ? min(basescale*(totalmillis-starttime)/float(guiscaletime), basescale) : basescale;
         needsinput = allowinput;
         hastitle = wantstitle;
         passthrough = !allowinput;
@@ -861,8 +875,38 @@ struct gui : guient
             curx = -xsize/2;
 
             glPushMatrix();
-            glTranslatef(origin.x, origin.y, origin.z);
-            glScalef(scale.x, scale.y, scale.z);
+            glTranslatef(uiorigin.x, uiorigin.y, uiorigin.z);
+            glScalef(uiscale.x, uiscale.y, uiscale.z);
+            int x = curx-guibound[0]*2, y = cury-guibound[1], w = xsize+guibound[0]*4, h = ysize+guibound[1]*2;
+            if(hastitle)
+            {
+                y -= guibound[1]*3/2;
+                h += guibound[1]*3/2;
+            }
+            if(guibgcolour >= 0)
+            {
+                notextureshader->set();
+                glDisable(GL_TEXTURE_2D);
+                hud::drawblend(x, y, w, h, (guibgcolour>>16)/255.f, ((guibgcolour>>8)&0xFF)/255.f, (guibgcolour&0xFF)/255.f, true);
+                defaultshader->set();
+                glEnable(GL_TEXTURE_2D);
+            }
+            if(guibordercolour >= 0)
+            {
+                lineshader->set();
+                glDisable(GL_TEXTURE_2D);
+                glDisable(GL_BLEND);
+                glColor3ub(guibordercolour>>16, (guibordercolour>>8)&0xFF, guibordercolour&0xFF);
+                glBegin(GL_LINE_LOOP);
+                glVertex2f(x, y);
+                glVertex2f(x+w, y);
+                glVertex2f(x+w, y+h);
+                glVertex2f(x, y+h);
+                glEnd();
+                defaultshader->set();
+                glEnable(GL_TEXTURE_2D);
+                glEnable(GL_BLEND);
+            }
         }
     }
 
@@ -879,8 +923,8 @@ struct gui : guient
 
             if(!passthrough)
             {
-                hitx = (cursorx - origin.x)/scale.x;
-                hity = (cursory - origin.y)/scale.y;
+                hitx = (cursorx - uiorigin.x)/uiscale.x;
+                hity = (cursory - uiorigin.y)/uiscale.y;
                 if((mouseaction[0]&GUI_PRESSED) && (fabs(hitx-firstx) > 2 || fabs(hity - firsty) > 2)) mouseaction[0] |= GUI_DRAGGED;
             }
         }
@@ -900,7 +944,7 @@ TVARN(IDF_PERSIST|IDF_PRELOAD, guislidertex, "textures/guislider", gui::sliderte
 
 vector<gui::list> gui::lists;
 float gui::basescale, gui::maxscale = 1, gui::hitx, gui::hity;
-bool gui::passthrough, gui::shouldautotab = true, gui::hitfx = true;
+bool gui::passthrough, gui::hitfx = true;
 int gui::curdepth, gui::fontdepth, gui::curlist, gui::xsize, gui::ysize, gui::curx, gui::cury, gui::mergelist, gui::mergedepth;
 int gui::ty, gui::tx, gui::tpos, *gui::tcurrent, gui::tcolor;
 static vector<gui> guis;
@@ -939,8 +983,8 @@ namespace UI
 
         if(code<0) switch(code)
         { // fall-through-o-rama
-            case -5: if(fieldmode == FIELDEDIT) break; mouseaction[1] |= GUI_ALT;
-            case -4: if(fieldmode == FIELDEDIT) break; mouseaction[1] |= isdown ? GUI_DOWN : GUI_UP;
+            case -5: mouseaction[1] |= GUI_ALT;
+            case -4: mouseaction[1] |= isdown ? GUI_DOWN : GUI_UP;
                 if(active()) return true;
                 break;
             case -3: mouseaction[0] |= GUI_ALT;
@@ -949,7 +993,7 @@ namespace UI
                 if(active()) return true;
                 break;
             case -2:
-                cleargui(1);
+                if(isdown) cleargui(1);
                 if(active()) return true;
             default: break;
         }
@@ -958,7 +1002,11 @@ namespace UI
         switch(code)
         {
             case SDLK_ESCAPE: //cancel editing without commit
-                if(isdown) fieldmode = FIELDABORT;
+                if(isdown)
+                {
+                    fieldmode = FIELDABORT;
+                    e->unfocus = true;
+                }
                 return true;
             case SDLK_RETURN:
             case SDLK_TAB:
@@ -966,17 +1014,8 @@ namespace UI
             case SDLK_KP_ENTER:
                 if(isdown) fieldmode = FIELDCOMMIT; //signal field commit (handled when drawing field)
                 return true;
-            case SDLK_F4:
-                if(SDL_GetModState()&MOD_KEYS)
-                {
-                    quit();
-                    break;
-                }
-                // fall through
             case SDLK_HOME:
             case SDLK_END:
-            case SDLK_PAGEUP:
-            case SDLK_PAGEDOWN:
             case SDLK_DELETE:
             case SDLK_BACKSPACE:
             case SDLK_UP:
@@ -985,8 +1024,16 @@ namespace UI
             case SDLK_RIGHT:
             case SDLK_LSHIFT:
             case SDLK_RSHIFT:
+                break;
+            case SDLK_PAGEUP:
+            case SDLK_PAGEDOWN:
             case -4:
             case -5:
+                if(e->parent && *e->parent)
+                { // pass input on to parent
+                    editor *f = findeditor(e->parent);
+                    if(f) e = f;
+                }
                 break;
             default:
                 if(!cooked || (code<32)) return false;
@@ -1021,7 +1068,8 @@ namespace UI
     {
         if(actionon) mouseaction[0] |= GUI_PRESSED;
 
-        gui::reset(); guis.shrink(0);
+        gui::reset();
+        guis.shrink(0);
 
         // call all places in the engine that may want to render a gui from here, they call addcb()
         if(progressing) progressmenu();
@@ -1070,7 +1118,7 @@ namespace UI
         }
 
         flusheditors();
-        if(!fieldsactive) fieldmode = FIELDSHOW; //didn't draw any fields, so loose focus - mainly for menu closed
+        if(!fieldsactive) fieldmode = FIELDSHOW; //didn't draw any fields, so lose focus - mainly for menu closed
         if((fieldmode!=FIELDSHOW) != wasfocused)
         {
             SDL_EnableUNICODE(fieldmode!=FIELDSHOW);
@@ -1079,9 +1127,9 @@ namespace UI
         loopi(2) mouseaction[i] = 0;
     }
 
-    editor *geteditor(const char *name, int mode, const char *init)
+    editor *geteditor(const char *name, int mode, const char *init, const char *parent)
     {
-        return useeditor(name, mode, false, init);
+        return useeditor(name, mode, false, init, parent);
     }
 
     void editorline(editor *e, const char *str, int limit)
