@@ -21,13 +21,13 @@ bool getentboundingbox(extentity &e, ivec &o, ivec &r)
             {
                 vec center, radius;
                 m->boundbox(0, center, radius);
-                if(e.attrs[4])
+                if(e.attrs[5])
                 {
-                    float scale = max(e.attrs[4]/100.0f, 1e-3f);
+                    float scale = max(e.attrs[5]/100.0f, 1e-3f);
                     center.mul(scale);
                     radius.mul(scale);
                 }
-                rotatebb(center, radius, e.attrs[1], e.attrs[2]);
+                rotatebb(center, radius, e.attrs[1], e.attrs[2], e.attrs[3]);
                 o = vec(center).add(e.o);
                 r = vec(radius).add(1);
                 o.sub(r);
@@ -48,25 +48,26 @@ bool getentboundingbox(extentity &e, ivec &o, ivec &r)
 enum
 {
     MODOE_ADD      = 1<<0,
-    MODOE_UPDATEBB = 1<<1
+    MODOE_UPDATEBB = 1<<1,
+    MODOE_LIGHTENT = 1<<2
 };
 
-void modifyoctaentity(int flags, int id, cube *c, const ivec &cor, int size, const ivec &bo, const ivec &br, int leafsize, vtxarray *lastva = NULL)
+void modifyoctaentity(int flags, int id, extentity &e, cube *c, const ivec &cor, int size, const ivec &bo, const ivec &br, int leafsize, vtxarray *lastva = NULL)
 {
     loopoctabox(cor, size, bo, br)
     {
         ivec o(i, cor.x, cor.y, cor.z, size);
         vtxarray *va = c[i].ext && c[i].ext->va ? c[i].ext->va : lastva;
         if(c[i].children != NULL && size > leafsize)
-            modifyoctaentity(flags, id, c[i].children, o, size>>1, bo, br, leafsize, va);
+            modifyoctaentity(flags, id, e, c[i].children, o, size>>1, bo, br, leafsize, va);
         else if(flags&MODOE_ADD)
         {
             if(!c[i].ext || !c[i].ext->ents) ext(c[i]).ents = new octaentities(o, size);
             octaentities &oe = *c[i].ext->ents;
-            switch(entities::getents()[id]->type)
+            switch(e.type)
             {
                 case ET_MAPMODEL:
-                    if(loadmodel(NULL, entities::getents()[id]->attrs[0]))
+                    if(loadmodel(NULL, e.attrs[0]))
                     {
                         if(va)
                         {
@@ -91,10 +92,10 @@ void modifyoctaentity(int flags, int id, cube *c, const ivec &cor, int size, con
         else if(c[i].ext && c[i].ext->ents)
         {
             octaentities &oe = *c[i].ext->ents;
-            switch(entities::getents()[id]->type)
+            switch(e.type)
             {
                 case ET_MAPMODEL:
-                    if(loadmodel(NULL, entities::getents()[id]->attrs[0]))
+                    if(loadmodel(NULL, e.attrs[0]))
                     {
                         oe.mapmodels.removeobj(id);
                         if(va)
@@ -141,27 +142,45 @@ void modifyoctaentity(int flags, int id, cube *c, const ivec &cor, int size, con
     }
 }
 
-static bool modifyoctaent(int flags, int id)
+vector<int> outsideents;
+
+static bool modifyoctaent(int flags, int id, extentity &e)
 {
-    vector<extentity *> &ents = entities::getents();
-    if(!ents.inrange(id)) return false;
+    if(flags&MODOE_ADD ? e.inoctanode : !e.inoctanode) return false;
+
     ivec o, r;
-    extentity &e = *ents[id];
-    if((flags&MODOE_ADD ? e.inoctanode : !e.inoctanode) || !getentboundingbox(e, o, r)) return false;
-
-    int leafsize = octaentsize, limit = max(r.x, max(r.y, r.z));
-    while(leafsize < limit) leafsize *= 2;
-    int diff = ~(leafsize-1) & ((o.x^(o.x+r.x))|(o.y^(o.y+r.y))|(o.z^(o.z+r.z)));
-    if(diff && (limit > octaentsize/2 || diff < leafsize*2)) leafsize *= 2;
+    if(!getentboundingbox(e, o, r)) return false;
 
+    if(!insideworld(e.o))
+    {
+        int idx = outsideents.find(id);
+        if(flags&MODOE_ADD)
+        {
+            if(idx < 0) outsideents.add(id);
+        }
+        else if(idx >= 0) outsideents.removeunordered(idx);
+    }
+    else
+    {
+        int leafsize = octaentsize, limit = max(r.x, max(r.y, r.z));
+        while(leafsize < limit) leafsize *= 2;
+        int diff = ~(leafsize-1) & ((o.x^(o.x+r.x))|(o.y^(o.y+r.y))|(o.z^(o.z+r.z)));
+        if(diff && (limit > octaentsize/2 || diff < leafsize*2)) leafsize *= 2;
+        modifyoctaentity(flags, id, e, worldroot, ivec(0, 0, 0), hdr.worldsize>>1, o, r, leafsize);
+    }
     e.inoctanode = flags&MODOE_ADD ? 1 : 0;
-    modifyoctaentity(flags, id, worldroot, ivec(0, 0, 0), hdr.worldsize>>1, o, r, leafsize);
     if(e.type == ET_LIGHT || e.type == ET_SUNLIGHT) clearlightcache(id);
-    else if(flags&MODOE_ADD) lightent(e);
+    else if(flags&MODOE_LIGHTENT) lightent(e);
     return true;
 }
 
-static inline void addentity(int id)    { modifyoctaent(MODOE_ADD|MODOE_UPDATEBB, id); }
+static inline bool modifyoctaent(int flags, int id)
+{
+    vector<extentity *> &ents = entities::getents();
+    return ents.inrange(id) && modifyoctaent(flags, id, *ents[id]);
+}
+
+static inline void addentity(int id)    { modifyoctaent(MODOE_ADD|MODOE_UPDATEBB|MODOE_LIGHTENT, id); }
 static inline void removeentity(int id) { modifyoctaent(MODOE_UPDATEBB, id); }
 
 void freeoctaentities(cube &c)
@@ -182,7 +201,55 @@ void freeoctaentities(cube &c)
 void entitiesinoctanodes()
 {
     vector<extentity *> &ents = entities::getents();
-    loopv(ents) modifyoctaent(MODOE_ADD, i);
+    loopv(ents) modifyoctaent(MODOE_ADD, i, *ents[i]);
+}
+
+static inline void findents(octaentities &oe, int low, int high, bool notspawned, const vec &pos, const vec &radius, vector<int> &found)
+{
+    vector<extentity *> &ents = entities::getents();
+    loopv(oe.other)
+    {
+        int id = oe.other[i];
+        extentity &e = *ents[id];
+        if(e.type >= low && e.type <= high && (e.spawned || notspawned) && vec(e.o).mul(radius).squaredlen() <= 1) found.add(id);
+    }
+}
+
+static inline void findents(cube *c, const ivec &o, int size, const ivec &bo, const ivec &br, int low, int high, bool notspawned, const vec &pos, const vec &radius, vector<int> &found)
+{
+    loopoctabox(o, size, bo, br)
+    {
+        if(c[i].ext && c[i].ext->ents) findents(*c[i].ext->ents, low, high, notspawned, pos, radius, found);
+        if(c[i].children && size > octaentsize)
+        {
+            ivec co(i, o.x, o.y, o.z, size);
+            findents(c[i].children, co, size>>1, bo, br, low, high, notspawned, pos, radius, found);
+        }
+    }
+}
+
+void findents(int low, int high, bool notspawned, const vec &pos, const vec &radius, vector<int> &found)
+{
+    vec invradius(1/radius.x, 1/radius.y, 1/radius.z);
+    ivec bo = vec(pos).sub(radius).sub(1),
+         br = vec(radius).add(1).mul(2);
+    int diff = (bo.x^(bo.x+br.x)) | (bo.y^(bo.y+br.y)) | (bo.z^(bo.z+br.z)) | octaentsize,
+        scale = worldscale-1;
+    if(diff&~((1<<scale)-1) || uint(bo.x|bo.y|bo.z|(bo.x+br.x)|(bo.y+br.y)|(bo.z+br.z)) >= uint(hdr.worldsize))
+    {
+        findents(worldroot, ivec(0, 0, 0), 1<<scale, bo, br, low, high, notspawned, pos, invradius, found);
+        return;
+    }
+    cube *c = &worldroot[octastep(bo.x, bo.y, bo.z, scale)];
+    if(c->ext && c->ext->ents) findents(*c->ext->ents, low, high, notspawned, pos, invradius, found);
+    scale--;
+    while(c->children && !(diff&(1<<scale)))
+    {
+        c = &c->children[octastep(bo.x, bo.y, bo.z, scale)];
+        if(c->ext && c->ext->ents) findents(*c->ext->ents, low, high, notspawned, pos, invradius, found);
+        scale--;
+    }
+    if(c->children && 1<<scale >= octaentsize) findents(c->children, ivec(bo).mask(~((2<<scale)-1)), 1<<scale, bo, br, low, high, notspawned, pos, invradius, found);
 }
 
 extern bool havesel, selectcorners;
@@ -335,8 +402,8 @@ void entselectionbox(const extentity &e, vec &eo, vec &es)
     if(e.type == ET_MAPMODEL && (m = loadmodel(NULL, e.attrs[0])))
     {
         m->collisionbox(0, eo, es);
-        if(e.attrs[4]) { float scale = max(e.attrs[4]/100.f, 1e-3f); eo.mul(scale); es.mul(scale); }
-        rotatebb(eo, es, e.attrs[1], e.attrs[2]);
+        if(e.attrs[5]) { float scale = max(e.attrs[5]/100.f, 1e-3f); eo.mul(scale); es.mul(scale); }
+        rotatebb(eo, es, e.attrs[1], e.attrs[2], e.attrs[3]);
         if(m->collide)
             eo.z -= camera1->aboveeye; // wacky but true. see physics collide
         else
@@ -537,8 +604,8 @@ bool dropentity(extentity &e, int drop = -1)
         {
             vec center;
             m->boundbox(0, center, radius);
-            if(e.attrs[4]) { float scale = max(e.attrs[4]/100.f, 1e-3f); center.mul(scale); radius.mul(scale); }
-            rotatebb(center, radius, e.attrs[1], e.attrs[2]);
+            if(e.attrs[5]) { float scale = max(e.attrs[5]/100.f, 1e-3f); center.mul(scale); radius.mul(scale); }
+            rotatebb(center, radius, e.attrs[1], e.attrs[2], e.attrs[3]);
             radius.x += fabs(center.x);
             radius.y += fabs(center.y);
         }
@@ -593,7 +660,7 @@ extentity *newentity(bool local, const vec &o, int type, const attrvector &attrs
     else while(ents.length() < idx) ents.add(entities::newent())->type = ET_EMPTY;
     extentity &e = *entities::newent();
     e.o = o;
-    e.attrs.add(0, clamp(attrs.length(), 5, MAXENTATTRS) - e.attrs.length());
+    e.attrs.add(0, clamp(attrs.length(), entities::numattrs(type), MAXENTATTRS) - e.attrs.length());
     loopi(min(attrs.length(), e.attrs.length())) e.attrs[i] = attrs[i];
     e.type = type;
     e.spawned = false;
@@ -729,7 +796,7 @@ void entset(char *what, char *attr)
     entattrs(attr, attrs);
     groupedit({
         e.type = type;
-        e.attrs.add(0, clamp(attrs.length(), 5, MAXENTATTRS) - e.attrs.length());
+        e.attrs.add(0, clamp(attrs.length(), entities::numattrs(e.type), MAXENTATTRS) - e.attrs.length());
         loopk(min(attrs.length(), e.attrs.length())) e.attrs[k] = attrs[k];
     });
 }
@@ -843,9 +910,13 @@ void splitocta(cube *c, int size)
 
 void clearworldvars(bool msg)
 {
+    setvar("sunlight", 0, false);
+    setvar("sunlightyaw", 0, false);
+    setvar("sunlightpitch", 90, false);
+    setfvar("sunlightscale", 1, false);
     identflags |= IDF_WORLD;
     enumerate(idents, ident, id, {
-        if(id.flags&IDF_WORLD) // reset world vars
+        if(id.flags&IDF_WORLD && !(id.flags&IDF_SERVER)) // reset world vars
         {
             switch (id.type)
             {
@@ -881,6 +952,7 @@ void resetmap(bool empty)
     pruneundos();
     clearmapcrc();
     entities::clearents();
+    outsideents.setsize(0);
     game::resetmap(empty);
 }
 
@@ -919,11 +991,11 @@ bool emptymap(int scale, bool force, char *mname, bool nocfg)   // main empty wo
     if(!nocfg)
     {
         identflags |= IDF_WORLD;
-        execfile("map.cfg");
+        execfile("config/map/default.cfg");
         identflags &= ~IDF_WORLD;
     }
 
-    clearlights();
+    initlights();
     allchanged(true);
 
     game::startmap(nocfg ? "" : "maps/untitled", NULL, true);
@@ -939,6 +1011,8 @@ bool enlargemap(bool force)
     }
     if(hdr.worldsize >= 1<<16) return false;
 
+    while(outsideents.length()) removeentity(outsideents.pop());
+
     worldscale++;
     hdr.worldsize *= 2;
     cube *c = newcubes(F_EMPTY);
@@ -975,6 +1049,8 @@ void shrinkmap()
     }
     if(octant < 0) return;
 
+    while(outsideents.length()) removeentity(outsideents.pop());
+
     if(!worldroot[octant].children) subdividecube(worldroot[octant], false, false);
     cube *root = worldroot[octant].children;
     worldroot[octant].children = NULL;
@@ -1019,7 +1095,7 @@ void mpeditent(int i, const vec &o, int type, attrvector &attr, bool local)
         removeentity(i);
         e.type = type;
         e.o = o;
-        e.attrs.add(0, max(5, min(attr.length(), MAXENTATTRS)) - e.attrs.length());
+        e.attrs.add(0, max(entities::numattrs(e.type), min(attr.length(), MAXENTATTRS)) - e.attrs.length());
         loopk(min(attr.length(), e.attrs.length())) e.attrs[k] = attr[k];
         addentity(i);
     }
