@@ -2,12 +2,12 @@
 
 #include "engine.h"
 
-namemap mapexts[] = {
+extern const namemap mapexts[] = {
     { ".mpz", MAP_MAPZ },
     { ".ogz", MAP_OCTA },
 };
 
-namemap mapdirs[] = {
+extern const namemap mapdirs[] = {
     { "maps", MAP_MAPZ },
     { "base", MAP_OCTA },
 };
@@ -23,6 +23,8 @@ VAR(IDF_PERSIST, autosavemapshot, 0, 1, 1);
 
 void fixmaptitle()
 {
+    string s; // remove colour from these things in RE
+    if(filtertext(s, maptitle)) setsvar("maptitle", s, false);
     const char *title = maptitle, *author = strstr(title, " by ");
     if(author && *author)
     {
@@ -41,7 +43,7 @@ void fixmaptitle()
 }
 
 SVARF(IDF_WORLD, maptitle, "", fixmaptitle());
-SVAR(IDF_WORLD, mapauthor, "");
+SVARF(IDF_WORLD, mapauthor, "", { string s; if(filtertext(s, mapauthor)) setsvar("mapauthor", s, false); });
 
 void setnames(const char *fname, int type)
 {
@@ -103,7 +105,7 @@ void savec(cube *c, const ivec &o, int size, stream *f, bool nolms)
 
             loopj(6) f->putlil<ushort>(c[i].texture[j]);
 
-            if(oflags&0x40) f->putchar(c[i].material);
+            if(oflags&0x40) f->putlil<ushort>(c[i].material);
             if(oflags&0x80) f->putchar(c[i].merged);
             if(oflags&0x20)
             {
@@ -341,6 +343,11 @@ void convertoldsurfaces(cube &c, const ivec &co, int size, surfacecompat *srcsur
     setsurfaces(c, dstsurfs, verts, totalverts);
 }
 
+static inline int convertoldmaterial(int mat)
+{
+    return ((mat&7)<<MATF_VOLUME_SHIFT) | (((mat>>3)&3)<<MATF_CLIP_SHIFT) | (((mat>>5)&7)<<MATF_FLAG_SHIFT);
+}
+
 void loadc(stream *f, cube &c, const ivec &co, int size, bool &failed)
 {
     bool haschildren = false;
@@ -368,10 +375,10 @@ void loadc(stream *f, cube &c, const ivec &co, int size, bool &failed)
             int mat = f->getchar();
             if((maptype == MAP_OCTA && hdr.version <= 26) || (maptype == MAP_MAPZ && hdr.version <= 30))
             {
-                static uchar matconv[] = { MAT_AIR, MAT_WATER, MAT_CLIP, MAT_GLASS|MAT_CLIP, MAT_NOCLIP, MAT_LAVA|MAT_DEATH, MAT_AICLIP, MAT_DEATH };
-                mat = size_t(mat) < sizeof(matconv)/sizeof(matconv[0]) ? matconv[mat] : MAT_AIR;
+                static ushort matconv[] = { MAT_AIR, MAT_WATER, MAT_CLIP, MAT_GLASS|MAT_CLIP, MAT_NOCLIP, MAT_LAVA|MAT_DEATH, MAT_AICLIP, MAT_DEATH };
+                c.material = size_t(mat) < sizeof(matconv)/sizeof(matconv[0]) ? matconv[mat] : MAT_AIR;
             }
-            c.material = mat;
+            else c.material = convertoldmaterial(mat);
         }
         surfacecompat surfaces[12];
         normalscompat normals[6];
@@ -454,7 +461,15 @@ void loadc(stream *f, cube &c, const ivec &co, int size, bool &failed)
     }
     else
     {
-        if(octsav&0x40) c.material = f->getchar();
+        if(octsav&0x40)
+        {
+            if((maptype == MAP_OCTA && hdr.version <= 32) || (maptype == MAP_MAPZ && hdr.version <= 42))
+            {
+                int mat = f->getchar();
+                c.material = convertoldmaterial(mat);
+            }
+            else c.material = f->getlil<ushort>();
+        }
         if(octsav&0x80) c.merged = f->getchar();
         if(octsav&0x20)
         {
@@ -741,7 +756,7 @@ void saveslotconfig(stream *h, Slot &s, int index)
             if(s.params[j].type == SHPARAM_LOOKUP || s.params[j].type == SHPARAM_UNIFORM) h->printf(" %s", escapeid(s.params[j].name));
             else h->printf(" %d", s.params[j].index);
             loopk(4) h->printf(" %f", s.params[j].val[k]);
-            if(s.params[j].palette > 0 || s.params[j].palindex > 0) h->printf(" %d %d", s.params[j].palette, s.params[j].palindex);
+            if(s.params[j].palette || s.params[j].palindex) h->printf(" %d %d", s.params[j].palette, s.params[j].palindex);
             h->printf("\n");
         }
     }
@@ -773,7 +788,7 @@ void saveslotconfig(stream *h, Slot &s, int index)
             h->printf("texalpha %f %f\n", s.variants->alphafront, s.variants->alphaback);
         if(s.variants->colorscale != vec(1, 1, 1))
             h->printf("texcolor %f %f %f\n", s.variants->colorscale.x, s.variants->colorscale.y, s.variants->colorscale.z);
-        if(s.variants->palette > 0 || s.variants->palindex > 0) h->printf("texpalette %d %d\n", s.variants->palette, s.variants->palindex);
+        if(s.variants->palette || s.variants->palindex) h->printf("texpalette %d %d\n", s.variants->palette, s.variants->palindex);
         if(s.variants->coastscale != 1) h->printf("texcoastscale %f\n", s.variants->coastscale);
         if(s.texgrass)
         {
@@ -796,8 +811,13 @@ void save_config(char *mname)
     stream *h = openutf8file(fname, "w");
     if(!h) { conoutf("\frcould not write config to %s", fname); return; }
 
+    string title, author;
+    if(*maptitle) filtertext(title, maptitle);
+    else copystring(title, maptitle);
+    if(*mapauthor) filtertext(author, mapauthor);
+    else copystring(author, mapauthor);
     // config
-    h->printf("// %s by %s (%s)\n// Config generated by %s\n\n", *maptitle ? maptitle : "Untitled", *mapauthor ? mapauthor : "Unknown", mapname, RE_NAME);
+    h->printf("// %s by %s (%s)\n// Config generated by %s\n\n", title, author, mapname, versionname);
 
     int vars = 0;
     h->printf("// Variables stored in map file, may be uncommented here, or changed from editmode.\n");
@@ -807,7 +827,7 @@ void save_config(char *mname)
     loopv(ids)
     {
         ident &id = *ids[i];
-        if(id.flags&IDF_WORLD) switch(id.type)
+        if(id.flags&IDF_WORLD && !(id.flags&IDF_SERVER)) switch(id.type)
         {
             case ID_VAR: h->printf((id.flags&IDF_HEX && *id.storage.i >= 0 ? (id.maxval==0xFFFFFF ? "// %s 0x%.6X\n" : "// %s 0x%X\n") : "// %s %d\n"), escapeid(id), *id.storage.i); vars++;break;
             case ID_FVAR: h->printf("// %s %s\n", escapeid(id), floatstr(*id.storage.f)); vars++; break;
@@ -822,7 +842,7 @@ void save_config(char *mname)
     loopv(ids)
     {
         ident &id = *ids[i];
-        if(id.type == ID_ALIAS && id.flags&IDF_WORLD && strlen(id.name))
+        if(id.type == ID_ALIAS && id.flags&IDF_WORLD && !(id.flags&IDF_SERVER) && strlen(id.name))
         {
             const char *str = id.getstr();
             if(str[0])
@@ -840,25 +860,27 @@ void save_config(char *mname)
     int nummats = sizeof(materialslots)/sizeof(materialslots[0]);
     loopi(nummats)
     {
-        if(verbose) progress(float(i)/float(nummats), "saving material slots...");
+        progress(i/float(nummats), "saving material slots...");
 
-        if(i == MAT_WATER || i == MAT_LAVA)
+        switch(i&MATF_VOLUME)
         {
-            saveslotconfig(h, materialslots[i], -i);
+            case MAT_WATER: case MAT_LAVA:
+                saveslotconfig(h, materialslots[i], -i);
+                break;
         }
     }
     if(verbose) conoutf("\fasaved %d material slots", nummats);
 
     loopv(slots)
     {
-        if(verbose) progress(float(i)/float(slots.length()), "saving texture slots...");
+        progress(i/float(slots.length()), "saving texture slots...");
         saveslotconfig(h, *slots[i], i);
     }
     if(verbose) conoutf("\fasaved %d texture slots", slots.length());
 
     loopv(mapmodels)
     {
-        if(verbose) progress(float(i)/float(mapmodels.length()), "saving mapmodel slots...");
+        progress(i/float(mapmodels.length()), "saving mapmodel slots...");
         h->printf("mmodel %s\n", escapestring(mapmodels[i].name));
     }
     if(mapmodels.length()) h->printf("\n");
@@ -866,7 +888,7 @@ void save_config(char *mname)
 
     loopv(mapsounds)
     {
-        if(verbose) progress(float(i)/float(mapsounds.length()), "saving mapsound slots...");
+        progress(i/float(mapsounds.length()), "saving mapsound slots...");
         h->printf("mapsound %s", escapestring(mapsounds[i].name));
         if((mapsounds[i].vol > 0 && mapsounds[i].vol < 255) || mapsounds[i].maxrad > 0 || mapsounds[i].minrad >= 0)
             h->printf(" %d", mapsounds[i].vol);
@@ -959,14 +981,14 @@ void save_world(const char *mname, bool nodata, bool forcesave)
     // world variables
     int numvars = 0, vars = 0;
     enumerate(idents, ident, id, {
-        if((id.type == ID_VAR || id.type == ID_FVAR || id.type == ID_SVAR) && id.flags&IDF_WORLD && strlen(id.name)) numvars++;
+        if((id.type == ID_VAR || id.type == ID_FVAR || id.type == ID_SVAR) && id.flags&IDF_WORLD && !(id.flags&IDF_SERVER) && strlen(id.name)) numvars++;
     });
     f->putlil<int>(numvars);
     enumerate(idents, ident, id, {
-        if((id.type == ID_VAR || id.type == ID_FVAR || id.type == ID_SVAR) && id.flags&IDF_WORLD && strlen(id.name))
+        if((id.type == ID_VAR || id.type == ID_FVAR || id.type == ID_SVAR) && id.flags&IDF_WORLD && !(id.flags&IDF_SERVER) && strlen(id.name))
         {
             vars++;
-            if(verbose) progress(float(vars)/float(numvars), "saving world variables...");
+            progress(vars/float(numvars), "saving world variables...");
             f->putlil<int>((int)strlen(id.name));
             f->write(id.name, (int)strlen(id.name)+1);
             f->putlil<int>(id.type);
@@ -999,7 +1021,7 @@ void save_world(const char *mname, bool nodata, bool forcesave)
     if(!forcesave) entities::remapents(remapents);
     loopv(ents) // extended
     {
-        if(verbose) progress(float(i)/float(ents.length()), "saving entities...");
+        progress(i/float(ents.length()), "saving entities...");
         int idx = remapents.inrange(i) ? remapents[i] : i;
         extentity &e = *(extentity *)ents[idx];
         if(e.type!=ET_EMPTY || forcesave)
@@ -1069,13 +1091,8 @@ void save_world(const char *mname, bool nodata, bool forcesave)
             if(verbose) conoutf("\fasaved blendmap");
         }
     }
-
-    progress(0, "saving world...");
-    game::saveworld(f);
     delete f;
-
     conoutf("\fasaved map %s v.%d:%d (r%d) in %.1f secs", mapname, hdr.version, hdr.gamever, hdr.revision, (SDL_GetTicks()-savingstart)/1000.0f);
-
     if(istempname(mapname)) setnames(&mapname[5], MAP_MAPZ);
 }
 
@@ -1092,6 +1109,11 @@ static void sanevars()
     setvar("blankgeom", 0, false);
 }
 
+VAR(IDF_HEX, sunlight, 0, 0, 0xFFFFFF); // OCTA compatibility
+VAR(0, sunlightyaw, 0, 0, 360);
+VAR(0, sunlightpitch, -90, 90, 90);
+FVAR(0, sunlightscale, 0, 1, 16);
+
 bool load_world(const char *mname, bool temp)       // still supports all map formats that have existed since the earliest cube betas!
 {
     int loadingstart = SDL_GetTicks();
@@ -1100,6 +1122,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
     {
         loop(format, MAP_MAX)
         {
+            int mask = maskpackagedirs(format == MAP_OCTA ? ~0 : ~PACKAGEDIR_OCTA);
             setnames(mname, format);
             if(!tempfile) loopk(2)
             {
@@ -1108,7 +1131,11 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
             }
 
             stream *f = opengzfile(mapfile, "rb");
-            if(!f) continue;
+            if(!f)
+            {
+                maskpackagedirs(mask);
+                continue;
+            }
 
             bool samegame = true;
             int eif = 0;
@@ -1118,6 +1145,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                 conoutf("\frerror loading %s: malformatted universal header", mapname);
                 delete f;
                 maploading = 0;
+                maskpackagedirs(mask);
                 return false;
             }
             lilswap(&newhdr.version, 2);
@@ -1134,8 +1162,10 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                         conoutf("\frerror loading %s: malformatted mapz v%d[%d] header", mapname, newhdr.version, ver); \
                         delete f; \
                         maploading = 0; \
+                        maskpackagedirs(mask); \
                         return false; \
                     }
+
                 if(newhdr.version <= 25)
                 {
                     MAPZCOMPAT(25);
@@ -1185,6 +1215,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                         conoutf("\frerror loading %s: malformatted mapz v%d header", mapname, newhdr.version);
                         delete f;
                         maploading = 0;
+                        maskpackagedirs(mask);
                         return false;
                     }
                     lilswap(&newhdr.worldsize, 8);
@@ -1192,9 +1223,10 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
 
                 if(newhdr.version > MAPVERSION)
                 {
-                    conoutf("\frerror loading %s: requires a newer version of %s", mapname, RE_NAME);
+                    conoutf("\frerror loading %s: requires a newer version of %s", mapname, versionname);
                     delete f;
                     maploading = 0;
+                    maskpackagedirs(mask);
                     return false;
                 }
 
@@ -1213,13 +1245,17 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                     progress(0, "loading variables...");
                     loopi(numvars)
                     {
-                        if(verbose) progress(float(i)/float(numvars), "loading variables...");
+                        progress(i/float(numvars), "loading variables...");
                         int len = hdr.version >= 25 ? f->getlil<int>() : f->getchar();
                         if(len)
                         {
                             string name;
                             f->read(name, len+1);
-                            if(hdr.version <= 34 && !strcmp(name, "cloudcolour")) copystring(name, "cloudlayercolour");
+                            if(hdr.version <= 34)
+                            {
+                                if(!strcmp(name, "cloudcolour")) copystring(name, "cloudlayercolour");
+                                if(!strcmp(name, "cloudblend")) copystring(name, "cloudlayerblend");
+                            }
                             if(hdr.version <= 41)
                             {
                                 if(!strcmp(name, "liquidcurb")) copystring(name, "liquidcoast");
@@ -1237,7 +1273,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                                     type = ID_FVAR;
                                 else proceed = false;
                             }
-                            if(!id || !(id->flags&IDF_WORLD)) proceed = false;
+                            if(!id || !(id->flags&IDF_WORLD) || id->flags&IDF_SERVER) proceed = false;
 
                             switch(type)
                             {
@@ -1266,9 +1302,13 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                                 case ID_SVAR:
                                 {
                                     int slen = f->getlil<int>();
-                                    string val;
-                                    f->read(val, slen+1);
-                                    if(proceed && slen) setsvar(name, val, true);
+                                    if(slen >= 0)
+                                    {
+                                        char *val = newstring(slen);
+                                        f->read(val, slen+1);
+                                        if(proceed && slen) setsvar(name, val, true);
+                                        delete[] val;
+                                    }
                                     break;
                                 }
                                 default:
@@ -1314,6 +1354,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                         conoutf("\frerror loading %s: malformatted octa v%d[%d] header", mapname, ver, ohdr.version); \
                         delete f; \
                         maploading = 0; \
+                        maskpackagedirs(mask); \
                         return false; \
                     }
 
@@ -1370,6 +1411,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                         conoutf("\frerror loading %s: malformatted octa v%d header", mapname, ohdr.version);
                         delete f;
                         maploading = 0;
+                        maskpackagedirs(mask);
                         return false;
                     }
                     lilswap(&ohdr.worldsize, 7);
@@ -1380,6 +1422,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                     conoutf("\frerror loading %s: requires a newer version of Cube 2 support", mapname);
                     delete f;
                     maploading = 0;
+                    maskpackagedirs(mask);
                     return false;
                 }
 
@@ -1406,11 +1449,14 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                     f->read(name, min(ilen, OCTASTRLEN-1));
                     name[min(ilen, OCTASTRLEN-1)] = '\0';
                     if(ilen >= OCTASTRLEN) f->seek(ilen - (OCTASTRLEN-1), SEEK_CUR);
+                    if(!strcmp(name, "cloudblend")) copystring(name, "cloudlayerblend");
                     if(!strcmp(name, "cloudalpha")) copystring(name, "cloudblend");
-                    if(!strcmp(name, "cloudcolour")) copystring(name, "cloudlayercolour");
                     if(!strcmp(name, "grassalpha")) copystring(name, "grassblend");
+                    if(!strcmp(name, "skyboxcolour")) copystring(name, "skycolour");
+                    if(!strcmp(name, "cloudcolour")) copystring(name, "cloudlayercolour");
+                    if(!strcmp(name, "cloudboxcolour")) copystring(name, "cloudcolour");
                     ident *id = getident(name);
-                    bool exists = id && id->type == type;
+                    bool exists = id && id->type == type && id->flags&IDF_WORLD && !(id->flags&IDF_SERVER);
                     switch(type)
                     {
                         case ID_VAR:
@@ -1470,6 +1516,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
             else
             {
                 delete f;
+                maskpackagedirs(mask);
                 continue;
             }
 
@@ -1495,8 +1542,8 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
             vector<extentity *> &ents = entities::getents();
             loopi(hdr.numents)
             {
-                if(verbose) progress(float(i)/float(hdr.numents), "loading entities...");
-                extentity &e = *entities::newent(); ents.add(&e);
+                progress(i/float(hdr.numents), "loading entities...");
+                extentity &e = *ents.add(entities::newent());
                 if(maptype == MAP_OCTA || (maptype == MAP_MAPZ && hdr.version <= 36))
                 {
                     entcompat ec;
@@ -1513,7 +1560,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                     f->read(&e, sizeof(entbase));
                     lilswap(&e.o.x, 3);
                     int numattr = f->getlil<int>();
-                    e.attrs.add(0, clamp(numattr, 5, MAXENTATTRS));
+                    e.attrs.add(0, clamp(numattr, entities::numattrs(e.type), MAXENTATTRS));
                     loopk(numattr)
                     {
                         int val = f->getlil<int>();
@@ -1545,7 +1592,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                         int links = f->getlil<int>();
                         f->seek(sizeof(int)*links, SEEK_CUR);
                     }
-                    entities::deleteent(ents.pop());
+                    e.type = ET_EMPTY;
                     continue;
                 }
                 entities::readent(f, maptype, hdr.version, hdr.gameid, hdr.gamever, i);
@@ -1575,7 +1622,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                         int angle = e.attrs[0];
                         e.attrs[0] = e.attrs[1];
                         e.attrs[1] = angle;
-                        loopi(e.attrs.length()-2) e.attrs[i+2] = 0;
+                        loopk(e.attrs.length()-2) e.attrs[k+2] = 0;
                     }
                     if(maptype == MAP_MAPZ && hdr.version <= 37)
                     {
@@ -1583,17 +1630,36 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                         e.attrs[5] = e.attrs[4];
                         e.attrs[3] = e.attrs[4] = 0;
                     }
+                    if(maptype == MAP_MAPZ && hdr.gamever <= 219)
+                    { // insert pitch at index 2 between yaw and roll
+                        e.attrs.insert(2, 0);
+                        if(e.attrs.length() > MAXENTATTRS) e.attrs.setsize(MAXENTATTRS);
+                    }
                 }
                 if(e.type == ET_SUNLIGHT && hdr.version <= 38) e.attrs[1] -= 90; // reorient pitch axis
                 if((maptype == MAP_OCTA && hdr.version <= 30) || (maptype == MAP_MAPZ && hdr.version <= 39)) switch(e.type)
                 {
-                    case ET_MAPMODEL: case ET_PLAYERSTART: e.attrs[1] = (e.attrs[1] + 180)%360; break;
+                    case ET_PLAYERSTART: case ET_MAPMODEL: e.attrs[1] = (e.attrs[1] + 180)%360; break;
                     case ET_SUNLIGHT: e.attrs[0] = (e.attrs[0] + 180)%360; break;
+                    default: break;
                 }
                 if(verbose && !insideworld(e.o) && e.type != ET_LIGHT && e.type != ET_LIGHTFX && e.type != ET_SUNLIGHT)
                     conoutf("\frWARNING: ent outside of world: enttype[%s] index %d (%f, %f, %f)", entities::findname(e.type), i, e.o.x, e.o.y, e.o.z);
             }
             if(verbose) conoutf("\faloaded %d entities", hdr.numents);
+            if(maptype == MAP_OCTA && sunlight)
+            {
+                extentity &e = *ents.add(entities::newent());
+                e.attrs.add(0, 6);
+                e.type = ET_SUNLIGHT;
+                e.o = vec(hdr.worldsize/2, hdr.worldsize/2, hdr.worldsize*3/4);
+                e.attrs[0] = sunlightyaw;
+                e.attrs[1] = sunlightpitch-90;
+                e.attrs[2] = int(((sunlight>>16)&0xFF)*sunlightscale*5/8);
+                e.attrs[3] = int(((sunlight>>8)&0xFF)*sunlightscale*5/8);
+                e.attrs[4] = int((sunlight&0xFF)*sunlightscale*5/8);
+                e.attrs[5] = -1;
+            }
 
             progress(0, "loading slots...");
             loadvslots(f, hdr.numvslots);
@@ -1613,7 +1679,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
             {
                 if(hdr.version >= 7) loopi(hdr.lightmaps)
                 {
-                    if(verbose) progress(i/(float)hdr.lightmaps, "loading lightmaps...");
+                    progress(i/(float)hdr.lightmaps, "loading lightmaps...");
                     LightMap &lm = lightmaps.add();
                     if(hdr.version >= 17)
                     {
@@ -1637,27 +1703,7 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                 if(verbose) conoutf("\faloaded %d lightmaps", hdr.lightmaps);
             }
 
-            progress(0, "loading world...");
-            game::loadworld(f, maptype);
-            entities::initents(f, maptype, hdr.version, hdr.gameid, hdr.gamever);
-
-            mapcrc = f->getcrc();
-
-            identflags |= IDF_WORLD;
-            defformatstring(cfgname)("%s.cfg", mapname);
-            if(maptype == MAP_OCTA)
-            {
-                execfile("octa.cfg"); // for use with -pSAUER_DIR
-                execfile(cfgname);
-            }
-            else if(!execfile(cfgname, false)) execfile("map.cfg");
-            if(maptype == MAP_OCTA || (maptype == MAP_MAPZ && hdr.version <= 34))
-            {
-                extern float cloudblend;
-                setfvar("cloudlayerblend", cloudblend, true);
-            }
-            identflags &= ~IDF_WORLD;
-
+            progress(0, "initialising entities...");
             loopv(ents)
             {
                 extentity &e = *ents[i];
@@ -1685,12 +1731,27 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
                     }
                 }
             }
+            entities::initents(f, maptype, hdr.version, hdr.gameid, hdr.gamever);
 
+            progress(0, "initialising config...");
+            mapcrc = f->getcrc();
+            identflags |= IDF_WORLD;
+            defformatstring(cfgname)("%s.cfg", mapname);
+            if(maptype == MAP_OCTA)
+            {
+                execfile("config/map/octa.cfg"); // for use with -pSAUER_DIR
+                execfile(cfgname);
+            }
+            else if(!execfile(cfgname, false)) execfile("config/map/default.cfg");
+            identflags &= ~IDF_WORLD;
+
+            progress(0, "preloading models...");
             preloadusedmapmodels(true);
 
             delete f;
             conoutf("\faloaded map %s v.%d:%d (r%d) in %.1f secs", mapname, hdr.version, hdr.gamever, hdr.revision, (SDL_GetTicks()-loadingstart)/1000.0f);
 
+            progress(0, "checking world...");
             if((maptype == MAP_OCTA && hdr.version <= 25) || (maptype == MAP_MAPZ && hdr.version <= 26))
                 fixlightmapnormals();
             if((maptype == MAP_OCTA && hdr.version <= 31) || (maptype == MAP_MAPZ && hdr.version <= 40))
@@ -1700,7 +1761,9 @@ bool load_world(const char *mname, bool temp)       // still supports all map fo
             initlights();
             allchanged(true);
 
+            progress(0, "preloading textures...");
             preloadtextures(IDF_GAMEPRELOAD);
+            maskpackagedirs(mask);
 
             progress(0, "starting world...");
             game::startmap(mapname, mname);
