@@ -3,34 +3,70 @@ namespace bomber
 {
     bomberstate st;
 
+    void killed(gameent *d, gameent *actor)
+    {
+        if(actor && m_gsp1(game::gamemode, game::mutators) && (!m_team(game::gamemode, game::mutators) || d->team != actor->team))
+        {
+            loopv(st.flags) if(isbomberaffinity(st.flags[i]) && st.flags[i].owner == actor)
+                st.flags[i].taketime = lastmillis;
+        }
+    }
+
     bool carryaffinity(gameent *d)
     {
         loopv(st.flags) if(st.flags[i].owner == d) return true;
         return false;
     }
 
+    float fixrot(float c)
+    {
+        float angle = c;
+        while(angle < 0.0f) angle += 360.0f;
+        while(angle >= 360.0f) angle -= 360.0f;
+        return angle;
+    }
+
+    float offrot(float a, float b)
+    {
+        float angle = fixrot(a)-fixrot(b);
+        while(angle < -180.0f) angle += 360.0f;
+        while(angle >= 180.0f) angle -= 360.0f;
+        return fabs(angle);
+    }
+
+    VAR(IDF_PERSIST, bombertargetintersect, 0, 1, 1);
+    VAR(IDF_PERSIST, bombertargetangle, 0, 1, 1);
     int findtarget(gameent *d)
     {
-        float bestdist = 1e16f;
+        vec dest;
         gameent *e = NULL;
+        float bestangle = 1e16f, bestdist = 1e16f;
         int best = -1;
-        vec targ;
         int numdyns = game::numdynents();
         loopk(d->aitype != AI_NONE ? 4 : 2)
         {
-            loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && e->team == d->team && e->state == CS_ALIVE && (k%2 ? d->aitype == AI_BOT : d->aitype == AI_NONE))
+            if(bombertargetintersect)
             {
-                float fx = k >= 2 ? 360 : (d->ai ? d->ai->views[0] : curfov), fy = k >= 2 ? 360 : (d->ai ? d->ai->views[1] : fovy);
-                if(getsight(d->o, d->yaw, d->pitch, e->o, targ, bestdist, fx, fy))
+                findorientation(d->o, d->yaw, d->pitch, dest);
+                if((e = game::intersectclosest(d->o, dest, d)) && e->team == d->team && e->state == CS_ALIVE && (k%2 || e->aitype != AI_BOT))
+                    return e->clientnum;
+            }
+            float fx = k >= 2 ? 360 : (d->ai ? d->ai->views[0] : curfov), fy = k >= 2 ? 360 : (d->ai ? d->ai->views[1] : fovy);
+            loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && e->team == d->team && e->state == CS_ALIVE && (k%2 || e->aitype != AI_BOT))
+            {
+                if(getsight(d->o, d->yaw, d->pitch, e->o, dest, 1e16f, fx, fy))
                 {
-                    vec dir = vec(e->o).sub(d->o).normalize();
-                    float yaw, pitch; vectoyawpitch(dir, yaw, pitch);
-                    while(yaw < 0) yaw += 360; while(yaw >= 360) yaw -= 360;
-                    float dist = fabs(d->yaw-yaw);
-                    if(dist < bestdist)
+                    vec dir = vec(e->o).sub(d->o);
+                    float dist = dir.magnitude();
+                    if(dist > 1e-3f) dir.div(dist);
+                    float yaw = 0, pitch = 0;
+                    vectoyawpitch(dir, yaw, pitch);
+                    float offyaw = offrot(d->yaw, yaw), offpitch = offrot(d->pitch, pitch), offangle = offpitch+offyaw;
+                    if(best < 0 || (bombertargetangle ? (offangle < bestangle || (offangle <= bestangle && dist < bestdist)) : (dist < bestdist || (dist <= bestdist && offangle < bestangle))))
                     {
                         best = e->clientnum;
                         bestdist = dist;
+                        bestangle = offangle;
                     }
                 }
             }
@@ -47,7 +83,7 @@ namespace bomber
             vec o = d->feetpos(1), inertia;
             vecfromyawpitch(d->yaw, d->pitch, 1, 0, inertia);
             inertia.normalize().mul(bomberspeed).add(vec(d->vel).add(d->falling).mul(bomberrelativity));
-            bool guided = m_isteam(game::gamemode, game::mutators) && bomberlockondelay && lastmillis-d->actiontime[AC_AFFINITY] >= bomberlockondelay;
+            bool guided = m_team(game::gamemode, game::mutators) && bomberlockondelay && lastmillis-d->actiontime[AC_AFFINITY] >= bomberlockondelay;
             client::addmsg(N_DROPAFFIN, "ri8", d->clientnum, guided ? findtarget(d) : -1, int(o.x*DMF), int(o.y*DMF), int(o.z*DMF), int(inertia.x*DMF), int(inertia.y*DMF), int(inertia.z*DMF));
             d->action[AC_AFFINITY] = false;
             d->actiontime[AC_AFFINITY] = 0;
@@ -58,13 +94,13 @@ namespace bomber
 
     void preload()
     {
-        loadmodel("ball", -1, true);
+        preloadmodel("props/ball");
     }
 
     vec pulsecolour()
     {
         uint n = lastmillis/100;
-        return vec::hexcolor(pulsecols[2][n%PULSECOLOURS]).lerp(vec::hexcolor(pulsecols[2][(n+1)%PULSECOLOURS]), (lastmillis%100)/100.0f);
+        return vec::hexcolor(pulsecols[PULSE_DISCO][n%PULSECOLOURS]).lerp(vec::hexcolor(pulsecols[PULSE_DISCO][(n+1)%PULSECOLOURS]), (lastmillis%100)/100.0f);
     }
 
     void drawblips(int w, int h, float blend)
@@ -97,24 +133,38 @@ namespace bomber
 
     void drawnotices(int w, int h, int &tx, int &ty, float blend)
     {
-        if(game::player1->state == CS_ALIVE && hud::shownotices >= 3)
+        if(game::focus->state == CS_ALIVE && hud::shownotices >= 2)
         {
+            if(game::focus->lastbuff && hud::shownotices >= 3)
+            {
+                pushfont("reduced");
+                if(m_regen(game::gamemode, game::mutators) && bomberregenbuff && bomberregenextra)
+                    ty += draw_textx("Buffing: \fs\fo%d%%\fS damage, \fs\fg%d%%\fS shield, +\fs\fy%d\fS regen", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, int(bomberbuffdamage*100), int(bomberbuffshield*100), bomberregenextra)*hud::noticescale;
+                else ty += draw_textx("Buffing: \fs\fo%d%%\fS damage, \fs\fg%d%%\fS shield", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, int(bomberbuffdamage*100), int(bomberbuffshield*100))*hud::noticescale;
+                popfont();
+            }
             loopv(st.flags)
             {
                 bomberstate::flag &f = st.flags[i];
-                if(f.owner == game::player1)
+                if(f.owner == game::focus)
                 {
                     pushfont("emphasis");
-                    ty += draw_textx("You have: \fs\f[%d]\f(%s)bomb\fS", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, pulsecols[2][clamp((lastmillis/100)%PULSECOLOURS, 0, PULSECOLOURS-1)], hud::bombtex)*hud::noticescale;
+                    ty += draw_textx("Holding: \fs\f[%d]\f(%s)bomb\fS", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, pulsecols[PULSE_DISCO][clamp((lastmillis/100)%PULSECOLOURS, 0, PULSECOLOURS-1)], hud::bombtex)*hud::noticescale;
                     popfont();
-                    if(bombercarrytime)
+                    if(carrytime)
                     {
-                        int delay = bombercarrytime-(lastmillis-f.taketime);
+                        int delay = carrytime-(lastmillis-f.taketime);
                         pushfont("default");
-                        ty += draw_textx("Explodes in \fs\fzgy%s\fS", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, hud::timetostr(delay, -1))*hud::noticescale;
+                        ty += draw_textx("Explodes in \fs\fzgy%s\fS", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, timestr(delay, -1))*hud::noticescale;
                         popfont();
+                        if(m_gsp1(game::gamemode, game::mutators))
+                        {
+                            pushfont("reduced");
+                            ty += draw_textx("Killing enemies resets fuse timer", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED)*hud::noticescale;
+                            popfont();
+                        }
                     }
-                    SEARCHBINDCACHE(altkey)("action 9", 0);
+                    SEARCHBINDCACHE(altkey)("affinity", 0);
                     pushfont("reduced");
                     ty += draw_textx("Press \fs\fc%s\fS to throw", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, altkey)*hud::noticescale;
                     popfont();
@@ -146,8 +196,11 @@ namespace bomber
                 else skew = 1;
             }
             else if(millis <= 1000) skew += ((1.f-skew)-(clamp(float(millis)/1000.f, 0.f, 1.f)*(1.f-skew)));
+            float wait = f.droptime ? clamp((lastmillis-f.droptime)/float(bomberresetdelay), 0.f, 1.f) : (f.owner ? clamp((lastmillis-f.taketime)/float(carrytime), 0.f, 1.f) : 1.f);
             int oldy = y-sy;
-            sy += hud::drawitem(hud::bombtex, x, oldy, s, 0, true, false, colour.x, colour.y, colour.z, blend, skew);
+            if(!game::intermission && (f.owner || f.droptime))
+                sy += hud::drawitem(hud::bombtex, x, oldy, s, 0, true, false, colour.x, colour.y, colour.z, blend, skew, "super", "%d%%", int(wait*100.f));
+            else sy += hud::drawitem(hud::bombtex, x, oldy, s, 0, true, false, colour.x, colour.y, colour.z, blend, skew);
             if(f.owner)
             {
                 vec c2 = vec::hexcolor(TEAM(f.owner->team, colour));
@@ -156,20 +209,17 @@ namespace bomber
             else if(f.droptime) hud::drawitem(hud::bombdroptex, x, oldy, s, 0.5f, true, false, 0.25f, 1.f, 1.f, blend, skew);
             if(!game::intermission)
             {
-                if(f.droptime || (f.owner && bombercarrytime))
+                if(f.droptime || (f.owner && carrytime))
                 {
-                    int sx = x-int(s*skew);
-                    float wait = f.droptime ? clamp((lastmillis-f.droptime)/float(bomberresetdelay), 0.f, 1.f) : clamp((lastmillis-f.taketime)/float(bombercarrytime), 0.f, 1.f);
                     if(wait > 0.5f)
                     {
                         int delay = wait > 0.7f ? (wait > 0.85f ? 150 : 300) : 600, millis = lastmillis%(delay*2);
                         float amt = (millis <= delay ? millis/float(delay) : 1.f-((millis-delay)/float(delay)));
                         flashcolour(colour.r, colour.g, colour.b, 1.f, 0.f, 0.f, amt);
                     }
-                    if(wait < 1) hud::drawprogress(sx, oldy, wait, 1-wait, s, false, colour.r, colour.g, colour.b, blend*0.25f, skew);
-                    hud::drawprogress(sx, oldy, 0, wait, s, false, colour.r, colour.g, colour.b, blend, skew, "super", "%d%%", int(wait*100.f));
+                    hud::drawitembar(x, oldy, s, false, colour.r, colour.g, colour.b, blend, skew, wait);
                 }
-                if(f.owner == game::focus && m_isteam(game::gamemode, game::mutators) && bomberlockondelay && f.owner->action[AC_AFFINITY] && lastmillis-f.owner->actiontime[AC_AFFINITY] >= bomberlockondelay)
+                if(f.owner == game::focus && m_team(game::gamemode, game::mutators) && bomberlockondelay && f.owner->action[AC_AFFINITY] && lastmillis-f.owner->actiontime[AC_AFFINITY] >= bomberlockondelay)
                 {
                     gameent *e = game::getclient(findtarget(f.owner));
                     float cx = 0.5f, cy = 0.5f, cz = 1;
@@ -239,7 +289,7 @@ namespace bomber
         loopv(st.flags) // flags/bases
         {
             bomberstate::flag &f = st.flags[i];
-            if(!entities::ents.inrange(f.ent) || !f.enabled || (f.owner == game::focus && !game::thirdpersonview(true))) continue;
+            if(!entities::ents.inrange(f.ent) || !f.enabled || (f.owner == game::focus && !game::thirdpersonview(true) && rendernormally)) continue;
             vec above(f.pos(true, true));
             float trans = isbomberaffinity(f) ? 1.f : 0.5f;
             if(!isbomberaffinity(f) || !f.interptime) // || (!f.droptime && !f.owner))
@@ -253,8 +303,8 @@ namespace bomber
                 {
                     if(!f.owner && !f.droptime) above.z += enttype[AFFINITY].radius/4*trans;
                     entitylight *light = &entities::ents[f.ent]->light;
-                    float yaw = !f.owner && f.proj ? f.proj->yaw : (lastmillis/4)%360, pitch = !f.owner && f.proj ? f.proj->pitch : 0, roll = !f.owner && f.proj ? f.proj->roll : 0,
-                          wait = f.droptime ? clamp((lastmillis-f.droptime)/float(bomberresetdelay), 0.f, 1.f) : ((f.owner && bombercarrytime) ? clamp((lastmillis-f.taketime)/float(bombercarrytime), 0.f, 1.f) : 0.f);
+                    float size = trans, yaw = !f.owner && f.proj ? f.proj->yaw : (lastmillis/4)%360, pitch = !f.owner && f.proj ? f.proj->pitch : 0, roll = !f.owner && f.proj ? f.proj->roll : 0,
+                          wait = f.droptime ? clamp((lastmillis-f.droptime)/float(bomberresetdelay), 0.f, 1.f) : ((f.owner && carrytime) ? clamp((lastmillis-f.taketime)/float(carrytime), 0.f, 1.f) : 0.f);
                     int interval = lastmillis%1000;
                     vec effect = pulsecolour();
                     if(wait > 0.5f)
@@ -264,7 +314,8 @@ namespace bomber
                         flashcolour(effect.r, effect.g, effect.b, 1.f, 0.f, 0.f, amt);
                     }
                     light->material[0] = bvec::fromcolor(effect);
-                    rendermodel(light, "ball", ANIM_MAPMODEL|ANIM_LOOP, above, yaw, pitch, roll, MDL_DYNSHADOW|MDL_CULL_VFC|MDL_CULL_OCCLUDED|MDL_LIGHTFX, NULL, NULL, 0, 0, trans, trans);
+                    if(f.owner == game::focus && game::thirdpersonview(true)) trans *= 0.5f;
+                    rendermodel(light, "props/ball", ANIM_MAPMODEL|ANIM_LOOP, above, yaw, pitch, roll, MDL_DYNSHADOW|MDL_CULL_VFC|MDL_CULL_OCCLUDED|MDL_LIGHTFX, NULL, NULL, 0, 0, trans, size);
                     float fluc = interval >= 500 ? (1500-interval)/1000.f : (500+interval)/1000.f;
                     int pcolour = (int(light->material[0].x)<<16)|(int(light->material[0].y)<<8)|int(light->material[0].z);
                     part_create(PART_HINT_SOFT, 1, above, pcolour, enttype[AFFINITY].radius/4*trans+(2*fluc), fluc*trans);
@@ -281,7 +332,7 @@ namespace bomber
                 else if(!m_gsp1(game::gamemode, game::mutators))
                 {
                     part_explosion(above, enttype[AFFINITY].radius*trans, PART_SHOCKWAVE, 1, TEAM(f.team, colour), 1.f, trans*0.25f);
-                    part_explosion(above, enttype[AFFINITY].radius/2*trans, PART_SHOCKBALL, 1, TEAM(f.team, colour), 1.f, trans*0.65f);
+                    part_explosion(above, enttype[AFFINITY].radius/3*trans, PART_SHOCKBALL, 1, TEAM(f.team, colour), 1.f, trans*0.5f);
                     above.z += enttype[AFFINITY].radius*trans+2.5f;
                     defformatstring(info)("<super>%s base", TEAM(f.team, name));
                     part_textcopy(above, info, PART_TEXT, 1, TEAM(f.team, colour), 2, 1);
@@ -316,7 +367,7 @@ namespace bomber
         loopv(entities::ents) if(entities::ents[i]->type == AFFINITY)
         {
             gameentity &e = *(gameentity *)entities::ents[i];
-            if(!m_check(e.attrs[3], e.attrs[4], game::gamemode, game::mutators) || !isteam(game::gamemode, game::mutators, e.attrs[0], TEAM_NEUTRAL))
+            if(!m_check(e.attrs[3], e.attrs[4], game::gamemode, game::mutators) || !isteam(game::gamemode, game::mutators, e.attrs[0], T_NEUTRAL))
                 continue;
             st.addaffinity(e.o, e.attrs[0], i);
         }
@@ -410,14 +461,13 @@ namespace bomber
 
     void destroyaffinity(const vec &o)
     {
-        float radius = max(WEAPEX(WEAP_GRENADE, false, game::gamemode, game::mutators, 1), enttype[AFFINITY].radius);
-        part_create(PART_PLASMA_SOFT, 250, o, 0xAA4400, radius*0.5f, 0.5f);
-        part_explosion(o, radius, PART_EXPLOSION, 500, 0xAA4400, 1.f, 0.5f);
-        part_explosion(o, radius*2, PART_SHOCKWAVE, 250, 0xAA4400, 1.f, 0.1f);
-        part_create(PART_SMOKE_LERP_SOFT, 500, o, 0x333333, radius*0.75f, 0.5f, -15);
+        part_create(PART_PLASMA_SOFT, 250, o, 0xAA4400, enttype[AFFINITY].radius*0.5f, 0.5f);
+        part_explosion(o, enttype[AFFINITY].radius, PART_EXPLOSION, 500, 0xAA4400, 1.f, 0.5f);
+        part_explosion(o, enttype[AFFINITY].radius*2, PART_SHOCKWAVE, 250, 0xAA4400, 1.f, 0.1f);
+        part_create(PART_SMOKE_LERP_SOFT, 500, o, 0x333333, enttype[AFFINITY].radius*0.75f, 0.5f, -15);
         int debris = rnd(5)+5, amt = int((rnd(debris)+debris+1)*game::debrisscale);
         loopi(amt) projs::create(o, o, true, NULL, PRJ_DEBRIS, rnd(game::debrisfade)+game::debrisfade, 0, rnd(501), rnd(101)+50);
-        playsound(WEAPSND2(WEAP_GRENADE, false, S_W_EXPLODE), o, NULL, 0, 255);
+        playsound(WSND2(W_GRENADE, false, S_W_EXPLODE), o, NULL, 0, 255);
     }
 
     void resetaffinity(int i, int value)
@@ -432,8 +482,8 @@ namespace bomber
             {
                 if(value == 2)
                 {
-                    affinityeffect(i, TEAM_NEUTRAL, f.pos(true, true), f.spawnloc, 3, "RESET");
-                    game::announcef(S_V_BOMBRESET, CON_INFO, NULL, "\fathe \fs\fwbomb\fS has been reset");
+                    affinityeffect(i, T_NEUTRAL, f.pos(true, true), f.spawnloc, 3, "RESET");
+                    game::announcef(S_V_BOMBRESET, CON_INFO, NULL, true, "\fathe \fs\fwbomb\fS has been reset");
                     isreset = true;
                 }
                 entities::execlink(NULL, f.ent, false);
@@ -451,7 +501,8 @@ namespace bomber
         entities::execlink(NULL, f.ent, false);
         entities::execlink(NULL, g.ent, false);
         hud::teamscore(d->team).total = score;
-        game::announcef(S_V_BOMBSCORE, d == game::focus ? CON_SELF : CON_INFO, d, "\fa%s destroyed the \fs\f[%d]%s\fS base for team \fs\f[%d]%s\fS (score: \fs\fc%d\fS, time taken: \fs\fc%s\fS)", game::colorname(d), TEAM(g.team, colour), TEAM(g.team, name), TEAM(d->team, colour), TEAM(d->team, name), score, hud::timetostr(lastmillis-f.inittime));
+        defformatstring(gteam)("%s", game::colourteam(g.team, "bombtex"));
+        game::announcef(S_V_BOMBSCORE, CON_INFO, d, true, "\fa%s destroyed the %s base for team %s (score: \fs\fc%d\fS, time taken: \fs\fc%s\fS)", game::colourname(d), gteam, game::colourteam(d->team), score, timestr(lastmillis-f.inittime));
         st.returnaffinity(relay, lastmillis, false);
     }
 
@@ -465,14 +516,16 @@ namespace bomber
         if(!f.droptime)
         {
             affinityeffect(i, d->team, d->feetpos(), f.pos(true, true), 1, "TAKEN");
-            game::announcef(S_V_BOMBPICKUP, d == game::focus ? CON_SELF : CON_INFO, d, "\fa%s picked up the \fs\fwbomb\fS", game::colorname(d));
+            game::announcef(S_V_BOMBPICKUP, CON_INFO, d, true, "\fa%s picked up the \fs\fzwv\f($bombtex)bomb\fS", game::colourname(d));
             entities::execlink(NULL, f.ent, false);
         }
         st.takeaffinity(i, d, lastmillis);
     }
 
-    void checkaffinity(gameent *d)
+    void checkaffinity(dynent *e)
     {
+        if(e->state != CS_ALIVE || !gameent::is(e)) return;
+        gameent *d = (gameent *)e;
         vec o = d->feetpos();
         loopv(st.flags)
         {
@@ -481,11 +534,11 @@ namespace bomber
             if(f.owner)
             {
                 int takemillis = lastmillis-f.taketime;
-                if(bombercarrytime && d->ai && f.owner == d && takemillis >= bombercarrytime-550-bomberlockondelay)
+                if(carrytime && d->ai && f.owner == d && takemillis >= carrytime-550-bomberlockondelay)
                 {
                     if(d->action[AC_AFFINITY])
                     {
-                        if(takemillis >= bombercarrytime-500 || lastmillis-d->actiontime[AC_AFFINITY] >= bomberlockondelay)
+                        if(takemillis >= carrytime-500 || lastmillis-d->actiontime[AC_AFFINITY] >= bomberlockondelay)
                             d->action[AC_AFFINITY] = false;
                     }
                     else
@@ -520,7 +573,7 @@ namespace bomber
     bool aihomerun(gameent *d, ai::aistate &b)
     {
         vec pos = d->feetpos();
-        if(m_isteam(game::gamemode, game::mutators) && !m_gsp1(game::gamemode, game::mutators))
+        if(m_team(game::gamemode, game::mutators) && !m_gsp1(game::gamemode, game::mutators))
         {
             int goal = -1;
             loopv(st.flags)
@@ -583,17 +636,19 @@ namespace bomber
         {
             bomberstate::flag &f = st.flags[j];
             if(!entities::ents.inrange(f.ent) || !f.enabled) continue;
-            bool home = isbomberhome(f, ai::owner(d));
+            int owner = ai::owner(d);
+            bool home = isbomberhome(f, owner) || isbombertarg(f, owner);
+            if(d->aitype == AI_BOT && m_duke(game::gamemode, game::mutators) && home) continue;
             static vector<int> targets; // build a list of others who are interested in this
             targets.setsize(0);
-            bool regen = d->aitype != AI_BOT || f.team == TEAM_NEUTRAL || !m_regen(game::gamemode, game::mutators) || d->health >= m_health(game::gamemode, game::mutators);
+            bool regen = d->aitype != AI_BOT || f.team != owner || !m_regen(game::gamemode, game::mutators) || d->health >= m_health(game::gamemode, game::mutators, d->model);
             ai::checkothers(targets, d, home || d->aitype != AI_BOT ? ai::AI_S_DEFEND : ai::AI_S_PURSUE, ai::AI_T_AFFINITY, j, true);
             if(d->aitype == AI_BOT)
             {
                 gameent *e = NULL;
                 int numdyns = game::numdynents();
                 float mindist = enttype[AFFINITY].radius*4; mindist *= mindist;
-                loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && !e->ai && e->state == CS_ALIVE && ai::owner(d) == ai::owner(e))
+                loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && !e->ai && e->state == CS_ALIVE && owner == ai::owner(e))
                 {
                     if(targets.find(e->clientnum) < 0 && (f.owner == e || e->feetpos().squaredist(f.pos()) <= mindist))
                         targets.add(e->clientnum);
@@ -608,7 +663,7 @@ namespace bomber
                     gameent *t;
                     loopvk(targets) if((t = game::getclient(targets[k])))
                     {
-                        if((t->ai && !t->hasweap(t->ai->weappref, m_weapon(game::gamemode, game::mutators))) || (!t->ai && t->weapselect < WEAP_OFFSET))
+                        if((t->ai && !t->hasweap(t->ai->weappref, m_weapon(game::gamemode, game::mutators))) || (!t->ai && t->weapselect < W_OFFSET))
                         {
                             guard = true;
                             break;
@@ -623,7 +678,7 @@ namespace bomber
                     n.target = j;
                     n.targtype = ai::AI_T_AFFINITY;
                     n.score = pos.squaredist(f.pos())/(!regen ? 100.f : 1.f);
-                    n.tolerance = 0.25f;
+                    n.tolerance = f.team != owner ? 0.5f : 0.25f;
                     n.team = true;
                 }
             }
@@ -637,7 +692,7 @@ namespace bomber
                     n.target = j;
                     n.targtype = ai::AI_T_AFFINITY;
                     n.score = pos.squaredist(f.pos());
-                    n.tolerance = 0.25f;
+                    n.tolerance = 0.5f;
                     n.team = true;
                 }
                 else
@@ -646,13 +701,14 @@ namespace bomber
                     loopvk(targets) if((t = game::getclient(targets[k])))
                     {
                         ai::interest &n = interests.add();
-                        bool team = ai::owner(d) == ai::owner(t);
+                        bool team = owner == ai::owner(t);
+                        if(d->aitype == AI_BOT && m_duke(game::gamemode, game::mutators) && team) continue;
                         n.state = team ? ai::AI_S_DEFEND : ai::AI_S_PURSUE;
                         n.node = t->lastnode;
                         n.target = t->clientnum;
                         n.targtype = ai::AI_T_ACTOR;
                         n.score = d->o.squaredist(t->o);
-                        n.tolerance = 0.25f;
+                        n.tolerance = 0.5f;
                         n.team = team;
                     }
                 }
@@ -663,7 +719,10 @@ namespace bomber
     bool aidefense(gameent *d, ai::aistate &b)
     {
         if(d->aitype == AI_BOT)
+        {
             loopv(st.flags) if(st.flags[i].owner == d) return aihomerun(d, b);
+            if(m_duke(game::gamemode, game::mutators)) return false;
+        }
         if(st.flags.inrange(b.target))
         {
             bomberstate::flag &f = st.flags[b.target];
@@ -672,7 +731,7 @@ namespace bomber
             int walk = f.owner && ai::owner(f.owner) != ai::owner(d) ? 1 : 0;
             if(d->aitype == AI_BOT)
             {
-                bool regen = !m_regen(game::gamemode, game::mutators) || d->health >= m_health(game::gamemode, game::mutators);
+                bool regen = !m_regen(game::gamemode, game::mutators) || d->health >= m_health(game::gamemode, game::mutators, d->model);
                 if(regen && lastmillis-b.millis >= (201-d->skill)*33)
                 {
                     static vector<int> targets; // build a list of others who are interested in this
@@ -720,7 +779,6 @@ namespace bomber
         {
             bomberstate::flag &f = st.flags[b.target];
             if(!entities::ents.inrange(f.ent) || !f.enabled) return false;
-            b.idle = -1;
             if(isbomberaffinity(f))
             {
                 if(f.owner)
