@@ -24,7 +24,7 @@ namespace capture
 
     void preload()
     {
-        loadmodel("flag", -1, true);
+        preloadmodel("props/flag");
     }
 
     void drawblips(int w, int h, float blend)
@@ -84,17 +84,25 @@ namespace capture
 
     void drawnotices(int w, int h, int &tx, int &ty, float blend)
     {
-        if(game::player1->state == CS_ALIVE && hud::shownotices >= 3)
+        if(game::focus->state == CS_ALIVE && hud::shownotices >= 2)
         {
+            if(game::focus->lastbuff && hud::shownotices >= 3)
+            {
+                pushfont("reduced");
+                if(m_regen(game::gamemode, game::mutators) && captureregenbuff && captureregenextra)
+                    ty += draw_textx("Buffing: \fs\fo%d%%\fS damage, \fs\fg%d%%\fS shield, +\fs\fy%d\fS regen", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, int(capturebuffdamage*100), int(capturebuffshield*100), captureregenextra)*hud::noticescale;
+                else ty += draw_textx("Buffing: \fs\fo%d%%\fS damage, \fs\fg%d%%\fS shield", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, int(capturebuffdamage*100), int(capturebuffshield*100))*hud::noticescale;
+                popfont();
+            }
             static vector<int> hasflags, taken, droppedflags;
             hasflags.setsize(0); taken.setsize(0); droppedflags.setsize(0);
             loopv(st.flags)
             {
                 capturestate::flag &f = st.flags[i];
-                if(f.owner == game::player1) hasflags.add(i);
-                else if(f.team == game::player1->team)
+                if(f.owner == game::focus) hasflags.add(i);
+                else if(f.team == game::focus->team)
                 {
-                    if(f.owner && f.owner->team != game::player1->team) taken.add(i);
+                    if(f.owner && f.owner->team != game::focus->team) taken.add(i);
                     else if(f.droptime) droppedflags.add(i);
                 }
             }
@@ -102,9 +110,9 @@ namespace capture
             {
                 pushfont("emphasis");
                 char *str = buildflagstr(hasflags, hasflags.length() <= 3);
-                ty += draw_textx("You have: \fs%s\fS", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, str)*hud::noticescale;
+                ty += draw_textx("Holding: \fs%s\fS", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, str)*hud::noticescale;
                 popfont();
-                SEARCHBINDCACHE(altkey)("action 9", 0);
+                SEARCHBINDCACHE(altkey)("affinity", 0);
                 pushfont("reduced");
                 ty += draw_textx("Press \fs\fc%s\fS to drop", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, altkey)*hud::noticescale;
                 popfont();
@@ -131,7 +139,7 @@ namespace capture
         {
             if(y-sy-s < m) break;
             capturestate::flag &f = st.flags[i];
-            bool headsup = hud::chkcond(hud::inventorygame, game::player1->state == CS_SPECTATOR || f.team == TEAM_NEUTRAL || f.team == game::focus->team);
+            bool headsup = hud::chkcond(hud::inventorygame, game::player1->state == CS_SPECTATOR || f.team == T_NEUTRAL || f.team == game::focus->team);
             if(headsup || f.lastowner == game::focus)
             {
                 int millis = lastmillis-f.displaytime, colour = TEAM(f.team, colour);
@@ -159,7 +167,6 @@ namespace capture
                 else hud::drawitem(hud::teamtexname(f.team), x, oldy, s, 0.5f, true, false, c.r, c.g, c.b, blend, skew);
                 if(!game::intermission && (f.droptime || (m_gsp3(game::gamemode, game::mutators) && f.taketime && f.owner && f.owner->team != f.team)))
                 {
-                    int sx = x-int(s*skew);
                     float wait = f.droptime ? clamp((lastmillis-f.droptime)/float(capturedelay), 0.f, 1.f) : clamp((lastmillis-f.taketime)/float(captureprotectdelay), 0.f, 1.f);
                     if(wait > 0.5f)
                     {
@@ -167,8 +174,7 @@ namespace capture
                         float amt = (millis <= delay ? millis/float(delay) : 1.f-((millis-delay)/float(delay)));
                         flashcolour(c.r, c.g, c.b, 0.65f, 0.65f, 0.65f, amt);
                     }
-                    if(wait < 1) hud::drawprogress(sx, oldy, wait, 1-wait, s, false, c.r, c.g, c.b, blend*0.25f, skew);
-                    hud::drawprogress(sx, oldy, 0, wait, s, false, c.r, c.g, c.b, blend, skew, "super", "%d%%", int(wait*100.f));
+                    hud::drawitembar(x, oldy, s, false, c.r, c.g, c.b, blend, skew, wait);
                 }
             }
         }
@@ -235,15 +241,16 @@ namespace capture
             light->material[0] = bvec::fromcolor(effect);
             int pcolour = (int(light->material[0].x)<<16)|(int(light->material[0].y)<<8)|int(light->material[0].z);
             if(!f.owner && !f.droptime)
-                rendermodel(light, "flag", ANIM_MAPMODEL|ANIM_LOOP, f.pos(true), entities::ents[f.ent]->attrs[1], 0, entities::ents[f.ent]->attrs[2], MDL_DYNSHADOW|MDL_CULL_VFC|MDL_CULL_OCCLUDED, NULL, NULL, 0, 0, 1);
-            else
+                rendermodel(light, "props/flag", ANIM_MAPMODEL|ANIM_LOOP, f.pos(true), entities::ents[f.ent]->attrs[1], entities::ents[f.ent]->attrs[2], 0, MDL_DYNSHADOW|MDL_CULL_VFC|MDL_CULL_OCCLUDED, NULL, NULL, 0, 0, 1);
+            else if(!f.owner || f.owner != game::focus || game::thirdpersonview(true) || !(rendernormally))
             {
                 vec lac(f.pos(true));
-                float yaw = 0, roll = 0;
+                float yaw = 0, pitch = 0, roll = 0;
                 if(f.owner)
                 {
                     yaw = f.owner->yaw-45.f+(90/float(numflags[f.owner->clientnum]+1)*(iterflags[f.owner->clientnum]+1));
-                    roll = f.owner->roll;
+                    //pitch = f.owner->pitch; // broken by mdlyaw 270
+                    //roll = f.owner->roll;
                 }
                 else
                 {
@@ -251,7 +258,8 @@ namespace capture
                     if(f.proj) lac.z -= f.proj->height;
                 }
                 while(yaw >= 360.f) yaw -= 360.f;
-                rendermodel(light, "flag", ANIM_MAPMODEL|ANIM_LOOP, lac, yaw, 0, roll, MDL_DYNSHADOW|MDL_CULL_VFC|MDL_CULL_OCCLUDED|MDL_LIGHT|MDL_LIGHTFX, NULL, NULL, 0, 0, 1);
+                float trans = f.owner == game::focus && game::thirdpersonview(true) ? 0.5f : 1.f;
+                rendermodel(light, "props/flag", ANIM_MAPMODEL|ANIM_LOOP, lac, yaw, pitch, roll, MDL_DYNSHADOW|MDL_CULL_VFC|MDL_CULL_OCCLUDED|MDL_LIGHT|MDL_LIGHTFX, NULL, NULL, 0, 0, trans);
                 lac.z += enttype[AFFINITY].radius*2/3;
                 if(f.owner) { lac.z += iterflags[f.owner->clientnum]*2; iterflags[f.owner->clientnum]++; }
                 defformatstring(info)("<super>%s flag", TEAM(f.team, name));
@@ -311,7 +319,7 @@ namespace capture
         loopv(entities::ents) if(entities::ents[i]->type == AFFINITY)
         {
             gameentity &e = *(gameentity *)entities::ents[i];
-            if(!m_check(e.attrs[3], e.attrs[4], game::gamemode, game::mutators) || !isteam(game::gamemode, game::mutators, e.attrs[0], TEAM_FIRST))
+            if(!m_check(e.attrs[3], e.attrs[4], game::gamemode, game::mutators) || !isteam(game::gamemode, game::mutators, e.attrs[0], T_FIRST))
                 continue;
             st.addaffinity(e.o, e.attrs[0], i);
         }
@@ -368,7 +376,7 @@ namespace capture
     {
         if(!st.flags.inrange(i)) return;
         capturestate::flag &f = st.flags[i];
-        game::announcef(S_V_FLAGDROP, d == game::focus ? CON_SELF : CON_INFO, d, "\fa%s dropped the the \fs\f[%d]%s\fS flag", game::colorname(d), TEAM(f.team, colour), TEAM(f.team, name));
+        game::announcef(S_V_FLAGDROP, CON_INFO, d, true, "\fa%s dropped the the %s flag", game::colourname(d), game::colourteam(f.team, "flagtex"));
         st.dropaffinity(i, droploc, inertia, lastmillis);
     }
 
@@ -409,7 +417,7 @@ namespace capture
         if(!st.flags.inrange(i)) return;
         capturestate::flag &f = st.flags[i];
         affinityeffect(i, d->team, d->feetpos(), f.spawnloc, m_gsp(game::gamemode, game::mutators) ? 2 : 3, "RETURNED");
-        game::announcef(S_V_FLAGRETURN, d == game::focus ? CON_SELF : CON_INFO, d, "\fa%s returned the \fs\f[%d]%s\fS flag (time taken: \fs\fc%s\fS)", game::colorname(d), TEAM(f.team, colour), TEAM(f.team, name), hud::timetostr(lastmillis-(m_gsp1(game::gamemode, game::mutators) || m_gsp3(game::gamemode, game::mutators) ? f.taketime : f.droptime)));
+        game::announcef(S_V_FLAGRETURN, CON_INFO, d, true, "\fa%s returned the %s flag (time taken: \fs\fc%s\fS)", game::colourname(d), game::colourteam(f.team, "flagtex"), timestr(lastmillis-(m_gsp1(game::gamemode, game::mutators) ? f.droptime : f.taketime)));
         entities::execlink(NULL, f.ent, false);
         st.returnaffinity(i, lastmillis);
     }
@@ -420,8 +428,8 @@ namespace capture
         capturestate::flag &f = st.flags[i];
         if(value > 0)
         {
-            affinityeffect(i, TEAM_NEUTRAL, f.droploc, f.spawnloc, 3, "RESET");
-            game::announcef(S_V_FLAGRESET, CON_INFO, NULL, "\fathe \fs\f[%d]%s\fS flag has been reset", TEAM(f.team, colour), TEAM(f.team, name));
+            affinityeffect(i, T_NEUTRAL, f.droploc, f.spawnloc, 3, "RESET");
+            game::announcef(S_V_FLAGRESET, CON_INFO, NULL, true, "\fathe %s flag has been reset", game::colourteam(f.team, "flagtex"));
         }
         entities::execlink(NULL, f.ent, false);
         st.returnaffinity(i, lastmillis);
@@ -440,7 +448,8 @@ namespace capture
         else affinityeffect(goal, d->team, f.pos(true), f.spawnloc, 3, "CAPTURED");
         entities::execlink(NULL, f.ent, false);
         hud::teamscore(d->team).total = score;
-        game::announcef(S_V_FLAGSCORE, d == game::focus ? CON_SELF : CON_INFO, d, "\fa%s captured the \fs\f[%d]%s\fS flag for team \fs\f[%d]%s\fS (score: \fs\fc%d\fS, time taken: \fs\fc%s\fS)", game::colorname(d), TEAM(f.team, colour), TEAM(f.team, name), TEAM(d->team, colour), TEAM(d->team, name), score, hud::timetostr(lastmillis-f.taketime));
+        defformatstring(fteam)("%s", game::colourteam(f.team, "flagtex"));
+        game::announcef(S_V_FLAGSCORE, CON_INFO, d, true, "\fa%s captured the %s flag for team %s (score: \fs\fc%d\fS, time taken: \fs\fc%s\fS)", game::colourname(d), fteam, game::colourteam(d->team), score, timestr(lastmillis-f.taketime));
         st.returnaffinity(relay, lastmillis);
     }
 
@@ -452,13 +461,15 @@ namespace capture
         d->actiontime[AC_AFFINITY] = 0;
         playsound(S_CATCH, d->o, d);
         affinityeffect(i, d->team, d->feetpos(), f.pos(true), 1, f.team == d->team ? "SECURED" : "TAKEN");
-        game::announcef(f.team == d->team ? S_V_FLAGSECURED : S_V_FLAGPICKUP, d == game::focus ? CON_SELF : CON_INFO, d, "\fa%s %s the \fs\f[%d]%s\fS flag", game::colorname(d), f.team == d->team ? "secured" : (f.droptime ? "picked up" : "stole"), TEAM(f.team, colour), TEAM(f.team, name));
+        game::announcef(f.team == d->team ? S_V_FLAGSECURED : S_V_FLAGPICKUP, CON_INFO, d, true, "\fa%s %s the %s flag", game::colourname(d), f.team == d->team ? "secured" : (f.droptime ? "picked up" : "stole"), game::colourteam(f.team, "flagtex"));
         entities::execlink(NULL, f.ent, false);
         st.takeaffinity(i, d, lastmillis);
     }
 
-    void checkaffinity(gameent *d)
+    void checkaffinity(dynent *e)
     {
+        if(e->state != CS_ALIVE || !gameent::is(e)) return;
+        gameent *d = (gameent *)e;
         vec o = d->feetpos();
         loopv(st.flags)
         {
@@ -562,10 +573,12 @@ namespace capture
         loopvj(st.flags)
         {
             capturestate::flag &f = st.flags[j];
+            bool home = f.team == ai::owner(d);
+            if(d->aitype == AI_BOT && m_duke(game::gamemode, game::mutators) && home) continue;
             static vector<int> targets; // build a list of others who are interested in this
             targets.setsize(0);
-            bool regen = d->aitype != AI_BOT || f.team == TEAM_NEUTRAL || m_gsp3(game::gamemode, game::mutators) || !m_regen(game::gamemode, game::mutators) || d->health >= m_health(game::gamemode, game::mutators);
-            ai::checkothers(targets, d, f.team == ai::owner(d) || d->aitype != AI_BOT ? ai::AI_S_DEFEND : ai::AI_S_PURSUE, ai::AI_T_AFFINITY, j, true);
+            bool regen = d->aitype != AI_BOT || f.team == T_NEUTRAL || m_gsp3(game::gamemode, game::mutators) || !m_regen(game::gamemode, game::mutators) || d->health >= m_health(game::gamemode, game::mutators, d->model);
+            ai::checkothers(targets, d, home || d->aitype != AI_BOT ? ai::AI_S_DEFEND : ai::AI_S_PURSUE, ai::AI_T_AFFINITY, j, true);
             if(d->aitype == AI_BOT)
             {
                 gameent *e = NULL;
@@ -577,7 +590,7 @@ namespace capture
                         targets.add(e->clientnum);
                 }
             }
-            if(f.team == ai::owner(d))
+            if(home)
             {
                 bool guard = false;
                 if(f.owner || f.droptime || targets.empty()) guard = true;
@@ -586,7 +599,7 @@ namespace capture
                     gameent *t;
                     loopvk(targets) if((t = game::getclient(targets[k])))
                     {
-                        if((t->ai && !t->hasweap(t->ai->weappref, m_weapon(game::gamemode, game::mutators))) || (!t->ai && t->weapselect < WEAP_OFFSET))
+                        if((t->ai && !t->hasweap(t->ai->weappref, m_weapon(game::gamemode, game::mutators))) || (!t->ai && t->weapselect < W_OFFSET))
                         {
                             guard = true;
                             break;
@@ -640,8 +653,11 @@ namespace capture
 
     bool aidefense(gameent *d, ai::aistate &b)
     {
-        if(!m_gsp3(game::gamemode, game::mutators) && d->aitype == AI_BOT)
-            loopv(st.flags) if(st.flags[i].owner == d) return aihomerun(d, b);
+        if(d->aitype == AI_BOT)
+        {
+            if(!m_gsp3(game::gamemode, game::mutators)) loopv(st.flags) if(st.flags[i].owner == d) return aihomerun(d, b);
+            if(d->aitype == AI_BOT && m_duke(game::gamemode, game::mutators)) return false;
+        }
         if(st.flags.inrange(b.target))
         {
             capturestate::flag &f = st.flags[b.target];
@@ -650,7 +666,7 @@ namespace capture
             int walk = f.owner && ai::owner(f.owner) != ai::owner(d) ? 1 : 0;
             if(d->aitype == AI_BOT)
             {
-                bool regen = !m_regen(game::gamemode, game::mutators) || d->health >= m_health(game::gamemode, game::mutators);
+                bool regen = !m_regen(game::gamemode, game::mutators) || d->health >= m_health(game::gamemode, game::mutators, d->model);
                 if(regen && lastmillis-b.millis >= (201-d->skill)*33)
                 {
                     static vector<int> targets; // build a list of others who are interested in this
@@ -697,7 +713,6 @@ namespace capture
         if(st.flags.inrange(b.target) && d->aitype == AI_BOT)
         {
             capturestate::flag &f = st.flags[b.target];
-            b.idle = -1;
             if(f.team != ai::owner(d))
             {
                 if(f.owner)
