@@ -10,7 +10,7 @@ namespace defend
 
     void preload()
     {
-        loadmodel("flag", -1, true);
+        preloadmodel("props/flag");
     }
 
     static vec skewcolour(int owner, int enemy, float occupy)
@@ -18,7 +18,7 @@ namespace defend
         vec colour = vec::hexcolor(TEAM(owner, colour));
         if(enemy)
         {
-            int team = owner && enemy && !defendinstant ? TEAM_NEUTRAL : enemy;
+            int team = owner && enemy && !defendinstant ? T_NEUTRAL : enemy;
             int timestep = totalmillis%1000;
             float amt = clamp((timestep <= 500 ? timestep/500.f : (1000-timestep)/500.f)*occupy, 0.f, 1.f);
             colour.lerp(vec::hexcolor(TEAM(team, colour)), amt);
@@ -66,13 +66,16 @@ namespace defend
             float occupy = b.occupied(defendinstant, defendcount);
             entitylight *light = &entities::ents[b.ent]->light;
             light->material[0] = bvec::fromcolor(skewcolour(b.owner, b.enemy, occupy));
-            rendermodel(light, "flag", ANIM_MAPMODEL|ANIM_LOOP, b.o, entities::ents[b.ent]->attrs[2], entities::ents[b.ent]->attrs[3], 0, MDL_DYNSHADOW|MDL_CULL_VFC|MDL_CULL_OCCLUDED);
+            rendermodel(light, "props/flag", ANIM_MAPMODEL|ANIM_LOOP, b.o, entities::ents[b.ent]->attrs[1], entities::ents[b.ent]->attrs[2], 0, MDL_DYNSHADOW|MDL_CULL_VFC|MDL_CULL_OCCLUDED);
             if(b.enemy && b.owner)
-                formatstring(b.info)("\fs\f[%d]\f(%s)%s\fS vs. \fs\f[%d]\f(%s)%s\fS", TEAM(b.owner, colour), hud::teamtexname(b.owner), TEAM(b.owner, name), TEAM(b.enemy, colour), hud::teamtexname(b.enemy), TEAM(b.enemy, name));
+            {
+                defformatstring(bowner)("%s", game::colourteam(b.owner));
+                formatstring(b.info)("%s vs. %s", bowner, game::colourteam(b.enemy));
+            }
             else
             {
                 int defend = b.owner ? b.owner : b.enemy;
-                formatstring(b.info)("\fs\f[%d]\f(%s)%s\fS", TEAM(defend, colour), hud::teamtexname(defend), TEAM(defend, name));
+                formatstring(b.info)("%s", game::colourteam(defend));
             }
             vec above = b.o;
             above.z += enttype[AFFINITY].radius*2/3;
@@ -130,14 +133,22 @@ namespace defend
 
     void drawnotices(int w, int h, int &tx, int &ty, float blend)
     {
-        if(game::player1->state == CS_ALIVE && hud::shownotices >= 3)
+        if(game::focus->state == CS_ALIVE && hud::shownotices >= 2)
         {
-            loopv(st.flags) if(insideaffinity(st.flags[i], game::player1) && (st.flags[i].owner == game::player1->team || st.flags[i].enemy == game::player1->team))
+            if(game::focus->lastbuff && hud::shownotices >= 3)
+            {
+                pushfont("reduced");
+                if(m_regen(game::gamemode, game::mutators) && defendregenbuff && defendregenextra)
+                    ty += draw_textx("Buffing: \fs\fo%d%%\fS damage, \fs\fg%d%%\fS shield, +\fs\fy%d\fS regen", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, int(defendbuffdamage*100), int(defendbuffshield*100), defendregenextra)*hud::noticescale;
+                else ty += draw_textx("Buffing: \fs\fo%d%%\fS damage, \fs\fg%d%%\fS shield", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, int(defendbuffdamage*100), int(defendbuffshield*100))*hud::noticescale;
+                popfont();
+            }
+            loopv(st.flags) if(insideaffinity(st.flags[i], game::focus) && (st.flags[i].owner == game::focus->team || st.flags[i].enemy == game::focus->team))
             {
                 defendstate::flag &f = st.flags[i];
                 pushfont("emphasis");
                 float occupy = !f.owner || f.enemy ? clamp(f.converted/float((!defendinstant && f.owner ? 2 : 1) * defendcount), 0.f, 1.f) : 1.f;
-                bool overthrow = f.owner && f.enemy == game::player1->team;
+                bool overthrow = f.owner && f.enemy == game::focus->team;
                 ty += draw_textx("%s \fs\f[%d]\f(%s)\f(%s)\fS \fs%s%d%%\fS", tx, ty, 255, 255, 255, int(255*blend), TEXT_CENTERED, -1, -1, overthrow ? "Overthrow" : "Secure", TEAM(f.owner, colour), hud::teamtexname(f.owner), hud::flagtex, overthrow ? "\fy" : (occupy < 1.f ? "\fc" : "\fg"), int(occupy*100.f))*hud::noticescale;
                 popfont();
                 break;
@@ -170,18 +181,17 @@ namespace defend
                 if(f.hasflag) skew += (millis <= 1000 ? clamp(float(millis)/1000.f, 0.f, 1.f)*(1.f-skew) : 1.f-skew);
                 else if(millis <= 1000) skew += (1.f-skew)-(clamp(float(millis)/1000.f, 0.f, 1.f)*(1.f-skew));
                 int oldy = y-sy;
-                sy += hud::drawitem(hud::flagtex, x, oldy, s, 0, true, false, c.r, c.g, c.b, blend, skew);
+                if(hasflag || f.enemy)
+                    sy += hud::drawitem(hud::flagtex, x, oldy, s, 0, true, false, c.r, c.g, c.b, blend, skew, "super", "%s%d%%", hasflag ? (f.owner && f.enemy == game::focus->team ? "\fy" : (occupy < 1.f ? "\fc" : "\fg")) : "\fw", int(occupy*100.f));
+                else sy += hud::drawitem(hud::flagtex, x, oldy, s, 0, true, false, c.r, c.g, c.b, blend, skew);
                 if(f.enemy)
                 {
-                    int sx = x-int(s*skew);
                     vec c2 = vec::hexcolor(TEAM(f.enemy, colour));
-                    hud::drawprogress(sx, oldy, occupy, 1-occupy, s, false, c1.r, c1.g, c1.b, blend*0.25f, skew);
-                    hud::drawprogress(sx, oldy, 0, occupy, s, false, c2.r, c2.g, c2.b, blend, skew, "super", "%s%d%%", hasflag ? (f.owner && f.enemy == game::focus->team ? "\fy" : (occupy < 1.f ? "\fc" : "\fg")) : "\fw", int(occupy*100.f));
                     hud::drawitem(hud::attacktex, x, oldy, s, 0.5f, true, false, c2.r, c2.g, c2.b, blend, skew);
+                    hud::drawitembar(x, oldy, s, false, c.r, c.g, c.b, blend, skew, occupy);
                 }
                 else if(f.owner)
                     hud::drawitem(hud::teamtexname(f.owner), x, oldy, s, 0.5f, true, false, c1.r, c1.g, c1.b, blend, skew);
-                //hud::drawitemtext(x, oldy, s, false, skew, "reduced", blend, "%s", f.name);
             }
         }
         return sy;
@@ -203,20 +213,20 @@ namespace defend
             {
                 case 3:
                 {
-                    if(team && !isteam(game::gamemode, game::mutators, team, TEAM_NEUTRAL)) team = TEAM_NEUTRAL;
+                    if(team && !isteam(game::gamemode, game::mutators, team, T_NEUTRAL)) team = T_NEUTRAL;
                     break;
                 }
                 case 2:
                 {
-                    if(!isteam(game::gamemode, game::mutators, team, TEAM_FIRST)) continue;
+                    if(!isteam(game::gamemode, game::mutators, team, T_FIRST)) continue;
                     break;
                 }
                 case 1:
                 {
-                    if(team && !isteam(game::gamemode, game::mutators, team, TEAM_NEUTRAL)) continue;
+                    if(team && !isteam(game::gamemode, game::mutators, team, T_NEUTRAL)) continue;
                     break;
                 }
-                case 0: team = TEAM_NEUTRAL; break;
+                case 0: team = T_NEUTRAL; break;
             }
             defendstate::flag &b = st.flags.add();
             b.o = e->o;
@@ -228,20 +238,21 @@ namespace defend
             b.kinship = team;
             b.reset();
         }
-        int bases[TEAM_ALL] = {0};
+        if(!st.flags.length()) return; // map doesn't seem to support this mode at all..
+        int bases[T_ALL] = {0};
         bool hasteams = true;
         loopv(st.flags) bases[st.flags[i].kinship]++;
-        loopi(numteams(game::gamemode, game::mutators)-1) if(bases[i+1] != bases[i+2])
+        loopi(numteams(game::gamemode, game::mutators)-1) if(!bases[i+1] || (bases[i+1] != bases[i+2]))
         {
-            loopvk(st.flags) st.flags[k].kinship = TEAM_NEUTRAL;
+            loopvk(st.flags) st.flags[k].kinship = T_NEUTRAL;
             hasteams = false;
             break;
         }
-        if(m_gsp3(game::gamemode, game::mutators))
+        if(m_gsp2(game::gamemode, game::mutators))
         {
             vec average(0, 0, 0);
             int count = 0;
-            loopv(st.flags) if(!hasteams || st.flags[i].kinship != TEAM_NEUTRAL)
+            loopv(st.flags) if(!hasteams || st.flags[i].kinship != T_NEUTRAL)
             {
                 average.add(st.flags[i].o);
                 count++;
@@ -260,7 +271,7 @@ namespace defend
             if(!st.flags.inrange(smallest)) smallest = rnd(st.flags.length());
             int ent = st.flags[smallest].ent;
             copystring(st.flags[smallest].name, "the flag");
-            st.flags[smallest].kinship = TEAM_NEUTRAL;
+            st.flags[smallest].kinship = T_NEUTRAL;
             loopv(st.flags) if(st.flags[i].ent != ent) st.flags.remove(i--);
         }
     }
@@ -300,34 +311,37 @@ namespace defend
     {
         if(!st.flags.inrange(i)) return;
         defendstate::flag &b = st.flags[i];
-        if(owner)
+        if(converted >= 0)
         {
-            if(b.owner != owner)
+            if(owner)
+            {
+                if(b.owner != owner)
+                {
+                    gameent *d = NULL, *e = NULL;
+                    int numdyns = game::numdynents();
+                    loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && e->type == ENT_PLAYER && insideaffinity(b, e))
+                        if((d = e) == game::focus) break;
+                    game::announcef(S_V_FLAGSECURED, CON_INFO, d, true, "\fateam %s secured \fw%s", game::colourteam(owner), b.name);
+                    part_textcopy(vec(b.o).add(vec(0, 0, enttype[AFFINITY].radius)), "<super>\fzZeSECURED", PART_TEXT, game::eventiconfade, TEAM(owner, colour), 3, 1, -10);
+                    if(game::dynlighteffects) adddynlight(vec(b.o).add(vec(0, 0, enttype[AFFINITY].radius)), enttype[AFFINITY].radius*2, vec::hexcolor(TEAM(owner, colour)).mul(2.f), 500, 250);
+                    entities::execlink(NULL, b.ent, false);
+                }
+            }
+            else if(b.owner)
             {
                 gameent *d = NULL, *e = NULL;
                 int numdyns = game::numdynents();
                 loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && e->type == ENT_PLAYER && insideaffinity(b, e))
                     if((d = e) == game::focus) break;
-                game::announcef(S_V_FLAGSECURED, d == game::focus ? CON_SELF : CON_INFO, d, "\fateam \fs\f[%d]%s\fS secured \fw%s", TEAM(owner, colour), TEAM(owner, name), b.name);
-                part_textcopy(vec(b.o).add(vec(0, 0, enttype[AFFINITY].radius)), "<super>\fzZeSECURED", PART_TEXT, game::eventiconfade, TEAM(owner, colour), 3, 1, -10);
-                if(game::dynlighteffects) adddynlight(vec(b.o).add(vec(0, 0, enttype[AFFINITY].radius)), enttype[AFFINITY].radius*2, vec::hexcolor(TEAM(owner, colour)).mul(2.f), 500, 250);
+                game::announcef(S_V_FLAGOVERTHROWN, CON_INFO, d, true, "\fateam %s overthrew \fw%s", game::colourteam(enemy), b.name);
+                part_textcopy(vec(b.o).add(vec(0, 0, enttype[AFFINITY].radius)), "<super>\fzZeOVERTHROWN", PART_TEXT, game::eventiconfade, TEAM(enemy, colour), 3, 1, -10);
+                if(game::dynlighteffects) adddynlight(vec(b.o).add(vec(0, 0, enttype[AFFINITY].radius)), enttype[AFFINITY].radius*2, vec::hexcolor(TEAM(enemy, colour)).mul(2.f), 500, 250);
                 entities::execlink(NULL, b.ent, false);
             }
-        }
-        else if(b.owner)
-        {
-            gameent *d = NULL, *e = NULL;
-            int numdyns = game::numdynents();
-            loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && e->type == ENT_PLAYER && insideaffinity(b, e))
-                if((d = e) == game::focus) break;
-            game::announcef(S_V_FLAGOVERTHROWN, d == game::focus ? CON_SELF : CON_INFO, d, "\fateam \fs\f[%d]%s\fS overthrew \fw%s", TEAM(enemy, colour), TEAM(enemy, name), b.name);
-            part_textcopy(vec(b.o).add(vec(0, 0, enttype[AFFINITY].radius)), "<super>\fzZeOVERTHROWN", PART_TEXT, game::eventiconfade, TEAM(enemy, colour), 3, 1, -10);
-            if(game::dynlighteffects) adddynlight(vec(b.o).add(vec(0, 0, enttype[AFFINITY].radius)), enttype[AFFINITY].radius*2, vec::hexcolor(TEAM(enemy, colour)).mul(2.f), 500, 250);
-            entities::execlink(NULL, b.ent, false);
+            b.converted = converted;
         }
         b.owner = owner;
         b.enemy = enemy;
-        b.converted = converted;
     }
 
     void setscore(int team, int total)
@@ -359,7 +373,7 @@ namespace defend
                 targets.setsize(0);
                 ai::checkothers(targets, d, ai::AI_S_DEFEND, ai::AI_T_AFFINITY, j, true);
                 gameent *e = NULL;
-                bool regen = !m_regen(game::gamemode, game::mutators) || d->health >= m_health(game::gamemode, game::mutators);
+                bool regen = !m_regen(game::gamemode, game::mutators) || d->health >= m_health(game::gamemode, game::mutators, d->model);
                 int numdyns = game::numdynents();
                 loopi(numdyns) if((e = (gameent *)game::iterdynents(i)) && !e->ai && e->state == CS_ALIVE && ai::owner(d) == ai::owner(e))
                 {
@@ -387,7 +401,7 @@ namespace defend
         if(st.flags.inrange(b.target))
         {
             defendstate::flag &f = st.flags[b.target];
-            bool regen = d->aitype != AI_BOT || !m_regen(game::gamemode, game::mutators) || d->health >= m_health(game::gamemode, game::mutators);
+            bool regen = d->aitype != AI_BOT || !m_regen(game::gamemode, game::mutators) || d->health >= m_health(game::gamemode, game::mutators, d->model);
             int walk = f.enemy && f.enemy != ai::owner(d) ? 1 : 0;
             if(regen && (!f.enemy && ai::owner(d) == f.owner))
             {
@@ -417,7 +431,7 @@ namespace defend
                 }
                 else walk = 1;
             }
-            return ai::defense(d, b, f.o, !f.enemy || m_gsp3(game::gamemode, game::mutators) ? ai::CLOSEDIST : float(enttype[AFFINITY].radius), !f.enemy || m_gsp3(game::gamemode, game::mutators) ? ai::CLOSEDIST : float(enttype[AFFINITY].radius*walk*8), walk);
+            return ai::defense(d, b, f.o, !f.enemy || m_gsp2(game::gamemode, game::mutators) ? ai::CLOSEDIST : float(enttype[AFFINITY].radius), !f.enemy || m_gsp2(game::gamemode, game::mutators) ? ai::CLOSEDIST : float(enttype[AFFINITY].radius*walk*8), walk);
         }
         return false;
     }
