@@ -80,6 +80,7 @@ namespace ai
                 bbmin.min(vec(w.o).sub(radius));
                 bbmax.max(vec(w.o).add(radius));
             }
+            if(first < last) lastwp = max(lastwp, last-1);
             build(indices.getbuf(), indices.length(), bbmin, bbmax);
         }
 
@@ -184,7 +185,7 @@ namespace ai
     void buildwpcache()
     {
         loopi(NUMWPCACHES) if(wpcaches[i].maxdepth < 0)
-            wpcaches[i].build(i > 0 ? wpcaches[i-1].lastwp+1 : 0, i+1 >= NUMWPCACHES || wpcaches[i+1].maxdepth < 0 ? -1 : wpcaches[i+1].firstwp);
+            wpcaches[i].build(i > 0 ? wpcaches[i-1].lastwp+1 : 1, i+1 >= NUMWPCACHES || wpcaches[i+1].maxdepth < 0 ? -1 : wpcaches[i+1].firstwp);
         clearedwpcaches = 0;
         lastwpcache = waypoints.length();
 
@@ -207,17 +208,21 @@ namespace ai
 
         #define CHECKCLOSEST(index) do { \
             int n = (index); \
-            waypoint &w = waypoints[n]; \
-            if(!links || w.haslinks()) \
+            if(waypoints.inrange(n)) \
             { \
-                float dist = w.o.squaredist(pos); \
-                if(dist < mindist*mindist) { closest = n; mindist = sqrtf(dist); } \
+                waypoint &w = waypoints[n]; \
+                if(!links || w.haslinks()) \
+                { \
+                    float dist = w.o.squaredist(pos); \
+                    if(dist < mindist*mindist) { closest = n; mindist = sqrtf(dist); } \
+                } \
             } \
         } while(0)
         int closest = -1;
         wpcachenode *curnode;
         loop(which, NUMWPCACHES) for(curnode = &wpcaches[which].nodes[0], wpcachestack.setsize(0);;)
         {
+            if(!curnode) continue;
             int axis = curnode->axis();
             float dist1 = pos[axis] - curnode->split[0], dist2 = curnode->split[1] - pos[axis];
             if(dist1 >= mindist)
@@ -262,13 +267,17 @@ namespace ai
         float mindist2 = mindist*mindist, maxdist2 = maxdist*maxdist;
         #define CHECKWITHIN(index) do { \
             int n = (index); \
-            const waypoint &w = waypoints[n]; \
-            float dist = w.o.squaredist(pos); \
-            if(dist > mindist2 && dist < maxdist2) results.add(n); \
+            if(waypoints.inrange(n)) \
+            { \
+                const waypoint &w = waypoints[n]; \
+                float dist = w.o.squaredist(pos); \
+                if(dist > mindist2 && dist < maxdist2) results.add(n); \
+            } \
         } while(0)
         wpcachenode *curnode;
         loop(which, NUMWPCACHES) for(curnode = &wpcaches[which].nodes[0], wpcachestack.setsize(0);;)
         {
+            if(!curnode) continue;
             int axis = curnode->axis();
             float dist1 = pos[axis] - curnode->split[0], dist2 = curnode->split[1] - pos[axis];
             if(dist1 >= maxdist)
@@ -312,12 +321,16 @@ namespace ai
         float limit2 = limit*limit;
         #define CHECKNEAR(index) do { \
             int n = (index); \
-            const waypoint &w = ai::waypoints[n]; \
-            if(w.o.squaredist(pos) < limit2) add(owner, above, n); \
+            if(ai::waypoints.inrange(n)) \
+            { \
+                const waypoint &w = ai::waypoints[n]; \
+                if(w.o.squaredist(pos) < limit2) add(owner, above, n); \
+            } \
         } while(0)
         wpcachenode *curnode;
         loop(which, NUMWPCACHES) for(curnode = &wpcaches[which].nodes[0], wpcachestack.setsize(0);;)
         {
+            if(!curnode) continue;
             int axis = curnode->axis();
             float dist1 = pos[axis] - curnode->split[0], dist2 = curnode->split[1] - pos[axis];
             if(dist1 >= limit)
@@ -556,7 +569,7 @@ namespace ai
             if(dropping && d->lastnode != curnode && iswaypoint(d->lastnode))
             {
                 linkwaypoint(waypoints[d->lastnode], curnode);
-                if(!d->timeinair) linkwaypoint(waypoints[curnode], d->lastnode);
+                if(!d->airmillis) linkwaypoint(waypoints[curnode], d->lastnode);
             }
             d->lastnode = curnode;
             if(d->ai && iswaypoint(prevnode) && d->lastnode != prevnode) d->ai->addprevnode(prevnode);
@@ -634,7 +647,7 @@ namespace ai
     bool cleanwaypoints()
     {
         int cleared = 0;
-        loopv(waypoints)
+        for(int i = 1; i < waypoints.length(); i++)
         {
             waypoint &w = waypoints[i];
             if(clipped(w.o))
@@ -774,7 +787,7 @@ namespace ai
 
     bool getwaypoints(bool force, const char *mname, bool check)
     {
-        if(check && loadedwaypoints[0]) return false;
+        if(check && (client::demoplayback || loadedwaypoints[0])) return false;
         return loadwaypoints(force, mname) || importwaypoints();
     }
 
@@ -783,7 +796,7 @@ namespace ai
         if(noedit(true)) return;
         vec o = sel.o.tovec().sub(0.1f), s = sel.s.tovec().mul(sel.grid).add(o).add(0.1f);
         int cleared = 0;
-        loopv(waypoints)
+        for(int i = 1; i < waypoints.length(); i++)
         {
             waypoint &w = waypoints[i];
             if(w.o.x >= o.x && w.o.x <= s.x && w.o.y >= o.y && w.o.y <= s.y && w.o.z >= o.z && w.o.z <= s.z)
