@@ -38,7 +38,7 @@ enum
     RET_FLOAT  = VAL_FLOAT<<CODE_RET,
 };
 
-enum { ID_VAR, ID_FVAR, ID_SVAR, ID_COMMAND, ID_ALIAS };
+enum { ID_VAR, ID_FVAR, ID_SVAR, ID_COMMAND, ID_ALIAS, ID_LOCAL };
 #define VAR_MIN INT_MIN+1
 #define VAR_MAX INT_MAX-1
 #define FVAR_MIN -1e6f
@@ -292,6 +292,7 @@ extern ident *newident(const char *name, int flags = 0);
 extern ident *readident(const char *name);
 extern ident *writeident(const char *name, int flags = 0);
 extern bool addcommand(const char *name, identfun fun, const char *narg, int flags = IDF_COMPLETE);
+extern bool addkeyword(int type, const char *name, int flags = 0);
 
 extern uint *compilecode(const char *p);
 extern void keepcode(uint *p);
@@ -305,7 +306,8 @@ extern int execute(const char *p);
 extern int execute(const char *p, bool nonworld);
 extern bool executebool(const uint *code);
 extern bool executebool(const char *p);
-extern bool execfile(const char *cfgfile, bool msg = true, bool nonworld = false);
+enum { EXEC_NOWORLD = 1<<0, EXEC_VERSION = 1<<1 };
+extern bool execfile(const char *cfgfile, bool msg = true, int flags = 0);
 extern void alias(const char *name, const char *action);
 extern void alias(const char *name, tagval &v);
 extern void worldalias(const char *name, const char *action);
@@ -324,12 +326,16 @@ extern int listlen(const char *s);
 extern char *indexlist(const char *s, int pos);
 extern const char *indexlist(const char *s, int pos, int &len);
 extern int listincludes(const char *list, const char *needl, int needlelen);
-extern char *shrinklist(const char *list, const char *limit, int failover);
+extern char *shrinklist(const char *list, const char *limit, int failover = 0, bool invert = false);
 
 extern void checksleep(int millis);
 extern void clearsleep(bool clearworlds = true);
 
+extern char *logtimeformat, *filetimeformat;
+extern const char *gettime(time_t ctime = 0, const char *format = NULL);
+
 // nasty macros for registering script functions, abuses globals to avoid excessive infrastructure
+#define KEYWORD(flags, name, type) static bool __dummy_##name = addkeyword(type, #name, flags)
 #define COMMANDN(flags, name, fun, nargs) static bool __dummy_##fun = addcommand(#name, (identfun)fun, nargs, flags|IDF_COMPLETE)
 #define COMMAND(flags, name, nargs) COMMANDN(flags, name, name, nargs)
 
@@ -369,8 +375,8 @@ extern void clearsleep(bool clearworlds = true);
 
 #if defined(GAMEWORLD)
 #define IDF_GAME (IDF_CLIENT|IDF_REWRITE)
-#define GAME(name) (name)
-#define PHYS(name) ((GAME(name##force) >= 0 ? GAME(name##force) : physics::name)*GAME(name##scale))
+#define G(name) (name)
+#define PHYS(name) (G(name)*G(name##scale))
 #define GICOMMAND(flags, n, g, proto, svbody, ccbody) ICOMMAND(flags|IDF_GAME, n, g, proto, ccbody)
 #define GVARN(flags, name, global, min, cur, max) _VAR(name, global, min, cur, max, flags|IDF_GAME)
 #define GVAR(flags, name, min, cur, max) _VAR(name, name, min, cur, max, flags|IDF_GAME)
@@ -382,7 +388,7 @@ extern void clearsleep(bool clearworlds = true);
 #define GSVAR(flags, name, cur) _SVAR(name, name, cur, flags|IDF_GAME)
 #define GSVARF(flags, name, cur, svbody, ccbody) _SVARF(name, name, cur, ccbody, flags|IDF_GAME)
 #elif defined(GAMESERVER)
-#define GAME(name) (sv_##name)
+#define G(name) (sv_##name)
 #define IDF_GAME (IDF_SERVER|IDF_REWRITE)
 #define GICOMMAND(flags, n, g, proto, svbody, ccbody) ICOMMAND(flags|(IDF_GAME&~IDF_REWRITE), sv_##n, g, proto, svbody)
 #define GVARN(flags, name, global, min, cur, max) _VAR(sv_##name, global, min, cur, max, flags|IDF_GAME)
@@ -395,8 +401,8 @@ extern void clearsleep(bool clearworlds = true);
 #define GSVAR(flags, name, cur) _SVAR(sv_##name, sv_##name, cur, flags|IDF_GAME)
 #define GSVARF(flags, name, cur, svbody, ccbody) _SVARF(sv_##name, sv_##name, cur, svbody, flags|IDF_GAME)
 #else
-#define GAME(name) (name)
-#define PHYS(name) ((GAME(name##force) >= 0 ? GAME(name##force) : physics::name)*GAME(name##scale))
+#define G(name) (name)
+#define PHYS(name) (G(name)*G(name##scale))
 #define GICOMMAND(flags, n, g, proto, svbody, ccbody)
 #define GVARN(flags, name, global, min, cur, max) extern int name
 #define GVAR(flags, name, min, cur, max) extern int name
