@@ -57,6 +57,7 @@ struct vec
     float dist(const vec &e, vec &t) const { t = *this; t.sub(e); return t.magnitude(); }
     float dist2(const vec &o) const { float dx = x-o.x, dy = y-o.y; return sqrtf(dx*dx + dy*dy); }
     bool reject(const vec &o, float r) { return x>o.x+r || x<o.x-r || y>o.y+r || y<o.y-r; }
+    bool rejectxyz(const vec &o, float rx, float ry, float rz1, float rz2) { return x>o.x+rx || x<o.x-rx || y>o.y+ry || y<o.y-ry || z>o.z+rz2 || z<o.z-rz1; }
     template<class A, class B>
     vec &cross(const A &a, const B &b) { x = a.y*b.z-a.z*b.y; y = a.z*b.x-a.x*b.z; z = a.x*b.y-a.y*b.x; return *this; }
     vec &cross(const vec &o, const vec &a, const vec &b) { return cross(vec(a).sub(o), vec(b).sub(o)); }
@@ -91,7 +92,7 @@ struct vec
 
     vec &rotate_around_z(float c, float s) { float rx = x, ry = y; x = c*rx-s*ry; y = c*ry+s*rx; return *this; }
     vec &rotate_around_x(float c, float s) { float ry = y, rz = z; y = c*ry-s*rz; z = c*rz+s*ry; return *this; }
-    vec &rotate_around_y(float c, float s) { float rx = x, rz = z; x = c*rx-s*rz; z = c*rz+s*rx; return *this; }
+    vec &rotate_around_y(float c, float s) { float rx = x, rz = z; x = c*rx+s*rz; z = c*rz-s*rx; return *this; }
 
     vec &rotate_around_z(float angle) { return rotate_around_z(cosf(angle), sinf(angle)); }
     vec &rotate_around_x(float angle) { return rotate_around_x(cosf(angle), sinf(angle)); }
@@ -146,6 +147,8 @@ struct vec
     {
         return vec(((color>>16)&0xFF)*(1.0f/255.0f), ((color>>8)&0xFF)*(1.0f/255.0f), (color&0xFF)*(1.0f/255.0f));
     }
+
+    int tohexcolor() { return ((int(r*255)>>16)&0xFF)|((int(g*255)>>8)&0xFF)|(int(b*255)&0xFF); }
 };
 
 static inline bool htcmp(const vec &x, const vec &y)
@@ -216,6 +219,14 @@ struct vec4
     vec4 &neg()              { neg3(); w = -w; return *this; }
 
     void setxyz(const vec &v) { x = v.x; y = v.y; z = v.z; }
+
+    vec4 &rotate_around_z(float c, float s) { float rx = x, ry = y; x = c*rx-s*ry; y = c*ry+s*rx; return *this; }
+    vec4 &rotate_around_x(float c, float s) { float ry = y, rz = z; y = c*ry-s*rz; z = c*rz+s*ry; return *this; }
+    vec4 &rotate_around_y(float c, float s) { float rx = x, rz = z; x = c*rx+s*rz; z = c*rz-s*rx; return *this; }
+
+    vec4 &rotate_around_z(float angle) { return rotate_around_z(cosf(angle), sinf(angle)); }
+    vec4 &rotate_around_x(float angle) { return rotate_around_x(cosf(angle), sinf(angle)); }
+    vec4 &rotate_around_y(float angle) { return rotate_around_y(cosf(angle), sinf(angle)); }
 };
 
 inline vec::vec(const vec4 &v) : x(v.x), y(v.y), z(v.z) {}
@@ -608,21 +619,36 @@ struct matrix3x3
                    a.y*o.x + b.y*o.y + c.y*o.z,
                    a.z*o.x + b.z*o.y + c.z*o.z);
     }
-};
 
-struct matrix2x3
-{
-    vec a, b;
-
-    matrix2x3() {}
-    matrix2x3(const vec &a, const vec &b) : a(a), b(b) {}
-
-    vec2 transform(const vec &o) const { return vec2(a.dot(o), b.dot(o)); }
-    vec transposedtransform(const vec2 &o) const
+    void identity()
     {
-        return vec(a.x*o.x + b.x*o.y,
-                   a.y*o.x + b.y*o.y,
-                   a.z*o.x + b.z*o.y);
+        a = vec(1, 0, 0);
+        b = vec(0, 1, 0);
+        c = vec(0, 0, 1);
+    }
+
+    void rotate_around_x(float angle)
+    {
+        float ck = cosf(angle), sk = -sinf(angle);
+        a.rotate_around_x(ck, sk);
+        b.rotate_around_x(ck, sk);
+        c.rotate_around_x(ck, sk);
+    }
+
+    void rotate_around_y(float angle)
+    {
+        float ck = cosf(angle), sk = -sinf(angle);
+        a.rotate_around_y(ck, sk);
+        b.rotate_around_y(ck, sk);
+        c.rotate_around_y(ck, sk);
+    }
+
+    void rotate_around_z(float angle)
+    {
+        float ck = cosf(angle), sk = -sinf(angle);
+        a.rotate_around_z(ck, sk);
+        b.rotate_around_z(ck, sk);
+        c.rotate_around_z(ck, sk);
     }
 };
 
@@ -779,39 +805,30 @@ struct matrix3x4
         c = vec4(d.x*d.z*(1-ck)-d.y*sk, d.y*d.z*(1-ck)+d.x*sk, d.z*d.z*(1-ck)+ck, 0);
     }
 
-    #define ROTVEC(V, m, n) \
-    { \
-        float m = V.m, n = V.n; \
-        V.m = m*ck + n*sk; \
-        V.n = n*ck - m*sk; \
-    }
-
     void rotate_around_x(float angle)
     {
-        float ck = cosf(angle), sk = sinf(angle);
-        ROTVEC(a, y, z);
-        ROTVEC(b, y, z);
-        ROTVEC(c, y, z);
+        float ck = cosf(angle), sk = -sinf(angle);
+        a.rotate_around_x(ck, sk);
+        b.rotate_around_x(ck, sk);
+        c.rotate_around_x(ck, sk);
     }
 
     void rotate_around_y(float angle)
     {
-        float ck = cosf(angle), sk = sinf(angle);
-        ROTVEC(a, z, x);
-        ROTVEC(b, z, x);
-        ROTVEC(c, z, x);
+        float ck = cosf(angle), sk = -sinf(angle);
+        a.rotate_around_y(ck, sk);
+        b.rotate_around_y(ck, sk);
+        c.rotate_around_y(ck, sk);
     }
 
     void rotate_around_z(float angle)
     {
-        float ck = cosf(angle), sk = sinf(angle);
-        ROTVEC(a, x, y);
-        ROTVEC(b, x, y);
-        ROTVEC(c, x, y);
+        float ck = cosf(angle), sk = -sinf(angle);
+        a.rotate_around_z(ck, sk);
+        b.rotate_around_z(ck, sk);
+        c.rotate_around_z(ck, sk);
     }
 
-    #undef ROTVEC
-
     vec transform(const vec &o) const { return vec(a.dot(o), b.dot(o), c.dot(o)); }
     vec transposedtransform(const vec &o) const
     {
@@ -1021,6 +1038,7 @@ struct ivec
     ivec &cross(const ivec &a, const ivec &b) { x = a.y*b.z-a.z*b.y; y = a.z*b.x-a.x*b.z; z = a.x*b.y-a.y*b.x; return *this; }
     int dot(const ivec &o) const { return x*o.x + y*o.y + z*o.z; }
     float dist(const plane &p) const { return x*p.x + y*p.y + z*p.z + p.offset; }
+    static ivec fromcolor(int color) { return ivec(int((color>>16)&0xFF), int((color>>8)&0xFF), int(color&0xFF)); }
 };
 
 static inline bool htcmp(const ivec &x, const ivec &y)
@@ -1399,3 +1417,50 @@ extern vec closestpointcylinder(const vec &center, const vec &start, const vec &
 
 extern const vec2 sincos360[];
 
+struct cvec
+{
+    union
+    {
+        struct { int x, y, z, w; };
+        struct { int r, g, b, a; };
+        int v[4];
+    };
+
+    cvec() {}
+    cvec(const vec &v) : x(int(v.x)), y(int(v.y)), z(int(v.z)), w(255) {}
+    cvec(const vec4 &v) : x(int(v.x)), y(int(v.y)), z(int(v.z)), w(int(v.w)) {}
+
+    cvec(int a, int b, int c) : x(a), y(b), z(c), w(255) {}
+    cvec(int a, int b, int c, int d) : x(a), y(b), z(c) {}
+
+    vec4 tovec4() const { return vec4(x, y, z, w); }
+    int toint() const { return (x>0?1:0) + (y>0?2:0) + (z>0?4:0); }
+
+    int &operator[](int i)       { return v[i]; }
+    int  operator[](int i) const { return v[i]; }
+
+    bool operator==(const cvec &v) const { return x==v.x && y==v.y && z==v.z && w==v.w; }
+    bool operator!=(const cvec &v) const { return x!=v.x || y!=v.y || z!=v.z || w!=v.w; }
+    bool iszero() const { return x==0 && y==0 && z==0 && w==0; }
+    cvec &alpha(int n) { a = n; return *this; }
+    cvec &shl(int n) { x<<= n; y<<= n; z<<= n; w<<= n; return *this; }
+    cvec &shr(int n) { x>>= n; y>>= n; z>>= n; w>>= n; return *this; }
+    cvec &mul(int n) { x *= n; y *= n; z *= n; w *= n; return *this; }
+    cvec &div(int n) { x /= n; y /= n; z /= n; w /= n; return *this; }
+    cvec &add(int n) { x += n; y += n; z += n; w += n; return *this; }
+    cvec &sub(int n) { x -= n; y -= n; z -= n; w -= n; return *this; }
+    cvec &mul(const cvec &v) { x *= v.x; y *= v.y; z *= v.z; w *= v.w; return *this; }
+    cvec &div(const cvec &v) { x /= v.x; y /= v.y; z /= v.z; w /= v.w; return *this; }
+    cvec &add(const cvec &v) { x += v.x; y += v.y; z += v.z; w += v.w; return *this; }
+    cvec &sub(const cvec &v) { x -= v.x; y -= v.y; z -= v.z; w -= v.w; return *this; }
+    cvec &mask(int n) { x &= n; y &= n; z &= n; w &= n; return *this; }
+    cvec &neg() { return mul(-1); }
+    cvec &min(const cvec &o) { x = ::min(x, o.x); y = ::min(y, o.y); z = ::min(z, o.z); w = ::min(w, o.w); return *this; }
+    cvec &max(const cvec &o) { x = ::max(x, o.x); y = ::max(y, o.y); z = ::max(z, o.z); w = ::max(w, o.w); return *this; }
+    cvec &min(int n) { x = ::min(x, n); y = ::min(y, n); z = ::min(z, n); w = ::min(w, n); return *this; }
+    cvec &max(int n) { x = ::max(x, n); y = ::max(y, n); z = ::max(z, n); w = ::max(w, n); return *this; }
+    cvec &abs() { x = ::abs(x); y = ::abs(y); z = ::abs(z); w = ::abs(w); return *this; }
+    static cvec from24c(int color) { return cvec(int((color>>16)&0xFF), int((color>>8)&0xFF), int(color&0xFF), 255); }
+    static cvec from32c(int color) { return cvec(int((color>>24)&0xFF), int((color>>16)&0xFF), int((color>>8)&0xFF), int(color&0xFF)); }
+    static cvec from24ca(int color, int alpha) { return cvec(int((color>>16)&0xFF), int((color>>8)&0xFF), int(color&0xFF), alpha); }
+};
