@@ -1,20 +1,22 @@
 // the interface the game uses to access the engine
 
-extern int kidmode, verbose, curtime, lastmillis, totalmillis, timescale, paused;
+extern int verbose, curtime, lastmillis, totalmillis, timescale, paused;
 extern uint totalsecs;
+extern time_t clocktime;
 extern int servertype, serverport, servermasterport;
 extern char *servermaster, *serverip;
 extern ENetAddress masteraddress;
-extern void fatal(const char *s, ...);
-extern void console(int type, const char *s, ...);
-extern void conoutft(int type, const char *s, ...);
-extern void conoutf(const char *s, ...);
+extern void fatal(const char *s, ...) PRINTFARGS(1, 2);
+extern void console(int type, const char *s, ...) PRINTFARGS(2, 3);
+extern void conoutft(int type, const char *s, ...) PRINTFARGS(2, 3);
+extern void conoutf(const char *s, ...) PRINTFARGS(1, 2);
 
 extern FILE *logfile;
+extern FILE *getlogfile();
 extern void setlogfile(const char *fname);
 extern void closelogfile();
 extern void logoutfv(const char *fmt, va_list args);
-extern void logoutf(const char *fmt, ...);
+extern void logoutf(const char *fmt, ...) PRINTFARGS(1, 2);
 
 #ifdef __GNUC__
 #define _dbg_ fprintf(stderr, "%s:%d:%s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
@@ -22,7 +24,7 @@ extern void logoutf(const char *fmt, ...);
 #define _dbg_ fprintf(stderr, "%s:%d\n", __FILE__, __LINE__);
 #endif
 
-struct namemap { const char *name; uchar id; };
+struct namemap { const char *name; int id; };
 
 extern void lightent(extentity &e, float height = 8);
 extern void lightreaching(const vec &target, vec &color, vec &dir, bool fast = false, extentity *e = 0, float ambient = 0.4f);
@@ -90,12 +92,12 @@ extern bool noedit(bool view = false);
 extern void toggleedit(bool force = true);
 extern void mpeditface(int dir, int mode, selinfo &sel, bool local);
 extern void mpedittex(int tex, int allfaces, selinfo &sel, bool local);
-extern void mpeditmat(int matid, int style, selinfo &sel, bool local);
+extern void mpeditmat(int matid, int filter, int style, selinfo &sel, bool local);
 extern void mpflip(selinfo &sel, bool local);
 extern void mpcopy(editinfo *&e, selinfo &sel, bool local);
 extern void mppaste(editinfo *&e, selinfo &sel, bool local);
 extern void mprotate(int cw, selinfo &sel, bool local);
-extern void mpreplacetex(int oldtex, int newtex, selinfo &sel, bool local);
+extern void mpreplacetex(int oldtex, int newtex, bool insel, selinfo &sel, bool local);
 extern void mpdelcube(selinfo &sel, bool local);
 extern void mpremip(bool local);
 
@@ -162,7 +164,7 @@ extern bool pushfont(const char *name);
 extern bool popfont(int num = 1);
 extern int draw_text(const char *str, int rleft, int rtop, int r = 255, int g = 255, int b = 255, int a = 255, int flags = TEXT_SHADOW, int cursor = -1, int maxwidth = -1);
 extern int draw_textx(const char *fstr, int left, int top, int r = 255, int g = 255, int b = 255, int a = 255, int flags = TEXT_SHADOW, int cursor = -1, int maxwidth = -1, ...);
-extern int draw_textf(const char *fstr, int left, int top, ...);
+extern int draw_textf(const char *fstr, int left, int top, ...) PRINTFARGS(1, 4);
 extern float text_widthf(const char *str, int flags = 0);
 extern void text_boundsf(const char *str, float &width, float &height, int maxwidth = -1, int flags = 0);
 extern int text_visible(const char *str, float hitx, float hity, int maxwidth = -1, int flags = 0);
@@ -198,9 +200,8 @@ enum
     DL_KEEP   = 1<<3
 };
 
-extern void adddynlight(const vec &o, float radius, const vec &color, int fade = 0, int peak = 0, int flags = 0, float initradius = 0, const vec &initcolor = vec(0, 0, 0), physent *owner = NULL);
+extern void adddynlight(const vec &o, float radius, const vec &color, int fade = 0, int peak = 0, int flags = 0, float initradius = 0, const vec &initcolor = vec(0, 0, 0));
 extern void dynlightreaching(const vec &target, vec &color, vec &dir);
-extern void removetrackeddynlights(physent *owner);
 extern void makelightfx(extentity &e, extentity &f);
 
 // rendergl
@@ -322,24 +323,6 @@ extern void regularsplash(int type, int color, float radius, int num, int fade,
 extern void regularshape(int type, float radius, int color, int dir, int num, int fade, const vec &p, float size = 2, float blend = 1, int grav = 0, int collide = 0, float vel = 1);
 extern void regularflame(int type, const vec &p, float radius, float height, int color, int density = 3, int fade = 500, float size = 2, float blend = 1, int grav = -1, int collide = 0, float vel = 1);
 
-#if 0
-enum { PARTTYPE_CREATE = 0, PARTTYPE_SPLASH, PARTTYPE_SHAPE, PARTTYPE_FIRE, PARTTYPE_FLAME, PARTTYPE_ICON, PARTTYPE_FLARE, PARTTYPE_MAX };
-
-struct mapparticle
-{
-    const char *name;
-    int part, type, colour, grav, fade, attr;
-    float radius, height, size, blend, vel;
-
-    mapparticle() : name(NULL) {}
-    ~mapparticle()
-    {
-        if(name) delete name;
-    }
-};
-extern vector<mapparticle> mapparts;
-#endif
-
 // decal
 enum
 {
@@ -404,15 +387,13 @@ extern void setbbfrommodel(dynent *d, const char *mdl, float size = 1);
 extern bool validragdoll(dynent *d, int millis);
 extern void moveragdoll(dynent *d, bool smooth);
 extern void cleanragdoll(dynent *d);
-extern vec ragdollcenter(dynent *d);
+extern void warpragdoll(dynent *d, const vec &vel, const vec &offset = vec(0, 0, 0));
 
 // server
 #define MAXCLIENTS 256                  // in a multiplayer game, can be arbitrarily changed
 #define MAXTRANS 5000                  // max amount of data to swallow in 1 go
 
-extern int maxclients;
-
-enum { DISC_NONE = 0, DISC_EOP, DISC_CN, DISC_KICK, DISC_TAGT, DISC_IPBAN, DISC_PRIVATE, DISC_MAXCLIENTS, DISC_TIMEOUT, DISC_OVERFLOW, DISC_SHUTDOWN, DISC_NUM };
+enum { DISC_NONE = 0, DISC_EOP, DISC_CN, DISC_KICK, DISC_MSGERR, DISC_IPBAN, DISC_PRIVATE, DISC_MAXCLIENTS, DISC_TIMEOUT, DISC_OVERFLOW, DISC_SHUTDOWN, DISC_NUM };
 
 extern void *getinfo(int i);
 extern const char *gethostname(int i);
@@ -420,25 +401,10 @@ extern void sendf(int cn, int chan, const char *format, ...);
 extern void sendfile(int cn, int chan, stream *file, const char *format = "", ...);
 extern void sendpacket(int cn, int chan, ENetPacket *packet, int exclude = -1);
 extern void flushserver(bool force);
+extern int getservermtu();
 extern int getnumclients();
 extern uint getclientip(int n);
-extern void putint(ucharbuf &p, int n);
-extern void putint(packetbuf &p, int n);
-extern void putint(vector<uchar> &p, int n);
-extern int getint(ucharbuf &p);
-extern void putuint(ucharbuf &p, int n);
-extern void putuint(packetbuf &p, int n);
-extern void putuint(vector<uchar> &p, int n);
-extern int getuint(ucharbuf &p);
-extern void putfloat(ucharbuf &p, float f);
-extern void putfloat(packetbuf &p, float f);
-extern void putfloat(vector<uchar> &p, float f);
-extern float getfloat(ucharbuf &p);
-extern void sendstring(const char *t, ucharbuf &p);
-extern void sendstring(const char *t, packetbuf &p);
-extern void sendstring(const char *t, vector<uchar> &p);
-extern void getstring(char *t, ucharbuf &p, int len = MAXTRANS);
-extern void filtertext(char *dst, const char *src, bool newline = true, bool colour = true, bool whitespace = true, int len = sizeof(string)-1);
+extern bool filtertext(char *dst, const char *src, bool newline = true, bool colour = true, bool whitespace = true, int len = sizeof(string)-1);
 extern void disconnect_client(int n, int reason);
 extern void kicknonlocalclients(int reason);
 extern bool hasnonlocalclients();
@@ -450,7 +416,7 @@ extern bool connectedmaster();
 extern ENetSocket connectmaster(bool reuse = true);
 extern void disconnectmaster();
 extern bool requestmaster(const char *req);
-extern bool requestmasterf(const char *fmt, ...);
+extern bool requestmasterf(const char *fmt, ...) PRINTFARGS(1, 2);
 
 extern bool findoctadir(const char *name, bool fallback = false);
 extern void trytofindocta(bool fallback = true);
@@ -564,13 +530,13 @@ struct guient
     virtual void end() = 0;
 
     virtual int text(const char *text, int color, const char *icon = NULL, int icolor = 0xFFFFFF) = 0;
-    int textf(const char *fmt, int color, const char *icon = NULL, int icolor = 0xFFFFFF, ...)
+    int textf(const char *fmt, int color, const char *icon = NULL, int icolor = 0xFFFFFF, ...) PRINTFARGS(2, 6)
     {
         defvformatstring(str, icolor, fmt);
         return text(str, color, icon, icolor);
     }
     virtual int button(const char *text, int color, const char *icon = NULL, int icolor = 0xFFFFFF, bool faded = true) = 0;
-    int buttonf(const char *fmt, int color, const char *icon = NULL, int icolor = 0xFFFFFF, bool faded = true, ...)
+    int buttonf(const char *fmt, int color, const char *icon = NULL, int icolor = 0xFFFFFF, bool faded = true, ...) PRINTFARGS(2, 7)
     {
         defvformatstring(str, faded, fmt);
         return button(str, color, icon, icolor, faded);
@@ -580,8 +546,8 @@ struct guient
     virtual void pushlist(bool merge = false) {}
     virtual int poplist() { return 0; }
 
-    virtual void allowautotab(bool on) = 0;
     virtual void allowhitfx(bool on) = 0;
+    virtual bool visible() = 0;
     virtual bool shouldtab() { return false; }
     virtual void tab(const char *name = NULL, int color = 0xFFFFFF, bool front = false) = 0;
     virtual int title(const char *text, int color = 0xFFFFFF, const char *icon = NULL, int icolor = 0xFFFFFF) = 0;
@@ -596,9 +562,11 @@ struct guient
     virtual void strut(float size = 1) = 0;
     virtual void space(float size = 1) = 0;
     virtual void spring(int weight = 1) = 0;
-    virtual char *field(const char *name, int color, int length, int height = 0, const char *initval = NULL, int initmode = EDITORFOCUSED) = 0;
-    virtual char *keyfield(const char *name, int color, int length, int height = 0, const char *initval = NULL, int initmode = EDITORFOCUSED) = 0;
-    virtual int playerpreview(int model, int color, int team, int weap, float scale, bool overlaid = false) { return 0; }
+    virtual char *field(const char *name, int color, int length, int height = 0, const char *initval = NULL, int initmode = EDITORFOCUSED, bool focus = false, const char *parent = NULL) = 0;
+    virtual char *keyfield(const char *name, int color, int length, int height = 0, const char *initval = NULL, int initmode = EDITORFOCUSED, bool focus = false, const char *parent = NULL) = 0;
+    virtual int playerpreview(int model, int color, int team, int weap, const char *vanity, float sizescale, bool overlaid = false, float scale = 1, float blend = 1) { return 0; }
+    virtual int modelpreview(const char *name, int anim, float sizescale, bool overlaid = false, float scale = 1, float blend = 1) { return 0; }
+
 };
 
 struct guicb
@@ -623,7 +591,7 @@ namespace UI
     extern bool hit(bool on, bool act);
     extern void addcb(guicb *cb);
     extern void limitscale(float scale);
-    extern editor *geteditor(const char *name, int mode, const char *init = NULL);
+    extern editor *geteditor(const char *name, int mode, const char *init = NULL, const char *parent = NULL);
     extern void editorline(editor *e, const char *str, int limit = -1);
     extern void editorclear(editor *e, const char *init = "");
     extern void editoredit(editor *e);
@@ -666,13 +634,15 @@ enum
 
 enum
 {
-    MATF_VOLUME_SHIFT = 0,
-    MATF_CLIP_SHIFT   = 3,
-    MATF_FLAG_SHIFT   = 5,
+    MATF_INDEX_SHIFT  = 0,
+    MATF_VOLUME_SHIFT = 2,
+    MATF_CLIP_SHIFT   = 5,
+    MATF_FLAG_SHIFT   = 8,
 
+    MATF_INDEX  = 3 << MATF_INDEX_SHIFT,
     MATF_VOLUME = 7 << MATF_VOLUME_SHIFT,
-    MATF_CLIP   = 3 << MATF_CLIP_SHIFT,
-    MATF_FLAGS  = 7 << MATF_FLAG_SHIFT
+    MATF_CLIP   = 7 << MATF_CLIP_SHIFT,
+    MATF_FLAGS  = 0xFF << MATF_FLAG_SHIFT
 };
 
 enum // cube empty-space materials
