@@ -15,11 +15,7 @@ typedef signed long long int llong;
 typedef unsigned long long int ullong;
 
 #ifdef _DEBUG
-#ifdef __GNUC__
-#define ASSERT(c) if(!(c)) { asm("int $3"); }
-#else
-#define ASSERT(c) if(!(c)) { __asm int 3 }
-#endif
+#define ASSERT(c) assert(c)
 #else
 #define ASSERT(c) if(c) {}
 #endif
@@ -131,6 +127,12 @@ static inline T clamp(T a, T b, T c)
 #define PATHDIV '/'
 #endif
 
+#ifdef __GNUC__
+#define PRINTFARGS(fmt, args) __attribute__((format(printf, fmt, args)))
+#else
+#define PRINTFARGS(fmt, args)
+#endif
+
 // easy safe strings
 
 #define MAXSTRLEN 512 // must be at least 512 bytes to comply with rfc1459
@@ -145,14 +147,14 @@ struct stringformatter
 {
     char *buf;
     stringformatter(char *buf): buf((char *)buf) {}
-    void operator()(const char *fmt, ...)
+    void operator()(const char *fmt, ...) PRINTFARGS(2, 3)
     {
         va_list v;
         va_start(v, fmt);
         vformatstring(buf, fmt, v);
         va_end(v);
     }
-    void operator()(int len, const char *fmt, ...)
+    void operator()(int len, const char *fmt, ...) PRINTFARGS(3, 4)
     {
         va_list v;
         va_start(v, fmt);
@@ -169,6 +171,8 @@ struct stringformatter
 #define loopvj(v)   for(int j = 0; j<(v).length(); j++)
 #define loopvk(v)   for(int k = 0; k<(v).length(); k++)
 #define loopvrev(v) for(int i = (v).length()-1; i>=0; i--)
+#define loopvjrev(v) for(int j = (v).length()-1; j>=0; j--)
+#define loopvkrev(v) for(int k = (v).length()-1; k>=0; k--)
 
 template <class T>
 struct databuf
@@ -225,6 +229,15 @@ struct databuf
         return read;
     }
 
+    void offset(int n)
+    {
+        n = min(n, maxlen);
+        buf += n;
+        maxlen -= n;
+        len = max(len-n, 0);
+    }
+
+    bool empty() const { return len==0; }
     int length() const { return len; }
     int remaining() const { return maxlen-len; }
     bool overread() const { return (flags&OVERREAD)!=0; }
@@ -395,6 +408,40 @@ template<class T> struct isclass
     enum { yes = sizeof(test<T>(0)) == 1 ? 1 : 0, no = yes^1 };
 };
 
+static inline uint hthash(const char *key)
+{
+    uint h = 5381;
+    for(int i = 0, k; (k = key[i]); i++) h = ((h<<5)+h)^k;    // bernstein k=33 xor
+    return h;
+}
+
+static inline bool htcmp(const char *x, const char *y)
+{
+    return !strcmp(x, y);
+}
+
+static inline uint hthash(int key)
+{
+    return key;
+}
+
+static inline bool htcmp(int x, int y)
+{
+    return x==y;
+}
+
+#ifndef STANDALONE
+static inline uint hthash(GLuint key)
+{
+    return key;
+}
+
+static inline bool htcmp(GLuint x, GLuint y)
+{
+    return x==y;
+}
+#endif
+
 template <class T> struct vector
 {
     static const int MINSIZE = 8;
@@ -776,6 +823,25 @@ template <class T> struct smallvector
         return e;
     }
 
+    void drop() { buf[len-1].~T(); growbuf(len-1); }
+
+    T &insert(int i, const T &e)
+    {
+        add(T());
+        for(int p = len-1; p>i; p--) buf[p] = buf[p-1];
+        buf[i] = e;
+        return buf[i];
+    }
+
+    T *insert(int i, const T *e, int n)
+    {
+        growbuf(len+n);
+        loopj(n) add(T());
+        for(int p = len-1; p>=i+n; p--) buf[p] = buf[p-n];
+        loopj(n) buf[i+j] = e[j];
+        return &buf[i];
+    }
+
     bool inrange(size_t i) const { return i<size_t(len); }
     bool inrange(int i) const { return i>=0 && i<len; }
 
@@ -794,30 +860,15 @@ template <class T> struct smallvector
         loopi(len) if(buf[i]==o) return i;
         return -1;
     }
+
+    template<class K>
+    int htfind(const K &key)
+    {
+        loopi(len) if(htcmp(key, buf[i])) return i;
+        return -1;
+    }
 };
 
-static inline uint hthash(const char *key)
-{
-    uint h = 5381;
-    for(int i = 0, k; (k = key[i]); i++) h = ((h<<5)+h)^k;    // bernstein k=33 xor
-    return h;
-}
-
-static inline bool htcmp(const char *x, const char *y)
-{
-    return !strcmp(x, y);
-}
-
-static inline uint hthash(int key)
-{
-    return key;
-}
-
-static inline bool htcmp(int x, int y)
-{
-    return x==y;
-}
-
 template<class T> struct hashset
 {
     typedef T elem;
@@ -1225,7 +1276,7 @@ struct stream
     virtual bool getline(char *str, int len);
     virtual bool putstring(const char *str) { int len = (int)strlen(str); return write(str, len) == len; }
     virtual bool putline(const char *str) { return putstring(str) && putchar('\n'); }
-    virtual int printf(const char *fmt, ...);
+    virtual int printf(const char *fmt, ...) PRINTFARGS(2, 3);
     virtual uint getcrc() { return 0; }
 
     template<class T> int put(const T *v, int n) { return write(v, n*sizeof(T))/sizeof(T); }
@@ -1285,8 +1336,8 @@ static inline uchar uni2cube(int c)
     extern const uchar uni2cubechars[];
     return uint(c) <= 0x7FF ? uni2cubechars[uni2cubeoffsets[c>>8] + (c&0xFF)] : 0;
 }
-extern int decodeutf8(uchar *dst, int dstlen, uchar *src, int srclen, int *carry = NULL);
-extern int encodeutf8(uchar *dstbuf, int dstlen, uchar *srcbuf, int srclen, int *carry = NULL);
+extern int decodeutf8(uchar *dst, int dstlen, const uchar *src, int srclen, int *carry = NULL);
+extern int encodeutf8(uchar *dstbuf, int dstlen, const uchar *srcbuf, int srclen, int *carry = NULL);
 
 extern char *makerelpath(const char *dir, const char *file, const char *prefix = NULL, const char *cmd = NULL);
 extern char *makefile(const char *s, const char *e = "", int revision = 0, int start = 1, bool store = false, bool skip = false);
@@ -1314,7 +1365,25 @@ extern void backup(const char *fname, const char *ext, int revision = 0, int sta
 
 extern void endianswap(void *, int, int);
 extern void seedMT(uint seed);
-extern uint randomMT(void);
+extern uint randomMT();
+extern int guessnumcpus();
+extern void putint(ucharbuf &p, int n);
+extern void putint(packetbuf &p, int n);
+extern void putint(vector<uchar> &p, int n);
+extern int getint(ucharbuf &p);
+extern void putuint(ucharbuf &p, int n);
+extern void putuint(packetbuf &p, int n);
+extern void putuint(vector<uchar> &p, int n);
+extern int getuint(ucharbuf &p);
+extern void putfloat(ucharbuf &p, float f);
+extern void putfloat(packetbuf &p, float f);
+extern void putfloat(vector<uchar> &p, float f);
+extern float getfloat(ucharbuf &p);
+extern void sendstring(const char *t, ucharbuf &p);
+extern void sendstring(const char *t, packetbuf &p);
+extern void sendstring(const char *t, vector<uchar> &p);
+extern void getstring(char *t, ucharbuf &p, int len);
+template<size_t N> static inline void getstring(char (&t)[N], ucharbuf &p) { getstring(t, p, int(N)); }
 
 #endif
 
